//------------------------------------------------------------------------------
// Projektname:		Module Datenstruktur
// Dateiname:		CFahrplanElement.cpp
// Beschreibung:	Klasse fuer ein Fahrplanelement, welches vom Fahrplanmodul
//                      an das Programmmodul uebergeben werden muss
// Autor(en):		Mike Sarnowski
// Version:		0.1
// Gendert von:	Mike Sarnowski
// Gendert am:		23.01.2001
// Zustand:		Erste Implementierung
//------------------------------------------------------------------------------

#include "CFahrplanElement.h"

//------------------------------------------------------------------------------
// Funktionsname:	CFahrplanElement
// Beschreibung:	Default Konstruktor
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
CFahrplanElement::CFahrplanElement(){
  this->haltestellenID = 0;
  this->linienID = 0;
  this->startHaltestelleID = 0;
  this->abfahrtszeitStunde = 0;
  this->abfahrtszeitMinute = 0;
}

//------------------------------------------------------------------------------
// Funktionsname:	CFahrplanElement
// Beschreibung:	Ueberladener Konstruktor. Fuellt das anzulegende Objekt mit allen Daten
// Parameter:		haltestellenID, linienID, startHaltestellenID, stunde, minute
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
CFahrplanElement::CFahrplanElement(int haltestellenID,int linienID,int startHaltestellenID,int stunde,int minute){
  this->haltestellenID = haltestellenID;
  this->linienID = linienID;
  this->startHaltestelleID = startHaltestellenID;
  this->abfahrtszeitStunde = stunde;
  this->abfahrtszeitMinute = minute;
}

//------------------------------------------------------------------------------
// Funktionsname:	CFahrplanElement
// Beschreibung:	Kopier - Konstruktor
// Parameter:		Das originale Objekt, welches kopiert werden soll
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
CFahrplanElement::CFahrplanElement(const CFahrplanElement& originalObjekt){
  this->haltestellenID = originalObjekt.haltestellenID;
  this->linienID = originalObjekt.linienID;
  this->startHaltestelleID = originalObjekt.startHaltestelleID;
  this->abfahrtszeitStunde = originalObjekt.abfahrtszeitStunde;
  this->abfahrtszeitMinute = originalObjekt.abfahrtszeitMinute;  
}

//------------------------------------------------------------------------------
// Funktionsname:	operator=
// Beschreibung:	= Operator
// Parameter:		Das originale Objekt, welches kopiert werden soll
// Rueckgabe:		Das kopierte Objekt
// Besonderheiten:	
//------------------------------------------------------------------------------
CFahrplanElement CFahrplanElement::operator= (const CFahrplanElement& originalObjekt){
  if (&originalObjekt!=this) {
    this->haltestellenID = originalObjekt.haltestellenID;
    this->linienID = originalObjekt.linienID;
    this->startHaltestelleID = originalObjekt.startHaltestelleID;
    this->abfahrtszeitStunde = originalObjekt.abfahrtszeitStunde;
    this->abfahrtszeitMinute = originalObjekt.abfahrtszeitMinute;  
  }
  return *this;
}

//------------------------------------------------------------------------------
// Funktionsname:	operator>>
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
istream& operator>> (istream& is, CFahrplanElement& originalFahrplan){
  return is;
}

//------------------------------------------------------------------------------
// Funktionsname:	operator<<
// Beschreibung:	Es werden die einzelnen Daten ausgegeben
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
ostream& operator<< (ostream& os, const CFahrplanElement& originalFahrplan){
  os << "Haltestelle: " << originalFahrplan.haltestellenID << ", Linie: " << originalFahrplan.linienID \
     << ", Linien Starthaltestelle: " << originalFahrplan.startHaltestelleID \
     << ", Abfahrtszeit: " << originalFahrplan.abfahrtszeitStunde << ":" << originalFahrplan.abfahrtszeitMinute;
  return os;
}

//------------------------------------------------------------------------------
// Funktionsname:	setzenHaltestelle
// Beschreibung:	Setzt das Attribut HaltestellenID
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CFahrplanElement::setzenHaltestelle(int haltestellenID){
  this->haltestellenID = haltestellenID;
}

//------------------------------------------------------------------------------
// Funktionsname:	setzenLinie
// Beschreibung:	Setzt das Attribut linienID
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CFahrplanElement::setzenLinie(int linienID){
  this->linienID = linienID;
}
    
//------------------------------------------------------------------------------
// Funktionsname:	setzenLinienStartHaltestell
// Beschreibung:	Setzt das Attribut satrtHaltestellenID
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CFahrplanElement::setzenLinienStartHaltestelle(int linienStartID){
  this->startHaltestelleID = linienStartID;
}

//------------------------------------------------------------------------------
// Funktionsname:	setzenAbfahrtszeitStunde
// Beschreibung:	Setzt das Attribut abfahrtszeitStunde
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CFahrplanElement::setzenAbfahrtszeitStunde(int stunde){
  this->abfahrtszeitStunde = stunde;
}
    
//------------------------------------------------------------------------------
// Funktionsname:	setzenAbfahrtszeitMinute
// Beschreibung:	Setzt das Attribut abfahrtszeitMinute
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CFahrplanElement::setzenAbfahrtszeitMinute(int minute){
  this->abfahrtszeitMinute = minute;
}

//------------------------------------------------------------------------------
// Funktionsname:	gebenHaltestelle
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
int CFahrplanElement::gebenHaltestelle(){
  return this->haltestellenID;
}

//------------------------------------------------------------------------------
// Funktionsname:	gebenLinie
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
int CFahrplanElement::gebenLinie(){
  return this->linienID;
}
    
//------------------------------------------------------------------------------
// Funktionsname:	gebenLinienStartHaltestelle
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
int CFahrplanElement::gebenLinienStartHaltestelle(){
  return this->startHaltestelleID;
}

//------------------------------------------------------------------------------
// Funktionsname:	gebenAbfahrtszeitStunde
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
int CFahrplanElement::gebenAbfahrtszeitStunde(){
  return this->abfahrtszeitStunde;
}
    
//------------------------------------------------------------------------------
// Funktionsname:	gebenAbfahrtszeitMinute
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
int CFahrplanElement::gebenAbfahrtszeitMinute(){
  return this->abfahrtszeitMinute;
}
