
#ifndef CFORTSCHRITT_H
#define CFORTSCHRITT_H

#include <LEDA/rat_point.h>
#include <LEDA/window.h>
#include <LEDA/list.h>
#include "Debug.h"

#define NR_SAMPLES 30
#define LAENGE 100

class CFortschritt {
public:
  CFortschritt(const char *titel="Vorgang wird bearbeitet...",bool batch=false);
  ~CFortschritt();

  /**
   * Zeigt den Fortschritt des aktuellen Arbeitsschrittes an.
   *
   * @param aktuell Aktuell erreichter Wert. Muss groesser oder gleich Null sein.
   * @param maximum Maximal zu erreichender Wert. Muss groesser als Null sein.
   */
  void anzeigenFortschritt(unsigned long aktuell, unsigned long maximum);
  void anzeigenFortschrittSteigenderAufwand(unsigned long aktuell, unsigned long maximum);

  /**
   * Meldet Programmaktivitaet in Form eines kleinen Rotators.
   */
  void anzeigenAktivitaet(void);


  /**
    * Gibt eine Meldung fuer den Benutzer aus. dient dem detaillierten
    * kommentieren von Teilschritten.
    */
  void meldenLaufzeitAusgabe(char*);

  /**
   * Dient dem Aufnehmen von Teilschriiten, welche zur Visualisierung
   * eines Gesamtfortschrittes herangezogen werden.
   */
  void setzenTeilschritte(unsigned int anzahl, char ** Teilschritte);

  /**
   * Setzt den aktuellen Teuilschritt.
   *
   * Die ParameterloseVariante aktiviert den naechsten Schritt, wohingegen
   * die Parameterbehaftete den entsprechenden Array-Eintrag hervorhebt.
   */
  void setzenNaechstenSchritt(void);
  void setzenNaechstenSchritt(unsigned int Schritt);

  /**
   * Berechnet die Summe von 1 bis Hoechstwert.
   */
  unsigned long int Summe(unsigned long int Hoechstwert);

private:
  bool BatchModus;
  window *W;
  unsigned int Status;
  struct timeval *zeit;
  unsigned long timeold;
  list<string> schritte;
  unsigned int arbeitsSchritt;
  int Promille;
  panel_item FortschrittLabel;
  string LaufzeitMeldung;
  panel_item schrittFeld[5];
  string FortschrittAnalyse;
  char ** Teilschritte;
  unsigned int AnzahlTeilschritte;
  unsigned long samples[NR_SAMPLES][2];
  int current_sample;
  void reset_samples();
  unsigned long ElapsedTime;
  unsigned long LastUpdateTime;
  unsigned long CurrentTime;
  int LastUpdateProgress;
  unsigned long ETA;
  unsigned long do_prediction(unsigned long time, int fortschritt);
  void neuzeichnen(bool zwingen=false);
};

#endif // CFORTSCHRITT_H
