#ifndef CGUETEMASSE_H
#define CGUETEMASSE_H

#include <LEDA/string.h>
#include <LEDA/h_array.h>
#include <LEDA/tuple.h>
#include <LEDA/map2.h>
#include <CReisebewertung.h>
#include <LEDA/matrix.h>
#include "Debug.h"

/**
 * In dieser Klasse werden die Guetemasse aller Module abgelegt.
 */
class CGuetemasse {
	
public:
	/**
	 * Standardkonstruktor.
	 */
	CGuetemasse();
	
	/**
	 * Destruktor
	 */
	~CGuetemasse();

// Haltestellen
private:    
    
	double bewertung;
	int anzahlHaltestellen; // Anzahl Haltestellen im Wegenetzgraph
	double bevoelkerungsUeberversorgung;
	double bevoelkerungsUnterversorgung;
	double flaechenUeberversorgung;
	double flaechenUnterversorgung;
	double flaechenVersorgunginProzent;
	double bevoelkerungsVersorgunginProzent;
	double fusswegVonHS;
	matrix * quelleZielMatrix;
	
	//die map2 - Datenstruktur hat die Form <HSnr, HSnr, ReisezeitHS2HSAuto>
	map2<int, int, double > * ReisezeitHS2HSAuto;
public:
	/**
	 * Liest die Gesamtbewertung. Dabei ist sie die Summe
	 * aus den Einzelbewertungen.
	 */
	double holenHaltestellenBewertung();	

	void speichernAnzahlHaltestellen(int);
	int holenAnzahlHaltestellen();

	void speichernBevoelkerungsUeberversorgung(double);
	double holenBevoelkerungsUeberversorgung();

	void speichernBevoelkerungsUnterversorgung(double);
	double holenBevoelkerungsUnterversorgung();

	void speichernFlaechenUeberversorgung(double);
	double holenFlaechenUeberversorgung();

	void speichernFlaechenUnterversorgung(double);
	double holenFlaechenUnterversorgung();

	void speichernFlaechenVersorgunginProzent(double);
	double holenFlaechenVersorgunginProzent();

	void speichernBevoelkerungsVersorgunginProzent(double);
	double holenBevoelkerungsVersorgunginProzent();
	
	/**
	 * Der durchschnittliche Fussweg pro Haltestelle
	 */
	void speichernFusswegProHaltestelle(double);
	double holenFusswegProHaltestelle(void);
	
	/**
	 * die direkte Reisezeit von Haltestelle zu Haltestelle (mit dem Auto)
	 * Methode fuer das Programm
	 */
	void speichernReisezeitHS2HSAuto(map2<int, int, double > *);
	/**
	 * diese Methode wird vom Haltestellen-Modul benoetigt. Es werden die ENTFERNUNGEN
	 * von jeder Haltestelle zu jeder anderen gesetzt. Die Entfernungen sind die kuerzesten Wege.
	 * Diese Methode berechnet daraus die Reisezeit von jeder Haltestelle zu jeder anderen.
	 */
	void setzenReisezeitHS2HSAuto(map2<int, int, double > *);
	map2<int, int, double > * holenReisezeitHS2HSAuto(void);
	
	/**
	 * Setzen-Methode fuer die Quelle-Ziel-Matrix. Diese wird fuer die Berechnung der Personenanzahl pro Takt benoetigt
	 * Diese Methode wird von den Haltestellen aufgerufen, sodass die Quelle Ziel-Matrix die aktuell von den Haltestellen
	 * berechneten Werte enthaelt.
	 */
	void setzenQuelleZielMatrix(matrix *);
	
// Linienplan
private:
	int anzahlUmstiege;
	double anzahlVerbindungenmitUmstiegzwang;
	int gesamtfahrer;
	int direktfahrer;
	int mittlereReisezeit;
	double prozentsatzVerbindungendiekeineDirektfahrten;
	double Direktfahreranteil;
	int Streckenkilometer; //des Linienplans
	int gesamteStreckenkilometer;
	int mittlereMindestreisezeit;
	double Reisezeitverlaengerung; // Wirkung zw. Reisezeit und mittlerer Mindestreisezeit
	double mittlererVerknuepfungsgrad;
	
	int linienAnzahl;
public:
	/**
	 * Speichert und ldt die Anzahl der Umstiege im gesamten Linienplan.
	 */
	void speichernAnzahlUmstiege(int anzUmstiege);
        	int holenAnzahlUmstiege();

	/**
	 * Speichert und ldt die durchgschnittliche Anzahl der Umstiege.
	 */
	void speichernAnzahlVerbindungenmitUmstiegzwang(double verbindungenmitUmstiegezwang);
	double holenAnzahlVerbindungenmitUmstiegzwang();
	
	/**
	 * die Gesamtfahrer
	 */
	void speichernGesamtfahrer(int gesFahrer);
	int holenGesamtfahrer();

	/**
	 * Speichert und ldt die Anzahl der Direktfahrer.
	 */
	void speichernDirektfahrer(int dirFahrer);
	int holenDirektfahrer();

	/**
	 * Speichert und ldt die mittlere Reisezeit.
	 */
	void speichernMittlereReisezeit(int mittlereReisezt);
	int holenMittlereResezeit();

	/**
	 * Speichert und ldt den Anteil der Verbindungen mit Umstiegszwang.
	 */
	void speichernProzentsatzVerbindungendiekeineDirektfahrten(double prozentVerbdiekeineDirektfahrten);
	double holenProzentsatzVerbindungendiekeineDirektfahrten();

	/**
	 * Der Anteil der Direktfahrer unter allen Fahrern.
	 */
	void speichernDirektfahreranteil(double direktFahrerAnt);
	double holenDirektfahreranteil();

	/**
	 * Die Streckenkilometer des Liniennetzes.
	 */
	void speichernStreckenkilometer(int streckenkm);
	int holenStreckenkilometer();

	/**
	 * Die Gesamtstreckenkilometer
	 */
	void speichernGesamteStreckenkilometer(int gesamteStreckenkm);
	int holenGesamteStreckenkilometer();

	/**
	 * Die mittlere Mindestreisezeit
	 */
	void speichernMittlereMindestreisezeit(int mittleremindestreisezt);
	int holenMittlereMindestreisezeit();

	/**
	 * Die Reisezeitverlaengerung
	 */
	void speichernReisezeitverlaengerung(double reiseztverlaengerung);
	double holenReisezeitverlaengerung();
	
	/**
	 * Anzahl der Linien insgesamt
	 */
	void setzenLinienanzahl(int);
	int holenLinienanzahl(void);
		
	/**
	 * Der mittlere Verknuepfungsgrad
	 */
	void speichernMittlerenVerknuepfungsgrad(double);
	double holenMittlerenVerknuepfungsgrad(void);

// Fahrplan
private:
	double anzahlBusse;
	double fahrplanwirkungsgrad;
	double gesamtZeitaufwand;
	double netzbedingteWartezeiten;
	double wartezeitaufwand;
	double  mindestwartezeitaufwand;
	int fahrgastfahrten;
	
public:
	void speichernAnzahlBusse(double);
	double holenAnzahlBusse(void);
	
	void speichernNetzbedingteWartezeitenGesamt(double);
	double holenNetzbedingteWartezeitenGesamt(void);
	
	void speichernWartezeiteAufwandGesamt(double);
	double holenWartezeitenAufwandGesamt(void);
	
	void speichernMindestwartezeitaufwand(double);
	double holenMindestwartezeitaufwand(void);
	
	void speichernGesamtzeitaufwand(double);
	double holenGesamtzeitaufwand(void);
	
	void speichernFahrplanwirkungsgrad(double);
	double holenFahrplanwirkungsgrad(void);

	void speichernFahrgastfahrten(int);
	int holenFahrgastfahrten(void);
	
// Global ermittelte Guetemasse
private:
	double durchschnittlicheAnzahlBusse;
	double mittlereReisezeitGlobal;
	double anzahlPersonenProBus;	
	double reisezeitZuDirektfahrer;
	double mindestreisezeitZuDirektfahrer;
	double fahrplanintervall;
		
	//Das three_tuple hat die Struktur <LinienNrVon, HSNr, LinienNrNach>
	h_array< three_tuple<int, int, int>*, int > *  NetzbedingtenWartezeitenArray;
	
	//die map2 - Datenstruktur hat die Form <anzahlHS, anzahlHS, CReisebewertung> und wird initialisiert
	//sobald die Anzahl der Haltestellen gesetzt wird
	map2<int, int, CReisebewertung > * reisebewertung;
	
public:

	/**
	 * Setzt das Fahrplaninterval
	 */
	void setzenFahrplanintervall(double);
	
	
	/**
	 * Setzt die durchschnittliche Anzahl an Bussen pro Linie
	 */
	void speichernDurchschnittlicheAnzahlBusse(double);
	
	/**
	 * Gibt die durchschnittliche Anzahl an Bussen pro Linie zurueck
	 */
	double holenDurchschnittlicheAnzahlBusse(void);
	
	/**
	 * Setzt die mittlere Reisezeit (Fussweg, Mindestreisezeit, Wartezeit)
                  */
                 void speichernMittlereReisezeit(double);

                 /**
	* Gibt die mittlere Reisezeit (Fussweg, Fahrtreisezeit, Wartezeit) zurueck
                 */
              	double holenMittlereReisezeit(void);

 	/**
	*  Setzt die durchschnittliche Anzahl Personen pro Bus in einem Takt
	*/
	void speichernAnzahlPersonen(double);
			
	/**
	 * Gibt die durchschnittliche Anzahl Personen pro Bus in einem Takt zurueck
	 */
	double holenAnzahlPersonen(void);
	
	/**
	 * Verhaeltnis der mittleren Reisezeit im Vergleich zur Direktfahrt (mit Auto)
	 * Das Ergebnis ist als Prozentwert zu interpretieren
	 */
	void speichernReisezeitZuDirektfahrer(double);
	
	/**
	 * Verhaeltnis der mittleren Reisezeit im Vergleich zur Direktfahrt (mit Auto)
	 * Das Ergebnis ist als Prozentwert zu interpretieren
	 */
	double holenReisezeitZuDirektfahrer(void);
	
	/**
	 * Verhaeltnis der mittleren Mindestreisezeit im Vergleich zur Direktfahrt (mit Auto)
	 * Das Ergebnis ist als Prozentwert zu interpretieren
	 */
	void speichernMindestreisezeitZuDirektfahrer(double);
	
	/**
	 * Verhaeltnis der mittleren Mindestreisezeit im Vergleich zur Direktfahrt (mit Auto)
	 * Das Ergebnis ist als Prozentwert zu interpretieren
	 */
	double holenMindestreisezeitZuDirektfahrer(void);
	
	
	//-----------------Vom Fahrplan gebraucht:-----------------	
	
	/**
	 * Erzeugen eines HashArrays fuer die Netzbedingten Wartezeiten
	 */
	 void erzeugenNetzbedingtenWartezeitenArray(void);
	
	/**
	 * Setzen der Netzbedingten Wartezeiten
	 */
	 void setzenNetzbedingteWartezeiten( h_array< three_tuple<int, int, int>*, int> * );
	
	
	/**
	 * Gibt die Netzbedingten Wartezeiten zurueck
	 */
	h_array< three_tuple<int, int , int>*, int> * holenNetzbedingteWartezeiten(void);
	
	
	/**
	 * Gibt die map2 Datenstruktur mit <anhHS, anzHS, CReisebewertung> zurueck
	 */
	map2 <int, int, CReisebewertung > * holenReisebewertung(void);	

	/**
	 *Speichert die map2 Datenstruktur mit <anhHS, anzHS, CReisebewertung> zurueck
	 */
	void speichernReisebewertung(map2 <int, int, CReisebewertung > *);	
};
#endif //CGUETEMASSE_H
