
//------------------------------------------------------------------------------
// Projektname:		
// Dateiname:		$RCSfile: CParameterHaltestelle.h,v $
// Beschreibung:	
// Version:	        $Revision: 1.16 $
// Gendert von:	$Author: pompl $
// Gendert am:		$Date: 2001/07/10 13:04:12 $
// Zustand:	        $State: Exp $
//------------------------------------------------------------------------------

#ifndef CPARAMETERHALTESTELLE_H
#define CPARAMETERHALTESTELLE_H

#include <LEDA/list.h>
#include <LEDA/matrix.h>
#include <LEDA/graph.h>

#include <assert.h>

#include "CPunkt.h"
#include "CBevoelkerungsdichteElement.h"
#include "CQuelleZielListeElement.h"
#include "CPendlerListeElement.h"

/**
 * Bestimmt den zu verwendenen Algorithmus fuer die Ermittlung der
 * Haltestellen.
 *
 */
enum TAlgorithmus {stochastisch,stochastischUmgebung, evolutionaereStrategie, nurPflichthaltestellen};

/**
 * Beschreibt die Modi der fuer die Ermittlung von Entfernungen
 * zugrundeliegenden Berechnungsarten.
 *
 * luftlinie - Die Luftlinie wird als Entfernungsmass zwischen zwei Punkten angenommen.
 *
 * realweg - Der Realweg wird als Entfernungsmass zwischen zwei Punkten angenommen.
 */
enum TRadius {luftlinie,realweg};

/**
 * Beschreibt die Modi der fuer die Anpassung der Haltestellen
 * an das Strassennetz zugrundeliegenden Berechnungsarten.
 *
 * lot - Das Lot wird von der aktuellen Position zu dem naechstliegenden Strassenzug gefaellt und der Fusspunkt ist die Position der Haltestelle.
 *
 * kreuzung - Die naechstligende Kreuzung wird die neue Haltestellenposition.
 */
enum TAnpassung {lot,kreuzung};

/**
 * Dient der Wahl der grundlegenden Strategie der ES.
 */
enum TStrategie {plus,komma};

/**
 * Dient der Aufnahme aller Parameter, welche fuer das Modul Haltestelle bestimmt sind.
 * Dient der einfacheren Uebergabe von Werten.
 */
class CParameterHaltestelle {
public:
	/**
	 * Standard-Konstruktor. Belegt alle Variablen mit
	 * definierten Vorgabewerten.
	 */
	CParameterHaltestelle(list<list<CPunkt*>*> * Strassennetzliste, list<CPunkt*> * Pflichthaltestellen, list<list<CPunkt*>*> * Wege,  list<CBevoelkerungsdichteElement*> * Bevoelkerungsdichtenliste,matrix * QuelleZielMatrix,list<CQuelleZielListeElement*> * QuelleZielListe,matrix * Pendlermatrix,list<CPendlerListeElement* > * Pendlerliste, list<list<CPunkt *> *> * Gebaeudeliste, TAlgorithmus Algorithmustyp=stochastisch,int Zufallszahlengeneratorinitialisierung=-1,double minimalesEinzugsgebiet=150,double maximalesEinzugsgebiet=450,double verbindungsradius=1200,TRadius Radiustyp=luftlinie,double Haltestellenlage=0,double toleranzgrenze=0.5,unsigned int BevoelkerungsrasterEinheiten=10,double koeffizientBevoelkerungsueberversorgung=0.5,double koeffizientBevoelkerungsunterversorgung=0.5,double koeffizientFlaechenueberversorgung=0.5,double koeffizientFlaechenunterversorgung=0.5,double koeffizientFahrtwunscheinbeziehung=0.5,double koeffizientStrassenanpassung=0.5, bool AnzeigenLegende = true, bool AnzeigenStrassennetz = true, bool AnzeigenBevoelkerung = true, bool AnzeigenFlaeche = true);

	~CParameterHaltestelle();
	
	/**
	 * Laedt eine zuvor gespeicherte Konfiguration mit dem angegebenen
	 * Dateinamen.
	 */
	void laden(const char * Dateiname);
	/**
	 * Speichert eine Konfiguration unter dem angegebenen Dateinamen.
	 */
	void speichern(const char * Dateiname);
	
	TAlgorithmus holenAlgorithmustyp(void);
	void setzenAlgorithmustyp(TAlgorithmus Algorithmustyp);
	
	int holenZufallszahlengeneratorinitialisierung(void);
	void setzenZufallszahlengeneratorinitialisierung(int Zufallszahlengeneratorinitialisierung);
	
	double holenMinimalesEinzugsgebiet(void);
	void setzenMinimalesEinzugsgebiet(double minimalesEinzugsgebiet);
	
	double holenMaximalesEinzugsgebiet(void);
	void setzenMaximalesEinzugsgebiet(double maximalesEinzugsgebiet);
	
	double holenVerbindungsradius(void);
	void setzenVerbindungsradius(double verbindungsradius);
	
	/**
	 * Ermittelt den Modus fuer die Ermittlung der von Entfernungen
	 * zugrundeliegenden Berechnungsart.
	 */
	TRadius holenRadiustyp(void);
	void setzenRadiustyp(TRadius Radiustyp);
	
	double holenHaltestellenlage(void);
	void setzenHaltestellenlage(double Haltestellenlage);
	
	double holenToleranzgrenze(void);
	void setzenToleranzgrenze(double toleranzgrenze);
	
	list<CPunkt*> * holenPflichthaltestellen(void);
	void setzenPflichthaltestellen(list<CPunkt*> * Pflichthaltestellen);
	
	list<list<CPunkt*>*> * holenStrassennetzliste(void);
	void setzenStrassennetzliste(list<list<CPunkt*>*> * Strassennetzliste);

	list<list<CPunkt*>*> * holenWegeliste(void);
	void setzenWegeliste(list<list<CPunkt*>*> * wege);
	
	unsigned int holenBevoelkerungsrasterEinheiten(void);
	void setzenBevoelkerungsrasterEinheiten(unsigned int BevoelkerungsrasterEinheiten);
	
	double holenKoeffizientBevoelkerungsueberversorgung(void);
	void setzenKoeffizientBevoelkerungsueberversorgung(double koeffizientBevoelkerungsueberversorgung);
	
	double holenKoeffizientBevoelkerungsunterversorgung(void);
	void setzenKoeffizientBevoelkerungsunterversorgung(double koeffizientBevoelkerungsunterversorgung);
	
	double holenKoeffizientFlaechenueberversorgung(void);
	void setzenKoeffizientFlaechenueberversorgung(double koeffizientFlaechenueberversorgung);
	
	double holenKoeffizientFlaechenunterversorgung(void);
	void setzenKoeffizientFlaechenunterversorgung(double koeffizientFlaechenunterversorgung);
	
	double holenKoeffizientFahrtwunscheinbeziehung(void);
	void setzenKoeffizientFahrtwunscheinbeziehung(double koeffizientFahrtwunscheinbeziehung);
	
	double holenKoeffizientStrassenanpassung(void);
	void setzenKoeffizientStrassenanpassung(double koeffizientStrassenanpassung);

	bool pruefenAnzeigenLegende(void);
	void setzenAnzeigenLegende(bool anzeigen);
	
	bool pruefenAnzeigenStrassennetz(void);
	void setzenAnzeigenStrassennetz(bool anzeigen);
	
	bool pruefenAnzeigenBevoelkerung(void);
	void setzenAnzeigenBevoelkerung(bool anzeigen);
	
	bool pruefenAnzeigenFlaeche(void);
	void setzenAnzeigenFlaeche(bool anzeigen);

	bool pruefenBatchModus(void);
	void setzenBatchModus(bool istbatchmodus);
	
	TStrategie pruefenStrategie(void);
	void setzenStrategie(TStrategie strategie);
	
	unsigned int pruefenAnzahlEltern(void);
	void setzenAnzahlEltern(unsigned int anzahl);
	
	unsigned int pruefenAnzahlKinder(void);
	void setzenAnzahlKinder(unsigned int anzahl);
	
	unsigned int pruefenAnzahlGenerationen(void);
	void setzenAnzahlGenerationen(unsigned int anzahl);
	
	bool pruefenWerteunterschreitung(void);
	void setzenWerteunterschreitung(bool aktiv);
	
	double pruefenGrenzwert(void);
	void setzenGrenzwert(double wert);

	unsigned int holenAnzahlHaltestellen(void);
	void setzenAnzahlHaltestellen(unsigned int anzahl);	
	
private:

	/**
	 */
	TAlgorithmus Algorithmustyp;
	
	/**
	 */
	int Zufallszahlengeneratorinitialisierung;
	
	/**
	 * Das minimale Einzugsgebiet einer Haltestelle in Form eines Radius.
	 * Liegt in GRASS-Einheiten vor.
	 */
	double minimalesEinzugsgebiet;

	/**
	 * Das max. Einzugsgebiet einer Haltestelle in Form eines Radius
	 * Liegt in GRASS-Einheiten vor.
	 */
	double maximalesEinzugsgebiet;

	/**
	 * Der Verbindungsradius, der angibt, in welchem Bereich alle Haltestellen untereinander verbunden werden.
	 * Liegt in GRASS-Einheiten vor.
	 */
	double verbindungsradius;

	/**
	 * Enthaelt den Modus der fuer die Ermittlung von Entfernungen
	 * zugrundeliegenden Berechnungsart.
	 */
	TRadius Radiustyp;
	
	/**
	 * Enthaelt den Modus der fuer die Anpassung der Haltestellen
	 * an das Strassennetz zugrundeliegenden Berechnungsart.
	 */
	TAnpassung Anpassungstyp;
	
	/**
	 */
        double Haltestellenlage;

	/**
	 * Die Tolereanzgrenze gibt den Weg an, den die Fahrgaeste noch bereit sind zu laufen.
	 * Liegt in GRASS-Einheiten vor.
	 */
	double toleranzgrenze;

	/**
	 * Stellt einen Zeiger auf eine LEDA-Liste dar, die Punkte als Elemente enthaelt. Dies sind die Pflichthaltestellen.
	 */
	list<CPunkt*> * Pflichthaltestellen;

	/**
	 * Beschreibt die Anzahl der Rasterungseinheiten fuer die Bevoelkerungsdichte.
	 */
	unsigned int BevoelkerungsrasterEinheiten;

	/**
	 */
	double koeffizientBevoelkerungsueberversorgung;
	/**
	 */
	double koeffizientBevoelkerungsunterversorgung;
	/**
	 */
	double koeffizientFlaechenueberversorgung;
	/**
	 */
	double koeffizientFlaechenunterversorgung;
	/**
	 */
	double koeffizientFahrtwunscheinbeziehung;
	/**
	 */
	double koeffizientStrassenanpassung;

	/**
	 */
	bool AnzeigenLegende;
	
	/**
	 */
	bool AnzeigenStrassennetz;
	
	/**
	 */
	bool AnzeigenBevoelkerung;
	
	/**
	 */
	bool AnzeigenFlaeche;

	/**
	 */
        bool BatchModus;
	
	TStrategie Strategie;
	
	unsigned int AnzahlEltern;
	
	unsigned int AnzahlKinder;
	
	unsigned int AnzahlGenerationen;
	
	bool Werteunterschreitung;
	
	double Grenzwert;
	
	unsigned int AnzahlHaltestellen;
	
	
public:

	list<CBevoelkerungsdichteElement*> * holenBevoelkerungsdichtenliste(void);
	void setzenBevoelkerungsdichtenliste(list<CBevoelkerungsdichteElement*> * Bevoelkerungsdichtenliste);
	
	matrix * holenQuelleZielMatrix(void);
	void setzenQuelleZielMatrix(matrix * QuelleZielMatrix);
	
	list<CQuelleZielListeElement*> * holenQuelleZielListe(void);
	void setzenQuelleZielListe(list<CQuelleZielListeElement*> * QuelleZielListe);
	
	matrix * holenPendlermatrix(void);
	void setzenPendlermatrix(matrix * Pendlermatrix);
	
	list<CPendlerListeElement* > * holenPendlerliste(void);
	void setzenPendlerliste(list<CPendlerListeElement* > * Pendlerliste);

	list<list<CPunkt* > * > * holenGebaeudeliste(void);
	void setzenGebaeudeliste(list<list<CPunkt* > * > * Gebaeudeliste);

private:
	/**
	 * Das Strassenentz als Liste von Listen von CPunkten.
	 */
	list<list<CPunkt*>*> * Strassennetzliste;
	/**
	 * Alle Fusswege als Liste von Listen von CPunkten.
	 */
	list<list<CPunkt*>*> * Wegeliste;
	/**
	 * Das Attribut Bevoelkerungsdichtenliste ist eine Liste der Bevoelkerungsdichte. Das Attribut ist so angelegt, dass die Bevoelkerungsdichte fuer einen Punkt angegeben wird. Um diesen Punkt wird ein Radius gezogen, der angibt, fuer welches Gebiet der Wert der Bevoelkerungsdichte gilt.
	 */
	list<CBevoelkerungsdichteElement*> * Bevoelkerungsdichtenliste;
	/**
	 *
	 */
	matrix * QuelleZielMatrix;

	/**
	 *
	 */
	list<CQuelleZielListeElement*> * QuelleZielListe;

	/**
	 * Enthaelt die Daten ueber die Pendlerbewegung in Matrixform. Der Konstruktor CHaltestellen() uebergibt die Referenz auf ein solches Matrix-Objekt die Methode setzenHaltestellen() benutzt die Pendlermatrix, um die Haltestellen zu setzen.
	 */
	matrix * Pendlermatrix;

	/**
	 * Die Pendlerliste verknuepft die Pendlermatrix mit dem Koordinatensystem.
	 */
	list<CPendlerListeElement* > * Pendlerliste;

	/**
	 * Liste mit den Umrissen aller Gebaeude.
	 */
	list<list<CPunkt *> *> * Gebaeudeliste;
};

#endif // CPARAMETERHALTESTELLE_H
