#ifndef FEHLER_H
#define FEHLER_H

#include "LEDA/string.h"


/*
 * Einige Klassen, welche Fehler enthalten, die bei Bedarf
 * geworfen werden koennen...
 */

/**
 * Allgemeine Fehlerklassen, von allen Modulen verwendbar
 */


/**
 * Wird beim nicht korrekten Aufruf eines Konstruktors oder
 * eines nicht erlaubten Konstruktors verwendet.
 */
class UnsinnigeErzeugung{};

/**
 * Wird bei Auftreten eines nicht instanzierten Parameter(objekt)s
 * verwendet.
 */
class NichtInstanzierteParameter{};

/**
 * Wird erzeugt, wenn ein Parameter ausserhalb des zulaessigen
 * Bereiches gesetzt wurde.
 */
class AusserhalbWertebereich{};

/**
 * Wird zum (halbwegs) kontrollierten Ausstieg aus einem Modul
 * verwendet. Von Gebrauch her aehnlich einem Breakpoint.
 */
class WohldefinierterAbbruch {};



/**
 * Default Exception der Haltestellegruppe, es wird im Konstruktor eine Meldung uebergeben
 */
class CHaltestelleExceptions {
 public:

  CHaltestelleExceptions(){};
  /**
   * Der Konstruktor enthaelt eine Meldung zur Fehler beschreibung
   */
  CHaltestelleExceptions(string);
  string gebenMeldung();

 private:
  string meldung;

};

/**
 * Default Exception der Fahrplangruppe, es wird im Konstruktor eine Meldung uebergeben
 */
class CFahrplanExceptions {
 public:

  CFahrplanExceptions(){};
  /**
   * Der Konstruktor enthaelt eine Meldung zur Fehler beschreibung
   */
  CFahrplanExceptions(string);
  string gebenMeldung();

 private:
  string meldung;

};

/**
 * Wird vom Fahrplanmodul benutzt, wenn der Benutzer bei der Auswahl von festen Bindungen 
 * dabei einen Kreis auswaehlt.
 */
class CKreisGewaehlt : public CFahrplanExceptions {
 public:
  CKreisGewaehlt(string);
};

/**
 * Wird geworfen, falls irgendwelche uebergebenen Variablen keine brauchbaren Informationen enthalten
 */
class CUnsinnigerWertGeliefert : public CFahrplanExceptions {
 public:
  CUnsinnigerWertGeliefert(string);
};

class CFesteBindungNichtGefunden : public CFahrplanExceptions {
 public:
  CFesteBindungNichtGefunden(string);
};


class CGuetemasseFehler {
    public:

     CGuetemasseFehler() {};
      //Konstruktor
     CGuetemasseFehler(string);
     //CGuetemasseFehler(char*);
     string ausgebenMeldung(void);

   private:

     string meldung;
};

#endif //FEHLER_H
