//------------------------------------------------------------------------------
// Projektname:		
// Dateiname:		$RCSfile: CFahrplan.cpp,v $
// Beschreibung:	
// Autor(en):		

// Version:			$Revision: 1.72 $
// Gendert von:	$Author: sarnowsk $
// Gendert am:		$Date: 2001/07/15 21:08:44 $

// Zustand:			$State: Exp $
//------------------------------------------------------------------------------

#include "CFahrplan.h"
#include "CSchnittstellenTest.h"
#include "CFPSchnittstelle.h"

//#define SCHNITTSTELLE this->FPSchnittstelleTest
#define SCHNITTSTELLE this->FPSchnittstelle

//------------------------------------------------------------------------------
// Funktionsname:	CFahrplan
// Beschreibung:	Konstruktor
// Parameter:		Pointer auf die Test-Programm-Schnittstelle
//                      Pointer auf das Netz der Transportverbindungen
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
CFahrplan::CFahrplan(CSchnittstellenTest* SchnittstelleTest, GRAPH<CFPlanGraphKnoten*, CFPlanGraphKante*>* Netz){
  // falls kein objekt uebergeben wird, werfen einer
  // fehlermeldung
  if ( (Netz == 0) || (SchnittstelleTest == 0)) {
    throw NichtInstanzierteParameter ();
  }
  this->NetzDerTransportverbindungen = Netz;
  this->FPSchnittstelleTest = SchnittstelleTest;
} // ende konstruktor

//------------------------------------------------------------------------------
// Funktionsname:	CFahrplan
// Beschreibung:	Konstruktor
// Parameter:		Pointer auf die Programm-Schnittstelle
//                      Pointer auf das Netz der Transportverbindungen
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
CFahrplan::CFahrplan(CFPSchnittstelle* Schnittstelle, GRAPH<CFPlanGraphKnoten*, CFPlanGraphKante*>* Netz){
  // falls kein objekt uebergeben wird, werfen einer
  // fehlermeldung
  if ((Netz == 0) || (Schnittstelle == 0)){
    throw NichtInstanzierteParameter ();
  }
  this->NetzDerTransportverbindungen = Netz;
  this->FPSchnittstelle = Schnittstelle;
} // ende konstruktor                                                                                                                   
//------------------------------------------------------------------------------
// Funktionsname:	~Fahrplan
// Beschreibung:	Destruktor
// Parameter:		keine
// Rueckgabe:		keine
// Besonderheiten:	keine
//------------------------------------------------------------------------------
CFahrplan::~CFahrplan () {
  node currentNode;
  edge currentEdge;
  CFPlanGraphKante* informationCurrentEdge;
  CFPlanGraphKnoten* informationCurrentNode;

  // loeschen netz der transportverbindungen
  // iteration ueber alle kanten
  if (this->NetzDerTransportverbindungen) {
    forall_edges (currentEdge, *(this->NetzDerTransportverbindungen)) {
      informationCurrentEdge = this->NetzDerTransportverbindungen->inf (currentEdge);
      if (informationCurrentEdge) {
        delete informationCurrentEdge;
      }
    } // iteration ueber alle kanten
  } // ende falls definiert

  // iteration ueber alle knoten
  if (this->NetzDerTransportverbindungen) {
    forall_nodes (currentNode, *(this->NetzDerTransportverbindungen)) {
      informationCurrentNode = this->NetzDerTransportverbindungen->inf (currentNode);
      if (informationCurrentNode) {
	delete informationCurrentNode;
      }
    } // iteration ueber alle knoten
  } // ende falls definiert

  // loeschen des netzes der transportverbindungen
  if (this->NetzDerTransportverbindungen)
    delete this->NetzDerTransportverbindungen;
  
  // loeschen des transportkettengraphen
  if (this->Transportkettengraph)
    delete this->Transportkettengraph;
} // ende destruktor

//------------------------------------------------------------------------------
// Funktionsname:       starten	
// Beschreibung:	Erzeugung eines neuen Transportkettengraphen
//                      Aufruf der Schnittstelle
//                      nach Absolvierung entsprechender Aufruf in der
//                      Schnittstelle mit dem Ergebnis der Berechnung
// Parameter:		keine
// Rueckgabe:		keine
// Besonderheiten:	keine
//------------------------------------------------------------------------------
void CFahrplan::starten(){
  this->Transportkettengraph = new CTransportkettengraph(NetzDerTransportverbindungen);
  SCHNITTSTELLE->fahrplanerstellungZusammenhangskomponenten (
    this->Transportkettengraph->berechnenListenDerHaltestellenInAllenZusammenhangskomponenten());
} // ende starten


//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CFahrplan::stoppen(){
  if (this->Transportkettengraph != 0) {
    this->Transportkettengraph->setzenStoppenFlag ();
  }
} // ende stoppen


//------------------------------------------------------------------------------
// Funktionsname:	berechnenHeuristischeFahrplanerstellung
// Beschreibung:	starten bei der Fahrplanberechnung
//                      fuer die uebergebene Zusammenhangskomponente
//                      Verwendung der heuristisches Ausgangsloesung nach
//                      Weigand
// Parameter:		ID der Zusammenhangskomponente
// Rueckgabe:		void
// Besonderheiten:	
//------------------------------------------------------------------------------
void CFahrplan::berechnenHeuristischeFahrplanerstellung(int zusammenhangskomponentenID){
  int haltezeit, umsteigezeit, fahrzeuganzahl, takt;
  SCHNITTSTELLE->holenFahrplanModulParameter(fahrzeuganzahl,takt,haltezeit,umsteigezeit);
  this->holenAllgemeinerTakt ();
  this->Transportkettengraph->InitMindestHaltezeit (haltezeit);
  this->Transportkettengraph->InitMindestUmsteigezeit (umsteigezeit);
  this->holenMindesthaltezeit ();
  this->holenMindestumsteigezeit ();

  this->zusammenhangskomponentenID = zusammenhangskomponentenID; 
  this->holenFesteVerbindungen ();
  this->Transportkettengraph->initialisierenGeruest();
  this->Transportkettengraph->setzenTakt (zusammenhangskomponentenID, this->allgemeinerTakt);
  CLoesungsgenerator* loesungsgenerator = new CHeuristischesLoesungsverfahren (
  		       	              zusammenhangskomponentenID,
  		  	              this->Transportkettengraph);

  loesungsgenerator->starten ();
  delete loesungsgenerator;
} // ende berechnenHeuristischeFahrplanerstellung

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CFahrplan::berechnenSukzessiveLoesungsverbesserung(int zusammenhangskomponentenID){
  Debug (2, "Berechnen lokale Suche");

  CLoesungsgenerator* loesungsgenerator = new CSukzessivVerfahren (
			     zusammenhangskomponentenID,
  			     this->Transportkettengraph);
  this->zusammenhangskomponentenID = zusammenhangskomponentenID;
  loesungsgenerator->starten ();

  delete loesungsgenerator;
} // ende berechnenSukzessiveLoesungsverbesserung

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CFahrplan::berechnenEntscheidungsbaumverfahren(int zusammenhangskomponentenID,int baumtiefe){
  int haltezeit, umsteigezeit, fahrzeuganzahl, takt;
  SCHNITTSTELLE->holenFahrplanModulParameter(fahrzeuganzahl,takt,haltezeit,umsteigezeit);
  this->zusammenhangskomponentenID = zusammenhangskomponentenID;
  this->holenAllgemeinerTakt ();
  this->Transportkettengraph->InitMindestHaltezeit (haltezeit);
  this->Transportkettengraph->InitMindestUmsteigezeit (umsteigezeit);
  this->holenMindesthaltezeit ();
  this->holenMindestumsteigezeit ();
  
  this->holenFesteVerbindungen ();
  this->Transportkettengraph->initialisierenGeruest();
  this->Transportkettengraph->initialisierenFesterVerbindungen();
  this->Transportkettengraph->setzenTakt (zusammenhangskomponentenID, this->allgemeinerTakt);
  CLoesungsgenerator* loesungsgenerator = new CEntscheidungsbaumverfahren (
  		       	              zusammenhangskomponentenID,
  		  	              this->Transportkettengraph,baumtiefe);

  loesungsgenerator->starten ();
  delete loesungsgenerator;
}

//------------------------------------------------------------------------------
// Funktionsname:	ausgebenFahrplan
// Beschreibung:	Wird von der Programmgruppe aufgerufen,
//                      um einen Fahrplan zu erzeugen
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CFahrplan::ausgebenFahrplan(){

  // Holen der Abfahrtszeiten
  array<three_tuple<int,int,int>* > abfahrtszeit = holenAbfahrtszeit();

  // Holen des Fahrplanintervalls
  four_tuple<int,int,int,int> fahrplanIntervall = holenFahrplanIntervall();

//  std::cout << "Beginn des Berechens:" << endl;

  // Berechnen des Fahrplans
  Transportkettengraph->berechnenFahrplan(&abfahrtszeit,&fahrplanIntervall);

  // Fahrplan holen
  list<CFahrplanElement*>* fahrplan = Transportkettengraph->gebenFahrplan();

  // fahrzeugAnzahl holen
  int fahrzeugAnzahl = Transportkettengraph->gebenFahrzeugAnzahl ();

  // Fahrplan der Schnittstelle uebergeben
  SCHNITTSTELLE->fahrplanerstellungErgebnisrueckgabe(fahrplan);

  // Guetemasse holen
  CGuetemasse* guetemasse = SCHNITTSTELLE->holenGuetemasse();

//  std::cout << "Benoetigte Fahrzeuge: " << fahrzeugAnzahl << endl; 
  guetemasse->speichernAnzahlBusse((double)fahrzeugAnzahl);

  // holen des Wirkungsgrades
  double wirkungsgrad = (double)this->Transportkettengraph->gebenFahrplanWirkungsgrad();
  guetemasse->speichernFahrplanwirkungsgrad(wirkungsgrad);
//  cout << "Fahrplanwirkungsgrad: " << wirkungsgrad << endl;
//  cout.flush();

  // holen der Fahrgastfahrten
  int fahrgastfahrten = this->Transportkettengraph->gebenFahrgastfahrten();
  guetemasse->speichernFahrgastfahrten(fahrgastfahrten);
//  cout << "Fahrgastfahrten: " << fahrgastfahrten << endl;
//  cout.flush();

  // Guetemasse speichern
  SCHNITTSTELLE->speichernGuetemasse(guetemasse);

  delete fahrplan;
}

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CFahrplan::ausgebenFahrzeugAnzahl () {
  //  int fahrzeugAnzahl = Transportkettengraph->gebenFahrzeugAnzahl ();
  //  SCHNITTSTELLE->fahrplanerstellungAnzahlBenoetigterFahrzeuge 
  //  (fahrzeugAnzahl);
}

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CFahrplan::ausgebenWartezeiten () {
  //  double summeNetzbedingteWartezeiten, wartezeitAufwand;
  //  double mindestWartezeitAufwand, gesamtWartezeitAufwand;

  //  Transportkettengraph->gebenWartezeiten (summeNetzbedingteWartezeiten,
  //				  wartezeitAufwand,
  //				  mindestWartezeitAufwand,
  //				  gesamtWartezeitAufwand);
  //  SCHNITTSTELLE->fahrplanerstellungWartezeiten (
  //		       	summeNetzbedingteWartezeiten,
  //			wartezeitAufwand,
  //			mindestWartezeitAufwand,
  //			gesamtWartezeitAufwand);
  
  //Reisebewertung erstellen !!!!!!! Muss wieder entfernt werden !!!!!!
  //this->ausgebenBewertung();
}

//------------------------------------------------------------------------------
// Funktionsname:	holenMindesthaltezeit
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CFahrplan::holenMindesthaltezeit(){
  Debug (2, "Holen MinstestHaltezeit");
  this->Transportkettengraph->setzenMindesthaltezeit(SCHNITTSTELLE->fahrplanerstellungMindesthaltezeit ());
} // ende holenMindesthaltezeit
//------------------------------------------------------------------------------
// Funktionsname:	holenMindestumsteigezeit
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CFahrplan::holenMindestumsteigezeit(){
  Debug (1, "holenMindestumsteigezeit");
  this->Transportkettengraph->setzenMindestumsteigezeit (SCHNITTSTELLE->fahrplanerstellungMindestumsteigezeit ());
} // ende holenMindestumsteigezeit

//------------------------------------------------------------------------------
// Funktionsname:	holenFesteVerbindungen
// Beschreibung:	Erhaelt eine Liste von four_tuples und gibt diese an CTransportkettengraph weiter
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CFahrplan::holenFesteVerbindungen(){
  Debug (1, "Init Verbindungen");
  this->Transportkettengraph->initialisierenFesterVerbindungen();

  Debug (2, "Setzen fester Bindungen");
  this->Transportkettengraph->setzenFesterBindungen(SCHNITTSTELLE->fahrplanerstellungFesteVerbindungen());
}

//------------------------------------------------------------------------------
// Funktionsname:	holenAllgemeinerTakt
// Beschreibung:	Holt aus der Programmschnittstelle den Allgemeinen Takt
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CFahrplan::holenAllgemeinerTakt(){
  this->allgemeinerTakt = SCHNITTSTELLE->fahrplanerstellungAllgemeinerTakt();
}

//------------------------------------------------------------------------------
// Funktionsname:	holenAbfahrtszeit
// Beschreibung:	Holt aus der Programmgruppe die Abfahrtszeit fuer alle Zusammenhangskomponenten
// Parameter:		
// Rueckgabe:		liste aller Abfahrtszeiten (Haltestelle,Linie,Stunde,Minute)
// Besonderheiten:	
//------------------------------------------------------------------------------
array<three_tuple<int, int, int>* > CFahrplan::holenAbfahrtszeit(){
  array<three_tuple<int,int,int>* > abfahrtszeit = *(SCHNITTSTELLE->fahrplanerstellungAbfahrtszeiten());
  return abfahrtszeit;
}

//------------------------------------------------------------------------------
// Funktionsname:	holenFahrplanIntervall
// Beschreibung:	Holt aus der Programmgruppe das Fahrplanintervall
// Parameter:		
// Rueckgabe:		Intervall des Fahrplans (StundeVon,MinuteVon,StundeBis,MinuteBis)
// Besonderheiten:	
//------------------------------------------------------------------------------
four_tuple<int, int, int, int> CFahrplan::holenFahrplanIntervall(){
  four_tuple<int,int,int,int> fahrplanIntervall = *(SCHNITTSTELLE->fahrplanerstellungFahrplanIntervall());
  return fahrplanIntervall;
}

void CFahrplan::holenMaximaleFahrzeugAnzahl(){
}

//------------------------------------------------------------------------------
// Funktionsname:	ausgebenBewertung
// Beschreibung:	Berechnet die Bewertung des Fahrplans und uebergibt die Resultate an das Programm
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CFahrplan::ausgebenBewertung() {
  double summeNetzbedingteWartezeiten, wartezeitAufwand;
  double mindestWartezeitAufwand, gesamtWartezeitAufwand;

  // Holen der Guetemasse
  Debug(1,"ausgebenBewertung einstieg");
  CGuetemasse* guetemasse = SCHNITTSTELLE->holenGuetemasse();
  map2<int,int,CReisebewertung> *reisebewertung = new map2<int,int,CReisebewertung>();

  // Berechnen der Guetemasse
  Debug(5,"Netzbedingte Wartezeiten setzen");
  guetemasse->setzenNetzbedingteWartezeiten(this->Transportkettengraph->gebenListeNetzbedingtewartezeit());

  Debug(5,"Reisebewertung durchfuehren");
  int anzahlHaltestellen = guetemasse->holenAnzahlHaltestellen();

  Transportkettengraph->fuellenReisebewertung(*reisebewertung,anzahlHaltestellen);

  Transportkettengraph->gebenWartezeiten (summeNetzbedingteWartezeiten,
					  wartezeitAufwand,
					  mindestWartezeitAufwand,
					  gesamtWartezeitAufwand);
  guetemasse->speichernNetzbedingteWartezeitenGesamt(summeNetzbedingteWartezeiten);
  guetemasse->speichernWartezeiteAufwandGesamt(wartezeitAufwand);
  guetemasse->speichernMindestwartezeitaufwand(mindestWartezeitAufwand);
  guetemasse->speichernGesamtzeitaufwand(gesamtWartezeitAufwand);

  // Ausgabe der reisebewertung
/*
  cout << endl << endl << "ANZAHL DER HALTESTELLEN: " << anzahlHaltestellen << endl;
  cout.flush();
  CReisebewertung reise;
  list<three_tuple<int,int,int> > reiseweg;
  three_tuple<int,int,int> knoten;
  for (int i=0;i<anzahlHaltestellen;i++) {
    for (int j=0;j<anzahlHaltestellen;j++) {
      reise = (*reisebewertung)(i,j);
      cout << i << " -> " << j << " = Fahrzeit: " << reise.gebenFahrzeit() << " ; Wartezeit: " << reise.gebenWartezeit() << " ; Umstiege: " << reise.gebenAnzahlDerUmstiege() << endl;
      reiseweg = reise.gebenReiseweg();
      forall(knoten,reiseweg) {
	cout << "(" << knoten.first() << "," << knoten.second() << "," << knoten.third() << ")" << endl;
	cout.flush();
      }
      cout << endl;
      cout.flush();
    }
  }
  Debug(1,"ausgebenBewertung ausstieg");
*/

  guetemasse->speichernReisebewertung(reisebewertung);

  // Speichern der Guetemasse
  SCHNITTSTELLE->speichernGuetemasse(guetemasse);

/*
  // Anzeigen der Bewertung
  Debug(5,"Bewertung anzeigen");
  CGuetemasse* testGuetemasse = SCHNITTSTELLE->holenGuetemasse();
  h_array<three_tuple<int,int,int>*,int>* netzbedingteWartezeiten = testGuetemasse->holenNetzbedingteWartezeiten();
  three_tuple<int,int,int>* ort;
  Debug(9,"Ausgeben Netzbedingte wartezeiten !!");
  cout << "Netzbedingte Wartezeit: " << netzbedingteWartezeiten;
  cout.flush();
  forall_defined(ort,*(netzbedingteWartezeiten)){
    cout << "(" << ort->first() << "," << ort->second() << "," << ort->third() << ") : " << (*netzbedingteWartezeiten)[ort] << endl;
    cout.flush();
  }

  Debug(9,"Ausgeben Reisebewertung !!");
  map2<int,int,CReisebewertung>* bewertung = testGuetemasse->holenReisebewertung();
  int haltestellen = testGuetemasse->holenAnzahlHaltestellen();
  cout << endl << endl << "ANZAHL DER HALTESTELLEN: " << haltestellen << endl;
  cout.flush();
  CReisebewertung reise;
  list<three_tuple<int,int,int> > reiseweg;
  three_tuple<int,int,int> knoten;
  for (int i=0;i<haltestellen;i++) {
    for (int j=0;j<haltestellen;j++) {
      reise = (*bewertung)(i,j);
      cout << i << " -> " << j << " = Fahrzeit: " << reise.gebenFahrzeit() << " ; Wartezeit: " << reise.gebenWartezeit() << " ; Umstiege: " << reise.gebenAnzahlDerUmstiege() << endl;
      reiseweg = reise.gebenReiseweg();
      forall(knoten,reiseweg) {
	cout << "(" << knoten.first() << "," << knoten.second() << "," << knoten.third() << ")" << endl;
	cout.flush();
      }
      cout << endl;
      cout.flush();
    }
  }
  Debug(1,"ausgebenBewertung ausstieg");

*/

/*

  Debug(5,"Bewertung anzeigen Original");
  netzbedingteWartezeiten = guetemasse->holenNetzbedingteWartezeiten();
  Debug(9,"Ausgeben Netzbedingte wartezeiten !!");
  cout << "Netzbedingte Wartezeit: " << netzbedingteWartezeiten;
  cout.flush();
  forall_defined(ort,*(netzbedingteWartezeiten)){
    cout << "(" << ort->first() << "," << ort->second() << "," << ort->third() << ") : " << (*netzbedingteWartezeiten)[ort] << endl;
    cout.flush();
  }
  Debug(9,"Ausgeben Reisebewertung !!");
  bewertung = guetemasse->holenReisebewertung();
  haltestellen = guetemasse->holenAnzahlHaltestellen();
  cout << endl << endl << "ANZAHL DER HALTESTELLEN: " << haltestellen << endl;
  cout.flush();
  for (int i=0;i<haltestellen;i++) {
    for (int j=0;j<haltestellen;j++) {
      reise = (*bewertung)(i,j);
      cout << i << " -> " << j << " = Fahrzeit: " << reise.gebenFahrzeit() << " ; Wartezeit: " << reise.gebenWartezeit() << " ; Umstiege: " << reise.gebenAnzahlDerUmstiege() << endl;
      reiseweg = reise.gebenReiseweg();
      forall(knoten,reiseweg) {
	cout << "(" << knoten.first() << "," << knoten.second() << "," << knoten.third() << ")" << endl;
	cout.flush();
      }
      cout << endl;
      cout.flush();
    }
  }
  Debug(1,"ausgebenBewertung ausstieg");#
*/
}


