/* Generated by Together */

#ifndef FAHRPLAN_H
#define FAHRPLAN_H

#include "CHeuristischesLoesungsverfahren.h"
#include "CEntscheidungsbaumverfahren.h"
#include "CSukzessivVerfahren.h"
#include "CLoesungsgenerator.h"
#include "CKnotenBeschriftung.h"
#include "CKantenBeschriftung.h"
#include "CFPlanGraphKante.h"
#include "CFPlanGraphKnoten.h"
#include "CFahrplanElement.h"
#include "CTransportkettengraph.h"
#include <LEDA/tuple.h>
#include <LEDA/graph.h>
#include <LEDA/map2.h> 
#include "Debug.h"
#include "Fehler.h"
#include "CReisebewertung.h"
#include "CGuetemasse.h"

class CSchnittstellenTest;
class CFPSchnittstelle;
/**
 * Dies ist die Hauptklasse des Fahrplan - Moduls.
 * Die komplette Kommunikation zwischen Programm - Modul und Fahrplan - Modul geschieht ber diese Klasse.
 * Desweiteren wird von dieser Klasse aus das Fahrplan - Modul gesteuert.
 * Die Vorgehensweise wird in den einzelnen Methoden beschrieben.
 */
class CFahrplan {
    public:

        /**
         * Wir erhalten von der Schnittstelle einen gerichteten Graphen.
         * Dabei handelt es sich um einen parametrisierbaren LEDA Graphen.
         * 
         * An den Kanten wird als Information folgendes erwartet:
         * 1. Verkehrsstrom
         * 2. Fahrzeit
         * 3. Liniennummer
         * 
         * An den Knoten sind folgende Werte zu finden:
         * Haltestellen-ID
         */
         CFahrplan(CSchnittstellenTest*, GRAPH<CFPlanGraphKnoten*, CFPlanGraphKante*>*);
         CFahrplan(CFPSchnittstelle*, GRAPH<CFPlanGraphKnoten*, CFPlanGraphKante*>*);                                               

	 /**
	  * Destruktor
	  */
	 ~CFahrplan ();
 
        /**
         * Start - Methode, die vom Programmodul aufgerufen wird.
         * Diese Methode erzeugt das Objekt Transportkettengraph der Klasse CTransportkettengraph.
         * An den Konstruktor der Klasse CTransportkettengraph uebergeben wir das
         * Netz der Transportverbindungen. Nachdem alle Berechnungen abgeschlossen sind,
         * uebergeben wir die Listen der Elemente der Zusammenhangskomponenten
         * an die Programmgruppe (Funktion: berechnenListeDerHaltestellenInAllenZusammenhangskomponenten
         * in CTransportkettengraph) und teilen ihr im Anschluss daran mit, das wir mit unseren
         * Berechnungen fertig sind (Message).
         * Zu diesem Zeitpunkt werden dann die entsprechenden Menueeintraege im Menu Berechnungen enabled.
         */
        void starten();

        /**
         * Stop - Methode des Fahrplan - Moduls.
         * Mit dieser Methode wird das Fahrplan - Modul beendet.
         *
         *
         *
         */
        void stoppen();
 

        /**
         * Diese Methode startet die heuristische Fahrplanberechnung.
         * Dazu wird als erstes die Mindesthaltezeiten und Mindestumsteigezeiten eingelesen.
         * Dazu werden die Methoden holenMindestumsteigezeit und holenMindesthaltezeit aufgerufen.
         * Dann wird die Methode holenFesteVerbindungen aufgerufen, die die vom Anwender gewaehlten
         * Bindungen abruft, die keine netzbedingten Wartezeiten haben sollen. Gegebenenfalls wird eine
         * Exception gefangen, die dann zum Abbruch der Berechnung und zur Rueckgabe einer
         * Fehlermeldung an die Programmgruppe fuehrt. Weiterhin wird die Methode holenAllgemeinerTakt aufgerufen.
         * Als nchstes wird der Loesungsgenerator mit der Klasse CHeuristischesLoesungsverfahren
         * initialisiert, wobei die ID der zu berechneneden Zusammenhangskomponente uebergeben und
         * die Berechnung per Heuristik gestartet wird.
         *
         * Parameter: 1. ID der Zusammenhangskomponente
         */
        void berechnenHeuristischeFahrplanerstellung(int);

        /**
         * Diese Methode startet die sukzessive Loesungsverbesserung.
         * Dazu wird als erstes die Mindesthaltezeiten und Mindestumsteigezeiten eingelesen.
         * Dazu werden die Methoden holenMindestumsteigezeit und holenMindesthaltezeit aufgerufen.
         * Als nchstes wird der Loesungsgenerator mit der Klasse CSukzessivVerfahren
         * initialisiert, wobei die ID der zu berechneneden Zusammenhangskomponente uebergeben wird und
         * die gegebene Ausgangsloesung verbessert.
         *
         * PRECONDITION: Ausgangsloesung liegt vor (Objekt Transportkettengraph)
         *
         * Parameter: 1. ID der Zusammenhangskomponente
         *
         *
         *
         */
        void berechnenSukzessiveLoesungsverbesserung(int);

        /**
         * Diese Methode startet die Fahrplanberechnung per Entscheidungsbaumverfahren.
         * Dazu wird als erstes die Mindesthaltezeiten und Mindestumsteigezeiten eingelesen.
         * Dazu werden die Methoden holenMindestumsteigezeit und holenMindesthaltezeit aufgerufen.
         * Als nchstes wird der Loesungsgenerator mit der Klasse CEntscheidungsbaumverfahren
         * initialisiert, wobei die ID der zu berechneneden Zusammenhangskomponente uebergeben wird und
         * der Fahrplan berechnet.
         *
         * Parameter: 1. ID der Zusammenhangskomponente
         *            2. Tiefe des Baumes
         *
         */
        void berechnenEntscheidungsbaumverfahren(int,int);

        /**
         * Diese Methode wird von der GUI aufgerufen, sobald der Benutzer sich einen Fahrplan
         * erstellen lassen moechte.
         * Dies geht erst, nachdem wir eine Loesung fuer wenigstens eine
         * Zusammenhangskomponenten erstellt haben. Vorher ist der Button in der GUI Disabled.
         * Dann werden ueber die Methode holenAbfahrtszeit die vom Benuzter eingegebenen
         * Abfahrtszeiten fuer alle Zusammenhangskomponenten geholt.
         * Dann werden ueber die Methode holenFahrplanIntervall, die Anfangs- und Endzeiten
         * geholt, fuer die der Fahrplan generiert werden soll.
	 * Die beiden geholten Werte werden dann an die Methode berechnenFahrplan in CTransportkettengraph 
	 * uebergeben.
	 * Dort wird dann mit Hilfe der CZusammenhangskomponenten eine liste von CFahrplanElement (Datenstruktur) Objekten
	 * erzeugt.
	 * Diese Liste wird dann an die Programmgruppe uebergeben.
         */
        void ausgebenFahrplan();

	/**
	 * Uebergabe der Wartezeiten der berechneten Fahrplaene
	 * an die Schnittstelle
	 */
	void ausgebenWartezeiten ();

	/**
	 * Uebergabe der benoetigten FahrzeugAnzahl an
	 * die Schnittstelle
	 */
	void ausgebenFahrzeugAnzahl ();

	/**
	 * Diese Methode benoetigt als Parameter eine "Matrix" in der fuer jedes Haltestellenpaar ein Object
	 * vom Typ CReisebewertung gespeichert ist.
	 */
	void ausgebenBewertung();

    private: // Attribute

        /**
         * @supplierCardinality 1
         * @supplierQualifier transportkettengraph
         * @directed
         *
         * Hier wird das Objekt der Klasse CTransportkettengraph gehalten
         */
        CTransportkettengraph* Transportkettengraph;

        /**
         * Hier wird das Netz der Transportverbindungen, welches von der Programmgrupe bergeben wird gespeichert. 
         */
         
        GRAPH<CFPlanGraphKnoten*, CFPlanGraphKante*>* NetzDerTransportverbindungen;

        /**
         * @supplierCardinality 1
         * @supplierQualifier ProgrammSchnittstelle
         *
         */
         /**
         * Hier wird das Objekt der Schnittstellenklasse aus der Programmgruppe gehalten
         */
        CFPSchnittstelle* FPSchnittstelle;
	CSchnittstellenTest* FPSchnittstelleTest;

        /**
         * Hier wird das Objekt der Klasse CLoesungsgenerator gehalten
         * @supplierCardinality 1
         */
        CLoesungsgenerator* Loesungsgenerator;

        int allgemeinerTakt;

	int zusammenhangskomponentenID;

    private: // Methoden

        /**
         * Wir erwarten von der Programmgruppe ein Hash-Array mit den Werten:
         * KEY: Haltestelle
         * VALUE: Mindesthaltezeit als int (gegebenenfalls gerundet)
         *
         * Dabei mu von der Programmgruppe geprueft werden, ob wir die aktuelle Mindesthaltezeit
         * schon eingelesen haben. Wenn dies der Fall ist, mu uns die Programmgruppe einen
         * NULL - Zeiger zurckliefern. Ansonsten werden die Daten bergeben.
         * Die geholten Daten werden an die Methode setzenMindesthaltezeit der Klasse
         * CTransportkettengraph uebergeben.
         */
        void holenMindesthaltezeit();

        /**
         * Wir erwarten von der Programmgruppe ein Hash-Array mit den Werten:
         * KEY: Haltestelle
         * VALUE: Liste von three_tupel(Liniennummer1, Liniennummer2, Minuten als int) (gegebenfalls gerundet)
         *
         * Dabei mu von der Programmgruppe geprueft werden, ob wir die aktuelle Mindestumsteigezeit
         * schon eingelesen haben. Wenn dies der Fall ist, mu uns die Programmgruppe einen
         * NULL - Zeiger zurckliefern. Ansonsten werden die Daten bergeben.
         * Die geholten Daten werden dann an die Methode setzenMindestumsteigezeit der Klasse
         * CTransportkettengraph uebergeben.
         */
        void holenMindestumsteigezeit();

        /**
         * Wir erwarten von der Programmgruppe eine Liste von four_tupeln (LEDA)
         * aller, durch den User gewaehlten festen Verbindungen. Ein four_tupel
         * enthaelt folgende Informationen:
         * 1. ID der Zusammenhangskomponente (eigentlich redundant)
         * 2. ID Liniennummer 1
         * 3. ID Liniennummer 2
         * 4. ID Umsteigehaltestelle
         *
         * Dabei steigen wir von Linie 1 in Linie 2 um. Diese Info reichen aus, um
         * die entsprechende Kante im Transportkettengraphen zu finden, dies geschieht wie folgt:
         * Wir rufen zu Beginn die Methode berechnenKnotenPartition mit der ZusammenhangsID im
         * CTransportkettengraph auf.
         * Danach wird die Methode setzenFesterBindungen mit der entsprechenden Liste festerVerbindungen
         * in CTransportkettengraph aufgerufen.
         * Diese Methode mu weiterhin eine Exception KreisGewaehlt auffangen, die gegebenenfalls von der
         * Methode setzenUmsteigekante in CZusammenhangskomponente geworfen wird.
         * Ihrerseits wirft sie gegebenenfalls eine KreisGewaehlt Exception ???
         */
	 void holenFesteVerbindungen();

        /**
         * Mit dieser Methode wird in der Programmgruppe der allgemeine Takt (integer Wert) abgerufen.
         * Dieser Takt wird als nchstes an die Klasse CTransportkettengraph uebergeben. Dies
         * geschieht mittels der Methode setzenTakt in CTransportkettengraph.
         */
        void holenAllgemeinerTakt();

        /**
         *  Wir fordern von der GUI ein Array mit den Abfahrtszeiten einer Linie/Haltestelle pro
	 *  Zusammenhangskomponente an.
         *  Die GUI liefert uns dann in dem Array (Gleicher Aufbau wie das Array der Zusammenhangskomponenten)
	 *  ein four_tupel mit folgenden Werten zurueck:
         *  - HaltestellenID
         *  - Liniennr
         *  - stunde
         *  - minute
         */
        array<three_tuple<int, int, int>* > holenAbfahrtszeit();

        /**
         * Diese Methode holt aus der Schnittstelle ein four_tupel, das
         * die beiden Uhrzeiten Anfangszeit und Endzeit, jeweils gesplittet
         * in Stunden und Minuten enthaelt. Fuer diesen Zeitraum soll der Fahrplan zurueckgegeben
         * werden.
         */
        four_tuple<int,int,int,int> holenFahrplanIntervall();

	void holenMaximaleFahrzeugAnzahl();
};
#endif //FAHRPLAN_H










