
#ifndef SCHNITTSTELLENTEST_H
#define SCHNITTSTELLENTEST_H

#include "CTransportkettengraph.h"
#include <LEDA/graph.h>
#include "CFPlanGraphKante.h"
#include "CFPlanGraphKnoten.h"
#include "CFahrplan.h"
#include <LEDA/list.h>
#include <LEDA/array.h>
#include <LEDA/tuple.h>
#include <LEDA/h_array.h>
#include "CFahrplanElement.h"
#include "CGuetemasse.h"

/**
 * Diese Klasse dient als Schnittstellentest
 */
class CSchnittstellenTest{

public:    

  ~CSchnittstellenTest ();

  /**
   * Diese Methode wird nur fuer Testzwecke benoetigt. Sie erstellt ein Netz der Transportverbindungen
   * und ruft mit diesem Graphen die Klasse CFahrplan auf.
   */
  void fahrplanerstellungWegenetzplan();

  /**
   * Hier soll die Maximale Fahrzeuganzahl uebergeben werden.
   */
  int fahrplanerstellungMaximaleFahrzeuganzahl();

  /**
   * Bei dieser Methode wird der Allgemeine Takt (Minuten - Wert) uebergeben.
   */
  int fahrplanerstellungAllgemeinerTakt();
    
  /**
   * Diese Methode erhaelt den erstellten Fahrplan zur Speicherung in die Datenbank.
   */
  void fahrplanerstellungErgebnisrueckgabe(list<CFahrplanElement*>*);

  /**
   * Uebergabe der Ergebnisse der Fahrplanberechnung
   * hinsichtlich der Wartezeiten
   * 1. Summe der netzbedingten Wartezeiten
   * 2. Wartezeit-Aufwand
   * 3. Mindestwartezeit-Aufwand
   * 4. Gesamt-Wartezeit-Aufwand
   */
  void fahrplanerstellungWartezeiten (double, double, double, double);
        
  /**
   * Uebergabe der benoetigten Fahrzeug-Anzahl
   */
  void fahrplanerstellungAnzahlBenoetigterFahrzeuge (int);

  /**
   * Diese Methode erhaelt die Bewertung der Ausgangsloesung
   *
   *
   *!!!!!!!!!!!!!!!!!!!!! NOCH BEARBEITEN !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   *
   */
  //  void fahrplanerstellungAusgangsloesung(int,int,int,int,int);
    
  /**
   * Starten der Berechnung einer Ausgangsloesung fuer
   * die entsprechende Zusammenhangskomponete gekennzeichnet
   * durch die ID
   */
  void fahrplanerstellungHeuristischeLoesung (int);

  /**
   * Diese Methode erhaelt die Zusammenhangskomponente
   * Als Parameter wird ein Array der Zusammenhangskomponenten uebergeben, in dem wiederum eine Liste von three_tuple steht
   * Die Werte sind dabei 1. HaltestellenID, 2. HaltestellenID und LinienID
   */
  void fahrplanerstellungZusammenhangskomponenten(array<list<three_tuple<int,int,int>* >* > *);

  /**
   * Die Methode liefert ein Array von four_tuple zurueck, welche folgende Werte haben:
   *  - HaltestellenID
   *  - Liniennr
   *  - minute (Reicht aus, da in dem Intervall ein fester Takt gilt);
   *
   * Das Array muss dabei die gleiche Reihenfolge haben, wie das Array der Zusammenhangskomponenten
   */
  array<three_tuple<int,int,int>* >* fahrplanerstellungAbfahrtszeiten();
    
  /**
   * Methode wird zum Aktivieren/Deaktivieren von Menueintraegen benoetigt
   */
  void fahrplanerstellungMenueaktivierung();
    
  /**
   * Diese Methode uebergibt die Mindesthaltezeit fuer jede Haltestelle
   * Die Daten werden in einem Hash_Array zurueckgegeben, indem der erste Wert
   * die HaltestellenID und der zweite Wert die Haltezeit an dieser Haltestelle
   * zurueckgegeben werden.
   */
  h_array<int,int>* fahrplanerstellungMindesthaltezeit();
    
  /**
   * Diese Methode uebergibt die Mindestumsteigezeit fuer jede Kombination von
   * Haltestelle und Liniennummern.
   * Der Rueckgabewert baut sich wie folgt auf:
   *  Das erste Element im Hash_Array ist die HaltestellenID bei dem zweiten Element handelt es sich
   *  um eine Liste bestehend aus Liniennummer1, Liniennummer2 und die Mindestumsteigezeit zwischen diesen Linien.
   */
  h_array<int,list<three_tuple<int,int,int>* >*>* fahrplanerstellungMindestumsteigezeit();
    
  /**
   * WIRD ZURZEIT NOCH NICHT BENOETIGT !!!!!!!!!
   */
  //  void fahrplanerstellungNeuerLinienplan();

  /**
   * Diese Methode ruft im Fahrplan die Methode ausgebenFahrplan auf. Nach Beendigung der Methode wurde ein
   * Fahrplan uebergeben
   */
  void fahrplanerstellungAusgabeFahrplan();

  /**
   *
   */
  void fahrplanerstellungBerechnenFahrzeugAnzahl ();

  /**
   *
   */
  void fahrplanerstellungBerechnenWartezeiten ();

  /**
   * Bei dieser Methode wird das Intervall zurueckgegeben, fuer das der Fahrplan gueltig sein soll
   * Dabei gilt folgende Reihenfolge bei den int - Werten :
   * Stunde Von, Minute Von, Stunde Bis, Minute Bis
   */
  four_tuple<int,int,int,int>* fahrplanerstellungFahrplanIntervall();

  /**
   * Setzen der Liste von festen Verbindungen
   * Diese Methode liefert in einer Liste alle tupel zurueck die eine feste Bindung haben
   * sollen. Dabei wird die ZusammenhangskomponentenID, die ID der Linie 1, die ID der 2. Linie und die Umsteigehaltestellen ID uebergeben.
   */
  list<four_tuple<int, int, int, int>* >* fahrplanerstellungFesteVerbindungen ();

  /**
   * Starten der Berechnung des Entscheidungsbaumverfahrens
   * der entsprechenden Zusammenhangskomponete gekennzeichnet
   * durch die ID
   */
  void fahrplanerstellungEntscheidungsbaumverfahren (int,int);
 
  /**
   * Starten der Berechnung der Lokalen Suche
   * mit der entsprechenden Zusammenhangskomponete gekennzeichnet
   * durch die ID
   */
  void fahrplanerstellungSukzessiveLoesungsverbesserung (int);

  void speichernGuetemasse(CGuetemasse*);
  CGuetemasse* holenGuetemasse();

 private:

  /**
   * In diesem Attribut wird ein Objekt der Klasse CFahrplan gehalten
   */
  CFahrplan* FahrplanKlasse;
};
#endif //SCHNITTSTELLENTEST












