///------------------------------------------------------------------------------
// Projektname:		Fahrplan-Modul
// Dateiname:	        CSimplexTableau.cpp
// Beschreibung:	Berechnen des SimplexTableaus einer
//                      Zusammenhangskomponente
// Autor(en):		Michael Wintergerst
// Version:		0.1
// Gendert von:	
// Gendert am:		
// Zustand:		Erst-Version
//------------------------------------------------------------------------------

#include "CSimplexTableau.h"
 
//------------------------------------------------------------------------------
// Funktionsname:  CSimplexTableau	
// Beschreibung:   Konstruktor	
// Parameter:	   Anzahl Zeilen, Spalten und die Taktzeit	
// Rueckgabe:	   void	
// Besonderheiten:	
//------------------------------------------------------------------------------       
CSimplexTableau::CSimplexTableau(int zeilen, int spalten, int taktzeit){
  //  Debug (1, "CSimplexTableau::CSimplexTableau()");
  this->anzahlZeilen = zeilen;
  this->anzahlSpalten = spalten;
  this->takt = taktzeit;
  this->stoppFlag = false;
  this->spaltenMapping = new array<edge> (spalten);
  this->zeilenMapping = new array<edge> (zeilen);
  this->simplexTableau = new TSparseMatrix (zeilen, spalten);
  this->resultatsVektor = new vector (zeilen);
} // ende des konstruktors

//------------------------------------------------------------------------------
// Funktionsname:  ~CSimplexTableau	
// Beschreibung:   Definiton des Destruktors	
// Parameter:	   void	
// Rueckgabe:	   void	
// Besonderheiten:	
//------------------------------------------------------------------------------
CSimplexTableau::~CSimplexTableau () {
  //  Debug (2, "CSimplexTableau::~CSimplexTableau()");
  delete this->zeilenMapping;
  delete this->spaltenMapping;
  delete this->simplexTableau;
  delete this->resultatsVektor;
} // ende des destruktors


//------------------------------------------------------------------------------
// Funktionsname:  einfuegenKantenInSimplexTableau	
// Beschreibung:   Einfuegen der Kanten in das SimplexTableau	
// Parameter:	   Zeiger auf Liste der Kanten in der entsprechenden
//                 Zusammenhangskomponente	
// Rueckgabe:	   void	
// Besonderheiten:	
//------------------------------------------------------------------------------
void CSimplexTableau::einfuegenKantenInSimplextableau(GRAPH<CKnotenBeschriftung*,
						      CKantenBeschriftung*>& graph) {

  //  Debug (2, "CSimplexTableau::einfuegenKantenInSimplexTableau()");

  edge aktKante;
  CKantenBeschriftung* informationAktKante;
  int aktZeilenPosition = 0;
  int aktSpaltenPosition = 0;
  stack<edge>* kantenStack;

  // iteration ueber alle kanten des graphen
  // einfuegen der kanten in zeilen- oder spaltenmapping
  // je nach dem, ob geruestkante oder nicht und vermerk 
  // der position im simplextableau in der kantenbeschriftung

  forall_edges (aktKante, graph) {
    informationAktKante = graph.inf (aktKante);
    informationAktKante->setzenNetzbedingteWartezeit (0);
    // aktKante im Maximalgeruest
    if (informationAktKante->gebenImMaximalgeruest ()) {
      informationAktKante->setzenPositionImSimplexTableau (aktSpaltenPosition);
      this->spaltenMapping->set(aktSpaltenPosition, aktKante);
      //      Debug(9,"Aktuelle Spalten Position" << aktSpaltenPosition);
      aktSpaltenPosition++;
    } // ende if
    else {
      informationAktKante->setzenPositionImSimplexTableau (aktZeilenPosition);
      this->zeilenMapping->set(aktZeilenPosition, aktKante);
      //      Debug(9,"Aktuelle Zeilen Position" << aktZeilenPosition);
      aktZeilenPosition++;
    } // ende else
  } // ende der iteration ueber alle kanten des graphen

  // berechnung der zykluskanten fuer jede nichtgeruestkante
  CFortschritt fort;
  for (int i=0; i < this->anzahlZeilen; i++) {
    fort.anzeigenFortschritt (i, anzahlZeilen-1);
    kantenStack = this->berechnenZyklusKanten (graph, (*zeilenMapping)[i]);
    this->fuellenSimplexTableau (graph, (*zeilenMapping)[i], *kantenStack);
    this->berechnenNetzbedingteWartezeit (graph, (*zeilenMapping)[i]);
    if (kantenStack != 0)
    {
      delete kantenStack;
    }
  } // ende for-schleife ueber alle nichtgeruestkanten
} // ende einfuegenKantenInSimplextableau

//------------------------------------------------------------------------------
// Funktionsname:  fuellenSimplexTableau	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CSimplexTableau::fuellenSimplexTableau(GRAPH<CKnotenBeschriftung*,
					          CKantenBeschriftung*>& graph,
					    edge nichtGeruestKante, 
					    stack<edge>& kantenStack){

  //  Debug (2, "CSimplexTableau::fuellenSimplexTableau()");

  int aktRichtung = 1;
  int vorgaengerID;
  int aktZeileImSimplexTableau;
  edge aktKante;
  node aktSourceKnoten, aktTargetKnoten;
  node sourceKnoten, targetKnoten;
  CKnotenBeschriftung *informationSourceKnoten, *informationTargetKnoten;
  CKnotenBeschriftung *informationAktSourceKnoten, *informationAktTargetKnoten;
  CKantenBeschriftung *informationKante, *informationAktKante;

  // initialisierungen
  informationKante = graph.inf (nichtGeruestKante);
  sourceKnoten = graph.source (nichtGeruestKante);
  targetKnoten = graph.target (nichtGeruestKante);
  informationSourceKnoten = graph.inf (sourceKnoten);
  informationTargetKnoten = graph.inf (targetKnoten);
  vorgaengerID = informationSourceKnoten->gebenKnotenID ();
  aktZeileImSimplexTableau = informationKante->gebenPositionImSimplexTableau ();

  // iteration ueber alle stackelemente
  while (! (kantenStack.empty())) {
    aktKante = kantenStack.pop ();
    informationAktKante = graph.inf (aktKante);
    aktSourceKnoten = graph.source (aktKante);
    aktTargetKnoten = graph.target (aktKante);
    informationAktSourceKnoten = graph.inf (aktSourceKnoten);
    informationAktTargetKnoten = graph.inf (aktTargetKnoten);
    // Richtung berechnen

    // vergleiche vorgaenger mit target neu
    if (aktRichtung == 1) {
      if (informationAktTargetKnoten->gebenKnotenID() ==
	  vorgaengerID) {
	vorgaengerID = informationAktSourceKnoten->gebenKnotenID();
      }
      else {
	aktRichtung = -1;
	vorgaengerID = informationAktTargetKnoten->gebenKnotenID();
      }
    }
    // vergleiche vorgaenger mit source neu
    else {
      if (informationAktSourceKnoten->gebenKnotenID() ==
	  vorgaengerID) {
	vorgaengerID = informationAktTargetKnoten->gebenKnotenID();
      }
      else {
	aktRichtung = 1;
	vorgaengerID = informationAktSourceKnoten->gebenKnotenID();
      }
    }
    this->setzenSimplexTableauEintrag (aktZeileImSimplexTableau,
			informationAktKante->gebenPositionImSimplexTableau(),
			aktRichtung);
  } // iteration ueber alle kantenelemente
} // fuellenSimplexTableau

//------------------------------------------------------------------------------
// Funktionsname:  setzenSimplexTableauEintrag	
// Beschreibung:   setzt einen bestimmten Eintrag im SimplexTableau	
// Parameter:	   Zeile, Spalte, Wert	
// Rueckgabe:	   void	
// Besonderheiten:	
//------------------------------------------------------------------------------
void CSimplexTableau::setzenSimplexTableauEintrag(int zeile, int spalte, int richtung){

  //  Debug (2, "CSimplexTableau::setzenSimplexTableauEintrag()");
  this->simplexTableau->InsertEnd (zeile, spalte, richtung);
} // ende setzenSimplexTableauEintrag


//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CSimplexTableau::berechnenNetzbedingteWartezeit(GRAPH<CKnotenBeschriftung*, CKantenBeschriftung*>& graph, edge nichtGeruestKante){

  //  Debug (2, "CSimplexTableau::berechnenNetzbedingteWartezeit()");

  int netzbedingteWartezeit = 0;
  int aktZeile, aktSpalte, aktMindestHaltezeit;
  int aktRichtung, aktMindestUmsteigezeit, fahrzeit;
  CKantenBeschriftung *informationNichtGeruestKante, *informationAktKante;
  CKnotenBeschriftung *informationAktKnoten,
                      *informationSourceKnoten;
  node aktKnoten, sourceKnoten;
  edge aktKante;

  informationNichtGeruestKante = graph.inf (nichtGeruestKante);
  aktZeile = informationNichtGeruestKante->gebenPositionImSimplexTableau();
  aktMindestHaltezeit = informationNichtGeruestKante->gebenMindestHaltezeit ();
  aktMindestUmsteigezeit = informationNichtGeruestKante->gebenMindestUmsteigezeit ();
  sourceKnoten = graph.source (nichtGeruestKante);
  informationSourceKnoten = graph.inf (sourceKnoten);
  fahrzeit = informationSourceKnoten->ausgebenFahrzeit();

 

  netzbedingteWartezeit += (-1 *  (aktMindestHaltezeit +
				   fahrzeit + 
				   aktMindestUmsteigezeit));

  aktKnoten = graph.source (nichtGeruestKante);
  informationAktKnoten = graph.inf (aktKnoten);
  fahrzeit = informationAktKnoten->ausgebenFahrzeit ();

  // iteration ueber alle spaltenelemente
  TRow *lastRow = this->simplexTableau->GetLastRowPtr ();
  TEntry *entry = lastRow->FirstEntry;
  while (entry)
  {
    aktSpalte = entry->Index;
    aktRichtung = (int) entry->Value;
    aktKante = (*spaltenMapping)[aktSpalte];
    informationAktKante = graph.inf (aktKante);
    aktMindestHaltezeit = informationAktKante->gebenMindestHaltezeit ();
    aktMindestUmsteigezeit = informationAktKante->gebenMindestUmsteigezeit ();
    sourceKnoten = graph.source (aktKante);
    informationSourceKnoten = graph.inf (sourceKnoten);
    fahrzeit = informationSourceKnoten->ausgebenFahrzeit ();

    netzbedingteWartezeit += (-aktRichtung *  (aktMindestHaltezeit
					       + aktMindestUmsteigezeit + fahrzeit));
    entry = entry->Next;
  } // ende iteration ueber alle spaltenelemente

  // setzen der netzbedingten wartezeit an entsprechender
  // position im loesungsvektor
  while (netzbedingteWartezeit < 0) {
    netzbedingteWartezeit += this->takt;
    if (this->takt == 0) {
      break;
    }
  }
  while (netzbedingteWartezeit >= this->takt) {
    netzbedingteWartezeit -= this->takt;
    if (this->takt == 0) {
      break;
    }
  }

  (*(this->resultatsVektor)) [aktZeile] = netzbedingteWartezeit;
  (graph.inf ((*this->zeilenMapping)[aktZeile]))->setzenNetzbedingteWartezeit (netzbedingteWartezeit);
} // ende berechnenNetzbedingteWartezeit

//------------------------------------------------------------------------------
// Funktionsname:  berechnenTiefenSuche
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CSimplexTableau::berechnenTiefenSuche (GRAPH<CKnotenBeschriftung*,CKantenBeschriftung*>& graph,
			   node startKnoten, node zielKnoten, 
		           stack<edge>& stackZyklusKanten, 
			   node_array<bool>& besuchtKnotenArray) {
  edge aktKante;
  node targetKnoten;
  CKantenBeschriftung* informationAktKante;
  // markiere startKnoten als besucht
  besuchtKnotenArray[startKnoten] = true;

  // falls zielKnoten noch nicht besucht
  if (! besuchtKnotenArray[zielKnoten]) {
    // iteration ueber alle out-kanten des startknotens
    forall_out_edges (aktKante, startKnoten) {
      // weitere suche nur falls zielknoten noch nicht gefunden
      if (! besuchtKnotenArray[zielKnoten]) {
	informationAktKante = graph.inf (aktKante);
	// ueberpruefe, ob aktkante im maximalgeruest
	if (informationAktKante->gebenImMaximalgeruest()) {
	  targetKnoten = graph.target (aktKante);
	  if (! besuchtKnotenArray[targetKnoten]) {
	    stackZyklusKanten.push (aktKante);
	    this->berechnenTiefenSuche (graph, targetKnoten, zielKnoten,
					stackZyklusKanten, besuchtKnotenArray);
	  }
	} // ende-if kante im maximalGeruest
      } // ende if zielknoten noch nicht besucht
    } // ende iteration ueber alle out-kanten

    // iteration ueber alle in-kanten des startknotens
    forall_in_edges (aktKante, startKnoten) {
      // weitere suche nur falls zielknoten noch nicht gefunden
      if (! besuchtKnotenArray[zielKnoten]) {
	informationAktKante = graph.inf (aktKante);
	// ueberpruefe, ob aktkante im maximalgeruest
	if (informationAktKante->gebenImMaximalgeruest()) {
	  targetKnoten = graph.source (aktKante);
	  if (! besuchtKnotenArray[targetKnoten]) {
	    stackZyklusKanten.push (aktKante);
	    this->berechnenTiefenSuche (graph, targetKnoten, zielKnoten,
					stackZyklusKanten, besuchtKnotenArray);
	  }
	} // ende-if kante im maximalGeruest
      } // ende if zielknoten noch nicht besucht
    } // ende der iteration ueber alle  in-kanten
    
    // kein weg von startknoten zu zielknoten

    if (! besuchtKnotenArray[zielKnoten]) {
      stackZyklusKanten.pop ();
    }

  } // ende if zielKnoten noch nicht besucht
} // ende berechnenTiefenSuche

//------------------------------------------------------------------------------
// Funktionsname:  berechnenZyklusKanten	
// Beschreibung:   Berechnung der Zykluskante fuer eine Nichtgeruestkante	
// Parameter:	   Referenz auf Graph der Zusammenhangskomponente und
//                 eine NichtGeruestkante	
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
stack<edge>* CSimplexTableau::berechnenZyklusKanten (GRAPH<CKnotenBeschriftung*,
						           CKantenBeschriftung*>&
						     graph, edge nichtGeruestKante) {

  //  Debug (2, "CSimplexTableau::berechnenZyklusKanten()");

  // markiere alle knoten des graphen als unmarkiert
  node_array<bool> besuchtKnotenArray (graph);
  node aktKnoten;
  // stack fuer aufnahme der zyklus-kanten
  stack<edge>* stackZyklusKanten = new stack<edge>;

  // iteration ueber alle knoten
  // setzen des besucht-status auf false
  forall_nodes (aktKnoten, graph) {
    besuchtKnotenArray[aktKnoten] = false;
  } // ende der iteration ueber alle knoten

  this->berechnenTiefenSuche
    (graph, graph.target(nichtGeruestKante),graph.source(nichtGeruestKante),
     *stackZyklusKanten, besuchtKnotenArray);

  // zyklus gefunden
  if (besuchtKnotenArray[graph.source(nichtGeruestKante)]) {
    return stackZyklusKanten;
  }

  return 0;
} // ende berechnenZyklusKanten

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CSimplexTableau::AusgebenSimplexTableau () {
//  std::cout << "Ausgabe Simplextableau:" << endl;

  //  std::cout << *(this->simplexTableau);
//  std::cout << *(this->resultatsVektor);
} // ende AusgebenSimplexTableau

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
list<two_tuple<int,int>*>* CSimplexTableau::erstellenListeZeilenElemente ()
{
  list<two_tuple<int, int>* >* listZeilen = new list<two_tuple<int, int>* >;
  int reisezeitAufwand;
  int reisendenZahl, netzbedingteWartezeit;
  CKantenBeschriftung* informationEdge;
  two_tuple<int, int>* currentTuple;

  // iteriere ueber alle zeilen
  for (int i=0; i < this->anzahlZeilen; i++)
  {
    informationEdge =  (graphLokal->inf ((*this->zeilenMapping)[i]));
    reisendenZahl = informationEdge -> abfragenVerkehrsstrom ();
    netzbedingteWartezeit = (int) (*(this->resultatsVektor)) [i];
    reisezeitAufwand = reisendenZahl * netzbedingteWartezeit;
    currentTuple = new two_tuple <int, int> (i, reisezeitAufwand);
    listZeilen->append (currentTuple);
  } // ende iteration ueber alle zeilen

  return listZeilen;
} // erstellenZeilenElemente

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
int VergleichenTupel (two_tuple<int, int> * const & p, 
		      two_tuple<int, int> * const & q)
{
  if (p->second() < q->second()) return 1;
  if (p->second () > q->second()) return -1;
  return 0;
} // ende VergleichenTupel

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
double CSimplexTableau::BerechnenReisezeitAufwand (vector *endVektor, 
						int zeile,
						int spalte)
{
  double reisezeitAufwand = 0.0;
  int reisendenZahl, netzbedingteWartezeit;
  CKantenBeschriftung* informationEdge;
 
  // iteriere ueber alle zeilen
  for (int i=0; i < this->anzahlZeilen; i++)
  {
    if (i != zeile) 
    {
      informationEdge =  (graphLokal->inf ((*this->zeilenMapping)[i]));
    } // ende iteration ueber alle zeilen
    else
    {
      informationEdge =  (graphLokal->inf ((*this->spaltenMapping)[spalte]));
    }
    reisendenZahl = informationEdge -> abfragenVerkehrsstrom ();
    netzbedingteWartezeit = (int) (*(endVektor)) [i];
    reisezeitAufwand += (double) (reisendenZahl * netzbedingteWartezeit);
  }

  return reisezeitAufwand;  
} // ende BerechnenReisezeitAufwand

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CSimplexTableau::SpeichernZeile (int zeile, vector *tmp)
{
  double currentValue;

  // iteration ueber alle spalten
  for (int i=0; i < this->anzahlSpalten; i++)
  {
    currentValue = (*simplexTableau).GetElement (zeile, i);
    (*tmp)[i] = currentValue;
  } // ende iteration ueber spalten
} // ende SpeichernZeile

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CSimplexTableau::SpeichernSpalte (int spalte, vector *tmp)
{
  double currentValue;

  // iteration ueber alle zeilen
  for (int i=0; i < this->anzahlZeilen; i++)
  {
    currentValue = simplexTableau->GetElement (i, spalte);
    (*tmp)[i] = currentValue;
  } // ende iteration ueber zeilen
} // ende SpeichernSpalte

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CSimplexTableau::UmrechnenSimplexTableau (int zeile, int spalte)
{
  double currentValue;
  bool neuesElement;
  vector *vektorTmpZeile = new vector (this->anzahlSpalten);
  vector *vektorTmpSpalte = new vector (this->anzahlZeilen);

  SpeichernZeile (zeile, vektorTmpZeile);
  SpeichernSpalte (spalte, vektorTmpSpalte);

  // iteration ueber alle zeilen
  for (int i=0; i < this->anzahlZeilen; i++)
  {
    // iteration ueber alle spalten
    for (int j=0; j < this->anzahlSpalten; j++)
    {
      currentValue = (*simplexTableau).GetElement (i,j);

      neuesElement = false;
      if (currentValue == 0.0)
      {
	neuesElement = true;
      }

      if ( (i == zeile) && (j == spalte) )
      {
  	currentValue = (*vektorTmpZeile)[spalte];
      }

      if ( (i == zeile) && (j != spalte) )
      {
  	currentValue = (*vektorTmpZeile)[j] /
  	  (*vektorTmpZeile)[spalte];
      }

      if ( (i != zeile) && (j == spalte) )
      {
  	currentValue = - ( (*vektorTmpSpalte)[i] /  
			   (*vektorTmpZeile)[spalte] );
      }

      if ( (i != zeile) && (j != spalte) )
      {
	currentValue = currentValue - (
		     ( (*vektorTmpSpalte)[i] * (*vektorTmpZeile)[j] ) / 
					(*vektorTmpZeile)[spalte] );
      }
      
      if ( (neuesElement) && (currentValue != 0.0) )
      {
	simplexTableau->Insert (i, j, currentValue);
      }
      
      if ( (! neuesElement) && (currentValue != 0.0))
      {
	simplexTableau->Update (i, j, currentValue);
      }

      if ( (! neuesElement) && (currentValue == 0.0))
      {
	simplexTableau->Delete (i,j);
      }
    } // ende iteration ueber alle spalten
  } // ende iteration ueber alle zeilen
  
  delete vektorTmpZeile;
  delete vektorTmpSpalte;
} // ende UmrechnenSimplexTableau

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CSimplexTableau::UmkopierenResultatsVektor (vector *neuerResultatsVektor)
{
  // iteration ueber alle zeilen
  for (int i=0; i < this->anzahlZeilen; i++)
  {
    (*resultatsVektor)[i] = (*neuerResultatsVektor)[i];
  } // ende iteration ueber alle zeilen
} // ende UmkopierenResultatsVektor

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
vector* CSimplexTableau::UmrechnenResultatsVektor (int zeile, int spalte)
{
  vector *vektorTmp = new vector (this->anzahlZeilen);
  double currentValue, divisor, divisor2;

  // iteration ueber alle zeilen
  for (int i=0; i < this->anzahlZeilen; i++)
  {
    // umrechnung der resultatsvektor
    // dabei sonderbehandlung fuer die zeile, die
    // getauscht werden soll
    // genauere details siehe weigand seite 50
    // currentValue = (*vektorTmp)[i];
    divisor = simplexTableau->GetElement (zeile, spalte);
   
    if (i == zeile)
    {
      currentValue = (*resultatsVektor)[i] / divisor;
    }
    else
    {
      divisor2 = simplexTableau->GetElement (i, spalte);
      currentValue = (*resultatsVektor)[i] - ( ((*resultatsVektor)[zeile] *  
  	     divisor2) / 
  	     divisor);	   
    }

    while (currentValue < 0) {
      currentValue += this->takt;
      if (this->takt == 0) {
	break;
      }
    }

    while (currentValue >= this->takt) {
      currentValue -= this->takt;
      if (this->takt == 0) {
	break;
      }
    }

    (*vektorTmp)[i] = currentValue;
  } // ende iteration ueber alle zeilen

  return vektorTmp;
} // ende UmrechnenResultatsVektor

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CSimplexTableau::VertauschenKanten (int zeile, int spalte)
{
  edge kanteTmp = (*zeilenMapping)[zeile];
  edge kanteTmp2 = (*spaltenMapping)[spalte];
  (*spaltenMapping)[spalte] = kanteTmp;
  (*zeilenMapping)[zeile] = kanteTmp2;
} // ende VertauschenKanten

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
bool CSimplexTableau::SuchenVerbesserung (int zeile, double &reisezeitAufwand)
{
  double neuerReisezeitAufwand;
  vector* neuerVektor;
  
  // iteriere ueber alle spalten
  TRow *currentRow = this->simplexTableau->GetRowPtr (zeile);
  TEntry *entry = currentRow->FirstEntry;
  while (entry)
  {
    neuerVektor = UmrechnenResultatsVektor (zeile, entry->Index);
    neuerReisezeitAufwand = BerechnenReisezeitAufwand (
				    neuerVektor, zeile, entry->Index);
    if (neuerReisezeitAufwand < reisezeitAufwand)
    {
      reisezeitAufwand = neuerReisezeitAufwand;
//      std::cout << "Neuer Reisezeitaufwand: " << reisezeitAufwand << endl;
      VertauschenKanten (zeile, entry->Index);
      UmrechnenSimplexTableau (zeile, entry->Index);
      UmkopierenResultatsVektor (neuerVektor);
      delete neuerVektor;
      return true;
    }
    delete neuerVektor;
    entry = entry->Next;
  }

  return false;
  //  for (int i=0; i < this->anzahlSpalten; i++)
  // {
  //  value = (int) (*this->simplexTableau) (zeile,i);
  //  if (value != 0)
  // {
  //   neuerVektor = UmrechnenResultatsVektor (zeile, i);
  //   neuerReisezeitAufwand = BerechnenReisezeitAufwand (
  //				 neuerVektor, zeile, i);
  //   if (neuerReisezeitAufwand < reisezeitAufwand)
  //   {
  //	reisezeitAufwand = neuerReisezeitAufwand;
  //VertauschenKanten (zeile, i);
  //UmrechnenSimplexTableau (zeile, i);
  //UmkopierenResultatsVektor (neuerVektor);
  //	delete neuerVektor;
  //	return true;
  //   }
  //   delete neuerVektor;
  // } // ende if (value != 0)
  //} // ende iteration ueber alle spalten
}

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
double CSimplexTableau::BerechnenReisezeitAufwand ()
{
  double reisezeitAufwand = 0.0;
  int reisendenZahl, netzbedingteWartezeit;
  CKantenBeschriftung* informationEdge;
 
  // iteriere ueber alle zeilen
  for (int i=0; i < this->anzahlZeilen; i++)
  {
    informationEdge =  (graphLokal->inf ((*this->zeilenMapping)[i]));
    reisendenZahl = informationEdge -> abfragenVerkehrsstrom ();
    netzbedingteWartezeit = (int) (*(this->resultatsVektor)) [i];
    reisezeitAufwand += (double) (reisendenZahl * netzbedingteWartezeit);
  } // ende iteration ueber alle zeilen

  return reisezeitAufwand;
} // berechnenReisezeitAufwand


//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CSimplexTableau::startenLokaleSuche (GRAPH<CKnotenBeschriftung*, 
					  CKantenBeschriftung*>& graph)
{
  double reisezeitAufwand;
  bool lokalOptimal = false;
  bool verbessert = false;
  list<two_tuple<int,int> *> *listeZeilenElemente;
  two_tuple<int, int>* currentTuple;


  this->stoppFlag = false;
  graphLokal = &graph;
  reisezeitAufwand = this->BerechnenReisezeitAufwand ();

  // iteriere solange bis keine lokal 
  // optimale loesung gefunden ist
  int i;
  while (! lokalOptimal)
  {
    CFortschritt *fort = new CFortschritt ();
    i = 0;
    listeZeilenElemente = erstellenListeZeilenElemente ();
    listeZeilenElemente->sort (&VergleichenTupel, 1);
    
    // iteration ueber alle listenelemete
    forall (currentTuple, *listeZeilenElemente)
    {
      fort->anzeigenFortschritt (i+1, listeZeilenElemente->length ());
      // suche nach verbesserung in der aktuellen zeile
      verbessert = SuchenVerbesserung (currentTuple->first(),
				       reisezeitAufwand);
      if (verbessert)
      {
	break;
      }
      i++;
    } // ende iteration ueber alle listenelemente
    
    if (! verbessert)
    {
      lokalOptimal = true;
    }  
    delete fort;
    delete listeZeilenElemente;

    if (this->stoppFlag) break;
  } // ende while (! lokalOptimal)

  // setzen der netzbedingten wartezeiten an den kanten
  // iteration ueber alle spalten
  for (int i=0; i < this->anzahlSpalten; i++)
  {
    (graph.inf ((*this->spaltenMapping)[i]))->setzenNetzbedingteWartezeit (0);
    (graph.inf ((*this->spaltenMapping)[i]))->setzenImMaximalgeruest (true);
  } // ende iteration ueber alle spalten

  // iteration ueber alle zeilen
  for (int i=0; i < this->anzahlZeilen; i++)
  {
    (graph.inf ((*this->zeilenMapping)[i]))->setzenNetzbedingteWartezeit ((*resultatsVektor)[i]);
    (graph.inf ((*this->zeilenMapping)[i]))->setzenImMaximalgeruest (false);
  }

} // ende startenLokaleSuche

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CSimplexTableau::setzenStoppenFlag ()
{
  this->stoppFlag = true;
}
