/* Generated by Together */

#ifndef SIMPLEXTABLEAU_H
#define SIMPLEXTABLEAU_H
#include "CKantenBeschriftung.h"
#include "CKnotenBeschriftung.h"
#include "LEDA/stack.h"
#include "LEDA/graph.h"
#include "LEDA/matrix.h"
#include "LEDA/vector.h"
#include "LEDA/list.h"
#include "LEDA/array.h"
#include "LEDA/integer.h"
#include "Debug.h"
#include "LEDA/tuple.h"
#include "CSpMatrix.h"
#include "CFortschritt.h"

class CSimplexTableau {
    public:
        /**
         * Im Konstruktor werden die Anzahl Zeilen und Spalten und die Taktzeit
         * uebergeben. Diese Werte werden in den entsprechenden Attributen gespeichert.
         */
        CSimplexTableau(int, int, int);

	/**
	 * Deklaration des Destruktors
	 */
	~CSimplexTableau ();

	/**
	 * setzen des Stoppen-Flag
	 */
	void setzenStoppenFlag ();

        /**
         * Diese Funktion fuegt die Kanten in das Simplextableau ein. Dabei gehen
         * wir wie folgt vor:
         * Wir lesen die KantenID der Kante und merken sie uns kurz. Dann schauen wir nach,
         * ob imMaximalgeruest in CKantenBeschriftung TRUE ist, ist dies der Fall, so fuegen
         * wir sie in die naechste freie Spalte im spaltenMapping ein. Diesen Spaltenwert schreiben wir in die
         * Variable positionImSimplextableau in CKantenBeschriftung.
         * Analog mit den Nichtgeruestkanten hinsichtlich des zeilenMappings.
         * Dann wird die Methode berechnenZyklusKanten fuer jede Nichtgeruestkante aufgerufen.
         * Dann benutzen wir die Methode fuellenSimplexTableau, die den jeweiligen Stack und die
         * Nichtgeruestkante bekommt, und die entsprechende Zeile im Simplextableau fuellt.
         * Dann wird die Methode berechnenNetzbedingteWartezeit mit der Nichtgeruestkante
         * und dem entsprechenden Stack aufgerufen, die den Loesungsvektor berechnet.
         */
        void einfuegenKantenInSimplextableau(GRAPH<CKnotenBeschriftung*, CKantenBeschriftung*>&);
	
	/**
	 * Hilfsfunktion zur Ausgabe
	 * des Simplextableau auf dem Bildschirm
	 */
	void AusgebenSimplexTableau ();

	/**
	 * Diese Methode sucht in der erhaltenen Zeile
	 * nach der Loesungsverbesserung
	 * und liefert bei Auffinden dieser true zurueck und
	 * ansonsten einen False-Wert
	 * Weiterhin wird bei Auffinden einer Verbesserung der aktuelle
	 * Reisezeitaufwand geaendert
	 */
	bool SuchenVerbesserung (int, double &);

	/**
	 * Anstoss der Berechnung der Lokalen Suche auf dem existierenden
	 * Simplextableau
	 * Vorgehensweise:
	 * Anfangs wird die lokale Variable lokalOptimal auf false
	 * gesetzt.
	 * Dann wird solange iteriert bis die Bedingung lokalOptimal
	 * erfuellt ist.
	 * Im Koerper dieser Iteration wird eine Liste von Tupeln erstellt.
	 * Diese Tupeln enthalten zwei Werte: Zeilennummer und Reiseaufwand.
	 * Der Reiseaufwand ergibt sich aus netzbedingter Wartezeit multipliziert
	 * mit der Reisendenzahl. Diese Liste wird nach dem Reiseaufwand
	 * absteigend sortiert.
	 * Dann wird solange die Liste nicht leer ist und keine lokale
	 * Verbesserung gefunden worden ist iteriert.
	 * In diesem Koerper wird das aktuelle Listenelement genommen und fuer
	 * die entsprechende Zeile versucht, eine Verbesserung zu erzielen.
	 * Falls dieses moeglich wird die das Simplextableau und der
	 * Resultatsvektor geaendert und die lokale Variable gefundenVerbesserung
	 * auf true gesetzt, so dass dieser innere Koerper verlassen und die 
	 * aeussere Ueberpruefung auf Lokale Optimalitaet, die nicht
	 * gegeben ist, ueberprueft.
	 * Falls keine Verbesserung erzielt werden kann, wird das naechste 
	 * Listenelement betrachtet. Falls die Liste leer sein sollte und
	 * keine Verbesserung erreicht wurde, ist die gefunde Loesung
	 * lokal optimal.
	 */
	void startenLokaleSuche (GRAPH<CKnotenBeschriftung*,
				       CKantenBeschriftung*> &);
     private: // Methoden
        /**
         * Diese Methode erhaelt eine Nichtgeruestkante und den Stack von Geruestkanten,
         * der einen Zyklus mit der Nichtgeruestkante bildet.
         * Diese Funktion entfernt der Reihe nach die Kanten auf dem Stack und
         * ueberprueft die jeweilige Richtung zur NichtBasisKante.
         * Ist die jeweilige Kantenrichtung identisch, wird eine 1 in das Simplextableau
         * eingetragen. Ist sie Richtung entgegengesetzt, wird eine -1 eingetragen.
         * Dazu verwenden wird die in der Kantenbeschriftung abgelegten
         * Zeilen- bzw. Spaltenindeces im Simplextableau. Fuer das Setzen im Simplextableau
         * benutzen die Methode setzenSimplexTableauEintrag.
         */
        void fuellenSimplexTableau(GRAPH<CKnotenBeschriftung*,
				         CKantenBeschriftung*>&,edge, stack<edge>&);

        /**
         * Diese Funktion fuellt das simplexTableau an Position (zeile, spalte) mit dem
         * entsprechenden Wert. Dabei muss eventuell noch der bool Wert in einen int Wert
         * umgerechnet werden.
         */
        void setzenSimplexTableauEintrag(int,int,int);

        /**
         * Diese Methode bekommt als Parameter die Nichtgeruestkante, fuer die die
         * netzbedingte Wartezeit zu berechnen ist. Zusaetzlich wird der Stack der
         * Zyklus-Geruestkanten uebergeben.
         * Diese Methode berechnet fuer das Simplextableau die netzbedingten Wartezeiten fuer die entsprechende
         * NichtgeruestKante.
         * Im Resultatsvektor wird diese abgespeichert (an der entsprechenden Stelle)
         * Verfahren:
         * Die Wartezeiten (Umsteige- und Haltezeit) an allen Kanten des Zyklusses werden mit Beachtung der
         * Richtung (siehe Simplextableau) aufsummiert.
         * Dazu kommt die Summe der Fahrtzeiten.Diese berechnet sich aus der Summe (mit Beachtung
         * der Richtung) der Fahrtzeiten derjenigen Kanten, die sowohl einen Ingrad als auch
         * einen Outgrad von 1 hinsichtlich des Zyklusses haben. Dazu gehen wir die Stack-Kanten
         * der Reihe nach durch und ueberpruefen dieses anhand der Richtung.
         * Am Anfang vergleichen wird die NichtBasisKante mit der ersten StackKante. Haben diese
         * eine gemeinsamen Uebergangsknoten im Transportkettengrad, eine Kante geht raus und
         * eine Kante geht rein, wird die Fahrtzeit an diesen Knoten aufsummiert unter Beachtung
         * der Richtung (im Simplextableau vermerkt).
         * Diese wird entsprechend fortgesetzt, bis die letzte Kante des Stacks mit der Nichtbasiskante
         * verglichen wird.
         * Diese Summe wird insgesamt negiert und modulo Taktzeit gerechnet, was dann die
         * netzbedingte Wartezeit ergibt.
         */
        void berechnenNetzbedingteWartezeit(GRAPH<CKnotenBeschriftung*, CKantenBeschriftung*>&, edge);

	

    private: // Attribute
        /**
         * Diese Variable speichert die Anzahl der Zeilen des Zeigers simplexTableau, diese
         * ist natuerlich gleich der zyklomatischen Zahl.
         */
        int anzahlZeilen;

        /**
         * Diese Variable speichert die Anzahl der Spalten des Zeigers simplexTableau minus 1,
         * da wir die Ergebniszeile als eigenen Vektor realisieren.
         */
        int anzahlSpalten;

        /**
         * Taktzeit, die uns aus der Zusammenhangskomponente uebergibt.
         */
        int takt;

	/**
	 * stoppFlag zur Anzeige des Stoppens der Ausfuehrung
	 */
	bool stoppFlag;

	/**
	 *
	 */
	GRAPH<CKnotenBeschriftung*, CKantenBeschriftung*>* graphLokal;

        /**
         * Zeiger auf das Simplextableau, welches als LEDA Matrix realisiert ist.
         */
        TSparseMatrix* simplexTableau;

        /**
         * Zeiger auf den Resultatsvektor des Simplextableaus. (Siehe Weigand Seite 51)
         * In diesem Vektor werden die netzbedingten Wartezeiten eingetragen.
         */
        vector* resultatsVektor;

        /**
         * Um die netzbedingten Wartezeiten berechnen zu koennen, gehen wir die
         * Zeilen durch. Fuer jeden Eintrag verschieden von 0 muessen dann Berechnungen
         * durchgefuehrt werden. Die entsprechenden Werte, die wir fuer diese Berechnung
         * brauchen finden wir an den Kanten. Um auf diese zu greifen zu koennen, mappen
         * wir hier die Zeilen und Spalten des Arrays.
         */
        array<edge>* spaltenMapping;

        /**
         * Um die netzbedingten Wartezeiten berechnen zu koennen, gehen wir die
         * Zeilen ja durch. Fuer jeden Eintrag verschieden von 0 muessen dann Berechnungen
         * durchgefuehrt werden. Die entsprechenden Werte, die wir fuer diese Berechnung
         * brauchen finden wir an den Kanten. Um auf diese zu greifen zu koennen, mappen
q         * wir hier die Zeilen und Spalten des Arrays.
         */
        array<edge>* zeilenMapping;

    private: // Methoden
        /**
         * Diese Methode berechnet fuer eine eingebene Nichtgeruestkante diejenigen
         * Geruestkanten, die mit dieser Kanten einen Zyklus bilden.
         * Als Rueckgabe wird ein Stack verwendet, der diese Geruestkanten enthaelt.
         * Folgendes Vorgehen ist sinnvoll:
         * Wir starten einen DFS von dem Targetknoten der Nichtgeruestkanten
         * Der DFS beruecksichtigt alle Ingrad- und Outgradkanten, d.h. die Richtung wird
         * ignoriert. Falls eine Kante erreicht wird die eine Geruestkante darstellt, wird
         * diese auf einen Stack gelegt, und der DFS je nach Richtung der Kante vom Source- oder
         * Targetknoten fortgesetzt. Falls eine Nichtgeruestkante erreicht wird, wird der DFS
         * abgebrochen. Falls bei einem Knoten der DFS beendet wurde, aber kein Zyklus gefunden
         * wurde, wird die entsprechende Kante vom Stack entfernt und es wird weiter nach einem
         * Zyklus gesucht. Sollte eine Geruestkante den Source-Knoten der gegebenen Nichtgeruestkante
         * erreichen, ist der Zyklus gefunden.
         */
        stack<edge>* berechnenZyklusKanten (GRAPH<CKnotenBeschriftung*, CKantenBeschriftung*>&, edge);

	/**
	 *
	 */
	void berechnenTiefenSuche (GRAPH<CKnotenBeschriftung*,CKantenBeschriftung*>&,
				   node, node, 
				   stack<edge>&, 
				   node_array<bool>&);

	/**
	 * Diese Methode berechnet den neuen Resultatsvektor, der 
	 * sich ergibt, wenn man die Kanten in der spezifierten
	 * Zeile bzw. Spalte vertauscht.
	 * Dieses geschieht jedoch nur temporaer, so dass ein
	 * neuer Vektor erzeugt wird und zurueckgeliefert wird
	 */
	vector* UmrechnenResultatsVektor (int, int); 

	/**
	 * Berechnet auf dem zugrunde liegenden Simplextableau und
	 * dem abgespeicherten Graphen den gesamten Reisezeitaufwand
	 */
	double BerechnenReisezeitAufwand ();

	/**
	 *
	 */
	double BerechnenReisezeitAufwand (vector*, int, int);

	/**
	 * Speicherung der entsprechenden Zeile in dem
	 * spezifierten Vektor
	 */
	void SpeichernZeile (int, vector *);

	/**
	 * Speicherung der entsprechenden Spalte in dem
	 * spezifierten Vektor
	 */
	void SpeichernSpalte (int, vector *);

	/**
	 * Umrechnung des SimplexTableaus gemaess 
	 * weigand seite 50 bei Vertauschung
	 * 
	 */
	void UmrechnenSimplexTableau (int, int);

	/**
	 * Der uebergebene Vektor wird in dem Resultatsvektor
	 * gespeichert
	 */
	void UmkopierenResultatsVektor (vector *);

	/**
	 * Vertauschung der Kanten der Zeile bzw. Spalte
	 * in den entsprechenden Mappings
	 */
	void VertauschenKanten (int, int);

	/**
	 * erstelle von den zeilen two_tuple
	 * ersten Element: Zeilennummer
	 * zweites Element: Reiseaufwand (Netzbedingte Wartezeit
	 * multipliziert mit Reisendenzahl)
	 */
	list<two_tuple<int, int>* >* erstellenListeZeilenElemente();
};

int VergleichenTupel (two_tuple<int, int>* const &, 
		      two_tuple<int, int>* const &);
#endif //SIMPLEXTABLEAU_H
