//---------------------------------------------------------------------------
// Projektname:		GSSMC Kernel
// Dateiname:		$RCSfile: CSpMatrix.cpp,v $
// Beschreibung:	Implementierung einer dnn besetzten Matrix
// Autor(en):		Oliver P. Waldhorst
// Version:			$Revision: 1.3 $
// Gendert von:	$Author: winterge $
// Gendert am:		$Date: 2001/06/08 10:13:45 $
// Zustand:			$State: Exp $
//---------------------------------------------------------------------------

#define VM_BOUND_CHECK

// includes
#include <stdio.h>
#include "CSpMatrix.h"


//---------------------------------------------------------------------------
// Funktionsname:
// Beschreibung:	Konstruktor
// Parameter:
// Rueckgabe:
// Besonderheiten:
//---------------------------------------------------------------------------

TSparseMatrix::TSparseMatrix()
{
  // Initialisieren
  _FirstRow = NULL;
  _LastRow = NULL;
  _m = 0;
  _n = 0;
  _NonZeros = 0;
}


//----------------------------------------------------------------------------
// Funktionsname:
// Beschreibung:	Konstruktor
// Parameter:		m: Anzahl Zeilen; n: Anzahl Spalten
// Rueckgabe:
// Besonderheiten:
//---------------------------------------------------------------------------

TSparseMatrix::TSparseMatrix( UINT m, UINT n )
{
  // Initialisieren
  _FirstRow = NULL;
  _LastRow = NULL;
  _m = m;
  _n = n;
  _NonZeros = 0;
  _RowPtr = new TRow*[_m];
}


//---------------------------------------------------------------------------
// Funktionsname:
// Beschreibung:	Copy-Konstruktor
// Parameter:
// Rueckgabe:
// Besonderheiten:
//---------------------------------------------------------------------------

TSparseMatrix::TSparseMatrix( const TSparseMatrix &M )
{
  // Initialisieren
  _FirstRow = NULL;
  _LastRow = NULL;
  _NonZeros = 0;

	// neue Dimensionen
  _m = M._m;
  _n = M._n;

	// Alle Zeilen kopieren
  for( TRow *Row = M._FirstRow; Row != NULL; Row = Row->Next )
    {
      for( TEntry *Entry = Row->FirstEntry; Entry != NULL; Entry = Entry->Next )
	{
	  InsertEnd( Row->Number, Entry->Index, Entry->Value );
	}
    }
}


//---------------------------------------------------------------------------
// Funktionsname:
// Beschreibung:	Konstruktor
// Parameter:		m: Anzahl Zeilen; n: Anzahl Spalten
//                      NonZeros: Anzahl der Eintraege != 0
//                      Values: Pointer auf Speicherbereich mit den Eintrgen
//                      RowPtr:
//                      ColInd:
// Rueckgabe:
// Besonderheiten:
//---------------------------------------------------------------------------

TSparseMatrix::TSparseMatrix( UINT m, UINT n, UINT NonZeros,
			      double *Values, UINT *RowPtr, UINT *ColInd )
{
  // Initialisieren
  _FirstRow = NULL;
  _LastRow = NULL;
  _m = m;
  _n = n;
  _NonZeros = 0;

  for( UINT i= 0; i < _m; i++ )
    {
      // ber alle Eintrge der Zeile
      for( UINT j= RowPtr[i]; j < RowPtr[i+1]; j++ )
	{
	  // eintragen
	  InsertEnd( i, ColInd[j], Values[j] );
	}
    }	// for( int i= 0; i < Spm.dim(0); i++ )
}


//---------------------------------------------------------------------------
// Funktionsname:
// Beschreibung:	Destruktor
// Parameter:
// Rueckgabe:
// Besonderheiten:
//---------------------------------------------------------------------------

TSparseMatrix::~TSparseMatrix()
{
  // zurcksetzen, wenn nicht leer
  if( _NonZeros > 0 )
    {
      Reset();
    }
  delete[] _RowPtr;
}


//---------------------------------------------------------------------------
// Funktionsname:
// Beschreibung:	Zuweisungsoperator
// Parameter:		Ursprungsmatrix
// Rueckgabe:
// Besonderheiten:
//---------------------------------------------------------------------------

TSparseMatrix &
TSparseMatrix::operator= ( const TSparseMatrix &M )
{
#ifdef DEBUG_METHODS
  cout << "TSparseMatrix::operator= ( const TSparseMatrix &M )\n";
#endif

  // alte Zielmatrix zurcksetzen, wenn nicht leer
  if( _NonZeros > 0 )
    {
      Reset();
    }

  // neue Dimensionen
  _m = M._m;
  _n = M._n;

  // Alle Zeilen kopiernen
  for( TRow *Row= M._FirstRow; Row != NULL; Row= Row->Next )
    {
      for( TEntry *Entry = Row->FirstEntry; Entry != NULL; Entry = Entry->Next )
	{
	  InsertEnd( Row->Number, Entry->Index, Entry->Value );
	}
    }
  return *this;
}


//---------------------------------------------------------------------------
// Funktionsname:	operator+=(const TSparseMatrix &)
// Beschreibung:	this = this + B
// Parameter:		B	Quellmatrix
// Rueckgabe:		Ergebnismatrix
// Besonderheiten:	-
//---------------------------------------------------------------------------

TSparseMatrix &
TSparseMatrix::operator+=(const TSparseMatrix &B)
{

#ifdef VM_BOUND_CHECK
  if( (_m != B._m) || (_n != B._n) )
    {
      cerr << "Falsche Dimensionen in TSparseMatrix::operator+="
	"(const TSparseMatrix &B)" << endl;
      exit(1);
    }
#endif

  // hier noch Test, ob B == this ?

  TRow* rowPtr = _FirstRow;  // Zeiger auf Zeile der this-Matrix
  TRow* rowPrePtr = NULL;    // Zeiger auf Zeile vor rowPtr
  TRow* bRowPtr = B._FirstRow;  // Zeiger auf Zeile der B-Matrix

  while ((rowPtr != NULL) && (bRowPtr != NULL)) {

    if (rowPtr->Number < bRowPtr->Number) {
      rowPrePtr = rowPtr;
      rowPtr = rowPtr->Next;
      continue;
    }

    else if (rowPtr->Number > bRowPtr->Number) {
      // Zeile aus B vor aktueller Zeile einfuegen
      TRow *neu = new TRow;
      neu->Next = rowPtr;
      neu->Number = bRowPtr->Number;
      if (rowPrePtr == NULL) {
	_FirstRow = neu;
      }
      else {
	rowPrePtr->Next = neu;
      }
      rowPrePtr = neu;

      TEntry *tmp = bRowPtr->FirstEntry;
      TEntry *letzterEintrag = NULL;

      // Eintrge in Matrix bernehmen
      while (tmp != NULL) {
	TEntry *neuerEintrag = new TEntry;
	neuerEintrag->Index = tmp->Index;
	neuerEintrag->Value = tmp->Value;
	neuerEintrag->Next = NULL;
	_NonZeros++;
	if (letzterEintrag == NULL) {
	  // erster Eintrag der neuen Zeile
	  neu->FirstEntry = neuerEintrag;
	}
	else {
	  letzterEintrag->Next = neuerEintrag;
	}
	letzterEintrag = neuerEintrag;
	tmp = tmp->Next;
      }
      neu->LastEntry = letzterEintrag;

      bRowPtr = bRowPtr->Next;
      continue;
    } // if (rowPtr->Number > bRowPtr->Number) {


    // hier ist rowPtr->Number == bRowPtr->Number, also summieren
    TEntry *tmpEntry = rowPtr->FirstEntry;
    TEntry *tmpPreEntry = NULL;
    TEntry *tmpBEntry = bRowPtr->FirstEntry;

    while ((tmpEntry != NULL) && (tmpBEntry != NULL)) {

      if (tmpEntry->Index < tmpBEntry->Index) {
	tmpPreEntry = tmpEntry;
	tmpEntry = tmpEntry->Next;
	continue;
      }

      else if (tmpEntry->Index > tmpBEntry->Index) {
	// Wert aus B einfuegen
	TEntry *neuerEintrag = new TEntry;
	neuerEintrag->Index = tmpBEntry->Index;
	neuerEintrag->Value = tmpBEntry->Value;
	neuerEintrag->Next = tmpEntry;
	_NonZeros++;
	if (tmpPreEntry == NULL) {
	  rowPtr->FirstEntry = neuerEintrag;
	}
	else {
	  tmpPreEntry->Next = neuerEintrag;
	}
	tmpPreEntry = neuerEintrag;
	tmpBEntry = tmpBEntry->Next;
	continue;
      } // if (tmpEntry->Index > tmpBEntry->Index) {

      // tmpEntry->Index == tmpBEntry->Index
      double Summe = tmpEntry->Value + tmpBEntry->Value;

      if (Summe == 0.0) {
	// Eintrag loeschen
	if (tmpPreEntry == NULL) {
	  rowPtr->FirstEntry = tmpEntry->Next;
	  delete tmpEntry;
	  tmpEntry = rowPtr->FirstEntry;
	}
	else {
	  tmpPreEntry->Next = tmpEntry->Next;
	  delete tmpEntry;
	  tmpEntry = tmpPreEntry->Next;
	}
	tmpBEntry = tmpBEntry->Next;
	_NonZeros--;
	continue;
      }

      tmpEntry->Value = Summe;
      tmpPreEntry = tmpEntry;
      tmpEntry = tmpEntry->Next;
      tmpBEntry = tmpBEntry->Next;

    } // while ((tmpEntry != NULL) && (tmpBEntry != NULL)) {

    // von den folgenden While/If-Bedingungen ist hoechstens eine wahr!
    if (tmpEntry != NULL) {
      tmpPreEntry = rowPtr->LastEntry;
    }

    while (tmpBEntry != NULL) {
      TEntry *neuerEintrag = new TEntry;
      neuerEintrag->Index = tmpBEntry->Index;
      neuerEintrag->Value = tmpBEntry->Value;
      neuerEintrag->Next = NULL;
      _NonZeros++;
      if (tmpPreEntry == NULL) {
	// erster Eintrag der neuen Zeile
	rowPtr->FirstEntry = neuerEintrag;
      }
      else {
	tmpPreEntry->Next = neuerEintrag;
      }
      tmpPreEntry = neuerEintrag;
      tmpBEntry = tmpBEntry->Next;
    } // while (tmpBEntry != NULL) {

    // an dieser Stelle zeigt tmpPreEntry auf das letzte Element
    // der aktuellen Zeile

    if (tmpPreEntry == NULL) {
      // diese ganze zeile ist jetzt leer, also loeschen
      if (rowPrePtr == NULL) {
	_FirstRow = rowPtr->Next;
	delete rowPtr;
	rowPtr = _FirstRow;
      }
      else {
	rowPrePtr->Next = rowPtr->Next;
	delete rowPtr;
	rowPtr = rowPrePtr->Next;
      }
      bRowPtr = bRowPtr->Next;
      continue;
    }

    rowPtr->LastEntry = tmpPreEntry;
    rowPtr = rowPtr->Next;
    bRowPtr = bRowPtr->Next;
  } // while ((rowPtr != NULL) && (bRowPtr != NULL)) {

  // eventuelle Restzeilen einer Matrix anhngen
  // von den folgenden 2 while/if-Bedingungen ist hoechstens eine wahr

  if (rowPtr != NULL) {
    return (*this);
  }

  while (bRowPtr != NULL) {
    TRow *neu = new TRow;
    neu->FirstEntry = NULL;
    neu->Next = NULL;
    neu->Number = bRowPtr->Number;
    TEntry *tmp = bRowPtr->FirstEntry;
    TEntry *neuerEintrag;
    TEntry *letzterEintrag = NULL;

      // Eintrge in Matrix bernehmen
    while (tmp != NULL) {
      neuerEintrag = new TEntry;
      neuerEintrag->Index = tmp->Index;
      neuerEintrag->Value = tmp->Value;
      neuerEintrag->Next = NULL;
      _NonZeros++;
      if (letzterEintrag == NULL) {
	// erster Eintrag der neuen Zeile
	neu->FirstEntry = neuerEintrag;
      }
      else {
	letzterEintrag->Next = neuerEintrag;
      }
      letzterEintrag = neuerEintrag;
      tmp = tmp->Next;
    }

    if (rowPrePtr == NULL) {
      // erste Zeile der neuen Matrix
      _FirstRow = neu;
    }
    else {
      rowPrePtr->Next = neu;
    }
    rowPrePtr = neu;
    bRowPtr = bRowPtr->Next;
  } // while (bRowPtr != NULL) {

  _LastRow = rowPrePtr;

  return (*this);
}


//---------------------------------------------------------------------------
// Funktionsname:	InsertEnd( UINT, UINT, double )
// Beschreibung:	Fgt Elemente am Ende der letzten Zeile oder
//                      in neue Zeile ein
// Parameter:		i	Zeile
//			j	Spalte
//			Value	Wert
// Rueckgabe:		-
// Besonderheiten:	Reihenfolge beachten!!!
//---------------------------------------------------------------------------

void
TSparseMatrix::InsertEnd( UINT i, UINT j, double Value )
{
#ifdef VM_BOUND_CHECK
  // Grenzen prfen
  if ( (i >= _m) || (j >= _n) )
    {
      cerr << "Falsche Dimensionen in TSparseMatrix::InsertEnd"
	"(UINT, UINT, double)" << endl;
      exit(1);
    }
#endif

  // 0.0 nicht eintragen
  if ( Value == 0.0 )
    {
      return;
    }

  // Muss neue Zeile angelegt werden?
  if ( !_LastRow || (_LastRow->Number != i) )
    {
      // anlegen & fllen
      TRow *NewRow = new TRow;

      _RowPtr[i] = NewRow;

      NewRow->FirstEntry = NewRow->LastEntry= NULL;
      NewRow->Next = NULL;
      NewRow->Number = i;

      // Leer => vorne einhngen, sonst hinten
      if( !_FirstRow )
	{
	  _FirstRow = NewRow;
	}
      else
	{
	  _LastRow->Next = NewRow;
	}

      _LastRow = NewRow;

    }	// if( !LastRow || (LastRow->Number != Row) )

  // Neuen Eintrag anlegen & fllen
  TEntry *NewEntry = new TEntry;

  NewEntry->Value = Value;
  NewEntry->Index = j;
  NewEntry->Next = NULL;

  // Eintrag einhngen
  if ( _LastRow->FirstEntry == NULL )
    {
      // leere Liste -> vorne Einhngen
      _LastRow->FirstEntry = NewEntry;
    }
  else
    {
      // hinten anhngen
      _LastRow->LastEntry->Next = NewEntry;
    }

  // Neues Ende
  _LastRow->LastEntry = NewEntry;

  // Einrge erhhen
  _NonZeros++;

  return;
}	// InsertEnd


//---------------------------------------------------------------------------
// Funktionsname:
// Beschreibung:	loescht alle Eintraege und setzt die Matrix zurueck
// Parameter:
// Rueckgabe:
// Besonderheiten:
//---------------------------------------------------------------------------

void
TSparseMatrix::Reset()
{
  // Fr alle Zeilen
  for( TRow *Row = _FirstRow, *TmpRow; Row != NULL; Row = TmpRow )
    {
      // Liste der Eintrge lschen
      TEntry *Entry = Row->FirstEntry, *TmpEntry;
      while( Entry )
	{
	  TmpEntry = Entry->Next;
	  delete Entry;
	  Entry = TmpEntry;
	}

      // Zeile lschen
      TmpRow = Row->Next;
      delete Row;

    }	// for( TRow *Row= _FirstRow, TmpRow; Row != NULL; Row= Row->Next )

  // Member zurcksetzen
  _FirstRow = _LastRow = NULL;
  _NonZeros = 0;
  _m = _n = 0;
}


//---------------------------------------------------------------------------
// Funktionsname:
// Beschreibung:	loescht alle Eintraege und setzt Matrixgroesse
// Parameter:
// Rueckgabe:
// Besonderheiten:
//---------------------------------------------------------------------------

void
TSparseMatrix::Reset( UINT m, UINT n )
{
  // alten Inhalt lschen
  if( _NonZeros > 0)
    {
      Reset();
    }

  // Initialisieren
  _m = m;
  _n = n;
}


//---------------------------------------------------------------------------
// Funktionsname:
// Beschreibung:	Loescht alle Zeiger auf die Reihen, setzt die Matrix-
//                      Dimensionen
// Parameter:
// Rueckgabe:
// Besonderheiten:	Eintraege bleiben erhalten
//---------------------------------------------------------------------------

void
TSparseMatrix::ResetRowsWithoutEntries( UINT m, UINT n )
{
  // Fr alle Zeilen
  for( TRow *Row = _FirstRow, *TmpRow; Row != NULL; Row = TmpRow )
    {
      // Zeile lschen
      TmpRow = Row->Next;
      delete Row;

    }	// for( TRow *Row= _FirstRow, TmpRow; Row != NULL; Row= Row->Next )

  // Member zurcksetzen
  _FirstRow = _LastRow = NULL;
  _NonZeros = 0;

  // Dimensionen initialisieren
  _m = m;
  _n = n;
}


//---------------------------------------------------------------------------
// Funktionsname:	AddScaledMatrix(double, const TSparseMatrix &)
// Beschreibung:	A = A + d*B
// Parameter:		d	Skalar
//			B	Quellmatrix
// Rueckgabe:		Ergebnismatrix
// Besonderheiten:	-
//---------------------------------------------------------------------------

TSparseMatrix &
TSparseMatrix::AddScaledMatrix(double d, const TSparseMatrix &B)
{
  bool deleteRow = false;

#ifdef VM_BOUND_CHECK
  if( (_m != B._m) || (_n != B._n) )
    {
      cerr << "Falsche Dimensionen in AddScaledMatrix!\n";
      exit(1);
    }
#endif

  if (d == 0) {
    return (*this);
  }

  // Reiverschluss ber Matrix-Zeilen
  TRow *ARow= _FirstRow;
  TRow *PrevARow= _FirstRow;
  TRow *BRow= B._FirstRow;

  UINT ARowNo, BRowNo;

  while( (ARow != NULL) || (BRow != NULL) )
    {
      ARowNo= (ARow != NULL) ? ARow->Number : 10000000;
      BRowNo= (BRow != NULL) ? BRow->Number : 10000000;

      if( ARowNo < BRowNo )
	{
	  // Zeile bernehmen

	  // Nchste Zeile
	  PrevARow = ARow;
	  ARow= ARow->Next;

	  continue;

	}	// if( ARow < BRow )

      if( ARowNo > BRowNo )
	{
	  // Zeile aus B einfgen

	  TRow *TempRow = new TRow;

	  TempRow->Number = BRowNo;
	  TempRow->Next = ARow;
	  TempRow->FirstEntry = NULL;
	  TempRow->LastEntry = NULL;

	  if(ARow == NULL)
	    {
	      if(_FirstRow == NULL)
		{
		  // leere Matrix
		  _FirstRow = TempRow;
		  _LastRow  = TempRow;
		  PrevARow  = TempRow;
		}
	      else
		{
		  // Ende der Matrix erreicht
		  PrevARow->Next = TempRow;
		  _LastRow  = TempRow;
		  PrevARow = TempRow;
		}
	    }
	  else
	    {
	      if(ARow == _FirstRow)
		{
		  _FirstRow = TempRow;
		  PrevARow  = TempRow;
		}
	      else
		{
		  PrevARow->Next = TempRow;
		  PrevARow = TempRow;
		}
	    }

	  // Zeile aus B einfgen
	  for( TEntry *Entry= BRow->FirstEntry; Entry != NULL; Entry= Entry->Next )
	    {
				// Neuen Eintrag anlegen & fllen
	      TEntry *NewEntry= new TEntry;

	      NewEntry->Value = d * Entry->Value;
	      NewEntry->Index = Entry->Index;
	      NewEntry->Next  = NULL;

				// Eintrag einhngen
	      if( TempRow->FirstEntry == NULL )
		{
		  // leere Liste -> vorne Einhngen
		  TempRow->FirstEntry = NewEntry;
		}
	      else
		{
		  // hinten anhngen
		  TempRow->LastEntry->Next= NewEntry;
		}

				// Neues Ende
	      TempRow->LastEntry= NewEntry;

				// Einrge erhhen
	      _NonZeros++;
	    }

	  // Nchste Zeile
	  BRow = BRow->Next;

	  continue;

	}	// if( ARow > BRow )


      // ( ARow == BRow ) => Reiverschluss ber Eintrge

      TEntry *AEntry= ARow->FirstEntry;
      TEntry *PrevAEntry= ARow->FirstEntry;
      TEntry *BEntry= BRow->FirstEntry;

      UINT AIndex, BIndex;

      while( (AEntry != NULL) || (BEntry != NULL) )
	{
	  AIndex= (AEntry != NULL) ? AEntry->Index : 10000000;
	  BIndex= (BEntry != NULL) ? BEntry->Index : 10000000;

	  if( AIndex < BIndex )
	    {
				// Nchster Eintrag
	      PrevAEntry = AEntry;
	      AEntry= AEntry->Next;

	      continue;

	    }	// if( AIndex < BIndex )

	  if( AIndex > BIndex )
	    {
				// Neuen Eintrag anlegen & fllen
	      TEntry *NewEntry= new TEntry;

	      NewEntry->Value = d * BEntry->Value;
	      NewEntry->Index = BIndex;
	      NewEntry->Next  = AEntry;

	      if(ARow->FirstEntry == AEntry)
		{
		  ARow->FirstEntry = NewEntry;
		  PrevAEntry = NewEntry;
		}
	      else
		{
		  PrevAEntry->Next = NewEntry;
		  PrevAEntry = NewEntry;
		}
				// Nchster Eintrag
	      BEntry= BEntry->Next;

				// Einrge erhhen
	      _NonZeros++;

	      continue;

	    }	// if( AIndex < BIndex )

	  // ( AIndex == BIndex ) => Addieren - dabei Ergebnis auf 0 testen
	  AEntry->Value +=  d * BEntry->Value;
	  BEntry= BEntry->Next;

	  if (AEntry->Value != 0) {
	    // Nchste Eintrge
	    PrevAEntry = AEntry;
	    AEntry= AEntry->Next;
	  }
	  else {
	    // Eintrag lschen
	    // erster Eintrag:
	    if (PrevAEntry == NULL) {
	      TEntry *tmp = AEntry->Next;
	      delete AEntry;
	      AEntry = tmp;
	      if ((AEntry == NULL) && (BEntry == NULL)) {
		// aktuelle Zeile lschen
		deleteRow = true;
	      }
	    }
	    else { // schon Eintrge in Zeile vorhanden
	      PrevAEntry->Next = AEntry->Next;
	      delete AEntry;
	      AEntry = PrevAEntry->Next;
	    }
	    _NonZeros--;
	  }

	}	// while( (AEntry != NULL) || (BEntry != NULL) )

      // Nchste Zeilen
      BRow= BRow->Next;
      if (deleteRow) {
	TRow *tmp = ARow->Next;
	delete ARow;
	ARow = tmp;
	deleteRow = false;
      }
      else {
	PrevARow = ARow;
	ARow= ARow->Next;
      }

    }	// while( (ARow != NULL) || (BRow != NULL) )

  return (*this);
}


//---------------------------------------------------------------------------
// Funktionsname:
// Beschreibung:
// Parameter:
// Rueckgabe:
// Besonderheiten:
//---------------------------------------------------------------------------

TSparseMatrix &
TSparseMatrix::Cut( UINT n )
{
#ifdef DEBUG_METHODS
  cout << "TSparseMatrix::Cut(UINT)\n";
#endif

#ifdef VM_BOUND_CHECK
  // Dimensionen prfen
  if( (n > _m) || (n > _n) )
    {
      cerr << "Falsche Dimensionen in TSparseMatrix::Cut(UINT)" << endl;
      return  *this;
    }
#endif

  /*
  _NonZeros = 0;

  // Die ersten n Zeilen lschen
  TRow *Row= _FirstRow, *TmpRow;

  // for( ; i < n; i++ )
  while( Row && (Row->Number < n) )
    {
      // Liste der Eintrge lschen
      TEntry *Entry= Row->FirstEntry, *TmpEntry;

      while( Entry != NULL )
	{
	  TmpEntry= Entry->Next;
	  delete Entry;
	  Entry= TmpEntry;

	}	// while( Entry != NULL )

      // Zeile lschen
      TmpRow= Row->Next;
      delete Row;
      Row= TmpRow;

    }	// while( Row && (Row.Number < n) )

  TRow *NewFirstRow= NULL;
  TRow *NewLastRow= NULL;

  // Restliche ab n. Element umhngen, Index ndern
  while( Row )
    {
      TRow *NextRow= Row->Next;
      TEntry *Entry= Row->FirstEntry;

      while( Entry != NULL )
	{
	  if( Entry->Index < n )
	    {
				// wegwerfen
	      TEntry *TmpEntry= Entry->Next;
	      delete Entry;
	      Entry= TmpEntry;

				// Wenn alle Eintrge gelscht, Zeile lschen
	      if( Entry == NULL )
		{
		  delete Row;
		  Row= NextRow;
		}

	      continue;
	    }
	  else
	    {
				// Zeile in neue Liste einhngen
	      if( NewFirstRow == NULL )
		{
		  NewFirstRow= Row;
		}
	      else
		{
		  NewLastRow->Next= Row;
		}

	      Row->Number -= n;
	      Row->Next = NULL;
	      NewLastRow = Row;

				// Liste hier beginnen
	      Row->FirstEntry= Entry;

				// Indices ndern
	      while( Entry != NULL )
		{
		  _NonZeros++;
		  Entry->Index -= n;
		  Entry = Entry->Next;
		}

				// Nchste Zeile
	      Row= NextRow;

	      break;
	    }
	}	// while( Entry != NULL )
    }	// while( Row )

  // Alte Zeilenzeiger durch neue ersetzen
  _FirstRow= NewFirstRow;
  _LastRow= NewLastRow;

  // Neue Dimensionen
  _m -= n;
  _n -= n;

  return *this;
  */

  for (UINT i = 0; i < n; i++) {
    DeleteFirstRow();
  }
  for (UINT i = 0; i < n; i++) {
    DeleteFirstColumn();
  }

  return *this;
} // Cut


//---------------------------------------------------------------------------
// Funktionsname:
// Beschreibung:
// Parameter:
// Rueckgabe:
// Besonderheiten:
//---------------------------------------------------------------------------

TSparseMatrix &
TSparseMatrix::AddIAndMult( const TSparseMatrix &M, double d )
{
#ifdef DEBUG_METHODS
  cout << "TSparseMatrix::AddIAndMult( const TSparseMatrix &M, double d )\n" << flush;
#endif

  // Zielmatrix zurcksetzen, wenn ntig
  if( _NonZeros > 0 )
    {
      Reset();
    }

  // Zeilen und Spalten bernehmen
  _m = M._m;
  _n = M._n;
  _FirstRow = _LastRow = NULL;

  UINT i = 0;	// Zeilennummer

  // ber alle Zeilen
  for( TRow *Row = M._FirstRow; Row != NULL; Row = Row->Next, i++ )
    {
      // Zwischenzeilen mit 1 auf Hauptdiagonale fllen
      while( i < Row->Number )
	{
	  InsertEnd( i, i, 1 );
	  i++;
	}

      bool found = false; // Diagonale noch nicht berschritten

      // ber alle Elemente der Zeilen
      for( TEntry *Entry= Row->FirstEntry; Entry != NULL; Entry= Entry->Next )
	{
	  // Wenn Diagonale gerade berschritten...
	  if( (Entry->Index > i) && (!found) )
	    {
				// ...1 auf (i,i) eintragen
	      InsertEnd( i, i, 1);
	      found= true;

	    }	// if( (Entry->Index > i) && (!found) )

	  // Wenn Element auf der Diagonalen...
	  if( i == Entry->Index )
	    {
				// ...Wert*r+1 auf (i,i) eintragen
	      InsertEnd( i, i, (Entry->Value * d + 1) );
	      found= true;

	    }	// if( i == M.col_ind(j) )
	  else
	    {
				// sonst Wert*r auf (i,M.col_ind(j)) eintragen
	      InsertEnd( i, Entry->Index, (Entry->Value * d) );
	    }
	}	// for( TEntry Entry= _M.Row[i]; TEntry != NULL; TEntry= TEntry->Next )

      // Wenn Diagonale nicht berschritten
      if( !found )
	{
	  // ...1 auf (i,i) eintragen
	  InsertEnd( i, i, 1 );

	}	// if( !found )
    }	// for( TRow *Row= M._FirstRow; Row != NULL; Row= Row->Next )

  // restliche Zeilen mit 1 auf Hauptdiagonale fllen
  while( i < _m )
    {
      InsertEnd( i, i, 1 );
      i++;
    }

  return *this;
}	// AddIAndMult


//---------------------------------------------------------------------------
// Funktionsname:	AddVector( const TSparseVector &, UINT )
// Beschreibung:	Addiert einen Vektor zu einer Matrix-Zeile
// Parameter:		v		Vektor
//			i		Zeilennummer
// Rueckgabe:		Zielmatrix
// Besonderheiten:	Nur fuer letzte Matrix-Zeile aufrufen!!!
//---------------------------------------------------------------------------

TSparseMatrix &
TSparseMatrix::AddVector( const TSparseVector & v, UINT i )
{
#ifdef VM_BOUND_CHECK
  // Grenzen prfen
  if( i >= _m )
    {
      cerr << "Falsche Dimensionen in TSparseMatrix::AddVector!\n";
      exit(1);
    }
#endif

  // Nur wenn auch was zu tun ist...
  if( v._NonZeros == 0 )
    {
      return ( *this );
    }

  // Muss neue Zeile erzeugt werden?
  if( !_LastRow || (_LastRow->Number < i) )
    {
      TRow *NewRow= new TRow;
      NewRow->FirstEntry= NewRow->LastEntry= NULL;
      NewRow->Next= NULL;
      NewRow->Number= i;

      // einhngen
      if( _FirstRow == NULL )
	{
	  _FirstRow= NewRow;
	}
      else
	{
	  _LastRow->Next= NewRow;
	}

      _LastRow= NewRow;

    }	// if( !_LastRow || (_Lastrow->Number < i) )

  TEntry *DestEntry= _LastRow->FirstEntry;
  TEntry *DestPreEntry= NULL;
  TEntry *SourceEntry= v._Entry;
  UINT Num1, Num2;

  // Addieren
  while((DestEntry != NULL) || (SourceEntry != NULL))
    {
      Num1 = DestEntry == NULL ? 10000000 : DestEntry->Index;
      Num2 = SourceEntry == NULL ? 10000000 : SourceEntry->Index;

      if (Num1 < Num2)
	{
	  // Zielvektor weitersetzen
	  DestPreEntry = DestEntry;
	  DestEntry = DestEntry->Next;
	}
      else if (Num1 == Num2)
	{
	  // Addieren und beide weiter
	  // noch auf 0 zu testen
	  DestEntry->Value += SourceEntry->Value;
	  DestPreEntry = DestEntry;
	  DestEntry = DestEntry->Next;
	  SourceEntry = SourceEntry->Next;
	}
      else // Num1 > Num2
	{
	  // Neuen Eintrag einhngen
	  TEntry *NewEntry = new TEntry;
	  NewEntry->Index = Num2;
	  // auf 0 testen oder annehmen, dass Vektor keine 0 enthaelt?
	  NewEntry->Value = SourceEntry->Value;
	  _NonZeros++;
	  NewEntry->Next = DestEntry;
	  if (DestPreEntry == NULL)
	    _LastRow->FirstEntry = NewEntry;
	  else
	    DestPreEntry->Next = NewEntry;
	  DestPreEntry = NewEntry;
	  SourceEntry = SourceEntry->Next;
	}
    }

  _LastRow->LastEntry= DestPreEntry;

  return *this;

}	// AddVector


//---------------------------------------------------------------------------
// Funktionsname:	AddVector2(const TSparseVector &, UINT, Trow *)
// Beschreibung:	Addiert einen Vektor zu einer Matrix-Zeile
// Parameter:		v	  Vektor
//			i	  Zeilennummer
//			StartRow  Zeile, bei der Suche begonnen wird
// Rueckgabe:		Zielmatrix
// Besonderheiten:	Fr alle Zeilen aufrufbar!
//---------------------------------------------------------------------------

TRow * TSparseMatrix::AddVector2( const TSparseVector & v, UINT i, TRow *StartRow )
{
#ifdef VM_BOUND_CHECK
  // Grenzen prfen
  if( i >= _m )
    {
      cerr << "Falsche Dimensionen in TSparseMatrix::AddVector2!\n";
      exit(1);
    }
#endif

  // Startzeile setzen
  if( (StartRow == NULL) || (StartRow->Number > i) )
    {
      StartRow= _FirstRow;
    }

  TRow *Row= StartRow;
  TRow *PreRow= NULL;

  // Bis zur Zeile laufen
  if( (_LastRow != NULL) && (_LastRow->Number == i) )
    {
      // Speedup: Knnte schon die Richtige Zeile sein
      Row= _LastRow;

    }	// if( (_LastRow != NULL) && (_LastRow->Number == i) )
  else
    {
      while( (Row != NULL) && (Row->Number < i) )
	{
	  PreRow= Row;
	  Row= Row->Next;
	}

      if( (Row == NULL) || (Row->Number > i) )
	{
	  // Zeile hinten anhngen
	  TRow *NewRow= new TRow;
	  NewRow->Number= i;
	  NewRow->Next= NULL;
	  NewRow->FirstEntry= NewRow->LastEntry= NULL;

	  if( _FirstRow == NULL )
	    {
				// Erste Zeile
	      _FirstRow= _LastRow= NewRow;
	    }
	  else
	    {
	      if( PreRow == NULL )
		{
		  // Vorne einfgen
		  NewRow->Next= _FirstRow;
		  _FirstRow= NewRow;
		}
	      else
		{
		  // in Mitte einfgen
		  NewRow->Next= PreRow->Next;
		  PreRow->Next= NewRow;

		  if( PreRow == _LastRow )
		    {
		      _LastRow= NewRow;
		    }
		}
	    }	// else if( _FirstRow == NULL )

	  Row= NewRow;

	}	// else if( (Row == NULL) || (Row->Number > i) )
    }	// if( (_LastRow != NULL) && (_LastRow->Number == i) )

  TEntry *DestEntry= Row->FirstEntry;
  TEntry *DestPreEntry= NULL;
  TEntry *SourceEntry= v._Entry;
  UINT Num1, Num2;

  // Addieren
  while((DestEntry != NULL) || (SourceEntry != NULL))
    {
      Num1 = DestEntry == NULL ? 10000000 : DestEntry->Index;
      Num2 = SourceEntry == NULL ? 10000000 : SourceEntry->Index;

      if (Num1 < Num2)
	{
	  // Zielvektor weitersetzen
	  DestPreEntry = DestEntry;
	  DestEntry = DestEntry->Next;
	}
      else if (Num1 == Num2)
	{
	  // Addieren und beide weiter
	  // test auf 0
	  DestEntry->Value += SourceEntry->Value;
	  DestPreEntry = DestEntry;
	  DestEntry = DestEntry->Next;
	  SourceEntry = SourceEntry->Next;
	}
      else // Num1 > Num2
	{
	  // Neuen Eintrag einhngen
	  TEntry *NewEntry = new TEntry;
	  NewEntry->Index = Num2;
	  // test auf 0 oder voraussetzen, dass es so ist?
	  NewEntry->Value = SourceEntry->Value;
	  _NonZeros++;
	  NewEntry->Next = DestEntry;
	  if (DestPreEntry == NULL)
	    _LastRow->FirstEntry = NewEntry;
	  else
	    DestPreEntry->Next = NewEntry;
	  DestPreEntry = NewEntry;
	  SourceEntry = SourceEntry->Next;
	}
    }

  Row->LastEntry= DestPreEntry;

  return Row;

}	// AddVector2


//---------------------------------------------------------------------------
// Funktionsname:	Compare( TSparseMatrix &, double )
// Beschreibung:	Vergleicht zwei Matrizen
// Parameter:		M			Vergleichsmatrix
//			Epsilon		max. Differenz
// Rueckgabe:		-
// Besonderheiten:	-
//---------------------------------------------------------------------------

void
TSparseMatrix::Compare( const TSparseMatrix &M, double Epsilon ) const
{
  // Reiverschluss ber Matrix-Zeilen
  TRow *ARow= _FirstRow;
  TRow *BRow= M._FirstRow;
  UINT ARowNo, BRowNo;

  while( (ARow != NULL) || (BRow != NULL) )
    {
      ARowNo= (ARow != NULL) ? ARow->Number : 10000000;
      BRowNo= (BRow != NULL) ? BRow->Number : 10000000;

      if( ARowNo < BRowNo )
	{
	  // Zeilenelemente ausgeben
	  for( TEntry *Entry= ARow->FirstEntry; Entry != NULL; Entry= Entry->Next )
	    {
	      if( (( Entry->Value )) > Epsilon )
		{
		  cout << "\tDifference (" << ARowNo << ",";
		  cout << Entry->Index << "):\t";
		  cout << Entry->Value << endl;
		  getchar();
		}	// if( ABS( Entry->Value ) > Epsilon )
	    }

	  // Nchste Zeile
	  ARow= ARow->Next;

	  continue;

	}	// if( ARow < BRow )

      if( ARowNo > BRowNo )
	{
	  // Zeilenelemente ausgeben
	  for( TEntry *Entry= BRow->FirstEntry; Entry != NULL; Entry= Entry->Next )
	    {
	      if( (( Entry->Value )) > Epsilon )
		{
		  cout << "\tDifference (" << BRowNo << ",";
		  cout << Entry->Index << "):\t";
		  cout << -Entry->Value << endl;
		  getchar();
		}	// if( ABS( Entry->Value ) > Epsilon )
	    }

	  // Nchste Zeile
	  BRow= BRow->Next;

	  continue;

	}	// if( ARow > BRow )

      // ( ARow == BRow ) => Reiverschluss ber Eintrge
      TEntry *AEntry= ARow->FirstEntry;
      TEntry *BEntry= BRow->FirstEntry;
      UINT AIndex, BIndex;

      while( (AEntry != NULL) || (BEntry != NULL) )
	{
	  AIndex= (AEntry != NULL) ? AEntry->Index : 10000000;
	  BIndex= (BEntry != NULL) ? BEntry->Index : 10000000;

	  if( AIndex < BIndex )
	    {
				// Eintrag ausgeben
	      if( (( AEntry->Value )) > Epsilon )
		{
		  cout << "\tDifference (" << ARowNo << ",";
		  cout << AIndex << "):\t";
		  cout << AEntry->Value << endl;
		  getchar();
		}	// if( ABS( AEntry->Value ) > Epsilon )

				// Nchster Eintrag
	      AEntry= AEntry->Next;

	      continue;

	    }	// if( AIndex < BIndex )

	  if( AIndex > BIndex )
	    {
				// Eintrag ausgeben
	      if( (( BEntry->Value )) > Epsilon )
		{
		  cout << "\tDifference (" << BRowNo << ",";
		  cout << BIndex << "):\t";
		  cout << -BEntry->Value << endl;
		  getchar();
		}	// if( ABS( AEntry->Value ) > Epsilon )

				// Nchster Eintrag
	      BEntry= BEntry->Next;

	      continue;

	    }	// if( AIndex < BIndex )

	  // ( AIndex == BIndex ) => Vergleichen
	  // Eintrag ausgeben
	  if( (((AEntry->Value - BEntry->Value))) > Epsilon )
	    {
	      cout << "\tDifference (" << ARowNo << ",";
	      cout << AIndex << "):\t";
	      cout << (AEntry->Value-BEntry->Value) << endl;
	      getchar();
	    }	// if( ABS( AEntry->Value ) > Epsilon )

	  // Nchste Eintrge
	  AEntry= AEntry->Next;
	  BEntry= BEntry->Next;

	}	// while( (AEntry != NULL) || (BEntry != NULL) )

      // Nchste Zeilen
      ARow= ARow->Next;
      BRow= BRow->Next;

    }	// while( (ARow != NULL) || (BRow != NULL) )

}	// Compare


//---------------------------------------------------------------------------
// Funktionsname:	AddBlock
// Beschreibung:	Fgt einen Bereich einer Matrix multipliziert mit einer
//					Konstanten an gegebener Position ein
// Parameter:		M		Quellmatrix
//			sm1		Zeile Quelle oben links
//			sn1		Spalte Quelle oben links
//			sm2		Zeile Quelle unten rechts
//			sn2		Spalte Quelle unten rechts
//			dm		Zeile Ziel oben links
//			dn		Spalte Ziel oben links
//			d		Faktor
// Rueckgabe:		-
// Besonderheiten:	Nur fr Einfgen hinter dem letzten Eintrag einer Zeile
//---------------------------------------------------------------------------

void
TSparseMatrix::AddBlock( TSparseMatrix &M, UINT sm1, UINT sn1, UINT sm2,
			 UINT sn2, UINT dm, UINT dn, double d )
{
#ifdef VM_BOUND_CHECK
  // Koordinaten berprfen
  if( (sm1 > sm2) || /* (sn1 > sn2) || */ (sm2 > M._m) || (sn2 > M._n)
      || (dm > _m) || (dn > _n) )
    {
      cerr << "Falsche Grenzen in TSparseMatrix::AddBlock!\n";
      exit( 1 );
    }
#endif

  // Zeilezeiger holen
  TRow *SourceRow= M._FirstRow;
  TRow *DestRow= _FirstRow;

  // In Quelle bis zur Startzeile gehen
  while( (SourceRow != NULL) && (SourceRow->Number < sm1) )
    {
      SourceRow= SourceRow->Next;
    }

  // Alle Zeilen im Quellbereich durchlaufen
  while( (SourceRow != NULL) && (SourceRow->Number <= sm2) )
    {
      // Zeile fr Ziel berechnen
      UINT i= dm + SourceRow->Number - sm1;

      TEntry *SourceCol= SourceRow->FirstEntry;

      // In Quelle bis zur Startzeile gehen
      while( (SourceCol != NULL) && SourceCol->Index < sn1 )
	{
	  SourceCol= SourceCol->Next;
	}

      // Alle Spalten im Quellbereich durchlaufen
      while( (SourceCol != NULL) && SourceCol->Index <= sn2 )
	{
	  // Spalte fr Ziel berechnen
	  UINT j= dn + SourceCol->Index - sn1;

	  // Eintrag einfgen
	  DestRow= InsertLineEnd( i, j, d * SourceCol->Value, DestRow );

	  // Nchste Spalte
	  SourceCol= SourceCol->Next;

	}	// while( (SourceCol != NULL) && SourceCol->Index <= sn2 )

      // Nchste Zeile
      SourceRow= SourceRow->Next;

    }	// while( (SourceRow != NULL) && SourceRow->Number <= sm2 )

}	// AddBlock


//---------------------------------------------------------------------------
// Funktionsname:	InsertLineEnd( UINT, UINT, double, TRow * )
// Beschreibung:	Fgt einen Wert am Ende einer Zeile hizu
// Parameter:		i			Zeile des Eintrags
//			j			Spalte des Eintrags
//			StartRow	Startzeile fr Suche
// Rueckgabe:		aktuelle Zeile
// Besonderheiten:	-
//---------------------------------------------------------------------------

TRow * TSparseMatrix::InsertLineEnd( UINT i, UINT j, double v, TRow *StartRow )
{
#ifdef VM_BOUND_CHECK
  if( (i >= _m) || (j >= _n) )
    {
      cerr << "Falsche Dimensionen in TSparseMatrix::InsertLineEnd!\n";
      exit( 1 );
    }
#endif

  // Startzeile setzen
  if( (StartRow == NULL) || (StartRow->Number > i) )
    {
      StartRow= _FirstRow;
    }

  TRow *Row= StartRow;
  TRow *PreRow= NULL;

  // Neues Element einfgen
  TEntry *NewEntry= new TEntry;
  NewEntry->Index= j;
  NewEntry->Value= v;
  NewEntry->Next= NULL;

  // Bis zur Zeile laufen
  if( (_LastRow != NULL) && (_LastRow->Number <= i) )
    {
      // Speedup: Knnte schon die Richtige Zeile sein
      Row= _LastRow;
    }	// if( (_LastRow != NULL) && (_LastRow->Number == i) )

  while( (Row != NULL) && (Row->Number < i) )
    {
      PreRow= Row;
      Row= Row->Next;
    }

  if( (Row == NULL) || (Row->Number > i) )
    {
      // Zeile hinten anhngen
      TRow *NewRow= new TRow;
      NewRow->Number= i;
      NewRow->Next= NULL;
      NewRow->FirstEntry= NewRow->LastEntry= NULL;

      if( _FirstRow == NULL )
	{
	  // Erste Zeile
	  _FirstRow= _LastRow= NewRow;
	}
      else
	{
	  if( PreRow == NULL )
	    {
				// Vorne einfgen
	      NewRow->Next= _FirstRow;
	      _FirstRow= NewRow;
	    }
	  else
	    {
				// in Mitte einfgen
	      NewRow->Next= PreRow->Next;
	      PreRow->Next= NewRow;

	      if( PreRow == _LastRow )
		{
		  _LastRow= NewRow;
		}
	    }
	}	// else if( _FirstRow == NULL )

      Row= NewRow;

    }	// else if( (Row == NULL) || (Row->Number > i) )

  // In Zeile einfgen
  if( Row->LastEntry == NULL )
    {
      // erstes Element
      Row->FirstEntry= Row->LastEntry= NewEntry;

    }	// if( Entry == NULL )
  else
    {
      // hinten anhngen
      Row->LastEntry->Next= NewEntry;
      Row->LastEntry= NewEntry;

    }	// else if( Entry == NULL )

  _NonZeros++;

  // aktuelle Zeile zurck fr schnelles suchen
  return Row;

}	// InsertLineEnd


//---------------------------------------------------------------------------
// Funktionsname:	AddVectorPos( TSparseVector &, UINT, UINT )
// Beschreibung:	Fgt einen Vector ab einer bestimmten Position in eine
//			Matrix-Zeile ein
// Parameter:		v		Referenz auf Vektor
//			i		Ziel-Zeile
//			j		Ziel-Spalte
// Rueckgabe:
// Besonderheiten:
//---------------------------------------------------------------------------

TSparseMatrix &
TSparseMatrix::AddVectorPos( const TSparseVector &v, UINT i, UINT j )
{
#ifdef VM_BOUND_CHECK
  // Grenzen prfen
  if( (i >= _m) || ((j + v._n - 1) >= _n) )
    {
      cerr << "Falsche Dimensionen in TSparseMatrix::AddVectorPos"
	"(const TSparseVector&, UINT, UINT)" << endl;
      exit(1);
    }
#endif

  // Nur wenn auch was zu tun ist...
  if( v._NonZeros == 0 )
    {
      return ( *this );
    }

  // Muss neue Zeile erzeugt werden?
  if( !_LastRow || (_LastRow->Number < i) )
    {
      TRow *NewRow= new TRow;
      NewRow->FirstEntry= NewRow->LastEntry= NULL;
      NewRow->Next= NULL;
      NewRow->Number= i;

      // einhngen
      if( _FirstRow == NULL )
	{
	  _FirstRow= NewRow;
	}
      else
	{
	  _LastRow->Next= NewRow;
	}

      _LastRow= NewRow;

    }	// if( !_LastRow || (_Lastrow->Number < i) )

  TEntry *DestEntry= _LastRow->FirstEntry;
  TEntry *DestPreEntry= NULL;
  TEntry *SourceEntry= v._Entry;
  UINT Num1, Num2;

  // Addieren
  while((DestEntry != NULL) || (SourceEntry != NULL))
    {
      Num1 = (DestEntry == NULL) ? 10000000 : (DestEntry->Index);
      Num2 = (SourceEntry == NULL) ? 10000000 : (SourceEntry->Index+j);

      if (Num1 < Num2)
	{
	  // Zielvektor weitersetzen
	  DestPreEntry = DestEntry;
	  DestEntry = DestEntry->Next;
	}
      else if (Num1 == Num2)
	{
	  // Addieren und beide weiter
	  DestEntry->Value += SourceEntry->Value;
	  DestPreEntry = DestEntry;
	  DestEntry = DestEntry->Next;
	  SourceEntry = SourceEntry->Next;
	}
      else // Num1 > Num2
	{
	  // Neuen Eintrag einhngen
	  TEntry *NewEntry = new TEntry;
	  NewEntry->Index = Num2;
	  NewEntry->Value = SourceEntry->Value;
	  NewEntry->Next = DestEntry;
	  if (DestPreEntry == NULL)
	    _LastRow->FirstEntry = NewEntry;
	  else
	    DestPreEntry->Next = NewEntry;
	  _NonZeros++;
	  DestPreEntry = NewEntry;
	  SourceEntry = SourceEntry->Next;
	}
    }

  _LastRow->LastEntry= DestPreEntry;

  return *this;

}	// AddVectorPos


//---------------------------------------------------------------------------
// Funktionsname:	operator << ( ostream &, const TSparseMatrix & )
// Beschreibung:
// Parameter:
// Rueckgabe:
// Besonderheiten:
//---------------------------------------------------------------------------

ostream& operator << ( ostream & os, const TSparseMatrix & M )
{
  os << M._m << " x " << M._n << endl;

  // Fr alle Zeilen
  for( TRow *Row = M._FirstRow; Row != NULL; Row = Row->Next )
    {
      // Liste ausgeben
      for( TEntry *Entry = Row->FirstEntry; Entry != NULL; Entry= Entry->Next )
	{
	  os << Row->Number << "    " << Entry->Index << "    " << Entry->Value << endl;
	}
    }

  return os;
}


//---------------------------------------------------------------------------
// Funktionsname:	TSparseMatrix::DeleteFirstColumn()
// Beschreibung:	die erste Spalte der Matrix loeschen
// Parameter:		-
// Rueckgabe:		-
// Besonderheiten:	-
//---------------------------------------------------------------------------

void
TSparseMatrix::DeleteFirstColumn(void)
{
  if (_n <= 0) {
    cerr << "Keine Spalte mehr da in TSparseMatrix::"
      "DeleteFirstColumn()" << endl;
    exit (1);
  }

  TRow *tmp = _FirstRow;
  TRow *tmpPreRow = NULL;

  while (tmp != NULL) {
    TEntry *tmpEntry = tmp->FirstEntry;

    if (tmpEntry->Index == 0) {
      tmp->FirstEntry = tmpEntry->Next;
      delete tmpEntry;
      _NonZeros--;
      tmpEntry = tmp->FirstEntry;

      if (tmpEntry == NULL) {
	// ganze Zeile loeschen und mit naechster Zeile weitermachen
	if (tmpPreRow == NULL) {
	  _FirstRow = tmp->Next;

	  // Matrix ganz leer?
	  if (_FirstRow == NULL) {
	    _LastRow = NULL;
	  }

	  delete tmp;
	  tmp = _FirstRow;
	  continue;
	}
	else {
	  tmpPreRow->Next = tmp->Next;
	  delete tmp;
	  tmp = tmpPreRow->Next;
	  continue;
	}
      } // if (tmpEntry == NULL) {
    } // if (tmpEntry->Index == 0) {

    // restliche Zeileneintraege um 1 nach links ruecken
    while (tmpEntry != NULL) {
      (tmpEntry->Index)--;
      tmpEntry = tmpEntry->Next;
    }

    tmpPreRow = tmp;
    tmp = tmp->Next;
  } // while (tmp != NULL) {

  _n--;
} // void TSparseMatrix::DeleteFirstColumn(void)


//---------------------------------------------------------------------------
// Funktionsname:	TSparseMatrix::DeleteFirstRow()
// Beschreibung:	die erste Zeile der Matrix lschen
// Parameter:		-
// Rueckgabe:		-
// Besonderheiten:	-
//---------------------------------------------------------------------------

void
TSparseMatrix::DeleteFirstRow(void)
{
  if (_m <= 0) {
    cerr << "Keine Zeile mehr da in TSparseMatrix::DeleteFirstRow()" << endl;
    exit (1);
  }

  TRow *Row = _FirstRow;

  if ((Row != NULL) && (Row->Number == 0)) {
    TEntry* Entry = Row->FirstEntry;
    TEntry* tmpEntry;

    // alle Eintraege loeschen
    while (Entry != NULL) {
      tmpEntry = Entry->Next;
      delete Entry;
      _NonZeros--;
      Entry = tmpEntry;
    }

    // erste Reihe loeschen
    _FirstRow = _FirstRow->Next;
    delete Row;
    if (_FirstRow == NULL) {
      _LastRow = NULL;
    }
    Row = _FirstRow;
  }

  while (Row != NULL) {
    (Row->Number)--;
    Row = Row->Next;
  }

  _m--;
}


//---------------------------------------------------------------------------
// Funktionsname:	TSparseMatrix::GetFirstRow()
// Beschreibung:	die erste Zeile der Matrix liefern
// Parameter:		-
// Rueckgabe:		neuer Vektor mit den Eintraegen
// Besonderheiten:	-
//---------------------------------------------------------------------------

TSparseVector
TSparseMatrix::GetFirstRow(void) const
{
  TSparseVector Result(_n);

  TRow *Row = NULL;

  Row = _FirstRow;

  if ((Row != NULL) && (Row->Number == 0))
    {
      for(TEntry *Entry = Row->FirstEntry; Entry != NULL; Entry = Entry->Next)
	{
	  Result.InsertEnd(Entry->Index, Entry->Value);
	}
    } // if ((Row != NULL) && (Row->Number == 0))

  return Result;
}


//---------------------------------------------------------------------------
// Funktionsname:	Get( UINT , UINT )
// Beschreibung:	Liefert einen Eintrag der Matrix
// Parameter:		i		Zeile
//			j		Spalte
// Rueckgabe:		-
// Besonderheiten:	-
//---------------------------------------------------------------------------

double
TSparseMatrix::Get( UINT i, UINT j ) const
{
#ifdef VM_BOUND_CHECK
  if( (i >= _m) || (j >= _n) )
    {
      cerr << "Falsche Dimensionen in TSparseMatrix::Get\n!";
      exit( 1 );
    }
#endif

  // Zur Zeile gehen
  TRow *Row = _FirstRow;

  while( (Row != NULL) && (Row->Number < i) )
    {
      Row= Row->Next;
    }

  if( (Row == NULL) || (Row->Number > i) )
    {
      // Zeile gibt es bicht
      return 0.0;
    }

  // Zur Spalte gehen
  TEntry *Entry = Row->FirstEntry;

  while( (Entry != NULL) && (Entry->Index < j) )
    {
      Entry = Entry->Next;
    }

  if( (Entry == NULL) || (Entry->Index > j) )
    {
      // Spalte gibt es bicht
      return 0.0;
    }

  return Entry->Value;

}	// Get

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
double
TSparseMatrix::GetElement (UINT row, UINT col)
{
  double value = 0.0;
  TRow *currentRow = _RowPtr[row];
  TEntry *entry = currentRow->FirstEntry;
  while (entry) 
  {
    if (entry->Index == col)
    {
      return entry->Value;
    }
    entry = entry->Next;
  }

  return value;
}

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void 
TSparseMatrix::Insert (UINT row, UINT col, double value)
{
  TRow *currentRow = _RowPtr[row];
  TEntry *firstEntry = currentRow->FirstEntry;
  
  TEntry *newEntry = new TEntry;
  newEntry->Index = col;
  newEntry->Value = value;
  newEntry->Next = firstEntry;
  
  currentRow->FirstEntry = newEntry;
} // ende Insert

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void
TSparseMatrix::Update (UINT row, UINT col, double value)
{
  bool found = false;
  TRow *currentRow = _RowPtr[row];
  TEntry *currentEntry = currentRow->FirstEntry;  

  while ( (! found) && (currentEntry) )
  {
    if (currentEntry->Index == col)
    {
      found = true;
      currentEntry->Value = value;
    }
    else
    {
      currentEntry = currentEntry->Next;
    }
  }
} // ende Update

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void
TSparseMatrix::Delete (UINT row, UINT col)
{
  TRow *currentRow = _RowPtr[row];
  TEntry *currentEntry = currentRow->FirstEntry;
  TEntry *prev;
  // am kopf loeschen
  if (currentEntry->Index == col)
  {
    currentRow->FirstEntry = currentEntry->Next;
    delete currentEntry;
  }
  else
  {
    prev = currentEntry;
    while (currentEntry->Index != col)
    {
      prev = currentEntry;
      currentEntry = currentEntry->Next;
    }
    prev->Next = currentEntry->Next;
    delete currentEntry;
  }
  
} // ende Delete

