//------------------------------------------------------------------------------
// Projektname:		
// Dateiname:		$RCSfile: CSpMatrix.h,v $
// Beschreibung:	Header der Klasse fr dnn besetzte Matrizen
// Autor(en):		Michael Wintergerst
// Version:	      	$Revision: 1.2 $
// Gendert von:	$Author: winterge $
// Gendert am:		$Date: 2001/06/08 07:47:51 $
// Zustand:		$State: Exp $
//------------------------------------------------------------------------------

// includes
#ifndef SPARSEMATRIX_H
#define SPARSEMATRIX_H

#include <iostream.h>
#include "CSpVektor.h"

#define UINT unsigned int

class TSparseVector;

typedef struct tagTRow
{
	UINT Number;
	TEntry * FirstEntry;
	TEntry * LastEntry;
	tagTRow *Next;
} TRow;

class TSparseMatrix
{
	public:

	// Konstruktoren
	TSparseMatrix();
	TSparseMatrix( UINT m, UINT n );
	TSparseMatrix( const TSparseMatrix &M );
	TSparseMatrix( UINT m, UINT n, UINT NonZeros,
			 double *Values, UINT *RowPtr, UINT *ColInd );

	// Destruktor
	~TSparseMatrix();

	// Operatoren
	TSparseMatrix & operator= ( const TSparseMatrix &M );
	TSparseMatrix & operator+=( const TSparseMatrix &M);

	// Operationen
	TSparseMatrix & Cut( UINT n );
	TSparseMatrix & AddIAndMult( const TSparseMatrix &M, double d );
	TSparseMatrix & AddVector( const TSparseVector &v, UINT i );
	TSparseMatrix & AddVectorPos( const TSparseVector &v, UINT i, UINT j );
	void Compare( const TSparseMatrix &M, double Epsilon= 0.0 ) const;

	TRow * TSparseMatrix::AddVector2( const TSparseVector & v, UINT i,
		TRow *StartRow );

	// Einfgen / Auslesen von Elementen
	void InsertEnd( UINT i, UINT j, double Value );
	TRow * InsertLineEnd( UINT i, UINT j, double v, TRow *StartRow= NULL );
	void AddBlock( TSparseMatrix &M, UINT sx1, UINT sy1, UINT sx2,
		UINT sy2, UINT dx, UINT dy, double d= 1.0 );
	double Get( UINT i, UINT j ) const;

	// zurcksetzen
	void Reset();
	void Reset( UINT m, UINT n );

	void ResetRowsWithoutEntries(UINT m, UINT n);
	TSparseMatrix & AddScaledMatrix(double d, const TSparseMatrix &B);

	// TRow * GetFirstRow() { return _FirstRow; }

	TRow * GetFirstRowPtr() const { return _FirstRow; }
	TRow * GetLastRowPtr() const { return _LastRow; }
	void SetFirstRowPtr(TRow *Row) { _FirstRow = Row; }
	void SetLastRowPtr(TRow *Row)  { _LastRow  = Row; }
	TRow* GetRowPtr (UINT row) { return _RowPtr[row]; }
	double GetElement (UINT row, UINT col);

	void Insert (UINT row, UINT col, double value);
	void Update (UINT row, UINT col, double value);
	void Delete (UINT row, UINT col);

	void DeleteFirstColumn(void);
	void DeleteFirstRow(void);
	TSparseVector GetFirstRow(void) const;

	// Elementzugriffe
	UINT Dim( UINT i ) const { return (i == 0) ? _m : _n; }
	UINT NonZeros() const { return _NonZeros; }

	protected:

	/**
	 *
	 */
	TRow **_RowPtr;

	// Implementierung der Datenstruktur
	// TEntry ** _Row;			// Array von Zeiger auf Zeilen
	// TEntry ** _EndRow;		// Array von Zeiger auf Zeilenenden
	TRow *_FirstRow;
	TRow *_LastRow;

	UINT _m, _n;				// Dimensionen
	UINT _NonZeros;				// #Eintrge

	// Freundschaften
	friend class TSparseVector;
	friend ostream& operator << ( ostream &, const TSparseMatrix & );
	friend TSparseVector TransMult( const TSparseVector &, const TSparseMatrix & );
	friend TSparseMatrix operator + ( const TSparseMatrix &, const TSparseMatrix & );
	friend void FastTransMult( TSparseVector &d, const TSparseVector &v, const TSparseMatrix &M );
};

#endif // #ifndef SPARSEMATRIX_H
