//------------------------------------------------------------------------------
// Projektname:		
// Dateiname:		$RCSfile: CSpVektor.cpp,v $
// Beschreibung:	Implementierung eines dnn besetzten Vektors
// Autor(en):		Michael Wintergerst
// Version:	       	$Revision: 1.1 $
// Gendert von:	$Author: winterge $
// Gendert am:		$Date: 2001/05/29 09:58:35 $
// Zustand:		$State: Exp $
//------------------------------------------------------------------------------

// #define DEBUG_METHODS

// Includes
#include "stdlib.h"
#include "CSpVektor.h"


//------------------------------------------------------------------------------
// Funktionsname:
// Beschreibung:
// Parameter:
// Rueckgabe:
// Besonderheiten:
//------------------------------------------------------------------------------

TSparseVector::TSparseVector()
{
	// Initialisieren
	_Entry= NULL;
	_EndEntry= NULL;
	_n= 0;
	_NonZeros= 0;
	_CurrentEntry= NULL;
};


//------------------------------------------------------------------------------
// Funktionsname:
// Beschreibung:
// Parameter:
// Rueckgabe:
// Besonderheiten:
//------------------------------------------------------------------------------

TSparseVector::TSparseVector( UINT n )
{
	// Initialisieren
	_Entry= _EndEntry= NULL;
	_n= n;
	_NonZeros= 0;
	_CurrentEntry= NULL;
};


//------------------------------------------------------------------------------
// Funktionsname:
// Beschreibung:
// Parameter:
// Rueckgabe:
// Besonderheiten:
//------------------------------------------------------------------------------

TSparseVector::TSparseVector( const TSparseVector &v )
{
#ifdef DEBUG_METHODS
	cout << "TSparseVector::TSparseVector( const TSparseVector &v )\n";
#endif

	// Initialisieren
	_Entry= _EndEntry= NULL;
	_NonZeros= 0;
	_CurrentEntry= NULL;

	// neue Lnge
	_n= v._n;

	// Eintrge kopiernen
	for( TEntry *Entry= v._Entry; Entry != NULL; Entry= Entry->Next )
	{
		InsertEnd( Entry->Index, Entry->Value );
	}
}


//------------------------------------------------------------------------------
// Funktionsname:
// Beschreibung:
// Parameter:
// Rueckgabe:
// Besonderheiten:
//------------------------------------------------------------------------------

TSparseVector::~TSparseVector()
{
#ifdef DEBUG_METHODS
	cout << "TSparseVector::~TSparseVector()\n";
#endif

	// Vector zurcksetzen
	Reset();
};


//------------------------------------------------------------------------------
// Funktionsname:
// Beschreibung:
// Parameter:
// Rueckgabe:
// Besonderheiten:
//------------------------------------------------------------------------------

void TSparseVector::InsertEnd( UINT Index, double Value )
{
	// Grenzen prfen
	if( Index >= _n )
	{
		return;
	}

	if( Value == 0.0 )
	{
		return;
	}

	// Neuen Eintrag anlegen & fllen
	TEntry *NewEntry= new TEntry;

	NewEntry->Value= Value;
	NewEntry->Index= Index;
	NewEntry->Next= NULL;

	// Eintrag einhngen
	if( _EndEntry == NULL )
	{
		// leere Liste -> vorne Einhngen
		_Entry= NewEntry;
	}
	else
	{
		// hinten anhngen
		_EndEntry->Next= NewEntry;
	}

	// Neues Ende
	_EndEntry= NewEntry;

	// Eintrge erhhen
	_NonZeros++;

	return;
};


//------------------------------------------------------------------------------
// Funktionsname:
// Beschreibung:
// Parameter:
// Rueckgabe:
// Besonderheiten:
//------------------------------------------------------------------------------

void TSparseVector::Reset(const UINT n = 0)
{
#ifdef DEBUG_METHODS
	cout << "TSparseVector::Reset()\n";
#endif

	// Liste lschen
	TEntry *Lauf= _Entry, *Tmp;
	while( Lauf )
	{
		Tmp= Lauf->Next;
	 	delete Lauf;
		Lauf= Tmp;
	}

	// Member zurcksetzen
	_Entry= _EndEntry= NULL;
	_CurrentEntry= NULL;
	_NonZeros= 0;
	_n= n;
}


//------------------------------------------------------------------------------
// Funktionsname:
// Beschreibung:
// Parameter:
// Rueckgabe:
// Besonderheiten:
//------------------------------------------------------------------------------
TSparseVector & TSparseVector::operator *=( const double d )
{
	// if (d == 0) {
	//	reset(_n);
	//	return(this);
	// }
	// Eintrge mit d multiplizieren
	TEntry *Lauf= _Entry;

	while( Lauf )
	{
		Lauf->Value = Lauf->Value * d;
		Lauf = Lauf->Next;
	}

	return( *this );
}


//------------------------------------------------------------------------------
// Funktionsname:
// Beschreibung:
// Parameter:
// Rueckgabe:
// Besonderheiten:
//------------------------------------------------------------------------------
TSparseVector operator *( const double d, TSparseVector V)
{
  return (V*d);
}


//------------------------------------------------------------------------------
// Funktionsname:
// Beschreibung:
// Parameter:		Vektor v, double d
// Rueckgabe:		neuer Vektor w = v*d
// Besonderheiten:
//------------------------------------------------------------------------------
TSparseVector operator *(TSparseVector V, const double d)
{

	TSparseVector Result(V.Dim());

	if (d == 0)
	  return (Result);

	// Eintrge mit d multiplizieren
	TEntry *Lauf= V.GetEntry();

	while (Lauf)
	{
		Result.InsertEnd(Lauf->Index, (Lauf->Value) * d);
		Lauf = Lauf->Next;
	}

	return (Result);
}


//------------------------------------------------------------------------------
// Funktionsname:
// Beschreibung:
// Parameter:
// Rueckgabe:
// Besonderheiten:
//------------------------------------------------------------------------------
double TSparseVector::operator ()( const UINT d )
{
	TEntry *Lauf;

	// wenn aktuelles weiter vorne, am Anfang beginnen
	if( _CurrentEntry && (_CurrentEntry->Index <= d) )
	{
		Lauf= _CurrentEntry;
	}
	else
	{
		Lauf= _Entry;
	}

	while( Lauf )
	{
		if(Lauf->Index == d)
		{
			_CurrentEntry= Lauf;
			return(Lauf->Value);
		}
		else if(Lauf->Index > d)
		{
			_CurrentEntry= Lauf;
			return 0;
		}

		Lauf = Lauf->Next;
	}

	return(0);
}


//------------------------------------------------------------------------------
// Funktionsname:
// Beschreibung:
// Parameter:
// Rueckgabe:
// Besonderheiten:
//------------------------------------------------------------------------------

TSparseVector TSparseVector::operator +( const  TSparseVector v)
{
	TSparseVector Result(_n);

	// Dimensionen prfen
	if( (_n != v._n) || (_n == 0) ) return Result;

	// bernommen aus rand_procs.c

	TEntry *DestEntry= _Entry;
	TEntry *SourceEntry= v._Entry;
	UINT Num1, Num2;

	while((DestEntry != NULL) || (SourceEntry != NULL))
	{
		Num1 = DestEntry == NULL ? 10000000 : DestEntry->Index;
		Num2 = SourceEntry == NULL ? 10000000 : SourceEntry->Index;

		if (Num1 < Num2)
		{
			Result.InsertEnd(Num1, DestEntry->Value);
			DestEntry = DestEntry->Next;
		}
		else if (Num1 == Num2)
		{
			Result.InsertEnd(Num1, DestEntry->Value + SourceEntry->Value);
			DestEntry = DestEntry->Next;
			SourceEntry = SourceEntry->Next;
		}
		else // Num1 > Num2
		{
			Result.InsertEnd(Num2, SourceEntry->Value);
			SourceEntry = SourceEntry->Next;
		}
	}

	return Result;
}


//------------------------------------------------------------------------------
// Funktionsname:
// Beschreibung:
// Parameter:
// Rueckgabe:
// Besonderheiten:
//------------------------------------------------------------------------------

TSparseVector TSparseVector::operator -( const  TSparseVector v)
{
	TSparseVector Result(_n);

	// Dimensionen prfen
	if( (_n != v._n) || (_n == 0) ) return Result;

	// bernommen aus rand_procs.c

	TEntry *DestEntry= _Entry;
	TEntry *SourceEntry= v._Entry;
	UINT Num1, Num2;

	while((DestEntry != NULL) || (SourceEntry != NULL))
	{
		Num1 = DestEntry == NULL ? 10000000 : DestEntry->Index;
		Num2 = SourceEntry == NULL ? 10000000 : SourceEntry->Index;

		if (Num1 < Num2)
		{
			Result.InsertEnd(Num1, DestEntry->Value);
			DestEntry = DestEntry->Next;
		}
		else if (Num1 == Num2)
		{
			Result.InsertEnd(Num1, DestEntry->Value - SourceEntry->Value);
			DestEntry = DestEntry->Next;
			SourceEntry = SourceEntry->Next;
		}
		else // Num1 > Num2
		{
			Result.InsertEnd(Num2, -(SourceEntry->Value));
			SourceEntry = SourceEntry->Next;
		}
	}

	return Result;
}


//------------------------------------------------------------------------------
// Funktionsname:
// Beschreibung:
// Parameter:
// Rueckgabe:
// Besonderheiten:
//------------------------------------------------------------------------------

TSparseVector & TSparseVector::operator +=(const TSparseVector &v)
{
	// Dimensionen prfen
	if( (_n != v._n) || (_n == 0) ) return *this;

	// bernommen aus rand_procs.c

	TEntry *DestEntry= _Entry;
	TEntry *DestPreEntry= NULL;
	TEntry *SourceEntry= v._Entry;
	UINT Num1, Num2;

	while((DestEntry != NULL) || (SourceEntry != NULL))
	{
		Num1 = DestEntry == NULL ? 10000000 : DestEntry->Index;
		Num2 = SourceEntry == NULL ? 10000000 : SourceEntry->Index;

		if (Num1 < Num2)
		{
			DestPreEntry = DestEntry;
			DestEntry = DestEntry->Next;
		}
		else if (Num1 == Num2)
		{
			// diese Summe sollte noch auf 0 getestet werden, falls ja, Eintrag streichen
			/*
			double Sum = (DestEntry->Value += SourceEntry->Value);
			if (Sum == 0) {
				TEntry *tmp = DestEntry;
				DestEntry = DestEntry->Next;
				if (DestPreEntry == NULL) {
					_Entry = DestEntry;
				}
				else {
					DestPreEntry->Next = DestEntry;
				}
				delete tmp;
				_NonZeros--;
			} // if (Sum == 0)
			else {
				DestPreEntry = DestEntry;
				DestEntry = DestEntry->Next;
			}
			*/  // die nchsten 3 Zeilen dann lschen.
			DestEntry->Value += SourceEntry->Value;
			DestPreEntry = DestEntry;
			DestEntry = DestEntry->Next;
			SourceEntry = SourceEntry->Next;
		}
		else // Num1 > Num2
		{
			TEntry *NewEntry = new TEntry;
			NewEntry->Index = Num2;
			NewEntry->Value = SourceEntry->Value;
			NewEntry->Next = DestEntry;
			if (DestPreEntry == NULL)
				_Entry = NewEntry;
			else	DestPreEntry->Next = NewEntry;
			_NonZeros++;
			DestPreEntry = NewEntry;
			SourceEntry = SourceEntry->Next;
		}
	}

	_EndEntry= DestPreEntry;

	return *this;
}


//------------------------------------------------------------------------------
// Funktionsname:
// Beschreibung:
// Parameter:
// Rueckgabe:
// Besonderheiten:
//------------------------------------------------------------------------------

TSparseVector & TSparseVector::operator -=(const TSparseVector &v)
{
	// Dimensionen prfen
	if( (_n != v._n) || (_n == 0) ) return *this;

	// bernommen aus rand_procs.c

	TEntry *DestEntry= _Entry;
	TEntry *DestPreEntry= NULL;
	TEntry *SourceEntry= v._Entry;
	UINT Num1, Num2;

	while((DestEntry != NULL) || (SourceEntry != NULL))
	{
		Num1 = DestEntry == NULL ? 10000000 : DestEntry->Index;
		Num2 = SourceEntry == NULL ? 10000000 : SourceEntry->Index;

		if (Num1 < Num2)
		{
			DestPreEntry = DestEntry;
			DestEntry = DestEntry->Next;
		}
		else if (Num1 == Num2)
		{
			// noch test auf 0 wie bei +=
			DestEntry->Value -= SourceEntry->Value;
			DestPreEntry = DestEntry;
			DestEntry = DestEntry->Next;
			SourceEntry = SourceEntry->Next;
		}
		else // Num1 > Num2
		{
			TEntry *NewEntry = new TEntry;
			NewEntry->Index = Num2;
			NewEntry->Value = -SourceEntry->Value;
			NewEntry->Next = DestEntry;
			if (DestPreEntry == NULL)
				_Entry = NewEntry;
			else	DestPreEntry->Next = NewEntry;
			_NonZeros++;
			DestPreEntry = NewEntry;
			SourceEntry = SourceEntry->Next;
		}
	}

	_EndEntry= DestPreEntry;

	return *this;

}	// Operator -=


//------------------------------------------------------------------------------
// Funktionsname:
// Beschreibung:
// Parameter:
// Rueckgabe:
// Besonderheiten:
//------------------------------------------------------------------------------

TSparseVector & TSparseVector::MultAndAdd( const TSparseVector &v, double d )
{
	// Dimensionen prfen
	if( (_n != v._n) || (_n == 0) ) return *this;

	// bernommen aus rand_procs.c

	TEntry *DestEntry= _Entry;
	TEntry *DestPreEntry= NULL;
	TEntry *SourceEntry= v._Entry;
	UINT Num1, Num2;

	while((DestEntry != NULL) || (SourceEntry != NULL))
	{
		Num1 = DestEntry == NULL ? 10000000 : DestEntry->Index;
		Num2 = SourceEntry == NULL ? 10000000 : SourceEntry->Index;

		if (Num1 < Num2)
		{
			DestPreEntry = DestEntry;
			DestEntry = DestEntry->Next;
		}
		else if (Num1 == Num2)
		{
			// noch test auf 0 wie bei +=
			DestEntry->Value += SourceEntry->Value * d;
			DestPreEntry = DestEntry;
			DestEntry = DestEntry->Next;
			SourceEntry = SourceEntry->Next;
		}
		else // Num1 > Num2
		{
			TEntry *NewEntry = new TEntry;
			NewEntry->Index = Num2;
			NewEntry->Value = SourceEntry->Value * d;
			NewEntry->Next = DestEntry;
			if (DestPreEntry == NULL)
				_Entry = NewEntry;
			else	DestPreEntry->Next = NewEntry;
			_NonZeros++;
			DestPreEntry = NewEntry;
			SourceEntry = SourceEntry->Next;
		}
	}

	return *this;
}


//------------------------------------------------------------------------------
// Funktionsname:
// Beschreibung:
// Parameter:
// Rueckgabe:
// Besonderheiten:
//------------------------------------------------------------------------------

TSparseVector & TSparseVector::operator =( const TSparseVector &v )
{
#ifdef DEBUG_METHODS
	cout << "TSparseVector::operator =( const TSparseVector &v )\n";
#endif

	// alten Zielvektor zurcksetzen
	Reset();

	// neue Lnge
	_n= v._n;

	// Eintrge kopiernen
	for( TEntry *Entry= v._Entry; Entry != NULL; Entry= Entry->Next )
	{
		InsertEnd( Entry->Index, Entry->Value );
	}

	return *this;
}


//------------------------------------------------------------------------------
// Funktionsname:
// Beschreibung:
// Parameter:
// Rueckgabe:
// Besonderheiten:
//------------------------------------------------------------------------------

TSparseVector & TSparseVector::operator *=( const TSparseMatrix &M )
{
#ifdef DEBUG_METHODS
	cout << "TSparseVector::operator *=( const TSparseMatrix &M )\n";
#endif

	if( _n != M._m )
	{
		cerr << "Falsche Dimensionen in operator *=!\n";
		exit(1);
	}

	// Hilfsarray zum Summieren
/*	double *Sum= new double[M._n];

	for( UINT i= 0; i < M._n; i++ )
	{
		Sum[i]= 0.0;
	}
*/

	double *Sum= (double *) calloc( M._n, sizeof( double ) );

	// Reiverschluss ber Vektor und Matrix-Zeilen
	TEntry *VecEntry= _Entry, *Tmp;
	TRow *MatRow= M._FirstRow;

	UINT MaxIndex= 0;

	while( (VecEntry != NULL) && (MatRow != NULL) )
	{
		if( VecEntry->Index < MatRow->Number )
		{
			// Vektor weiter setzen und Eintrag lschen
			Tmp= VecEntry->Next;
			delete VecEntry;
			VecEntry= Tmp;
			continue;
		}

		if( VecEntry->Index > MatRow->Number )
		{
			MatRow= MatRow->Next;
			continue;
		}

		// (VecEntry->Index == MatRow->Number) => Multiplizieren
		for( TEntry *MatEntry= MatRow->FirstEntry; MatEntry != NULL; MatEntry= MatEntry->Next )
		{
			Sum[MatEntry->Index]+= VecEntry->Value * MatEntry->Value;

			if( MatEntry->Index > MaxIndex )
			{
				MaxIndex= MatEntry->Index;
			}
		}

		// beide weiter setzen & Vektoreintrag lschen
		Tmp= VecEntry->Next;
		delete VecEntry;
		VecEntry= Tmp;
		MatRow= MatRow->Next;

	}	// while( (VecEntry != NULL) && (MatRow != NULL) )

	// ggf. Rest des Vektors lschen
	while( VecEntry != NULL )
	{
		Tmp= VecEntry->Next;
		delete VecEntry;
		VecEntry= Tmp;

	}	// while( VecEntry != NULL )

	// Vektor zurcksetzen
	_Entry= _EndEntry= NULL;
	_NonZeros= 0;
	_n= M._n;

	// In Ergebnisvektor eintragen
	TEntry *NewEntry= new TEntry;
	TEntry *Anfang= NewEntry;

	// for( UINT i= 0; i < M._n; i++ )
	for( UINT i= 0; i <= MaxIndex; i++ )
	{
		if( Sum[i] != 0.0 )
		{

			// InsertEnd( i, Sum[i] ); per Hand ge-inline-t

			// Neuen Eintrag anlegen & fllen
			NewEntry= NewEntry->Next= new TEntry;

			NewEntry->Value= Sum[i];
			NewEntry->Index= i;

			_NonZeros++;
		}
	}

	// Ordentlich einhngen
	if( _NonZeros != 0 )
	{
		_EndEntry= NewEntry;
		_EndEntry->Next= NULL;
		_Entry= Anfang->Next;
	}

	delete Anfang;

//	delete [] Sum;

	free( Sum );

	return  *this;
}


//------------------------------------------------------------------------------
// Funktionsname:	Add( const UINT , const double )
// Beschreibung:	Adiert eine Zahl auf einen Eintrag des Vektors
// Parameter:		i		Index des Eintrags
//					d		zu addierende Zahl
// Rueckgabe:		-
// Besonderheiten:	-
//------------------------------------------------------------------------------

void TSparseVector::Add( const UINT i, const double d )
{
	TEntry *Entry= _Entry, *PreEntry= NULL;
	bool Found= false;

	while( Entry )
	{
		// Gefunden => addieren
		if( Entry->Index == i )
		{
			// noch test auf Ergebnis == 0
			Entry->Value+= d;
			Found= true;
			break;

		}	// if( Entry->Index == i )

		// zu weit => einfgen
		if( Entry->Index > i )
		{
			// noch Test auf d == 0
			TEntry *NewEntry= new TEntry;
			NewEntry->Index= i;
			NewEntry->Value= d;
			NewEntry->Next= Entry;

			if( PreEntry == NULL )
			{
				// Am Anfang einfgen
				_Entry= NewEntry;
			}
			else
			{
				// sonst vor aktuellem
				PreEntry->Next= NewEntry;
			}

			_NonZeros++;

			Found= true;
			break;

		}	// if( Entry->Index > i )

		// Sonst weiter in Liste
		PreEntry= Entry;
		Entry= Entry->Next;

	}	// while( Entry )

	// am Ende einfgen, wenn noch nicht gefunden
	if( !Found )
	{
		// noch test auf d == 0
		TEntry *NewEntry= new TEntry;
		NewEntry->Index= i;
		NewEntry->Value= d;
		NewEntry->Next= NULL;

		// Liste leer?
		if( _Entry )
		{
			// Nein => hinten einhngen
			_EndEntry->Next= NewEntry;
			_EndEntry= NewEntry;
		}
		else
		{
			// Ja => vorne einhngen
			_Entry= _EndEntry= NewEntry;
		}

		_NonZeros++;

	}	// if( !Found )

}	// Add


//------------------------------------------------------------------------------
// Funktionsname:	Set( const UINT , const double )
// Beschreibung:	trgt einen Wert in einen Vektor ein und berschreibt ggf.
//					den alten Eintrag an dieser Stelle
// Parameter:		i		Index des Eintrags
//					d		eizutragender Wert
// Rueckgabe:		-
// Besonderheiten:	-
//------------------------------------------------------------------------------

void TSparseVector::Set( const UINT i, const double d )
{
	TEntry *Entry= _Entry, *PreEntry= NULL;
	bool Found= false;

	while( Entry )
	{
		// Gefunden => setzen
		if( Entry->Index == i )
		{
			// noch test auf d == 0
			Entry->Value= d;
			Found= true;
			break;

		}	// if( Entry->Index == i )

		// zu weit => einfgen
		if( Entry->Index > i )
		{
			// noch test auf d == 0
			TEntry *NewEntry= new TEntry;
			NewEntry->Index= i;
			NewEntry->Value= d;
			NewEntry->Next= Entry;

			if( PreEntry == NULL )
			{
				// Am Anfang einfgen
				_Entry= NewEntry;
			}
			else
			{
				// sonst vor aktuellem
				PreEntry->Next= NewEntry;
			}

			_NonZeros++;

			Found= true;
			break;

		}	// if( Entry->Index > i )

		// Sonst weiter in Liste
		PreEntry= Entry;
		Entry= Entry->Next;

	}	// while( Entry )

	// am Ende einfgen, wenn noch nicht gefunden
	if( !Found )
	{
		// noch test auf d == 0
		TEntry *NewEntry= new TEntry;
		NewEntry->Index= i;
		NewEntry->Value= d;
		NewEntry->Next= NULL;

		// Liste leer?
		if( _Entry )
		{
			// Nein => hinten einhngen
			_EndEntry->Next= NewEntry;
			_EndEntry= NewEntry;
		}
		else
		{
			// Ja => vorne einhngen
			_Entry= _EndEntry= NewEntry;
		}

		_NonZeros++;

	}	// if( !Found )

}	// Set


//------------------------------------------------------------------------------
// Funktionsname:
// Beschreibung:
// Parameter:
// Rueckgabe:
// Besonderheiten:
//------------------------------------------------------------------------------

ostream& operator << (ostream & os, const TSparseVector & v)
{
	// Liste ausgeben
	for( TEntry *Entry= v._Entry; Entry != NULL; Entry= Entry->Next )
	{
		os << Entry->Index << "    " << Entry->Value << endl;
	}

	return os;
}


//------------------------------------------------------------------------------
// Funktionsname:	TSparseVector::ConcatVector(UINT, const TSparseVector &)
// Beschreibung:	Konkatenation von Vektoren
// Parameter:		StartIndex	StartIndex
// Rueckgabe:		v			anzufgender Vektor
// Besonderheiten:
//------------------------------------------------------------------------------

void TSparseVector::InsertVector(UINT StartIndex, const TSparseVector &v)
{
	// alle Eintrge auslesen und einfgen
	for(TEntry *Entry = v.GetEntry(); Entry != NULL; Entry = Entry->Next)
	{
		InsertEnd( (StartIndex + Entry->Index), Entry->Value);
	}
}
