//------------------------------------------------------------------------------
// Projektname:		
// Dateiname:		$RCSfile: CSpVektor.h,v $
// Beschreibung:	Header der Klasse fr dnn besetzte Vektoren
// Autor(en):		Michael Wintergerst
// Version:		$Revision: 1.1 $
// Gendert von:	$Author: winterge $
// Gendert am:		$Date: 2001/05/29 09:58:36 $
// Zustand:	       	$State: Exp $
//------------------------------------------------------------------------------

#ifndef SPARSEVECTOR_H
#define SPARSEVECTOR_H

#include <iostream.h>

#define UINT unsigned int

class TSparseMatrix;

typedef struct tagTEntry
{
	UINT Index;
	double Value;
	tagTEntry * Next;
} TEntry;

#include "CSpMatrix.h"

class TSparseVector
{
	public:

	// Konstruktoren
	TSparseVector();
	TSparseVector( UINT n );
	TSparseVector( const TSparseVector &v );

	// Destruktor
	~TSparseVector();

	// Operatoren
	TSparseVector & operator =( const TSparseVector &v );
	TSparseVector & operator *=( const TSparseMatrix &M );
	TSparseVector & operator += ( const  TSparseVector &v);
	TSparseVector & operator -= ( const  TSparseVector &v);

	TSparseVector & operator *=( const double d );
	double 		    operator ()( const UINT d );
	TSparseVector   operator  + ( const  TSparseVector v);
	TSparseVector   operator  - ( const  TSparseVector v);

	// Operationen
	TSparseVector & MultAndAdd( const TSparseVector &v, double d );

	// Auslesen der Eintrge
	TEntry *GetEntry() const { return _Entry; }
	TEntry *GetEndEntry() const { return _EndEntry; }

	// Einfgen / Setzen von Elementen
	void InsertEnd( const UINT Index, const double Value );
	void Add( const UINT i, const double d );
	void Set( const UINT i, const double d );

	// Konkatenation von Vektoren
	void InsertVector(UINT StartIndex, const TSparseVector &v);

	// zurcksetzen
	void Reset( const UINT n= 0 );

	// Elementzugriffe
	UINT Dim() { return _n; }
	UINT NonZeros() { return _NonZeros; }

	protected:

	// Datenstruktur
	TEntry * _Entry;		// Zeiger auf Eintrge
	TEntry * _EndEntry;		// Zeiger auf Eintragsende
	UINT _n;				// Dimension
	UINT _NonZeros;			// #Eintrge

	// Zeiger fr schnelle sequentielle Elementzugriffe
	TEntry * _CurrentEntry;

	// Freundschaften
	friend class TSparseMatrix;
	friend ostream& operator << (ostream & os, const TSparseVector & v);
	friend TSparseVector TransMult( const TSparseVector &v, const TSparseMatrix &M );
	friend TSparseVector & MultAndAdd( TSparseVector &, const TSparseVector &, double);
	friend void FastTransMult( TSparseVector &d, const TSparseVector &v, const TSparseMatrix &M );

	friend TSparseVector operator *( const double d, TSparseVector V);
	friend TSparseVector operator *( TSparseVector V, const double d);
};
#endif // #ifndef SPARSEVECTOR_H
