/* Generated by Together */

#ifndef TRANSPORTKETTENGRAPH_H
#define TRANSPORTKETTENGRAPH_H

#include "CZusammenhangskomponenten.h"
#include "CKantenBeschriftung.h"
#include "CKnotenBeschriftung.h"
#include <iostream.h>
#include "LEDA/graph.h"
#include "LEDA/h_array.h"
#include "LEDA/list.h"
#include "LEDA/basic.h"
#include "LEDA/array.h"
#include "LEDA/tuple.h"
#include "LEDA/map2.h"
#include "CFPlanGraphKante.h"
#include "CFPlanGraphKnoten.h"
#include "CFahrplanElement.h"
#include "LEDA/graph_alg.h"
#include "LEDA/node_array.h"
#include "Debug.h"
#include "CVisualFahrplan.h"
#include "CReisebewertung.h"

class CSimplexTableau;

/**
 * In dieser Klasse stehen alle Methoden und Attribute, die fr den Transportkettengraphen
 * wichtig sind.
 */
class CTransportkettengraph {
    public:
        /**
         * Nachdem wir von der Funktion starten aus der Klasse CFahrplan das Netz der Transportverbindungen
         * erhalten haben, beginnen wir damit, dieses in einen Transportkettengraphen
         * umzubauen. Zu diesem Zweck rufen wir die Funktion umbauenTransporkettengraph auf.
         *
         * Im Anschluss an den Umbau, rufen wir die Funktion reduzierenTransportkettengraph
         * auf (erst im zweiten Prototypen). Im Anschluss an diesen Aufruf steht uns ein reduzierter
         * Transportkettengraph zur Verfuegung.
         * Jetzt werden die Zusammenhangskomponenten berechnet. Hierzu wird die Funktion
         * berechnenZusammenhangskomponenten aufgerufen. Dann wird fuer jede dieser Zusammenhangskomponenten
         * die zyklomatische Zahl berechnet. Funktion: berechnenZyklomatischeZahl.
         */
        CTransportkettengraph(GRAPH<CFPlanGraphKnoten*, CFPlanGraphKante*>*);

	/**
	 * Destruktor
	 */
	~CTransportkettengraph ();

	/**
	 * setzen des Stoppen Flags
	 */
	void setzenStoppenFlag ();

	/**
	 * Init der Mindesthaltezeit
	 */
	void InitMindestHaltezeit (int);

	/**
	 * Init der Mindestumsteigezeit
	 */
	void InitMindestUmsteigezeit (int);

	/**
	 * Geben der Fahrzeuganzahl
	 */
	int gebenFahrzeugAnzahl ();

	/**
	 * Berechnen der Wartezeit-Resultate
	 * in allen Zusammenhangskomponenten
	 */
	void berechnenWartezeitResultate ();

	/**
	 * Bestimmung der Summe der Wartezeiten in allen
	 * Zusammenhangskomponenten
	 */
	void gebenWartezeiten (double&, double&, double&, double&);

        /**
         * Bei Bedarf moechten wir die Mindesthaltezeit einer Haltestelle beruecksichtigen. Wir bekommen
         * von der Programmgruppe ein Hash-Array (LEDA) welches folgende Informationen enthaelt:
         * KEY: Eindeutige ID der Haltestelle (int).
         * VALUE: Mindesthaltezeit in Minuten in Minuten (int), also eventuell gerundet.
         */
        int  ausgebenMindesthaltezeit(int);
       
        /**
         * Diese Funktion iteriert ueber alle Zusammenhangskomponenten und
         * ruft fuer diese die Funktion initialisierenFesterVerbindungen auf
         */
        void initialisierenFesterVerbindungen();

	void initialisierenGeruest();

        /**
         * Diese Funktion speichert zu Beginn das Hash-Array. Dazu benutzt
         * sie mindestHaltezeit aus dem private Teil.
         * Im Anschluss daran geht die Methode geht wie folgt vor: Sie iteriert ueber alle
         * Zusammenhangskomponenten und uebergibt
         * dort das hash-array an die Methode setzenMindesthaltezeit. Diese schreibt die
         * Mindesthaltezeit dann ebtsprechend dem dort beschriebenen Vorgehen an die Kanten. Fuer naehere Infos
         * siehe setzenMindesthaltezeit in CZusammenhangskomponente.
         */
        void setzenMindesthaltezeit(h_array<int,int> *);

        /**
         * Diese Funktion gibt die Mindestumsteigezeiten von einer Linie zu einer
         * anderen Linie an einer Haltestelle aus. Zur Speicherung benutzen wir ein
         * Hash-Array (LEDA). Dabei sind die Werte wie folgt in dem Array abgelegt
         * KEY: HaltestellenID.
         * VALUE: Liste von Struct mit folgenden Eintraegen  (Liniennummer1, Liniennummer2, min)
         * dabei steigt von Liniennummer1 auf Liniennummer2 um.
         * Die entsprechende Mindestumsteigezeit finden wir wie folgt:
         * Wir hashen auf der ID und erhalten die LIste. Auf diese Liste wenden wir, zumindest im ersten
         * Prototypen eine lineare Suche an, um die entsprechende Zeit zu finden.
         */
        int  ausgebenMindestumsteigezeit(int, int, int);

	/**
	 * Setzen des Taktes in der entsprechenden Zusammenhangskomponente
	 */
	void setzenTakt (int, int);

        /**
         * Diese Funktion speichert das Hash-Array der MindestUmsteigezeiten, welches wir von der Programm-
         * gruppe erhalten haben. Dazu dient mindestUmsteigezeit im private Teil.
         * Im Anschluss daran geht die Methode geht wie folgt vor: Sie iteriert ueber alle
         * Zusammenhangskomponenten und uebergibt
         * dort das hash-array an die Methode setzenMindesthaltezeit. Diese schreibt die
         * Mindesthaltezeit dann ebtsprechend dem dort beschriebenen Vorgehen an die Kanten. Fuer naehere Infos
         * siehe setzenMindesthaltezeit in CZusammenhangskomponente.
         */
        void setzenMindestumsteigezeit(h_array<int,list< three_tuple<int,int,int>* >* >*);

        /**
         * Diese Funktion
         */
        void setzenNetzbedingteWartezeiten(h_array<int,int>);

        /**
        * Mit dieser Methode werden die festen Verbindungen zweier Linien gesetzt
        * als four_tupel erhalten wir von CFahrplan folgendes:
        * Parameter 1: ZusammenhangskomponentenID
        * Parameter 2: LinienID1
        * Parameter 3: LinienID2
        * Parameter 4: HaltestellenID
        *
        * Wir iterieren ueber die uebergebene Liste und
        * rufen fuer die gegebene Zusammenhangskomponente die Methode setzenFesterBindungen(LinienID1,LinienID2,HaltestellenID)
        * der Klasse CZusammenhangskomponente auf.
        */
        void setzenFesterBindungen(list<four_tuple<int,int,int,int>* >*);

        /**
         * Mit dieser Methode berechnet man ein Array, welches aus den Listen der IDs der Haltestellen
         * in jeweils einer Zusammenhangskomponente besteht. Im Prinzip iterieren wir nur ueber alle Zusammenhangskomponenten,
         * dazu haben wir ja das FeldDerZusaammenhangskomponenten und rufen fuer jede der vorhandenen Zusammenhangskomponenten
         * die Methode gebenZusammenhangskomponenteAlsListe in CZusammenhangskomponenten auf. Diese Methode liefert uns dann die
         * geforderte Liste, welche wir dann einfach in das neue Array von Listen einhaengen.
         */
        array<list<three_tuple<int,int,int>* >* >* berechnenListenDerHaltestellenInAllenZusammenhangskomponenten();
        /**
         * Wir rufen in der entsprechenden Zusammenhangskomponente die Methode berechnenSimplextableau
         * in CZusammenkhangskomponente auf. 
         */
        void berechnenSimplexTableau(int);

        /**
        * Bestimmt fuer die Zusammenhangskomponente die Partition
        * Parameter 1: ZusammenhangskomponentenID
        *
        * Ruft in CZusammenhangskomponente die Methode berechneKnotenPartiotion()
        */
        void berechnenKnotenPartition(int);
        
	array<CZusammenhangskomponenten>* ausgebenZusammenhangskomponenten();
        
	GRAPH<CKnotenBeschriftung,CKantenBeschriftung>* liefernZeigerAufGraph(int);
        /**
         * Diese Methode stoesst die Berechnung des Maximalgeruestes innerhalb der
         * angegebenen Zusammenhangskomponente an. Zu diesem Zweck ruft es in der
         * entsprechenden Zusammenhangskomponente, auf die wir mittels des Arrays der
         * Zusammenhangskomponenten zu greifen koennen, die Funktion
         * berechnenMaximalgeruest auf.
         */
        void berechnenMaximalgeruest(int);

	/**
	 * Diese Methode wird von CFahrplan aufgerufen um den Fahrplan zu erstellen.
	 * Es werden die in CFahrplan geholten Werte fuer die Abfahrtszeiten und FahrplanIntervall
	 * uebergeben.
	 * Danach wird ueber alle Zusammenhangskomponenten iteriert und dort die Methode berechnenFahrplan (CZusammenhangskomponente)
	 * aufgerufen. Dabei wird die Abfahrtszeit der Zusammenhangskomponente und das FahrplanIntervall uebergeben.
	 * In CZusammenhangskomponente werden dann die Abfahrtszeiten generiert und eine liste mit CFDahrplanElement Objekten
	 * erstellt.
	 * Im Anschluss daran, werden die listen der Zusammenhangskomponenten zusammengefuegt.
	 */
	void berechnenFahrplan(array<three_tuple<int,int,int>* >*,four_tuple<int,int,int,int>*);

	/** 
	 * Diese Methode gibt den erzeugten Fahrplan zurueck
	 */
	list<CFahrplanElement*>* gebenFahrplan();

	/**
	 * Diese Methode gibt die Zyklomatische Zahl einer bestimmten Zusammenhangskomponente zurueck
	 */
	int gebenZyklomatischeZahl(int);

	/**
	 * Diese Methode liefert die Anzahl der Kanten einer Zusammenhangskomponente zurueck
	 */
	int gebenAnzahlDerKanten(int);

	/**
	 * Diese Methode startet die Berechnung der Lokalen Suche
	 * in der uebergebenen Zusammenhangskomponente
	 */
	void startenLokaleSuche (int);

	/**
	 * Berechnet fuer die Zusammenhangskomponente (1. Parameter) den Entscheidungsbaum bis zur Tiefe
	 * (2. Parameter)
	 */
	void berechnenBesteLoesung(int,int);

	/**
	 * Gibt von allen Zusammenhangskomponenten ein Hash_array der Netzbedingten Wartezeiten aus.
	 */
	h_array<three_tuple<int,int,int>*,int>* gebenListeNetzbedingtewartezeit();

	void fuellenReisebewertung(map2<int,int,CReisebewertung>&, int);

	double gebenFahrplanWirkungsgrad();

	int gebenFahrgastfahrten();

    private:
        /**
         * Mindesthaltezeit Hash aus CFahrplan
         */
         h_array<int, int>* mindestHaltezeitHash;

        /**
         * Mindestumsteigezeit Hash aus CFahrplan
         */
         h_array<int, three_tuple<int,int,int> >* mindestUmsteigezeit;
        /**
         * An den Knoten des Transportkettengraphen sollten die Haltstellen-IDs
         * vermerkt werden, damit eine Ruecktransformation in das Netz der Transportverbindungen
         * ermoeglicht wird (quasi welche Kante respraesentiert der entsprechende Knoten). 
         * Auch wird die Liniennummer in den Knoten abgespeichert. Um eine eindeutige
         * Rekonstruktion durchfuehren zu koennen, werden diese Daten als Tripel in der Form
         * (IDHaltestelleA, IDHaltestelleB, Liniennr. X) gespeichert. Dieses bedeutet, dass der Knoten
         * eindeutig der gerichteten Kante von HaltestelleA (ID) nach HaltestelleB (ID),  
         * auf welcher die Linie X faehrt, im Netz der Transportverbindungen entspricht. 
         * Allerdings kann es bei der anschliessenden Reduktion des Transportkettengraphen dazu kommen, dass Knoten zusammengefasst werden.
         * Das Problem hierbei ist dann, dass unter Umstanden nicht nur ein Tripel an einem Knoten im TKG steht.
         * Loesung (vorlaeufig): Eine Liste von Tripeln der oben beschriebenen Gestalt kann an die entsprechenden Knoten
         * gebunden werden. Ferner vergeben wir fuer jeden Knoten eine interne ID, diese wird ebenfalls an den
         * Knoten gespeichert.
         * 
         * Weiterhin wird an den Knoten die Fahrzeit der entsprechenden Verbindung vermerkt.
         * An den Kanten stehen die jeweiligen Verkehrsstroeme, die in der entsprechenden Methode
         * umgelegt werden. 
         */
       GRAPH<CKnotenBeschriftung*,CKantenBeschriftung*>* transportkettengraph;

        /**
         *  Beinhaltet die Anzahl der Zusammenhangskomponenten
         */
        int anzahlZusammenhangskomponenten;

        /**
         * Dieses Attribut beinhaltet ein Array von Zusammenhangskomponenten der groesse anzahlZusammenhangskomponenten
         */
        array<CZusammenhangskomponenten*>* FeldDerZusammenhangskomponenten;

        /**
         * @supplierCardinality 0..* 
         */
        CZusammenhangskomponenten* Zusammenhangskomponenten;
        int takt;
       


	/**
         * 
         */
       
	array<list<three_tuple<int,int,int>*>*>* InhaltDerZusammenhangskomponenten;

        /**
         * Das Linienplan hash array.
         */
        h_array<int, list<list<three_tuple<int, int, int> > > >* linienplan ;

	/**
	 * Liste des Fahrplans
	 */
	list<CFahrplanElement*>* fahrplan;

    private:
        /**
         * Hash-Array der Haltestellen.
         */
        h_array<int, four_tuple<int, int, int, int> >* hashHaltestellenplanGesamt;
        /**
         * Diese Methode baut das von der Programmgruppe erhaltene Netz der Transportverbindungen
         * in einen Transportkettengraphen um. Dabei gehen wir wie folgt vor:
         * 1. Wir iterieren ueber alle Kanten des NetzDerTransportverbindungen und erzeugen fuer jede
         *    Kante einen Knoten und vergeben eine fortlaufende interne ID. Ferner setzen wir die HaltestellenID's
         *    entsprechend unseren Ueberlegungen.
         *    Weiterhin erzeugen wir ein temporaeres Hash, welches von den Kanten des Netz der Transpoortverbindungen
         *    auf die Knoten des Transportkettengraphen verweist. Vermutlich brauchen wir dieses nur in dieser
         *    Aufbauphase.
         * 2. Im Anschluss daran iterieren wir ueber alle Knoten im Netz der Transportverbindungen. Fuer jeden Knoten
         *    im Netz der Transpoertverbindungen machen wir dann folgendes:
         *    Wir iterieren ueber alle ingrad Kanten dieses Knoten (Befehl in LEDA:forall_in_edges()). Fuer jede Kante
         *    die wir hierbei erhalten, schauen wir uns alle outgrad Kanten des Knotens an. Dann wird folgende
         *    Vergleichsoperation ausgefuehrt: Wir vergleichen den Source Knoten der in-Kante mit dem Target Knoten
         *    der out Kante. Sind diese verschieden, dann greifen wir ueber den temporaeren Hash auf die entsprechende
         *    Knoten im Transportkettengraphen zu und fuegen zwischen diese eine Kante ein. Dabei initialisieren wir die
         *    CKantenbeschriftung und legen die Verkehrstroeme auf die neu eingefuegte Kante wie folgt um: Wir summieren
         *    die Verkehrsstroeme der Kanten im Netz der Tranpsportverbindungen auf und schreiben sie mittels der Methode
         *    setzenVerkehrsstrom in CKantenBeschriftung an die Kante im Transportkettengraph.
         *    Sind die angeforderten Source und Targetknoten gleich, so machen wir nichts und gucken den naechsten out Knoten an.
         */
        void umbauenTransportkettengraph(GRAPH<CFPlanGraphKnoten*, CFPlanGraphKante*>*);
        /**
         * Erst im zweiten Prototypen. 
         */
        void reduzierenTransportkettengraph();
        /**
         * Nach Durchlauf des DFS haben wir ein Array der Knoten (Index). Die Elemente des Arrays sind INTEGER,
         * diese geben die Zusammenhangskomponente an, in welcher sich der entsprechende Knoten befindet.
         * Wenn dieses Array komplett berechnet wurde, erzeugen wir fuer jede Zusammenhangskomponente einen
         * eigenen Graphen. Folgendes Vorgehen schwebt uns vor: Wir erzeugen uns soviele Instanzen, wie wir Zusammen-
         * hangskomponenten im Transportkettengraphen haben, im Konstruktor erhalten diese dann ihre
         * Zusammenhangskomponenten ID. Anschliessend iterieren wir ueber alle Knoten
         * und fuegen die entsprechenden Knoten in den neuen Graphen der jeweiligen Zusammenhangskomponente ein.
         * Die Knoten behalten dabei alle ihre Informationen.
         * Funktion: anfuegenKnoten in CZusammenhangskomponenten. Diese baut uns fuer die jeweilige Zusammenhangskomponente
         * ein Hasharray auf. Mittels diesem koennen wir dann ueber unsere interne KnotenID auf den entsprechenden Knoten in
         * der Zusammenhangskomponente zugreifen.
         * Problem: Wenn wir dies fuer alle Knoten machen, fehlen uns immer noch die Verbindungen zwischen diesen.
         * Um diese einzufuegen gehen wir wie folgt vor: Wir iterieren ueber alle Kanten im Transportkettengraphen. Fuer
         * jede Kante lassen wir uns dann den Source, sowie den Target Knoten geben. Diese liegen natuerlich in einer
         * Zusammenhangskomponente. Wir schauen dann in unserem DFS-Array nach, um welche Komponente es sich handelt,
         * sobald wir dies wissen, rufen wir die Methode anfuegenKante in CZusammenhangskomponente auf. Diese erhaelt als
         * Parameter die internen ID's des Source und Targetknotens, sowie den Verkehrsstrom auf der Kante.
         */
        void berechnenZusammenhangskomponenten();
        /**
        * Berechnet die zyklomatische Zahl der einzelnen Zusammenhangskomponenten.
        */

        void berechnenZyklomatischeZahl();

	/**
	 *
	 */
	int gesamtFahrzeugAnzahl;
       
        /**
         * Gibt den Takt zurueck
         */
        int gebenTakt();
	/**
         * Reine Testfunktion. Gibt den berechneten Transportkettengraph aus. Nur um zu schauen, ob Umbau korrekt.
         */
        void ausgebenTGraph();

	/**
	 * Zeichnen des Transportkettengraphen
	 */
	void ZeichnenTransportkettengraph ();

	/**
	 * Zeichnen Netz der Transportverbindungen
	 */
	void ZeichnenNetzDerTransportverbindungen (GRAPH<CFPlanGraphKnoten*,
					            CFPlanGraphKante*> *);

	/**
	 * Zeichnen der einzelnen Zusammenhangskomponenten
	 */
	void ZeichnenZusammenhangskomponente ();
};
#endif //TRANSPORTKETTENGRAPH_H



