//------------------------------------------------------------------------------
// Projektname:		Fahrplan Modul
// Dateiname:		CZusammenhangskomponenten.cpp
// Beschreibung:	Kapselung der einzelnen Zusammenhangskomponenten
//                      des Transportkettengraphen
// Autor(en):		Michael Wintergerst
// Version:		0.1
// Gendert von:	
// Gendert am:	       
// Zustand:		Teil-Bearbeitung
//------------------------------------------------------------------------------

#include "CZusammenhangskomponenten.h"

//------------------------------------------------------------------------------
// Funktionsname: CZusammenhangskomponenten	
// Beschreibung: Konstruktor	
// Parameter: Nummer der Zusammenhangskomponente		
// Rueckgabe: void		
// Besonderheiten: Erzeugung eines parametrisierten Graphen	
//------------------------------------------------------------------------------
CZusammenhangskomponenten::CZusammenhangskomponenten(int zusammenhangskomponentenID){
  Debug (1, "CZusammenhangskomponenten::CZusammenhangskomponenten");
  this->zusammenhangskomponentenID = zusammenhangskomponentenID;
  this->graphDerZusammenhangskomponente = new GRAPH<CKnotenBeschriftung*, CKantenBeschriftung*>;
  this->zuordnungIdNachKnoten = new h_array<int,node>;
  this->listeDerHaltestellen = new list<three_tuple<int,int,int>* >;    
  this->mappingUmsteigeKante = new h_array<three_tuple<int,int,int>, edge>;
  this->mappingKante = new h_array<three_tuple<int,int,int>, edge>;
  this->simplexTableau = 0;
  this->listeMaximalgeruest = 0;
  this->absoluteFahrtzeitLinienArray = new h_array<int,int>();
  this->startHaltestellenNachLinie = new h_array<int, list<node>*>;

} // ende des konstruktors

//------------------------------------------------------------------------------
// Funktionsname:	~CZusammenhangskomponenten
// Beschreibung:	Destruktor
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
CZusammenhangskomponenten::~CZusammenhangskomponenten () {
  Debug(1,"Zertstoere Zusammenhangskomponente");
  node currentNode;
  edge currentEdge;
  CKantenBeschriftung* informationCurrentEdge;
  CKnotenBeschriftung* informationCurrentNode;
  
  // loeschen zusammenhangskomponentengraph
  // iteration ueber alle kanten
  forall_edges (currentEdge, *(this->graphDerZusammenhangskomponente)) {
    informationCurrentEdge = this->graphDerZusammenhangskomponente->inf (currentEdge);
    delete informationCurrentEdge;
  } // iteration ueber alle kanten

  // iteration ueber alle knoten
  forall_nodes (currentNode, *(this->graphDerZusammenhangskomponente)) {
    informationCurrentNode = this->graphDerZusammenhangskomponente->inf (currentNode);
    delete informationCurrentNode;
  } // iteration ueber alle knoten

  if (this->mappingUmsteigeKante) {
    delete this->mappingUmsteigeKante;
  }

  if (this->mappingKante) {
    delete this->mappingKante;
  }

  if (this->startHaltestellenArray) {
    delete this->startHaltestellenArray;
  }

  if (this->graphDerZusammenhangskomponente) {
    delete this->graphDerZusammenhangskomponente;
  }

  if (this->zuordnungIdNachKnoten) {
    delete this->zuordnungIdNachKnoten;
  }

  if (this->simplexTableau) {
    delete this->simplexTableau; 
  }

  if (this->linienListe) {
    delete this->linienListe;
  }

  //delete this->startHaltestellenArray;
  if (this->absoluteFahrtzeitLinienArray) {
    delete this->absoluteFahrtzeitLinienArray;
  }

  if (this->startHaltestellenNachLinie) {
    delete this->startHaltestellenNachLinie;
  }
} // ende destruktor

//------------------------------------------------------------------------------
// Funktionsname: gebenZusammenhangskomponenteAlsListe	
// Beschreibung:  Rueckgabe einer Liste von Tripeln, die es ermoeglicht
//	          die Zusammenhangskomponente im Netz der Transportverbindungen
//                zu rekonstruieren
// Parameter:	  void	
// Rueckgabe:	  Liste von Tripeln: HaltestelleA nach HaltestelleB bzgl. LinieA	
// Besonderheiten:	
//------------------------------------------------------------------------------
list<three_tuple<int,int,int>* >* CZusammenhangskomponenten::gebenZusammenhangskomponenteAlsListe(){
  node aktKnoten;
  CKnotenBeschriftung* informationAktKnoten;
  three_tuple<int,int,int>* neuesTripel;
  
  // iteration ueber alle knoten der zusammenhangskomponente
  // jeweils entsprechendes tripel in die liste der haltestellen einfuegen

  forall_nodes (aktKnoten, *(this->graphDerZusammenhangskomponente)) {
    informationAktKnoten = this->graphDerZusammenhangskomponente->inf (aktKnoten);
    neuesTripel = new three_tuple <int, int, int> (
                        informationAktKnoten->ausgebenHaltestellenID1 (),
			informationAktKnoten->ausgebenHaltestellenID2 (),
			informationAktKnoten->ausgebenLiniennummer ());
    this->listeDerHaltestellen->append (neuesTripel);
  } // ende der iteration ueber alle knoten

  return this->listeDerHaltestellen;
}

//------------------------------------------------------------------------------
// Funktionsname: anfuegenKnoten	
// Beschreibung:  Anfuegen eines Knoten in den Graphen unter
//                Beachtung der uebergegbenen Knotenbeschriftung	
// Parameter: 	  Referenz auf eine CKnotenBeschriftung	
// Rueckgabe:	  void	
// Besonderheiten:	
//------------------------------------------------------------------------------
void CZusammenhangskomponenten::anfuegenKnoten(const CKnotenBeschriftung& knotenBeschriftung){
  CKnotenBeschriftung* neueKnotenBeschriftung = new CKnotenBeschriftung ();

  *neueKnotenBeschriftung = knotenBeschriftung;
  (*(this->zuordnungIdNachKnoten))[(*neueKnotenBeschriftung).gebenKnotenID()] =
    this->graphDerZusammenhangskomponente->new_node (neueKnotenBeschriftung);
} // ende anfuegenKnoten

//------------------------------------------------------------------------------
// Funktionsname: anfuegenKante	
// Beschreibung:  fuegt eine Kante in die Zusammenhangskomponente	
// Parameter:	  SourceKnotenID, TargetKnotenID und Verkehrsstrom	
// Rueckgabe:	  void
// Besonderheiten:	
//------------------------------------------------------------------------------
void CZusammenhangskomponenten::anfuegenKante(int knoten1, int knoten2, int kantenID, int verkehrsstrom,
					      bool istUmsteigeKante){
  node sourceNode, targetNode;
  CKantenBeschriftung *kantenBeschriftung 
    = new CKantenBeschriftung (kantenID, verkehrsstrom, istUmsteigeKante);

  edge currentEdge;
  CKnotenBeschriftung *informationSourceKnoten, *informationTargetKnoten;
  three_tuple<int, int, int> *currentTriple;

  sourceNode = (*(this->zuordnungIdNachKnoten))[knoten1];
  targetNode = (*(this->zuordnungIdNachKnoten))[knoten2];
  
  informationSourceKnoten = this->graphDerZusammenhangskomponente->inf (sourceNode);
  informationTargetKnoten = this->graphDerZusammenhangskomponente->inf (targetNode);

  currentEdge = this->graphDerZusammenhangskomponente->new_edge (sourceNode,
								 targetNode,
								 kantenBeschriftung);
 
  three_tuple<int,int,int> tripel (informationSourceKnoten->ausgebenLiniennummer(),
						     informationTargetKnoten->ausgebenLiniennummer(),
				   informationSourceKnoten->ausgebenHaltestellenID2());

  (*mappingKante)[tripel] = currentEdge;

  // falls eine umsteigekante vorliegt, wird das definierte
  // mappingUmsteigeKante-hasharray gefuellt
  if (kantenBeschriftung->gebenIstUmsteigekante()) {
    currentTriple = new three_tuple <int, int, int> (informationSourceKnoten->ausgebenLiniennummer(),
						     informationTargetKnoten->ausgebenLiniennummer(),
						     informationSourceKnoten->ausgebenHaltestellenID2());
    (*mappingUmsteigeKante)[*currentTriple] = currentEdge; 
    currentTriple = new three_tuple <int, int, int> (informationSourceKnoten->ausgebenLiniennummer(),
						     informationTargetKnoten->ausgebenLiniennummer(),
						     informationSourceKnoten->ausgebenHaltestellenID2());
   
}
 
} // ende anfuegenKante

GRAPH<CKnotenBeschriftung,CKantenBeschriftung>* CZusammenhangskomponenten::gebenGraphDerZusammenhangskomponte(){
  return 0;
}

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CZusammenhangskomponenten::berechnenSimplexTableau(){
  int anzahlZeilen = this->zyklomatischeZahl;
  int anzahlSpalten = this->listeMaximalgeruest->size ();
  //  Debug(5,"Loeschen altes SimplexTableau");
  if (this->simplexTableau != 0) {
    delete this->simplexTableau;
  }
  //  Debug(5,"Erzeugen neues SimplexTableau");
  this->simplexTableau = new CSimplexTableau (anzahlZeilen,
					      anzahlSpalten,
					      this->takt);
  this->simplexTableau->einfuegenKantenInSimplextableau (
	    *(this->graphDerZusammenhangskomponente));
} // ende berechnenSimplexTableau

//------------------------------------------------------------------------------
// Funktionsname: berechnenZyklomatischeZahl 	
// Beschreibung: berechnet die Zyklomatische Zahl	
// Parameter: void		
// Rueckgabe: void		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CZusammenhangskomponenten::berechnenZyklomatischeZahl(){
  this->zyklomatischeZahl = 
    this->graphDerZusammenhangskomponente->number_of_edges () -
    this->graphDerZusammenhangskomponente->number_of_nodes () + 1;
  this->knotenPartition = new node_partition (*(this->graphDerZusammenhangskomponente));  
  Debug(9,"Zyklomatische Zahl: " << this->zyklomatischeZahl);
} // ende berechnenZyklomatischeZahl

void CZusammenhangskomponenten::setzenIdDerZusammenhangskomponente(){
}

int CZusammenhangskomponenten::gebenIdDerZusammenhangskomponente(){
  return 0;
}

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
int CZusammenhangskomponenten::gebenZyklomatischeZahl(){
  return this->zyklomatischeZahl;
}
//------------------------------------------------------------------------------
// Funktionsname: initialisierenFesterVerbindungen 	
// Beschreibung: 	
// Parameter: void		
// Rueckgabe: void		
// Besonderheiten:	
//------------------------------------------------------------------------------    
void CZusammenhangskomponenten::initialisierenFesterVerbindungen (){
  edge aktKante;
  CKantenBeschriftung* infoAktKante;

  // Iterieren ueber alle Kanten und deren bool Wert fuer feste Verbindung auf false setzen.
  forall_edges(aktKante,*(this->graphDerZusammenhangskomponente)){
    infoAktKante=this->graphDerZusammenhangskomponente->inf(aktKante);
    infoAktKante->setzenFesteVerbindung(false);
  };
 
  // Abfrage, ob Knoten Partition schon erzeugt wurde, ansonsten initialisieren.
  //  if (this->knotenPartition == 0) {
  delete this->knotenPartition;
  this->knotenPartition = new node_partition (*(this->graphDerZusammenhangskomponente));
    //this->knotenPartition.init (*(this->graphDerZusammenhangskomponente));
    //}; 

  // initialisieren des Atributes mussInsMaximalgeruest der Graphkanten
  //  this->initialisierenMussInsMaximalgeruest();
}
    
//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CZusammenhangskomponenten::berechnenMaximalgeruest(){

  edge kante;
  CKantenBeschriftung* kantenListe;
  forall_edges(kante,*(this->graphDerZusammenhangskomponente)){
    kantenListe = this->graphDerZusammenhangskomponente->inf(kante);
    kantenListe->setzenImMaximalgeruest(false);
  }
  if (this->listeMaximalgeruest != 0) {
    delete this->listeMaximalgeruest;
  }
  this->listeMaximalgeruest=new list<edge>;
  vergleichenKanten cmp (*(this->graphDerZusammenhangskomponente));

  //  Debug(5,"Berechne MinSpanningTree");
  (*listeMaximalgeruest)=
    MIN_SPANNING_TREE( *graphDerZusammenhangskomponente, cmp ); 
  //  Debug(5,"MinSpanningTree fertig");
   this->setzenMaximalgeruestKanten ();

} // ende berechnenMaximalGeruest
    
void CZusammenhangskomponenten::setzenMaximalgeruest(list<edge>* maximalgeruestKanten){
}
    
list<edge>* CZusammenhangskomponenten::gebenMaximalgeruest(){
  return 0;
}
//------------------------------------------------------------------------------
// Funktionsname: setzenFesterBindungen
// Beschreibung: Setzt die von der Programmgruppe gegebene feste Verbindung in
//               in dieser Zusammenhangskomponente.
// Parameter: void		
// Rueckgabe: void		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CZusammenhangskomponenten::setzenFesterBindungen(int linienID1,int linienID2,int haltestellenID){
//  std::cout << "FN: " << linienID1 << " " << linienID2 << " " << haltestellenID << endl;
  edge dieUmsteigeKante;
  string Meldung ("Die gewaehlte feste Verbindung existiert nicht, Eingabe wurde ignoriert.");
//  cout.flush ();
  // Suchen der entsprechenden Kante mittels der Methode suchenUmsteigeKante in CZusammenhangskomponenten.
  // dieUmsteigeKante=suchenUmsteigeKante(linienID1, linienID2, haltestellenID);

  three_tuple<int,int,int> tripel (linienID1, linienID2, haltestellenID);

  dieUmsteigeKante=(*mappingKante)[tripel];
//  cout.flush ();
   
  if (dieUmsteigeKante==0) {throw CFesteBindungNichtGefunden(Meldung); }
  else {
//    std::cout << "Feste Bindung gefunden" << endl;
//    std::cout.flush ();
    this->setzenUmsteigeKante (dieUmsteigeKante); 
  }
} // ende setzenFesterBindungen
    
void CZusammenhangskomponenten::berechnenKnotenPartition(){
}
    
double CZusammenhangskomponenten::gebenNetzbedingteWartezeit(){
  return 0;
}


//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------    
void CZusammenhangskomponenten::setzenMaximalgeruestKanten(){
  edge aktKante;
  CKantenBeschriftung* InfoAktKante;

  // festhalten in der kantenbeschriftung, dass
  // enstsprechende kanten im maximalgeruest vorhanden sind
  for (int i=0; i < this->listeMaximalgeruest->length(); i++){
    aktKante=listeMaximalgeruest->contents(listeMaximalgeruest->get_item(i));
    InfoAktKante=this->graphDerZusammenhangskomponente->inf(aktKante);
    InfoAktKante->setzenImMaximalgeruest (true);
  };//Ende for-Schleife
} // ende setzenMaximalgeruestKanten
    
double  CZusammenhangskomponenten::ausgebenMindesthaltezeit(int haltestellenID){
  return 0;
}

//------------------------------------------------------------------------------
// Funktionsname:	setzenMindesthaltezeit
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------    
void CZusammenhangskomponenten::setzenMindesthaltezeit(h_array<int, int>* mindesthaltezeit){
  node currentNode;
  edge currentEdge;
  CKantenBeschriftung* informationCurrentEdge;
  CKnotenBeschriftung* informationCurrentNode;
  int currentHaltezeit;

  // iteration ueber alle knoten
  forall_nodes (currentNode, *(this->graphDerZusammenhangskomponente)) {
    informationCurrentNode = this->graphDerZusammenhangskomponente->inf (currentNode);
    currentHaltezeit = (*mindesthaltezeit)[informationCurrentNode->ausgebenHaltestellenID2()];
    // iteration ueber alle out-kanten
    forall_out_edges (currentEdge, currentNode) {
      informationCurrentEdge = this->graphDerZusammenhangskomponente->inf (currentEdge);
      informationCurrentEdge->setzenMindestHaltezeit (currentHaltezeit);
    } // ende iteration ueber alle out-kanten
  } // ende iteration ueber alle knoten
} // setzenMindesthaltezeit
    
double  CZusammenhangskomponenten::ausgebenMindestumsteigezeit(int haltestellenID, int Liniennummer1, int Liniennummer2){
  return 0;
}
    
//------------------------------------------------------------------------------
// Funktionsname:	setzenMindestumsteigezeit
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CZusammenhangskomponenten::setzenMindestumsteigezeit(h_array<int, list< three_tuple<int,int,int>* >* >* mindestumsteigezeit){
  node currentNode, targetNode;
  edge currentEdge;
  CKantenBeschriftung *informationCurrentEdge;
  CKnotenBeschriftung *informationCurrentNode, *informationTargetNode;
  int currentUmsteigezeit, umsteigeHaltestelle, linienID1, linienID2;
  list<three_tuple<int, int, int>*>* currentList;
  three_tuple <int, int, int>* currentTripel;

  // iteration ueber alle knoten
  forall_nodes (currentNode, *(this->graphDerZusammenhangskomponente)) {
    informationCurrentNode = this->graphDerZusammenhangskomponente->inf (currentNode);
    umsteigeHaltestelle = informationCurrentNode->ausgebenHaltestellenID2 ();
    currentList = (*mindestumsteigezeit)[umsteigeHaltestelle];
 
    // falls liste ueberhaupt angegeben
    if (currentList) {
     
      // iteration ueber alle out-kanten
      forall_out_edges (currentEdge, currentNode) {
	informationCurrentEdge = this->graphDerZusammenhangskomponente->inf (currentEdge);
	// falls umsteigekante, entsprechend suchen der mindestumsteigezeit
	if (informationCurrentEdge->gebenIstUmsteigekante ()) {
	  targetNode = this->graphDerZusammenhangskomponente->target (currentEdge);
	  informationTargetNode = this->graphDerZusammenhangskomponente->inf (targetNode);
	  linienID1 = informationCurrentNode->ausgebenLiniennummer ();
	  linienID2 = informationTargetNode->ausgebenLiniennummer ();
	  // suchen der umsteigezeit
	  forall (currentTripel, *currentList) {
	   
	    // falls passender eintrag gefunden
	    if (currentTripel->first () == linienID1 &&
		currentTripel->second () == linienID2) {
	      currentUmsteigezeit = currentTripel->third ();
	      informationCurrentEdge->setzenMindestUmsteigezeit (currentUmsteigezeit);
	      break;
	    } // ende if gefunden
	  }
	} // ende if umsteigeKante
      } // ende iteration ueber alle out-kanten
    } // ende if liste definiert
  } // ende iteration ueber alle knoten  
} // ende setzenMindestumsteigezeit
    
//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CZusammenhangskomponenten::setzenTakt(int takt){
  this->takt = takt;
} // ende setzenTakt
    
//------------------------------------------------------------------------------
// Funktionsname:	berechnenFahrplan
// Beschreibung:	Diese Methode ruft zuerst die Methode generieren Abfahrtszeiten auf
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CZusammenhangskomponenten::berechnenFahrplan(three_tuple<int,int,int>* anfangszeit,four_tuple<int,int,int,int>* fahrplanIntervall){
 // Berechne mit dem Fahrplanintervall und dem Takt die Anzahl der Abfahrten
  int vonZeit = (fahrplanIntervall->first() * 60) + fahrplanIntervall->second();
  int bisZeit = (fahrplanIntervall->third() * 60) + fahrplanIntervall->fourth();

  this->gesamtDauerFahrplan = bisZeit - vonZeit;

  this->generierenAbfahrtszeiten(anfangszeit);
 
  // Initialisieren der Elemente
  CFahrplanElement* fahrplanElement;
  node aktKnoten;
  fahrplan = new list<CFahrplanElement*>;

  // Falls der Takt noch auf 0 steht, setzen in per Default auf 20
  if (this->takt == 0) this->takt=20;
  int anzahlAbfahrten = (bisZeit - vonZeit) / this->takt;

  // Fuellen des Fahrplans
  Debug(5,"Erstellen des Fahrplans!");
  forall_nodes(aktKnoten,*(this->graphDerZusammenhangskomponente)){
    
    // Hole Knoteninformation
    CKnotenBeschriftung* knotenInfo = this->graphDerZusammenhangskomponente->inf(aktKnoten);
    int haltestellenID = knotenInfo->ausgebenHaltestellenID1();
    int linienID = knotenInfo->ausgebenLiniennummer();
    node startHaltestelle = (*(this->startHaltestellenArray))[linienID];
    CKnotenBeschriftung* startHaltestelleInfo = this->graphDerZusammenhangskomponente->inf(startHaltestelle);
    int linienStartID = startHaltestelleInfo->ausgebenHaltestellenID1();

    // Berechne mit dem Fahrplanintervall und dem Takt die Anzahl der Abfahrten
    for (int zaehler = 0; zaehler < anzahlAbfahrten; zaehler ++) {
      
      int abfahrtzeitMinute = vonZeit + (zaehler * this->takt) + knotenInfo->ausgebenAbfahrtzeit();
      int stunde = abfahrtzeitMinute / 60;
      int minute = abfahrtzeitMinute % 60;
      if (minute < 0) {
	minute = minute + 60;
	stunde = stunde - 1;
      }
      if (stunde > 23) stunde = stunde - 24;
      if (stunde < 0) stunde = stunde + 24;
      fahrplanElement = new CFahrplanElement(haltestellenID,linienID,linienStartID,stunde,minute);
      this->fahrplan->append(fahrplanElement);
    }
  }     
  // ENDE DER FAHRPLANBERECHNUNG...!!!!!!!!!!!!!!!!
  // Sortieren der ZeitenListe gebenFahrplan.
  // PLAN: erstmal sortieren nach Linie und Zeit. 
   
   this->fahrplan->sort(&sortierenNachLinie, 1);
   this->aufbauUmlaufGraph();
   this->sortierenFahrplanElemente ();
}

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CZusammenhangskomponenten::berechnenWartezeitResultate () {
  edge currentEdge;
  CKantenBeschriftung* informationCurrentEdge;

  // initialisieren der wartezeiten
  this->initWartezeitResultate ();

  // iteration ueber alle kanten der zusammenhangskomponente
  forall_edges (currentEdge, *(this->graphDerZusammenhangskomponente)) {
    informationCurrentEdge = this->graphDerZusammenhangskomponente->inf (
		       					 currentEdge);
    this->summeNetzbedingteWartezeiten += (double)
      informationCurrentEdge->gebenNetzbedingteWartezeit ();
    this->wartezeitAufwand += (double)
      (informationCurrentEdge->gebenNetzbedingteWartezeit () *
       informationCurrentEdge->abfragenVerkehrsstrom ());
    this->mindestWartezeitaufwand += (double)
      ( (informationCurrentEdge->gebenMindestHaltezeit () +
	 informationCurrentEdge->gebenMindestUmsteigezeit ()) *
	informationCurrentEdge->abfragenVerkehrsstrom ());
  } // ende der iteration ueber alle kanten

  this->gesamtWartezeitAufwand = 
    this->wartezeitAufwand + this->mindestWartezeitaufwand;
} // berechnenWartezeitResultate

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CZusammenhangskomponenten::initWartezeitResultate () {
  this->summeNetzbedingteWartezeiten = 0.0;
  this->wartezeitAufwand = 0.0;
  this->mindestWartezeitaufwand = 0.0;
  this->gesamtWartezeitAufwand = 0.0;
} // ende initWartezeitResultate

//------------------------------------------------------------------------------
// Funktionsname:	generierenAbfahrtszeiten
// Beschreibung:	Mit dieser Methode werden die Abfahrtszeiten aller Haltestellen der Zusammenhangskomponente
//                      bestimmt und in dem Parameter fahrplan gespeichert.
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CZusammenhangskomponenten::generierenAbfahrtszeiten(three_tuple<int, int, int>* anfangszeit){
  Debug(1,"generierenAbfahrtszeiten Einsprung");
  // Als erstes werden die Linien Starthaltestellen berechnet
  node aktKnoten, dfsStartKnoten, tempKnoten;
  this->linienListe = new list<int>;    
  this->startHaltestellenArray = new h_array<int,node>();
  this->endHaltestellenArray = new h_array<int, node>();
  delete this->absoluteFahrtzeitLinienArray;
  this->absoluteFahrtzeitLinienArray = new h_array<int,int>();
  relativeGesamtFahrtzeit = 0;

  // lokale variablen fuer berechnung der fahrzeuganzahl
  int absoluteFahrzeit = 0; // summe der fahrzeiten auf einer strecke
  int relativeFahrzeit = 0; // summe der fahrzeiten + wartezeiten auf einer strecke

  // Setzen der ersten Abfahrtszeit (HaltestellenID,liniennr,std,min) und
  // initialisieren des Flags fahrplanzeitGesetzt in CKnotenBeschriftung

  bool abfahrtszeitGesetzt = false; // Wird auf true gesetzt, sobald die Abfahrtszeit erfolgreich gesetzt wurde
  CKnotenBeschriftung* infoKnoten;

  Debug(5,"Setzen der ersten Abfahrtszeit");
  forall_nodes(aktKnoten,*(this->graphDerZusammenhangskomponente)){
    infoKnoten = this->graphDerZusammenhangskomponente->inf(aktKnoten);
    infoKnoten->initialisierenFahrplanzeitGesetzt();
    // Nachschauen, ob fuer die aktuelle Liniennummer schon die Starthaltestelle bestimmt wurde
    // wenn nicht, erfolgt die Berechnung. Tempknoten wird gespeichert, um spaeter Linienliste
    // aufabauen zu koennen.

    if (!((*(this->startHaltestellenArray)).defined(infoKnoten->ausgebenLiniennummer()))) {
      
     
      tempKnoten = (this->berechnenLinienStartHaltestelle(&aktKnoten));
     
      (*(this->startHaltestellenArray))[infoKnoten->ausgebenLiniennummer()]=tempKnoten;
      
      // Eintragen der Liniennummer in linienListe.
      //      Debug(1, "Fuellen der Linienliste");
      //mw CKnotenBeschriftung* infoTempKnoten=this->graphDerZusammenhangskomponente->inf(tempKnoten);
      //mw this->linienListe->append(infoTempKnoten->ausgebenLiniennummer());
    }
   
    // Nachschauen, ob es sich bei dem Knoten um den Startknoten handelt
    if ((infoKnoten->ausgebenHaltestellenID1() == anfangszeit->first()) &&
	(infoKnoten->ausgebenLiniennummer() == anfangszeit->second())) {
      infoKnoten->setzenAbfahrtzeit(anfangszeit->third());
      dfsStartKnoten = aktKnoten;
      abfahrtszeitGesetzt = true;
    }
   
  }

  if (!abfahrtszeitGesetzt) {
    string meldung("Die Abfahrtszeiten konnten nicht gesetzt werden!");
    throw CUnsinnigerWertGeliefert(meldung);
  }

    // Setzen aller anderer Zeiten per BFS Durchlauf beginnend bei dfsStartKnoten
  Debug(2,"Aufruf der Methode berechnenAbfahrtszeitenPerBFS");
  berechnenAbfahrtszeitenPerBFS(&dfsStartKnoten);    
  
  // Anpassen der Linien :
  int linienNummer;
  
  // setzen der fahrzeuganzahl auf 0
  this->fahrzeugAnzahl = 0;
  
  // iteration ueber alle linien der zusammenhangskomponenten
  Debug(5,"Die einzelnen Abfahrtszeiten werden auf die Linien angepasst");
  forall_defined(linienNummer,*(this->startHaltestellenArray)){
    this->linienListe->append(linienNummer);

    node haltestelle = (*(this->startHaltestellenArray))[linienNummer]; // Hole die erste Haltestelle der Linie
    CKnotenBeschriftung* haltestellenInfo = this->graphDerZusammenhangskomponente->inf(haltestelle);
    haltestellenInfo->setzenAbfahrtzeit(haltestellenInfo->ausgebenAbfahrtzeit() % this->takt); // Setze Abfahrtzeit
    if (haltestellenInfo->ausgebenAbfahrtzeit() < 0){ // Ueberpruefen, ob Abfahrtzeit kleiner 0 ist
      haltestellenInfo->setzenAbfahrtzeit(haltestellenInfo->ausgebenAbfahrtzeit() + this->takt);
    }
    
    // merken der relativen abfahrtszeit
    relativeFahrzeit = -haltestellenInfo->ausgebenAbfahrtzeit ();
    // merken der startfahrzeit
    absoluteFahrzeit = haltestellenInfo->ausgebenFahrzeit ();
    
    bool letztenKnotenNichtGefunden = true;
    node adjKnoten;
    list<edge> kantenListe;
    
    while (letztenKnotenNichtGefunden) {
      letztenKnotenNichtGefunden = false; // Falls der akt Knoten schon der letzte ist
      kantenListe = this->graphDerZusammenhangskomponente->out_edges(haltestelle);
      edge kante;
      forall(kante,kantenListe){
	CKantenBeschriftung* kantenInfo = this->graphDerZusammenhangskomponente->inf(kante);
	if (!(kantenInfo->gebenIstUmsteigekante())){
	  adjKnoten =this->graphDerZusammenhangskomponente->target(kante);
	  CKnotenBeschriftung* adjKnotenInfo = this->graphDerZusammenhangskomponente->inf(adjKnoten);
	  haltestellenInfo = this->graphDerZusammenhangskomponente->inf(haltestelle); // Da haltestelle sich aendert
	  adjKnotenInfo->setzenAbfahrtzeit(haltestellenInfo->ausgebenAbfahrtzeit() + haltestellenInfo->ausgebenFahrzeit() + kantenInfo->gebenWartezeit());
	  haltestelle = adjKnoten;
	  
	  absoluteFahrzeit += adjKnotenInfo->ausgebenFahrzeit ();
	  
	  letztenKnotenNichtGefunden = true; // Es gibt mindestens noch einen Knoten	    
	} // end if


	//HASHARRAY absolute Fahrtzeit aller Linien anlegen. Nicht die ideale Stelle, da noch in for-Schleife,
        // aber es gibt keine andere Stelle, an welcher die absoluteFahrzeit UND die Liniennummer gleichzeitig 
        // bekannt sind.
        (*absoluteFahrtzeitLinienArray)[linienNummer]=absoluteFahrzeit;


      } // end forall
    } // end while
    // letzen knoten noch beachten
    relativeFahrzeit += (haltestellenInfo->ausgebenFahrzeit () +
			 haltestellenInfo->ausgebenAbfahrtzeit ());
    relativeGesamtFahrtzeit += relativeFahrzeit;
    this->berechnenFahrzeugAnzahl (relativeFahrzeit, absoluteFahrzeit);
  } // end forall_defined
   
 

    // Berechnung der Endhaltestellen jeder Linie. Dabei wird die Symmetrie der Linien genutzt.
    // Ergebnis der Berechnung wird im hash Array endHaltestellenArray gespeichert. 
  
    for(int i=0;i< this->linienListe->length();i++){
      
      int aktLinie =this->linienListe->contents(this->linienListe->get_item(i));
      node startknotenLinieA;
     
      startknotenLinieA =(*startHaltestellenArray)[aktLinie];
      //      (*endHaltestellenArray)[aktLinie*(-1)]=startknotenLinieA;
      (*endHaltestellenArray)[aktLinie]=SuchenEndKnoten (startknotenLinieA);
    };  
    
 
} // ende generierenAbfahrtzeit

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CZusammenhangskomponenten::berechnenFahrzeugAnzahl (int relativeFahrzeit,
							 int absoluteFahrzeit) {
  int gesamtZeitUmlauf = relativeFahrzeit + absoluteFahrzeit;

  if (gesamtZeitUmlauf < this->gesamtDauerFahrplan) {
    this->fahrzeugAnzahl += (int) (ceil ( (double) gesamtZeitUmlauf / (double) takt));
    //  std::cout << "Fahrzeugthis->fahrzeugAnzahl << endl;
  } //ende if
  else {
    this->fahrzeugAnzahl += (int) (ceil ((double) this->gesamtDauerFahrplan / (double) takt));
  } // ende else
//  std::cout << " FahrzeugAnzahL: " << this->fahrzeugAnzahl << endl;

} // ende berechnenFahrzeugAnzahl

//------------------------------------------------------------------------------
// Funktionsname:	gebenFahrplan
// Beschreibung:	Es wird der Fahrplan fuer die Zusammenhangskomponente zurueckgegeben
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
list<CFahrplanElement*>* CZusammenhangskomponenten::gebenFahrplan(){
  return this->fahrplan;
}

double CZusammenhangskomponenten::netzbedingteWartezeit(){
  return 0;
}
//------------------------------------------------------------------------------
// Funktionsname:	suchenUmsteigeKante
// Beschreibung:	Gibt einen Zeiger auf die gesuchte Kante zurueck.
// Parameter:		linienID1, linienID2, haltestellenID
// Rueckgabe:		Die zu diesen Werten gehoerige Kante.
// Besonderheiten:	Diese Methode kann kein hebraeisch! 
//------------------------------------------------------------------------------    
edge CZusammenhangskomponenten::suchenUmsteigeKante(int linienID1, int linienID2, int haltestellenID){
  edge dieGesuchteKante;
  three_tuple<int, int, int> suchTripel (linienID1, linienID2, haltestellenID);
 
  dieGesuchteKante = (*mappingUmsteigeKante)[suchTripel];
   
  return (dieGesuchteKante);     
   

   // Die alte Art die kanten zu finden, wenn alles laeuft mal loeschen....


  // bool kanteGefunden=false;
  // Suchen der Kante mittels des hash_arrays mappingUmsteigekante
  
 
  // Iteration ueber alle Kanten dieser Zusammenhangskomponente, bis die gesuchte Kante ermittelt wurde!

  //forall_edges(aktKante,*(this-> graphDerZusammenhangskomponente)){

      // Holen Knotenbeschriftungen des Source und Target Knotens der aktuellen Kante.
      // infoSourceKnoten=this->graphDerZusammenhangskomponente->inf(this->graphDerZusammenhangskomponente->source(aktKante));
      //infoTargetKnoten=this->graphDerZusammenhangskomponente->inf(this->graphDerZusammenhangskomponente->target(aktKante));

      //if ( ( (infoSourceKnoten->ausgebenHaltestellenID2() == haltestellenID) && (infoSourceKnoten->ausgebenLiniennummer() == linienID1)) && 
      //	( (infoTargetKnoten->ausgebenHaltestellenID1() == haltestellenID) && (infoTargetKnoten->ausgebenLiniennummer() == linienID2))){

      // dieGesuchteKante=aktKante;
      //  kanteGefunden=true;
      //  break;
  //  }; 
      
   // if(!kanteGefunden) cout<<"Die gesuchte Kante existiert in dieser Zusammenhangskomponente nicht!"<<endl;
   
   //return dieGesuchteKante;
}
//------------------------------------------------------------------------------
// Funktionsname: setzenUmsteigeKante
// Beschreibung:  
// Parameter:	  
// Rueckgabe:	  
// Besonderheiten:	
//-------------------------------------------------------------------------------    
void CZusammenhangskomponenten::setzenUmsteigeKante(edge kanteMitFesterBindung){
  node sourceKnoten, targetKnoten;
  bool gleichePartition=false;
  CKantenBeschriftung* informationKanteMitFesterBindung = 
    this->graphDerZusammenhangskomponente->inf (kanteMitFesterBindung);

  sourceKnoten=this->graphDerZusammenhangskomponente->source(kanteMitFesterBindung);
  targetKnoten=this->graphDerZusammenhangskomponente->target(kanteMitFesterBindung);
  gleichePartition=this->knotenPartition->same_block(sourceKnoten, targetKnoten);
  
  // knoten in verschiedener partition
  // falls richtig setzenFesteVerbindung und
  // vereinigen der beiden partitionen
  if (! gleichePartition) {
    informationKanteMitFesterBindung->setzenFesteVerbindung (true);
    this->knotenPartition->union_blocks (sourceKnoten, targetKnoten);
  } else {
    string message("");
    throw CKreisGewaehlt(message);
  }
} // ende setzenUmsteigeKante
    
void CZusammenhangskomponenten::berechnenNetzbedingteWartezeit(){
}
    
list<two_tuple<bool,edge> >* CZusammenhangskomponenten::berechnenZyklusKanten(edge zyklusKante){
  return 0;
}

//------------------------------------------------------------------------------
// Funktionsname:	berechnenLinienStartHaltestellenID
// Beschreibung:	Sucht rekursiv von einem gegeben Knoten den Linienstartknoten
// Parameter:		Ein beliebiger Knoten der Linie
// Rueckgabe:		Anfangsknoten der Linie
// Besonderheiten:	
//------------------------------------------------------------------------------
node CZusammenhangskomponenten::berechnenLinienStartHaltestelle(node* knoten){
  Debug(1,"berechenLinienStartHaltestelle");
  node startHaltestelle = *knoten;
    bool found = false;

    // Knoteninformationen holen
    //  CKnotenBeschriftung* knotenInformation = this->graphDerZusammenhangskomponente->inf(*knoten);

    // Nachschauen, ob der Knoten der LinienStartknoten ist
    //list<edge> Kantenliste = this->graphDerZusammenhangskomponente->in_edges(*knoten);
    //   edge Kante;
 
    while (! found)
    {
      CKnotenBeschriftung* knotenInformation = this->graphDerZusammenhangskomponente->inf(startHaltestelle);
      list<edge> Kantenliste = this->graphDerZusammenhangskomponente->in_edges(startHaltestelle);
      edge Kante;
      forall (Kante, Kantenliste)
      {
	node vorherigerKnoten = this->graphDerZusammenhangskomponente->source(Kante);
	CKnotenBeschriftung* vorherigerKnotenInformation = this->graphDerZusammenhangskomponente->inf(vorherigerKnoten);
        if (vorherigerKnotenInformation->ausgebenLiniennummer() == knotenInformation->ausgebenLiniennummer()){
	  startHaltestelle = vorherigerKnoten;
	  found = true;
	  break;
	}
      }
      if (found == true) found = false;
      else found = true;
    }

    //   forall(Kante,Kantenliste){
    //  node vorherigerKnoten = this->graphDerZusammenhangskomponente->source(Kante);
    //  CKnotenBeschriftung* vorherigerKnotenInformation = this->graphDerZusammenhangskomponente->inf(vorherigerKnoten);
    // if (vorherigerKnotenInformation->ausgebenLiniennummer() == knotenInformation->ausgebenLiniennummer()){
    //	std::cout << "Rek: " << vorherigerKnotenInformation->ausgebenLiniennummer () << endl;
    //	startHaltestelle = this->berechnenLinienStartHaltestelle(&vorherigerKnoten);
	//CKnotenBeschriftung* tm = graphDerZusammenhangskomponente->inf (*startHaltestelle);
	//std::cout << "REkN: " << tm->ausgebenLiniennummer () << endl;
	//	std::cout.flush ();
    //      }
    //}
    
 
    // Wenn es dieser nicht ist, rekursiver abstieg
    // ansonsten Knoten ID zurueckliefern
    return startHaltestelle;
}

//------------------------------------------------------------------------------
// Funktionsname:		berechnenAbfahrtzeiten
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CZusammenhangskomponenten::berechnenAbfahrtszeitenPerBFS(const node* einstiegsKnoten){
  Debug(1,"Einsprung in berechenenAbfahrtszeitenPerBFS");
  if (!(einstiegsKnoten == 0)) {
    CKnotenBeschriftung* einstiegsKnotenInfo = this->graphDerZusammenhangskomponente->inf(*einstiegsKnoten);
    list<edge> adjKantenListe = this->graphDerZusammenhangskomponente->out_edges(*einstiegsKnoten);
    list<edge> hilfListe = this->graphDerZusammenhangskomponente->in_edges(*einstiegsKnoten);
    adjKantenListe.conc(hilfListe);
    edge adjKante;
    Debug(5,"Die relativen Abfahrtszeiten werden gesetzt");
    forall(adjKante,adjKantenListe){
      CKantenBeschriftung* adjKantenInfo = this->graphDerZusammenhangskomponente->inf(adjKante);
      if (adjKantenInfo->gebenImMaximalgeruest()){
	CKnotenBeschriftung* adjKnotenInfo;
	node adjKnoten;
	int abfahrtzeit;
	if (this->graphDerZusammenhangskomponente->source(adjKante) == *einstiegsKnoten){
	  adjKnoten = this->graphDerZusammenhangskomponente->target(adjKante);
	  adjKnotenInfo = this->graphDerZusammenhangskomponente->inf(adjKnoten);
	  abfahrtzeit = einstiegsKnotenInfo->ausgebenAbfahrtzeit() + einstiegsKnotenInfo->ausgebenFahrzeit() +
	    adjKantenInfo->gebenWartezeit();
	}
	else if (this->graphDerZusammenhangskomponente->target(adjKante) == *einstiegsKnoten){
	  adjKnoten = this->graphDerZusammenhangskomponente->source(adjKante);
	  adjKnotenInfo = this->graphDerZusammenhangskomponente->inf(adjKnoten);
	  abfahrtzeit = einstiegsKnotenInfo->ausgebenAbfahrtzeit() - adjKnotenInfo->ausgebenFahrzeit() -
	    adjKantenInfo->gebenWartezeit();
	}
	if (adjKnotenInfo->ausgebenFahrplanzeitGesetzt() == false){
	  adjKnotenInfo->setzenAbfahrtzeit(abfahrtzeit);
	  berechnenAbfahrtszeitenPerBFS(&adjKnoten);
	}
      }
    }
  }
  else {
    throw NichtInstanzierteParameter();
  }
}


//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CZusammenhangskomponenten:: ausgebenZGraph(){
  CKantenBeschriftung* InfoAktKante;
  CKnotenBeschriftung* InfoAktKnoten;
  CKnotenBeschriftung*  InfoTargetKnoten;
  node  aktKnoten;
  list<edge> KantenListeAktKnoten;
  edge aktKante;

  // Iteration ueber alle Knoten des Graphen
  forall_nodes (aktKnoten, *(this->graphDerZusammenhangskomponente))
     {
       // CKnotenBeschriftung holen und ausgeben mittels ueberladenem << Operator
       InfoAktKnoten=this->graphDerZusammenhangskomponente->inf(aktKnoten);
//       cout<< "Werte des aktuellen Knoten: " << (*InfoAktKnoten) << endl;

       // Holen aller adjazenten Kanten dieses Knotens und ausgeben dieser Werte, sofern
           // der Knoten ueber adjazente Kanten verfuegt.
       KantenListeAktKnoten=this->graphDerZusammenhangskomponente->adj_edges(aktKnoten);
       

       if (!KantenListeAktKnoten.empty()){

              for (int i=0; i < KantenListeAktKnoten.length(); i++){
//		cout<< "Werte der "<< i+1 << " . adjazenten Kante dieses Knoten: ";
	          aktKante=KantenListeAktKnoten.contents(KantenListeAktKnoten.get_item(i));
                  InfoAktKante=this->graphDerZusammenhangskomponente->inf(aktKante);
		  
		  // Holen und ausgeben des Target Knotens der jeweils aktuellen Kante. 
		  InfoTargetKnoten=this->graphDerZusammenhangskomponente->inf(this->graphDerZusammenhangskomponente->target(aktKante));
//                  cout<< (*InfoAktKante) << endl;   
//		  cout<< "Werte des Targetknoten dieser Kante: " << (*InfoTargetKnoten) << endl; 

               };//Ende for-Schleife

       }//Ende if Block
       else cout<< "Keine adjazenten Kanten an Knoten "<<InfoAktKnoten->gebenKnotenID() << "!"<< endl;
       cout << endl;

     };// Ende forall_nodes  
}

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
int CZusammenhangskomponenten::gebenFahrzeugAnzahl () {
  return this->fahrzeugAnzahl;
}

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
double CZusammenhangskomponenten::gebenSummeNetzbedingteWartezeiten () {
  return this->summeNetzbedingteWartezeiten;
}

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
double CZusammenhangskomponenten::gebenWartezeitAufwand () {
  return this->wartezeitAufwand;
}

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
double CZusammenhangskomponenten::gebenMindestWartezeitAufwand () {
  return this->mindestWartezeitaufwand;
}

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
double CZusammenhangskomponenten::gebenGesamtWartezeitAufwand () {
  return this->gesamtWartezeitAufwand;
}

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CZusammenhangskomponenten::startenLokaleSuche ()
{
  this->simplexTableau->startenLokaleSuche (*(this->graphDerZusammenhangskomponente));
  this->berechnenWartezeitResultate ();
} // ende startenLokaleSuche


//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CZusammenhangskomponenten::aufbauUmlaufGraph(){
  Debug(1,"Aufbau des Umlaufgraphen");

  // summe der gesamtfahrzeiten, jeder fahrgastfahrt 
  int gesamtFahrzeitRein = 0;
  // summe der gesamtfahrzeiten + fahrzeit zu der anfangsfahrt
  int gesamtUmlaufzeit = 0;

  node quelle, startKnotenFahrgastfahrt, senke;
  list<node>* listeDerEndhaltestellenKnoten=new list<node>;
  CFahrplanElement* aktuellerFahrplanEintrag;
  umlaufGraph = new GRAPH< four_tuple<int,int,int,int> , int>;
  delete this->startHaltestellenNachLinie;
  this->startHaltestellenNachLinie = new h_array<int, list<node>*>;
  int aktuelleLinie=0;
  list<node>* tempListeStarthaltestellen=new list<node>;
  list<edge> *listeAnfang = new list<edge>;
  list<edge> *listeEnde = new list<edge>;
  list<edge> *listeFahr = new list<edge>;

  // Depotknoten (Quelle) einfuegen
  four_tuple<int, int, int, int> depotQuelle ( -5, -16000, 0, 0 );
  quelle=umlaufGraph->new_node(depotQuelle );  

  // Depotknoten (Senke) einfuegen
  four_tuple<int, int, int, int> depotSenke ( -6, 16000, 0, 0 );
  senke=umlaufGraph->new_node(depotSenke );  

  // Eigentlicher Beginn des Umbaus.
  //Debug(1, "Suchen Startknoten");
  // Suchen des Startknotens aus dem generiertem Fahrplan
  Debug (2, "Erstellen Knoten Fahrgastfahrten");
  CFortschritt *fort = new CFortschritt ();
  for(int i=0; i < this->fahrplan->length();i++){
    fort->anzeigenFortschritt (i+1, this->fahrplan->length());
    aktuellerFahrplanEintrag=this->fahrplan->contents(fahrplan->get_item(i));
    if (aktuellerFahrplanEintrag->gebenHaltestelle()==aktuellerFahrplanEintrag->gebenLinienStartHaltestelle()){   
      // Startknoten erzeugen und in den Umlaufgraphen einfuegen.
      //Debug(1, "Startknoten erzeugen und einfuegen");
      four_tuple<int,int,int,int> fahrgastFahrtBeginn (
       aktuellerFahrplanEintrag->gebenHaltestelle(),aktuellerFahrplanEintrag->gebenLinie(),
       aktuellerFahrplanEintrag->gebenAbfahrtszeitStunde(),aktuellerFahrplanEintrag->gebenAbfahrtszeitMinute());
      startKnotenFahrgastfahrt=umlaufGraph->new_node(fahrgastFahrtBeginn);
      // Linienliste fuer das Hash Array erzeugen und gegebenenfalls einfuegen.
      // In leere Liste direkt einfuegen
      if (tempListeStarthaltestellen->empty() ) {
       tempListeStarthaltestellen->append(startKnotenFahrgastfahrt);
       aktuelleLinie=aktuellerFahrplanEintrag->gebenLinie();
      }//end if

      // falls Liste voll, Fallunterscheidung. Ist der aktuelle Knoten noch von der selben Linie dann einfuegen,
      // falls nicht, Liste in h_arrat einfuegen, dann loeschen und mit aktuellem Knoten neue Liste starten.
      else {
	if (aktuellerFahrplanEintrag->gebenLinie() == aktuelleLinie ){
	  tempListeStarthaltestellen->append(startKnotenFahrgastfahrt);
        }//end if
        else {
	  (*startHaltestellenNachLinie)[aktuelleLinie]=tempListeStarthaltestellen;
          tempListeStarthaltestellen=new list<node>;
          tempListeStarthaltestellen->append(startKnotenFahrgastfahrt);   
          aktuelleLinie=aktuellerFahrplanEintrag->gebenLinie();
	}//end else
      }//end else
      node endhaltestelle;
      CKnotenBeschriftung* endKnotenBeschriftung;
      endhaltestelle =(*endHaltestellenArray)[aktuellerFahrplanEintrag->gebenLinie()];
  
      //Debug(1,"Holen Knoten in ZUsammenhangskomponente.");
      endKnotenBeschriftung=this->graphDerZusammenhangskomponente->inf(endhaltestelle);

      // Zeiten des Endknotens berechnen mittels absoluteFahrtzeitLinienArray
      int dauerFahrt=(*absoluteFahrtzeitLinienArray)[aktuellerFahrplanEintrag->gebenLinie()];
      int stunde=aktuellerFahrplanEintrag->gebenAbfahrtszeitStunde();
      int minute=aktuellerFahrplanEintrag->gebenAbfahrtszeitMinute();

      gesamtFahrzeitRein += dauerFahrt;
      gesamtUmlaufzeit += dauerFahrt;

      // Gucken, ob wir in eine neue Stunde kommen, wenn ja entsprechend umrechnen.
      while (dauerFahrt > 0){
	if (dauerFahrt >= 60) {
	  dauerFahrt = dauerFahrt - 60;
          stunde = stunde + 1;
	}
        else {
          minute = minute +dauerFahrt;
          dauerFahrt = dauerFahrt - dauerFahrt;
          if (minute>=60){
	    minute = minute - 60;
            stunde= stunde +1;
	  }
	}
      }
      // Eigentliches Einfuegen des Knoten
      four_tuple<int,int,int,int> fahrgastFahrtEnde (
						     endKnotenBeschriftung->ausgebenHaltestellenID2(), endKnotenBeschriftung->ausgebenLiniennummer(), stunde, minute );
     node endKnotenFahrgastfahrt = umlaufGraph->new_node(fahrgastFahrtEnde);           
     listeDerEndhaltestellenKnoten->append(endKnotenFahrgastfahrt);
     // Debug(1, "Einfuegen der Kante");
     // Setzen der Kanten zwischen den eingefuegten Knoten!
     // Einfuegen Kante zwischen Startknoten und Quelle.
     listeAnfang->append (umlaufGraph->new_edge(quelle, startKnotenFahrgastfahrt));
     // Einfuegen Kante zwischen Endknoten und Senke
     listeEnde->append (umlaufGraph->new_edge(endKnotenFahrgastfahrt, senke));
     // Einfuegen Kante zwischen eigentlicher FahrgastfahrtenKnoten
     listeFahr->append(umlaufGraph->new_edge(startKnotenFahrgastfahrt,endKnotenFahrgastfahrt));
    } 
   }
  
  (*startHaltestellenNachLinie)[aktuelleLinie]=tempListeStarthaltestellen;
      
  // Kanten zwischen verschiedenen Fahrgastfahrten EINER Linie einfuegen, dabei dann auch direkt 
  // Edge Arrays fuellen.
  Debug(2," Kanten einfuegen Vergleich End- und Startknoten.");
  int minute, stunde, fahrzeit, minuteStart, stundeStart;
  list<node> *tmpListe;
  node start;
  list<edge> *listeQuer = new list<edge>;
  // CFortschritt *fort = new CFortschritt;
  for (int i=0; i < listeDerEndhaltestellenKnoten->length();i++){
    node einfuegeEndknoten;
    four_tuple<int,int,int,int> knotenBeschriftung, startKnotenBeschriftung;
   
    einfuegeEndknoten=listeDerEndhaltestellenKnoten->contents(listeDerEndhaltestellenKnoten->get_item(i));
    
    knotenBeschriftung = umlaufGraph->inf (einfuegeEndknoten);
    tmpListe = (*startHaltestellenNachLinie)[knotenBeschriftung.second() * (-1)];

    if (tmpListe != 0) {
      for (int j=0; j < tmpListe->length(); j++)
      {
	  start = tmpListe->contents(tmpListe->get_item(j));
	  startKnotenBeschriftung = umlaufGraph->inf (start);
	  stunde = knotenBeschriftung.third ();
	  minute = knotenBeschriftung.fourth ();
	  stundeStart = startKnotenBeschriftung.third ();
	  minuteStart = startKnotenBeschriftung.fourth ();
	  
	  if ( (stundeStart > stunde) || ( (stundeStart == stunde) && (minuteStart >= minute)))
	  {
	      listeQuer->append (umlaufGraph->new_edge (einfuegeEndknoten, start));
	  }
      } // ende iteriere ueber alle listenelemente
    }
    
    tmpListe = (*startHaltestellenNachLinie)[knotenBeschriftung.second()];

    if (tmpListe != 0) {
      for (int j=0; j < tmpListe->length(); j++)
      {
	start = tmpListe->contents(tmpListe->get_item(j));
	startKnotenBeschriftung = umlaufGraph->inf (start);
	stunde = knotenBeschriftung.third ();
	minute = knotenBeschriftung.fourth ();
	stundeStart = startKnotenBeschriftung.third ();
	minuteStart = startKnotenBeschriftung.fourth ();
	fahrzeit = (*absoluteFahrtzeitLinienArray)[startKnotenBeschriftung.second ()];
	while (fahrzeit >= 60)
	  {
	    stunde++;
	    fahrzeit -= 60;
	  }
	minute += fahrzeit;
	if (minute >= 60)
	  {
	    stunde++;
	    minute -= 60;
	  }
	if ( (stundeStart > stunde) || ( (stundeStart == stunde) && (minuteStart >= minute)))
	  {
	    listeQuer->append (umlaufGraph->new_edge (einfuegeEndknoten, start));
	  }
      } // ende iteriere ueber alle listenelemente
    }
  } // ende for
  delete fort;

  Debug(1,"Fuellen der Arrays.");
  // Anlegen Edge Arrays fuer MinFlow Algo
  edge_array<int> lcap (*umlaufGraph);
  edge_array<int> ucap (*umlaufGraph);
  edge_array<int> cost (*umlaufGraph);
  
  // iterationen ueber alle querverbindungen
  edge quer;
  CFortschritt* fort1;
  fort1 = new CFortschritt ();
  Debug (1, "Quer-Verbindungen");
  int i=0;
  forall (quer, *listeQuer)
    {
      i++;
      if ( (i % 100) == 0)
	fort1->anzeigenFortschritt (i, listeQuer->length ());
      lcap[quer]=0;
      ucap[quer]=1;
      cost[quer]=0;
    }
  
  delete fort1;

  // senke-quelle - edege
  edge sQ = umlaufGraph->new_edge (senke, quelle);
  lcap[sQ]=0;
  ucap[sQ]=umlaufGraph->outdeg (quelle);
  cost[sQ]=0;

  fort1 = new CFortschritt ();
  // iteration ueber alle anfangsverbindungen
  edge anfang;
  i=0;
  Debug (1, "Anfang-Verbindungen");
  forall (anfang, *listeAnfang) {
    i++;
    if ( (i % 100) ==0) 
      fort1->anzeigenFortschritt (i, listeAnfang->length ());
    lcap[anfang]=0;
    ucap[anfang]=1;
    cost[anfang]=1;
  } // ende iteration ueber alle anfangsverbindungen
  delete fort1;

  // iteration ueber alle fahrgastverbindungen
  edge fahr;
  fort1 = new CFortschritt ();
  i = 0;
   Debug (1, "Fahrgast-Verbindungen");
  forall (fahr, *listeFahr) {
    i++;
    if ( (i% 100) == 0)
      fort1->anzeigenFortschritt (i+1, listeFahr->length ());
    lcap[fahr]=1;
    ucap[fahr]=1;
    cost[fahr]=0;
  } // ende iteration ueber alle fahrgastverbindungen
  delete fort1;

// iteration ueber alle endeverbindungen
  edge ende;
  fort1 = new CFortschritt ();
  i=0;
  Debug (1, "End-Verbindungen");

  forall (ende, *listeEnde) {
    i++;
    if ( (i% 100) == 0)
      fort1->anzeigenFortschritt (i+1, listeEnde->length ());
    lcap[ende]=0;
    ucap[ende]=1;
    cost[ende]=1;
  } // ende iteration ueber alle endeverbindungen
  delete fort1;

  // min cost aufruf
  edge_array<int> flow (*umlaufGraph);
  node_array<int> supply (*umlaufGraph);
  node currentNode;
  forall_nodes (currentNode, *umlaufGraph) 
  {
    supply[currentNode] = 0;
  }
  Debug (1, "Fluss Berechnung");
  bool gef = MIN_COST_FLOW(*umlaufGraph,lcap,ucap,cost,supply, flow);
  if (gef)
  {
//    std::cout << "Fluss: " << flow[sQ] << endl;
    this->fahrzeugAnzahl = flow[sQ];

  }
  else
  {
//    std::cout << "Kein Fluss gefunden " << endl;
  }

  // querkanten und deren fluss betrachten
  // iterationen ueber alle querverbindungen
  fort1 = new CFortschritt ();
  Debug (1, "Quer-Verbindungen und deren Fluss betrachten");
  i=0;
  forall (quer, *listeQuer)
    {
      i++;
      if ( (i % 100) == 0)
	fort1->anzeigenFortschritt (i, listeQuer->length ());
      
      if (flow[quer] > 0) {
	node source = umlaufGraph->source (quer);
	node dest = umlaufGraph->target (quer);
	four_tuple<int,int,int,int> infSource, infDest;
	infSource = umlaufGraph->inf (source);
	infDest = umlaufGraph->inf (dest);
	int diff = infDest.third () * 60 + infDest.fourth () - 
	           infSource.third () * 60 - infSource.fourth ();
	gesamtUmlaufzeit += diff;
      }
    }
  
  delete fort1;  

  this->fahrplanWirkungsgrad = (double) gesamtFahrzeitRein / (double) gesamtUmlaufzeit;

  fahrgastfahrten = listeFahr->length();

  delete listeQuer;
  delete listeAnfang;
  delete listeEnde;
  delete listeFahr;
  delete umlaufGraph;
  delete listeDerEndhaltestellenKnoten;
  delete tempListeStarthaltestellen;
}


 node CZusammenhangskomponenten::vergleichenAbfahrtszeiten(list<node>* listeAnfang ,node endknoten, bool& gefunden) {
   // Suchen des Fahrgast Startknotens
  
 if ((listeAnfang && endknoten) ){
   int endeMinute=0, endeStunde=0, fahrzeit=0;
   

   // Berechnen Endzeit + Rueckfahrzeit, so dass moeglicher wiedereinsatz des Busses bekannt ist.
   endeStunde=(this->umlaufGraph->inf(endknoten)).third();
   endeMinute=(this->umlaufGraph->inf(endknoten)).fourth();
   //cout<<"Davor: " << endeStunde << "," << endeMinute << endl;
   int temp=(this->umlaufGraph->inf(endknoten)).second();
   fahrzeit=(*absoluteFahrtzeitLinienArray)[temp];
   
   endeMinute += fahrzeit;
   while (endeMinute>= 60){
     endeMinute -= 60;
     endeStunde += 1;
   }//end while
   
    
   for (int i=0; i < listeAnfang->length();i++){
     node tempStartknoten;
     int startStunde=0, startMinute=0; 

     // Debug Ausgabe
     //cout<< "MOEP!!!!" << endl;
     //cout.flush();
     
     // Startwerte fuer Vergleich holen.
     tempStartknoten=listeAnfang->contents(listeAnfang->get_item(i));
     startStunde=(this->umlaufGraph->inf(tempStartknoten)).third();
     startMinute=(this->umlaufGraph->inf(tempStartknoten)).fourth();
      
     // Debug Ausgabe
     //cout<< "Start Stunde: " << startStunde << " : "; 
     //cout<< "Start Minute" << startMinute << endl;
   
     // Vergleich der Zeiten, schauen, ob es moeglich ist, von diesem 
     // Endknoten zum naechsten Startknoten zu fahren. 
     
     if ((startStunde >= endeStunde) && (startMinute >= endeMinute)){

       // Debug Ausgabe
       // cout<< "Hab einen gefunden...";
       // cout << " Werte Start" << startStunde << ":" << startMinute << "Werte Ende" << endeStunde << ":" << endeMinute << endl; 
       gefunden = true;
       return tempStartknoten; 
     }//ende for
  
     //Debug Ausgabe
     //else {cout << "Nein, Werte Start" << startStunde << ":" << startMinute << "Werte Ende" << endeStunde << ":" << endeMinute<<endl; }
   } 
   // cout<<"Danach: " << endeStunde << "," << endeMinute << endl;
  }
 
 else { gefunden = false;  return 0; }  

 gefunden = false;
 return 0;
  }
 

//------------------------------------------------------------------------------
// Funktionsname: sortierenNachLinie	
// Beschreibung:	 
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
int sortierenNachLinie(CFahrplanElement* const & eintrag1, CFahrplanElement* const & eintrag2){
  
  if (eintrag1->gebenLinie()<eintrag2->gebenLinie()) return -1;
  if (eintrag1->gebenLinie()>eintrag2->gebenLinie()) return 1;

  if (eintrag1->gebenLinie()==eintrag1->gebenLinie()){
    if  (eintrag1->gebenAbfahrtszeitStunde()<eintrag2->gebenAbfahrtszeitStunde()) return -1;
    if  (eintrag1->gebenAbfahrtszeitStunde()>eintrag2->gebenAbfahrtszeitStunde()) return 1;

      if (eintrag1->gebenAbfahrtszeitStunde()==eintrag2->gebenAbfahrtszeitStunde()){ 
	if (eintrag1->gebenAbfahrtszeitMinute()<eintrag2->gebenAbfahrtszeitMinute()) return -1;
	if (eintrag1->gebenAbfahrtszeitMinute()>eintrag2->gebenAbfahrtszeitMinute()) return 1;
      };
  };
  return 0;
}//Ende sortierenNachLinie

//------------------------------------------------------------------------------
// Funktionsname:	berchnenBesteLoesung
// Beschreibung:	Diese Methode wendet das Entscheidungsbaumverfahren auf der 
//                      Zusammenhangskomponente an
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CZusammenhangskomponenten::berechnenBesteLoesung(int baumtiefe){
    Debug(1,"Einstieg in die Methode berechneBesteLoesung");
    
    // Initialisieren und Definieren der Datentypen
    int zaehler = 0;
    edge kante;
    CKantenBeschriftung* kantenInfo;
    this->besteWartezeit = 0;
    this->kantenlisteDesGraphen = new array<edge>(this->graphDerZusammenhangskomponente->number_of_edges());
    this->feldDerBestenLoesung = new array<int>(kantenlisteDesGraphen->size());
    this->maximaleGeruestGroesse = (this->graphDerZusammenhangskomponente->number_of_edges() - (this->zyklomatischeZahl));
    array<int>* aktAst = new array<int>(kantenlisteDesGraphen->size());

    // Fuellen des Feldes kantenlisteDesGraphen
    zaehler = 0;
    forall_edges(kante,*(this->graphDerZusammenhangskomponente)){
	this->kantenlisteDesGraphen->set(zaehler,kante);
	zaehler++;
    }

    // initialisieren des Atributes mussInsMaximalgeruest der Graphkanten
    this->initialisierenMussInsMaximalgeruest();

    // Erzeugen einer Ausgangsloesung mit Hilfe der heuristischen Loesung
    this->berechnenMaximalgeruest();
    this->berechnenSimplexTableau();
    this->berechnenWartezeitResultate();
    this->besteWartezeit = this->wartezeitAufwand;
    Debug(5,"Wartezeit der Ausgangsloesung: " << this->besteWartezeit);
    Debug(5,"Belegen des Arrays der besten Loesung und initialisieren des Aktuellen Astes");
    for(int i=0;i<this->kantenlisteDesGraphen->size();i++){
	kantenInfo = this->graphDerZusammenhangskomponente->inf(this->kantenlisteDesGraphen->get(i));
	if (kantenInfo->gebenImMaximalgeruest()) {
	    this->feldDerBestenLoesung->set(i,1);
	} else {
	    this->feldDerBestenLoesung->set(i,-1);
	}
	aktAst->set(i,0);
    }
    
    // Beginn des Entscheidungsbaum
    this->aktuelleGeruestGroesse = 0;
    this->aktuelleNichtGeruestGroesse = 0;
    berechnenEntscheidungsbaum(0,aktAst,baumtiefe);
    
    // Auswaehlern der besten Loesung
    Debug(5,"Auswahl der besten Loesung");
    for (int i=0;i<this->kantenlisteDesGraphen->size();i++) {
	    kantenInfo = this->graphDerZusammenhangskomponente->inf(this->kantenlisteDesGraphen->get(i));
	if (this->feldDerBestenLoesung->get(i) == 1) {
	    kantenInfo->setzenImMaximalgeruest(true); 
	} else if (this->feldDerBestenLoesung->get(i) == -1) {
	    kantenInfo->setzenImMaximalgeruest(false); 
	} else {
	    Debug(2,"Fehler im setzen der besten Loesung!");
	    kantenInfo->setzenImMaximalgeruest(false); 
	}
    }

    // Berechnen der besten Loesung;
    Debug(5," Berechnen der besten Loesung");
    this->berechnenSimplexTableau();
    this->berechnenWartezeitResultate();
    this->besteWartezeit = this->wartezeitAufwand;
    Debug(5,"Wartezeitaufwand = "<< this->wartezeitAufwand);

    // speicher Freigeben
    delete this->kantenlisteDesGraphen;
    delete this->feldDerBestenLoesung;
    delete aktAst;

} // ende berchnenBesteLoesung

//------------------------------------------------------------------------------
// Funktionsname:       initialisierenMussInsMaximalgeruest
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CZusammenhangskomponenten::initialisierenMussInsMaximalgeruest(){
  edge kante;
  CKantenBeschriftung* kantenInfo;
  forall_edges(kante,*(this->graphDerZusammenhangskomponente)){
    kantenInfo = this->graphDerZusammenhangskomponente->inf(kante);
    kantenInfo->setzenMussInsMaximalgeruest(false);
    kantenInfo->setzenNichtImMaximalgeruest(false);
  }
}

//------------------------------------------------------------------------------
// Funktionsname:	berchnenEntscheidungsbaum
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CZusammenhangskomponenten::berechnenEntscheidungsbaum(int aktKantenNummer,array<int>* aktAst,int baumtiefe){
  
  //  Debug(5,"Berechne Entscheidungsbaum. KantenNummer: " << aktKantenNummer);
  if ((aktKantenNummer > baumtiefe) && (baumtiefe > 0)) return;
  if (aktKantenNummer < this->graphDerZusammenhangskomponente->number_of_edges()){
    
    edge kante = this->kantenlisteDesGraphen->get(aktKantenNummer);
    CKantenBeschriftung* kantenInfo = this->graphDerZusammenhangskomponente->inf(kante);
    int knotenwartezeit = 0;
    
    // Wenn Geruest noch nicht voll: Kante ins Geruest aufnehmen
    if (this->aktuelleGeruestGroesse < this->maximaleGeruestGroesse) {
      //      Debug(5,"Kante reinnehmen " << aktKantenNummer);
      this->aktuelleGeruestGroesse++;
      kantenInfo->setzenMussInsMaximalgeruest(true);
      aktAst->set(aktKantenNummer,1);
      
      // Geruest berechnen
      try {
	//	Debug(6,"Geruest berechnen");
	this->berechnenMaximalgeruest();
	if (this->listeMaximalgeruest->size() == this->maximaleGeruestGroesse) {
	  this->berechnenSimplexTableau();
	  
	  // Wartezeitaufwand berechnen (Fuer Kante und Gesamt)
	  this->berechnenWartezeitResultate();
	  
	  // Wartezeit vergleichen
	  CKantenBeschriftung* wartezeitkantenInfo; 
	  knotenwartezeit = 0;
	  for (int i=0;i<=aktKantenNummer;i++){
	    if (aktAst->get(i) == 1) {
	      wartezeitkantenInfo = this->graphDerZusammenhangskomponente->inf(this->kantenlisteDesGraphen->get(i));
	      knotenwartezeit += (wartezeitkantenInfo->gebenNetzbedingteWartezeit() * wartezeitkantenInfo->abfragenVerkehrsstrom());
	    }
	  }
	  if (knotenwartezeit < this->besteWartezeit) {
	    // wenn wartezeit an der Kante > beste Wartezeit ist, dann verlasse den Ast
	    if (this->besteWartezeit > this->wartezeitAufwand) {
	      this->besteWartezeit = this->wartezeitAufwand;
	      for(int i=0;i<this->kantenlisteDesGraphen->size();i++){
		kantenInfo = this->graphDerZusammenhangskomponente->inf(this->kantenlisteDesGraphen->get(i));
		if (kantenInfo->gebenImMaximalgeruest()) {
		  this->feldDerBestenLoesung->set(i,1);
		} else {
		  this->feldDerBestenLoesung->set(i,-1);
		} // if .. else
	      } // for
	    } // if
	    this->berechnenEntscheidungsbaum(aktKantenNummer + 1,aktAst, baumtiefe);
	  } // if
	} else {
	  Debug(5,"Ast uebersprungen!");
	}
      } catch (...){}
      kantenInfo->setzenMussInsMaximalgeruest(false);
      this->aktuelleGeruestGroesse--;
      aktAst->set(aktKantenNummer,0);
    }
    
    // Wenn NichtGeruest noch nicht voll: Kante nicht ins Geruest aufnehmen
    if (this->aktuelleNichtGeruestGroesse < this->zyklomatischeZahl) {
      //      Debug(5,"Kante Rausnehmen " << aktKantenNummer);
      this->aktuelleNichtGeruestGroesse++;
      kantenInfo->setzenNichtImMaximalgeruest(true);
      aktAst->set(aktKantenNummer,-1);
      
      // Geruest berechnen
      try {
	//	Debug(6,"Geruest berechnen");
	this->berechnenMaximalgeruest();
	if (this->listeMaximalgeruest->size() == this->maximaleGeruestGroesse) {
	  this->berechnenSimplexTableau();
	  
	  // Wartezeitaufwand berechnen (Fuer Kante und Gesamt)
	  this->berechnenWartezeitResultate();
	  
	  // Wartezeit vergleichen
	  if (this->besteWartezeit > this->wartezeitAufwand) {
	    this->besteWartezeit = this->wartezeitAufwand;
	    for(int i=0;i<this->kantenlisteDesGraphen->size();i++){
	      kantenInfo = this->graphDerZusammenhangskomponente->inf(this->kantenlisteDesGraphen->get(i));
	      if (kantenInfo->gebenImMaximalgeruest()) {
		this->feldDerBestenLoesung->set(i,1);
	      } else {
		this->feldDerBestenLoesung->set(i,-1);
	      }
	    }
	  }
	  this->berechnenEntscheidungsbaum(aktKantenNummer + 1,aktAst,baumtiefe);
	}
      } catch(...){}
      kantenInfo->setzenNichtImMaximalgeruest(false);
      this->aktuelleNichtGeruestGroesse--;
      aktAst->set(aktKantenNummer,0);
    }
  }
}

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CZusammenhangskomponenten::ZeichnenZusammenhangskomponente ()
{
  node currentNode, node1, node2;
  edge currentEdge;
  CKnotenBeschriftung *information, *information1, *information2;
  CKantenBeschriftung *informationEdge;
  //  h_array<int, node> nodes (0, graphDerZusammenhangskomponente->
  //		       number_of_nodes ());
  
 h_array<int, node> nodes;

  GRAPH<CKnotenBeschriftung, CKantenBeschriftung> graphTmp;
  forall_nodes (currentNode, *graphDerZusammenhangskomponente) {
    information = graphDerZusammenhangskomponente->inf (currentNode);
    nodes[information->gebenKnotenID()] = graphTmp.new_node (*information);
  }

  forall_edges (currentEdge, *graphDerZusammenhangskomponente) {
    node1 = graphDerZusammenhangskomponente->source (currentEdge);
    information1 = graphDerZusammenhangskomponente->inf (node1);
    node2 = graphDerZusammenhangskomponente->target (currentEdge);
    information2 = graphDerZusammenhangskomponente->inf (node2);
    informationEdge = graphDerZusammenhangskomponente->inf (currentEdge);
    graphTmp.new_edge (nodes[information1->gebenKnotenID()],
		       nodes[information2->gebenKnotenID()],
		       *informationEdge);
  }
  
  CVisualFahrplan visual (&graphTmp);
  visual.Zeichnen (550, 800);
}

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
h_array<three_tuple<int,int,int>*,int>* CZusammenhangskomponenten::gebenListeNetzbedingtewartezeit() {
  h_array<three_tuple<int,int,int>*,int>* wartezeitListe = new h_array<three_tuple<int,int,int>*,int>();
  edge kante;
  
  forall_edges(kante,*(this->graphDerZusammenhangskomponente)){
    CKantenBeschriftung* kantenInfo = this->graphDerZusammenhangskomponente->inf(kante);
    if (kantenInfo->gebenNetzbedingteWartezeit() != 0) {
      CKnotenBeschriftung* knotenInfo1 = this->graphDerZusammenhangskomponente->
	inf(this->graphDerZusammenhangskomponente->source(kante));
      CKnotenBeschriftung* knotenInfo2 = this->graphDerZusammenhangskomponente->
	inf(this->graphDerZusammenhangskomponente->target(kante));
      three_tuple<int,int,int>* hash1 = new three_tuple<int,int,int>(knotenInfo1->ausgebenLiniennummer(),
								     knotenInfo1->ausgebenHaltestellenID2(),
								     knotenInfo2->ausgebenLiniennummer());
      int hash2 = kantenInfo->gebenNetzbedingteWartezeit();
      (*wartezeitListe)[hash1] = hash2;
    }
  }

  return wartezeitListe;
}

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CZusammenhangskomponenten::fuellenReisebewertung(map2<int,int,CReisebewertung>& reisebewertung) {

    Debug(1,"fuellen Reisebewertung begin");

    // Pruefen, ob reisebewertung existiert.
    if (&reisebewertung == 0) {
	reisebewertung = *(new CReisebewertung());
    }

    h_array<int,list<node> > abfahrtshaltestelle, ankunftshaltestelle; // Mapping der Haltestellen.
    node knoten;
    node zielKnoten, startKnoten, endKnoten;
    CKnotenBeschriftung* knotenInfo, *zielKnotenInfo, *knotenInfoQ, *knotenInfoZ;
    edge kante;
    CKantenBeschriftung* kantenInfo;
    int zaehler = 0;
    int startHaltestelle, endHaltestelle;
    double neueReisezeit,alteReisezeit;
    map2<int,int,four_tuple<node,node,int,list<edge>*>*>* kuerzesterWeg = new map2<int,int,four_tuple<node,node,int,list<edge>*>*>();
    list<edge>* kantenListe;
    // reiseweg: int=Dauer; list<edge>=Kantenliste des Weges
    four_tuple<node,node,int,list<edge>*>* reiseweg;

    // Aufbauen der Hash_Arrays fuer die Haltestellen. Im ersten Array stehen alle Haltestellen - Knoten Kombinationen,
    // wo die Haltestelle eine Abfahrtshaltestelle ist, im zweiten wo die Haltestelle eine Ankunftshaltestelle ist
    zaehler = 0;
    forall_nodes(knoten,*(this->graphDerZusammenhangskomponente)){
	knotenInfo = this->graphDerZusammenhangskomponente->inf(knoten);
	int haltestelle1 = knotenInfo->ausgebenHaltestellenID1();
	int haltestelle2 = knotenInfo->ausgebenHaltestellenID2();
	if (abfahrtshaltestelle.defined(haltestelle1)) {
	    (abfahrtshaltestelle[haltestelle1]).append(knoten);
	} else {
	    abfahrtshaltestelle[haltestelle1] = list<node>();
	    (abfahrtshaltestelle[haltestelle1]).append(knoten);
	}
	if (ankunftshaltestelle.defined(haltestelle2)) {
	    (ankunftshaltestelle[haltestelle2]).append(knoten);
	} else {
	    ankunftshaltestelle[haltestelle2] = list<node>();
	    (ankunftshaltestelle[haltestelle2]).append(knoten);
	}
    } // Haltestellen Mapping aufgebaut

    // Dijkstra Kostenarrays fuellen

    edge_array<double> kostenArray(*(this->graphDerZusammenhangskomponente),0); // Dijkstra

    forall_edges(kante,*(this->graphDerZusammenhangskomponente)){
	kantenInfo = this->graphDerZusammenhangskomponente->inf(kante);
	knotenInfoQ = this->graphDerZusammenhangskomponente->inf(this->graphDerZusammenhangskomponente->source(kante));
	knotenInfoZ = this->graphDerZusammenhangskomponente->inf(this->graphDerZusammenhangskomponente->target(kante));
	int kostenWert = 0;
	kostenWert += kantenInfo->gebenWartezeit();
	kostenWert += knotenInfoQ->ausgebenFahrzeit();
	kostenWert += knotenInfoZ->ausgebenFahrzeit();
	kostenArray[kante] = kostenWert;
    }

    // Berechnen des Dijkstra fuer alle Knoten zu jedem Knoten
    Debug(5,"Dijkstra berechnen");
    CFortschritt *fort1 = new CFortschritt("Suche alle Reisen");
    for (int i=0;i<abfahrtshaltestelle.size();i++){
	list<node> abfahrtsknotenListe = abfahrtshaltestelle[i];
	forall(knoten,abfahrtsknotenListe){
	    knotenInfo = this->graphDerZusammenhangskomponente->inf(knoten);

	    // Dijkstra berechnen
	    node_array<edge> ergebnisArray(*(this->graphDerZusammenhangskomponente),0); // Dijkstra
	    node_array<double> entfernungsArray(*(this->graphDerZusammenhangskomponente),0); // Dijkstra
	    DIJKSTRA_T(*(this->graphDerZusammenhangskomponente),knoten,kostenArray,entfernungsArray,ergebnisArray);

	    // Weg speichern
	    startHaltestelle = knotenInfo->ausgebenHaltestellenID1();
	    forall_nodes(zielKnoten,*(this->graphDerZusammenhangskomponente)){ // Fuer alle ZielKnoten
		neueReisezeit = entfernungsArray[zielKnoten]; // aktuelle Reisezeit 
		zielKnotenInfo = this->graphDerZusammenhangskomponente->inf(zielKnoten); // ZielKnotenInfo holen
		endHaltestelle = zielKnotenInfo->ausgebenHaltestellenID2(); // endHaltestelle bestimmen
		if ((knoten != zielKnoten) && (neueReisezeit > 0)) { // Wenn Knoten unterschiedlich ...
		    if (startHaltestelle != endHaltestelle){ // Wenn Haltestellen unterschiedlich, waehle kuerzesten Weg
			if (kuerzesterWeg->defined(startHaltestelle,endHaltestelle)) { // Wenn schon ein kuezesterWeg von a nach b vorhanden ist ...
			    alteReisezeit = (*kuerzesterWeg)(startHaltestelle,endHaltestelle)->third();
			    if (neueReisezeit < alteReisezeit) { // wenn die neue Reise kuerzer ist, nimm diese
				startKnoten = knoten; // Benenne Knoten um
				endKnoten = zielKnoten;
				kantenListe = new list<edge>();
				while (startKnoten != endKnoten){
				    kante = ergebnisArray[endKnoten];
				    if (kante == 0) {
					endKnoten = startKnoten;
					break;
				    }
				    kantenListe->push(kante);
				    endKnoten = this->graphDerZusammenhangskomponente->source(kante);
				}
				reiseweg = new four_tuple<node,node,int,list<edge>*>(knoten,zielKnoten,neueReisezeit,kantenListe);
				(*kuerzesterWeg)(startHaltestelle,endHaltestelle) = reiseweg;
			    } else { // sonst tue nichts
			    }
			} else { // ansonsten ...
			    startKnoten = knoten; // Benenne Knoten um
			    endKnoten = zielKnoten;
			    kantenListe = new list<edge>();
			    while (startKnoten != endKnoten){
				kante = ergebnisArray[endKnoten];
				if (kante == 0) {
				    endKnoten = startKnoten;
				    break;
				}
				kantenListe->push(kante);
				endKnoten = this->graphDerZusammenhangskomponente->source(kante);
			    }
			    reiseweg = new four_tuple<node,node,int,list<edge>*>(knoten,zielKnoten,neueReisezeit,kantenListe);
			    (*kuerzesterWeg)(startHaltestelle,endHaltestelle) = reiseweg;
			} // end: else
		    }// end: if (startHaltestelle != endHaltestelle){ // Wenn Haltestellen unterschiedlich, waehle kuerzesten Weg
		} else if (knoten == zielKnoten) { 
		    // Wenn Knoten gleich sind, die benachbarten Reisen ueberpruefen.
		    if (kuerzesterWeg->defined(startHaltestelle,endHaltestelle)) { // Wenn schon ein kuezesterWeg von a nach b vorhanden ist ...
			// Wenn es schon einen kuerzesten Weg gibt ...
			if (((*kuerzesterWeg)(startHaltestelle,endHaltestelle))->third() > zielKnotenInfo->ausgebenFahrzeit()) {
			    reiseweg = new four_tuple<node,node,int,list<edge>*>(knoten,knoten,zielKnotenInfo->ausgebenFahrzeit(),0);
			    (*kuerzesterWeg)(startHaltestelle,endHaltestelle) = reiseweg;
			}
		    } else {
			// ansonsten ...
			reiseweg = new four_tuple<node,node,int,list<edge>*>(knoten,knoten,zielKnotenInfo->ausgebenFahrzeit(),0);
			(*kuerzesterWeg)(startHaltestelle,endHaltestelle) = reiseweg;
		    } // end: if ((*kuerzesterWeg)(startHaltestelle,endHaltestelle) > zielKnotenInfo->ausgebenFahrzeit()) {} else {}
		} // end: if (knoten != zielKnoten) {} else {}
	    }
	}
	fort1->anzeigenFortschritt(i, abfahrtshaltestelle.size()-1);
    }
    int vonHaltestelle, nachHaltestelle;
    delete fort1;
    fort1 = new CFortschritt("Bewerte alle Reisen");
    zaehler = 0;
    fort1->anzeigenFortschritt (zaehler, abfahrtshaltestelle.size());
    CReisebewertung *bewertung;
    forall_defined(vonHaltestelle,abfahrtshaltestelle){
	zaehler++;
	fort1->anzeigenFortschritt (zaehler, abfahrtshaltestelle.size());
	forall_defined(nachHaltestelle,ankunftshaltestelle){
	    if (vonHaltestelle != nachHaltestelle) {
//		cout << endl << "von: " << vonHaltestelle << " nach: " << nachHaltestelle << endl;
//		cout.flush();

		if (kuerzesterWeg->defined(vonHaltestelle,nachHaltestelle)) {
//		    cout << "bewertung: " << bewertung << endl;
//		    cout << "kuerzesterWeg: " << (*kuerzesterWeg)(vonHaltestelle,nachHaltestelle) << endl;
//		    cout.flush();
		    reisebewertung(vonHaltestelle,nachHaltestelle) = bewerteReise((*kuerzesterWeg)(vonHaltestelle,nachHaltestelle));	
		} else
		    Debug(5,"Kein kuerzester Weg gefunden!");
	    }
	}
    }
    delete fort1;
}

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
CReisebewertung CZusammenhangskomponenten::bewerteReise (four_tuple<node,node,int,list<edge>*>* reise) {
//    Debug(5,"Bewerte Reise");

    CReisebewertung bewertung;
    int fahrzeit, wartezeit, umstiege;
    list<three_tuple<int,int,int> > reiseweg;
    node startKnoten, endKnoten, zwischenKnoten;
    CKnotenBeschriftung *startKnotenInfo, *endKnotenInfo, *zwischenKnotenInfo;
    edge kante;
    CKantenBeschriftung *kantenInfo;
    list<edge>* kantenListe = reise->fourth();

    startKnoten = reise->first();
    endKnoten = reise->second();
    startKnotenInfo = this->graphDerZusammenhangskomponente->inf(startKnoten);
    endKnotenInfo = this->graphDerZusammenhangskomponente->inf(endKnoten);

    reiseweg.clear();
    fahrzeit = 0;
    wartezeit = 0;
    umstiege = 0;
    if (startKnoten == endKnoten) {
	// Knoten sind gleich
	fahrzeit = startKnotenInfo->ausgebenFahrzeit();
	wartezeit = 0;
	umstiege = 0;
	reiseweg.append(*(new three_tuple<int,int,int>(0,
						       startKnotenInfo->ausgebenHaltestellenID1(),
						       startKnotenInfo->ausgebenLiniennummer())));
	reiseweg.append(*(new three_tuple<int,int,int>(startKnotenInfo->ausgebenLiniennummer(),
						       startKnotenInfo->ausgebenHaltestellenID2(),
						       0)));
    } else {
	// Knoten sind ungleich
	fahrzeit = startKnotenInfo->ausgebenFahrzeit();
	wartezeit = 0;
	int liniennummer = startKnotenInfo->ausgebenLiniennummer();
	reiseweg.append(three_tuple<int,int,int>(0,
						 startKnotenInfo->ausgebenHaltestellenID1(),
						 liniennummer));
	int listenGroesse = kantenListe->size();
	for (int zaehler=0;zaehler < listenGroesse;zaehler++){
	    kante = kantenListe->pop();
	    kantenInfo = this->graphDerZusammenhangskomponente->inf(kante);
	    zwischenKnoten = this->graphDerZusammenhangskomponente->target(kante);
	    zwischenKnotenInfo = this->graphDerZusammenhangskomponente->inf(zwischenKnoten);

	    // Wartezeit aufrechnen
	    wartezeit += kantenInfo->gebenWartezeit();

	    // Umstiege aufrechnen
	    if (liniennummer != zwischenKnotenInfo->ausgebenLiniennummer()){ 
		umstiege++;
	    }

	    // Fahrzeit aufrechnen
	    fahrzeit += zwischenKnotenInfo->ausgebenFahrzeit(); 

	    // Reiseweg fuellen
	    if (zwischenKnoten == endKnoten){ // Wenn ende erreicht
		reiseweg.append(*(new three_tuple<int,int,int>(liniennummer,
							       zwischenKnotenInfo->ausgebenHaltestellenID1(),
							       zwischenKnotenInfo->ausgebenLiniennummer())));
		reiseweg.append(*(new three_tuple<int,int,int>(zwischenKnotenInfo->ausgebenLiniennummer(),
							       zwischenKnotenInfo->ausgebenHaltestellenID2(),
							       0)));
	
	    } else {
		reiseweg.append(*(new three_tuple<int,int,int>(liniennummer,
							       zwischenKnotenInfo->ausgebenHaltestellenID1(),
							       zwischenKnotenInfo->ausgebenLiniennummer())));
	    }

	    liniennummer = zwischenKnotenInfo->ausgebenLiniennummer();
	}
    }
    // Bewertung fuellen und terminieren der Funktion
    bewertung.setzenFahrzeit(fahrzeit);
    bewertung.setzenWartezeit(wartezeit);
    bewertung.setzenAnzahlDerUmstiege(umstiege);
    bewertung.setzenReiseweg(reiseweg);
//    cout << "fahrzeit: " << fahrzeit << "; wartezeit: " << wartezeit << "; umstiege: " << umstiege << endl;
//    cout.flush();
    return bewertung;
}

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CZusammenhangskomponenten::InitMindestHaltezeit (int haltezeit)
{
  edge currentEdge;
  CKantenBeschriftung* informationEdge;
  
  // iteration ueber alle kanten
  forall_edges (currentEdge, *(this->graphDerZusammenhangskomponente)) {
    informationEdge = this->graphDerZusammenhangskomponente->inf (currentEdge);
    informationEdge->setzenMindestHaltezeit (haltezeit);
  }
} // InitMindestHaltezeit


//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CZusammenhangskomponenten::InitMindestUmsteigezeit (int haltezeit)
{
  edge currentEdge;
  CKantenBeschriftung* informationEdge;

  // iteration ueber alle kanten
  forall_edges (currentEdge, *(this->graphDerZusammenhangskomponente)) {
    informationEdge = this->graphDerZusammenhangskomponente->inf (currentEdge);
    informationEdge->setzenMindestUmsteigezeit (haltezeit);
  }
} // InitMindestHaltezeit

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CZusammenhangskomponenten::sortierenFahrplanElemente ()
{
  h_array<int, h_array<int, list<CFahrplanElement* > * >* > *sortList = 
    new h_array<int, h_array<int, list<CFahrplanElement*> * >* >;
  CFahrplanElement *current;
  int haltestelle, liniennr;
  h_array<int, list <CFahrplanElement* >* >* haltestellenHash;
  list<CFahrplanElement*> *listFahrplan;
  h_array<int, list<int>* > *listeHaltestellen = new h_array<int, list<int> *>;
  list<CFahrplanElement*>* newFahrplan = new list<CFahrplanElement*>;
  list<int> *haltestellenfolge;

  Debug (1, "Init");

  // init
  for(int i=0;i< this->linienListe->length();i++){  
    liniennr = this->linienListe->contents(this->linienListe->get_item(i));
    (*listeHaltestellen)[liniennr] = new list<int>;
    this->berechnenHaltestellenFolge (liniennr, (*listeHaltestellen)[liniennr]);
    haltestellenHash = new h_array<int, list <CFahrplanElement*> *>;
    (*sortList)[liniennr] = haltestellenHash;
  } // ende init
    

  Debug (2, "Iteration ueber alle elemente");

  // iteration ueber alle elemente
  forall (current, *fahrplan)
  {
    liniennr = current->gebenLinie ();
    haltestelle = current->gebenHaltestelle ();
    haltestellenHash = (*sortList)[liniennr];
    if ( (*haltestellenHash).defined (haltestelle)) {
      listFahrplan = (*haltestellenHash)[haltestelle];
    }
    else {
      listFahrplan = new list<CFahrplanElement *>;
      (*haltestellenHash)[haltestelle] = listFahrplan;
    }
    listFahrplan->append (current);
  } // ende iteration ueber alle elemente

  Debug (2, "Sortieren der Listen");

  // sortieren der listen nach zeit
  this->sortierenFahrplanListen (sortList);

  Debug (2, "Fuellen des neuen Fahrplan");
  
  list<CFahrplanElement*> *currentList;
  CFahrplanElement *currentElement;
  // fuellen pro linie
  for(int i=0;i< this->linienListe->length();i++){  
    liniennr = this->linienListe->contents(this->linienListe->get_item(i));
    haltestellenfolge = (*listeHaltestellen)[liniennr];
    // iteration ueber alle haltestellen
    for (int j=0; j < haltestellenfolge->length ()-1; j++)
    {
      haltestelle = haltestellenfolge->contents (haltestellenfolge->get_item (j));
      currentList = (*(*sortList)[liniennr])[haltestelle];
      for(int k=0;k < currentList->length(); k++) {  
	currentElement = currentList->contents(currentList->get_item(k));
	newFahrplan->append (currentElement);
      }
    } // ende iteration ueber alle haltestellen
  } // ende fuellen pro linie
  
  // loeschen der erzeugten objekte
  forall (haltestellenfolge, *listeHaltestellen) {
    delete haltestellenfolge;
  }
  delete listeHaltestellen;

  int linie;
  // iteration ueber alle linien
  forall_defined (linie, *sortList)
  {
    h_array<int, list<CFahrplanElement*> *> *haltestellenHash = (*sortList)[linie];
    // iteration ueber alle haltestellen
    forall_defined (haltestelle, *haltestellenHash)
    {
      currentList = (*haltestellenHash)[haltestelle];
      delete currentList;
    } // ende iteration ueber alle haltestellen
    delete haltestellenHash;
  } // ende iteration ueber alle linien
  delete sortList;

  delete fahrplan;

  fahrplan = newFahrplan;

} // ende sortierenFahrplanElemente

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CZusammenhangskomponenten::sortierenFahrplanListen (h_array<int, h_array<int, list<CFahrplanElement* > * >* > * sortList)
{
  list<CFahrplanElement*> *currentList;
  int linie, haltestelle;

  // iteration ueber alle linien
  forall_defined (linie, *sortList)
  {
    h_array<int, list<CFahrplanElement*> *> *haltestellenHash = (*sortList)[linie];
    // iteration ueber alle haltestellen
    forall_defined (haltestelle, *haltestellenHash)
    {
      currentList = (*haltestellenHash)[haltestelle];
      currentList->sort (&sortierenNachFahrzeit, 1);
    } // ende iteration ueber alle haltestellen
  } // ende iteration ueber alle linien
} // ende sortierenFahrplanListen

//------------------------------------------------------------------------------
// Funktionsname: sortierenNachFahrzeit	
// Beschreibung:	 
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
int sortierenNachFahrzeit(CFahrplanElement* const & eintrag1, CFahrplanElement* const & eintrag2){
  
  if ( (eintrag1->gebenAbfahrtszeitStunde() > eintrag2->gebenAbfahrtszeitStunde()) ||
       ( (eintrag1->gebenAbfahrtszeitStunde() == eintrag2->gebenAbfahrtszeitStunde()) &&
	 (eintrag1->gebenAbfahrtszeitMinute() > eintrag2->gebenAbfahrtszeitMinute()))) return 1;

  if ( (eintrag2->gebenAbfahrtszeitStunde() > eintrag1->gebenAbfahrtszeitStunde()) ||
       ( (eintrag2->gebenAbfahrtszeitStunde() == eintrag1->gebenAbfahrtszeitStunde()) &&
	 (eintrag2->gebenAbfahrtszeitMinute() > eintrag1->gebenAbfahrtszeitMinute()))) return -1;

  return 0;
}//Ende sortierenNachFahrzeit

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CZusammenhangskomponenten::berechnenHaltestellenFolge (int linie, list<int> *liste)
{
  node current;
  CKnotenBeschriftung *information;
  bool found = false;

  current = (*startHaltestellenArray)[linie];
  information = this->graphDerZusammenhangskomponente -> inf (current);
  liste->append (information->ausgebenHaltestellenID1());
  liste->append (information->ausgebenHaltestellenID2());
  
  while (! found)
  {
    list<edge> Kantenliste = this->graphDerZusammenhangskomponente->out_edges(current);
    edge Kante;
    forall (Kante, Kantenliste)
    {
      node nachfolgerKnoten = this->graphDerZusammenhangskomponente->target(Kante);
      CKnotenBeschriftung* nachfolgerKnotenInformation = this->graphDerZusammenhangskomponente->inf(nachfolgerKnoten);
      if (nachfolgerKnotenInformation->ausgebenLiniennummer() == linie){
	  current = nachfolgerKnoten;
	  liste->append (nachfolgerKnotenInformation->ausgebenHaltestellenID2());
	  found = true;
	  break;
      }
    }
    if (found == true) found = false;
    else found = true;
  }
} // ende berechnenHaltestellenFolge

int CZusammenhangskomponenten::gebenRelativeGesamtFahrzeit(){
  return this->relativeGesamtFahrtzeit;
}

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
node CZusammenhangskomponenten::SuchenEndKnoten (node startKnoten)
{
  node current;
  CKnotenBeschriftung *information;
  bool found = false;
  int linie;

  information = this->graphDerZusammenhangskomponente -> inf (startKnoten);
  linie = information->ausgebenLiniennummer ();

  current = startKnoten;
  while (! found)
  {
    list<edge> Kantenliste = this->graphDerZusammenhangskomponente->out_edges(current);
    edge Kante;
    forall (Kante, Kantenliste)
    {
      node nachfolgerKnoten = this->graphDerZusammenhangskomponente->target(Kante);
      CKnotenBeschriftung* nachfolgerKnotenInformation = this->graphDerZusammenhangskomponente->inf(nachfolgerKnoten);
      if (nachfolgerKnotenInformation->ausgebenLiniennummer() == linie){
	  current = nachfolgerKnoten;
	  found = true;
	  break;
      }
    }
    if (found == true) found = false;
    else found = true;
  }

  return current;
} // ende SuchenEndKnoten

int CZusammenhangskomponenten::gebenFahrgastfahrten(){
  return this->fahrgastfahrten;
}

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
double CZusammenhangskomponenten::gebenFahrplanWirkungsgrad ()
{
  return this->fahrplanWirkungsgrad;
}

//------------------------------------------------------------------------------
// Funktionsname:	
// Beschreibung:	
// Parameter:		
// Rueckgabe:		
// Besonderheiten:	
//------------------------------------------------------------------------------
void CZusammenhangskomponenten::setzenStoppenFlag ()
{
  if (this->simplexTableau != 0)
  {
    this->simplexTableau->setzenStoppenFlag ();
  }
}
