/* Generated by Together */

#ifndef CZUSAMMENHANGSKOMPONENTEN_H
#define CZUSAMMENHANGSKOMPONENTEN_H
#include "CKantenBeschriftung.h"
#include "CKnotenBeschriftung.h"
#include "CSimplexTableau.h"
#include "CFahrplanElement.h"
#include <iostream.h>
#include "LEDA/graph.h"
#include "LEDA/graph_alg.h"
#include "LEDA/tuple.h"
#include "LEDA/h_array.h"
#include "LEDA/list.h"
#include "LEDA/node_partition.h"
#include "LEDA/string.h"
#include "LEDA/map2.h"
#include "CVergleichsklasse.h"
#include <math.h>
#include "Debug.h"
#include "Fehler.h"
#include "CVisualFahrplan.h"
#include "CReisebewertung.h"
#include "LEDA/templates/shortest_path.t"
#include "CFortschritt.h"

/**
 * In dieser Klasse beschreibt eine Zusammenhangskomponente und gibt alle Methoden und Attribute an,
 * die zur Berechnung eines Fahrplans dienen.
 */
class CZusammenhangskomponenten {
public:
    /**
     *  Der Konstruktor erhaelt einen int Wert, dieser ist die ID der Zusammenhangskomponente.
     */
    CZusammenhangskomponenten(int);

    /**
     * Destruktor
     */
    ~CZusammenhangskomponenten ();

    /**
     * Initialisieren der Mindesthaltezeit auf allen Kanten
     */
    void InitMindestHaltezeit (int);

    /**
     * setzen des Stoppen-Flags
     */
    void setzenStoppenFlag ();

    /**
     * Initialisieren der Mindestumsteigezeit auf allen Kanten
     */
    void InitMindestUmsteigezeit (int);

    /**
     * Diese Methode gibt als Liste alle Haltestellen der Zusammenhangskomponente zurueck.
     * Um eine Rekonstruktion zu ermoeglichen, sieht ein Listenelement wie folgt aus:
     * Parameter1: HaltestellenID1
     * Parameter2: HaltestellenID2
    * Parameter3: Liniennummer
    * Wir erzeugen dabei eine Liste von Tripeln aus dem Standardtyp (Leda) three_tuple.
    * Zu lesen ist das Ganze dann wie folgt: Im Netz der Transportverbindungen ist die
    * Kante gemeint, auf welche die Linie mit der angebenen Nummer von Haltestelle A
    * nach Haltestelle B faehrt.
    */
    list<three_tuple<int,int,int>* >* gebenZusammenhangskomponenteAlsListe();
    /**
    * Diese Methode erzeugt einen neuen Knoten und kopiert alle bisherigen Beschriftungen.
    * Weiterhin fuellt diese Methode dann das Hash array zuordnungIdNachKnoten in CZusammenhangskomponente
    * aus. Dabei notieren wir als Key die interne KnotenID und als Value den entsprechenden Knoten in der
    * Zusammenhangskomponente.
    */
    void anfuegenKnoten(const CKnotenBeschriftung&);
    
    void anfuegenKante(int, int, int, int, bool);

    /**
     * Gibt den Graph der Zusammenhangskomponente
     * textuell aus
     */
    void ausgebenZGraph ();
    
    /**
     * Diese Methode gibt den Graphen der Zusammenhangskomponente an die aufrufende Methode
     * zurueck.
     */
    GRAPH<CKnotenBeschriftung,CKantenBeschriftung>* gebenGraphDerZusammenhangskomponte();
    /**
     * Wir erzeugen eine Instanz von CSimplexTableau und uebergeben an diese die Anzahl
     * Basis und Nichtbasisvariablen, sowie die Taktzeit (Konstruktor). Dann rufen wir die Funktion
     * einfuegenKantenInSimplextableau in CSimplexTableau auf, waehrend dieses Aufrufs erhaelt die
     * aufgerufene Methode die Liste aller Kanten der Zusammenhangskomponente.
     *
     */
    void berechnenSimplexTableau();

    void berechnenZyklomatischeZahl();
    void setzenIdDerZusammenhangskomponente();
    int gebenIdDerZusammenhangskomponente();
    int gebenZyklomatischeZahl();

    /**
     * Diese Methode startet im errechneten Simplextableau
     * die Berechnung der lokalen Suche
     */
    void startenLokaleSuche ();

    /**
     * setzen aller Kantenbeschriftungen des Graphen hinsichtlich festerBindungen
     * auf false
     */
    void initialisierenFesterVerbindungen ();

    /**
     * setzen aller Kantenbeschriftungen des Graphen hinsichtlich mussInsMaximalgeruest
     * auf false und die Werte fuer nichtImMaximalgeruest auf false
     */
    void initialisierenMussInsMaximalgeruest ();

    /**
     * Diese Methode berechnet das von Weigand als Ausgangsloesung geforderte
     * Maximalgeruest. Wir gehen dabei wie folgt vor:
     * Wir uebergeben den Graphen der Zuasmmenhangskomponente
     * (graphDerZusammenhagskomponente), sowie einen Zeiger auf eine Compare-Funktion
     * (vergleichenKantenkosten in CZusammenhangskomponenten) an die Funktion
     * MIN_SPANNING_TREE in LEDA.
     * Als Rueckgabe bekommen wir eine Liste von Kanten,
     * dieses sind die Kanten des Maximalgeruests. Diese Liste uebergeben wir an
     * die Methode setzenMaximalgeruest in CZusammenhangskomponenten.
     */
    void berechnenMaximalgeruest();
    /**
     * Diese Methode speichert die Kanten des Maximalgeruestes in listeMaximalgeruest
     * in CZusammenhangskomponente ab.
     */
    void setzenMaximalgeruest(list<edge>*);
    /**
     * Diese Methode gibt eine Liste von Kanten, welche das Maximalgeruest der entsprechenden
     * Zusammenhangskomponente darstellt, an die aufrufende Methode zurueck. 
     */
    list<edge>* gebenMaximalgeruest();

    /**
    * Mit dieser Methode wird die feste Verbindung zweier Linien gesetzt
    * Parameter 1: LinienID1
    * Parameter 2: LinienID2
    * Parameter 3: HaltestellenID
    *
    *  Um die entsprechende Kante, und damit auch Knoten zu finden, benutzen wir die Funktion
    * suchenUmsteigeKante in CZusammenhangskomponenten. Diese gibt die richtige Kante an uns zurueck.
    * Haben wir die Kante erhalten, so geben wir sie an die Funktion setzenUmsteigekante in
    * CZusammenhangskomponente weiter.
    */
    void setzenFesterBindungen(int,int,int);

    /**
    * Bei der Wahl der festen Verbindungen kann es passieren, dass der User die Kanten so
    * waehlt, dass die gewaehlten Kanten einen Zyklus ergeben. In einem solchen Fall ist es
    * aber nicht moeglich, die netzbedingten Wartezeiten dieser Verbindungen auf Null zu setzen.
    * Wir brauchen also einen "Zyklentest". Diesen wollen wir mittels einer UNION-FIND-DS realisieren.
    * Folgendes Vorgehen haben wir uns ueberlegt:
    * Beim ersten Aufruf der Funktion wird jeder Knoten in eine eigene Partition gepackt. Dieses erledigt diese
    * Funktion (setzenFesterBindungen in CZusammenhangskomponenten). Dazu benutzen wir die private Variable
    * knotenPartition.
    * Nach der Initialiserung durch diese Funktion werden wir bei jedem Setzen einer festen Verbindung mittels
    * der Methode setzenFesterBindungen (dort ist auch der detaillierte Abluaf zu finden) zwei FINDS auf die
    * Knoten der gefundenen Kante durchgefuehrt. Liegen diese in zwei verschiedenen Knotenpartitionen, so
    * erzeugen wir durch die Wahl der Kante als feste Verbindung keinen Zyklus. Tritt dieser Fall ein, so werden
    * alle Knoten der kleineren der beiden Partition in die groessere Knotenpartion "gemergt" (Dieses erledigt LEDA).
    * Befinden sich die Knoten bereits in einer Partition, so wird eine Fehlermeldung ausgegeben.
    *
    *
    */
    void berechnenKnotenPartition();
    /**
    * Mit dieser Methode wird die netzbedingte Wartezeit der Zusammenhangskomponente ausgegeben.
    * Liegt dieser Wert noch nicht vor, so wird er hier mit der Methode berechnenNetzbedingteWartezeit berechnet.
    */
    double gebenNetzbedingteWartezeit();
    /**
    * Es wird ueber alle Kanten des Maximalgeruestes iteriert. Diese Kanten haben wir ja in der
    * Variable listeMaximalgeruest in CZusammenhangskomponente gespeichert. Fuer alle Kanten in dieser
    * Liste wird in der booleschen Variable imMaximalgeruest, zu finden in CKantenBeschriftung mittels
    * der Methode setzenImMaximalgeruest auf TRUE gesetzt.
    */
    void setzenMaximalgeruestKanten();
    /**
     *
     */
    double  ausgebenMindesthaltezeit(int);
    /**
     *  Diese Methode wird von der Methode setzenMindesthaltezeit in CTransportkettengraph
     *  aufgerufen. Sie erhaelt das Hash-Array der vom Benutzer eingegebenen Mindesthaltezeiten.
     *  Sie macht damit folgendes: Wir iterieren ueber alle Knoten in der Zusammenhangskomponente
     *  und lassen uns jeweils HaltestellenID2 des Source Knoten geben. Diese benutzen wir als Key und schauen
     *  im erhaltenen Hasharray nach, wie gross die Mindesthaltezeit ist. Im Anschluss daran ermitteln
     *  wir alle von diesem Knoten ausgehenden Kanten und uebergeben den Wert an die Methode
     *  setzenMindesthaltezeit in CKantenBeschriftung.
     */
    void setzenMindesthaltezeit(h_array<int, int>*);
    /**
    *
    */
    double ausgebenMindestumsteigezeit(int,int,int);

    /**
     * Geben der FahrzeugAnzahl
     */
    int gebenFahrzeugAnzahl ();

    /**
     * Geben-Methode
     */
    double gebenSummeNetzbedingteWartezeiten();

    /**
     * Geben-Methode
     */
    double gebenWartezeitAufwand ();
      
    /**
     * Geben-Methode des Fahrplanwirkungsgrades
     */
    double gebenFahrplanWirkungsgrad ();

    /**
     * Geben-Methode
     */
    double gebenMindestWartezeitAufwand ();

    /**
     * Geben-Methode
     */
    double gebenGesamtWartezeitAufwand ();

     /**
     *  Diese Methode wird von der Methode setzenMindestumsteigezeit in CTransportkettengraph
     *  aufgerufen. Sie erhaelt das Hash-Array der vom Benutzer eingegebenen Mindestumsteigezeiten.
     *  Sie macht damit folgendes: Wir iterieren ueber alle Knoten in der Zusammenhangskomponente. Dann
     *  iterieren wir ueber alle out Kanten dieses Knotens, finden wir eine Kante mit
     *  istUmsteigekante==true, lesen wir die LinienIDs des Source und Targetknotens mittels ausgebenLiniennummer
     *  aus CKnotenBeschriftung aus. Dann suchen wir uns im erhaltenen 3-Tupel den Eintrag, welcher den
     *  erhaltenen Liniennummern (Reihenfolge Source - Target) entspricht. Der dritte Wert des 3-Tupels
     *  ist dann die MindestUmsteigezeit, diese wird an die Kante drangeschrieben.
     *  Hierzu dient die Methode setzenMindestumsteigezeit in CKantenBeschriftung.
     */
    void setzenMindestumsteigezeit(h_array<int,list< three_tuple<int,int,int>* >* >*);
    /**
     * Setzt den Takt
     */
    void setzenTakt(int);

    /**
     * Zeichnen der Zusammenhangskomponente
     */
    void ZeichnenZusammenhangskomponente ();

    /**
     * Berechnung der Wartezeit-Resultate in dieser
     * Zusammenhangskomponente
     */
    void berechnenWartezeitResultate ();

    /**
     * Diese Methode wird aufgerufen, um den Fahrplan fuer die Zusammenhangskomponente erstellen zu koennen.
     * Uebergeben wird die Abfahrtszeit und das Fahrplanintervall.
     * Diese Methode speichert das Fahrplanintervall und ruft die Private Methode generierenAbfahrtszeit mit der uebergebenen Anfangszeit auf.
     */
    void berechnenFahrplan(three_tuple<int,int,int>*,four_tuple<int,int,int,int>*);

    /**
     * Gibt den Fahrplan als liste von CFahrplanElement zurueck
     */
    list<CFahrplanElement*>* gebenFahrplan();

    /**
     * Berechnet den Entscheidungsbaum bis zur angegebenen Tiefe
     */
    void berechnenBesteLoesung(int);

    /**
     * Initialisiert die Wartezeiten
     */
    void initWartezeitResultate ();

    /**
     * Gibt eine Liste zurueck, in der die Netzbedingten Wartezeiten fuer die Kombination
     * LinieVon, Haltestelle, LinieNach stehen.
     */
    h_array<three_tuple<int,int,int>*,int>* gebenListeNetzbedingtewartezeit();

    void fuellenReisebewertung(map2<int,int,CReisebewertung>&);

    int gebenRelativeGesamtFahrzeit();

    int gebenFahrgastfahrten();

  private:
    /**
     * dieses mapping symbolisiert die abbildung
     * von einem drei-tupel (liniennr1, liniennr2, haltestellenid)
     * auf die entsprechende kante im transportkettengraph bzw.
     * in der zusammenhangskomponente, nuetzlich fuer das
     * auffinden einer umsteigekante
     */
    h_array<three_tuple<int, int, int>, edge> *mappingUmsteigeKante;

    h_array<three_tuple<int, int, int>, edge> *mappingKante;

    /**
     *
     * Die Methode berechnet den Fahrplan fuer diese Zusammenhangskomponente.
     * Dabei gehen wir wie folgt vor. Beginnend von dem Startknoten, welchen wir
     * ja mittgeteilt bekommen, beginnen wir einen BFS. Dabei muessen wir jeweils noch
     * zwischen ingrad und outgrad Kanten unterscheiden. Dann koennen wir fur jede Haltestelle
     * und jede Linie an dieser Haltestelle die Abfahrtszeit berechnen. Diese setzt sich zusammen
     * aus: Ankunftszeit an Haltestelle  + Summe Fahrtzeit+ Summe Wartezeiten fuer outgrad Kanten,
     * Ankunftszeit an Haltestelle - Summe Fahrtzeit + Summe Wartezeiten fuer ingrad Kanten.
     * Das Ergebnis schreiben wir an den Knoten im TKG, wobei sich der Wert auf die Abfahrtshaltestelle
     * (die zweite HaltestellenID) bezieht.
     * Diese Werte werden in dem Attribut fahrplan gespeichert.
     */
    void generierenAbfahrtszeiten(three_tuple<int, int, int>*);

    /**
     * Berechnen des Endknotens einer Linie;
     * respraesentiert durch einen Knoten dieser Linie
     */
    node SuchenEndKnoten (node);

    /**
     * Sortieren der Fahrplanelemente
     */
    void sortierenFahrplanElemente ();

    /**
     * Sortieren der einzelnen Fahrzeit-Listen
     * pro Haltestelle
     */
    void sortierenFahrplanListen (h_array<int, h_array<int, list<CFahrplanElement* > * >* > *);
    
    /**
     * Berechnung der Haltestellenfolge fuer eine Linie
     */
    void berechnenHaltestellenFolge (int, list<int> *);

    /**
     * In dieser Liste wird der Fahrplan erstellt.
     */
    list<CFahrplanElement*>* fahrplan;

    /**
     * Mindesthaltezeit Hash aus CFahrplan
     */
    h_array<int, int>* mindestHaltezeitHash;
    /**
     * Mindestumsteigezeit Hash aus CFahrplan
     */
    h_array<int, list<int> >* mindestUmsteigezeit;

     /**
     * @supplierCardinality 1 
     */
    CSimplexTableau * simplexTableau;
    /**
     * Die zyklomatische Zahl der Zusammenhangskomponente.
     */
    int zyklomatischeZahl;
    
    /**
     * Der Graph der Zusammenhangskomponente.
     */
    GRAPH<CKnotenBeschriftung*, CKantenBeschriftung*>* graphDerZusammenhangskomponente;
    /**
     * In dieser UNION-FIND-DS speichern wir uns die Knotenpartition, welche von der
     * Funktion berechnenKnotenPartition in CZusammenhangskomponenten initialisiert wird.
     */
    node_partition* knotenPartition;
    /**
     * Die Liste enthaelt alle Kanten des Maximalgeruestes.
     */
    list<edge>* listeMaximalgeruest;

    /**
    * Enthaelt die netzbedingte Wartezeit in min
    */
    double netzbedingteWartezeit();

    /**
    * Sucht die Umsteige Kante in der Zusammenhangskomponente
    * Parameter 1: LinienID1
    * Parameter 2: LinienID2
    * Parameter 3: HaltestellenID
    *
    * Ablauf der Methode: Wir iterieren ueber alle Kanten im Graphen und ueberpruefen mittels
    * Informationen des Source- und Targetknoten ob es sich um die Umsteigekante handelt.
    * Dazu iterieren wir ueber alle Kanten und sehen uns die Informationen an den Source
    * und Target Knoten an: es muss gelten:
    * source: haltestellenID2 = HaltestellenID und Liniennummer = LinienID1
    * target: haltestellenID1 = HaltestellenID und Liniennummer = LinienID2
    * Wenn wir die Kante dann gefunden haben, geben wir sie an den Aufrufer zurueck.
    */
    edge suchenUmsteigeKante(int,int,int);

    /**
     * Berechnung der benoetigten Fahrzeuganzahl
     */
    void berechnenFahrzeugAnzahl (int, int);

    /**
    *
    * Parameter 1: Kante die als feste Bindung gesetzt werden soll
    *
    * Ablauf der Methode:
    * Mittels der erhaltenen Kante identifizieren wir die beiden, uns interessierenden Knoten.
    * Haben wir diese gefunden, schauen unter Nutzung der LEDA-Knotenpartition nach, ob diese
    * beiden Knoten in der derselben Partition liegen. Es koennen nun zwi Faelle auftreten:
    * 1. Sie sind in unterschiedlichen Partitionen: Mittels der LEDA-Knotenpartition vereinigen wir
    * die beiden Knotenpartitionen. Anschliessend rufen wir die Methode setzenFesteVerbindung in
    * CKantenbeschriftung auf. Die Kante wurde als feste Verbindung markiert.
    * 2. Sie sind in der gleichen Partition: Wuerde die Kante gewaehlt, wuerde ein Zyklus entstehen, dies
    * ist nicht erlaubt. Eine Fehlermeldung muss ausgegeben werden mittels TRY-AND-CATCH.
    */
    void setzenUmsteigeKante(edge);
    /**
     * Diese Methode ist die in der Methode berechnenMaximalgeruest angesprochene Compare-Funktion.
     * Sie schaut als erstes nach, ob das festeVerbindung Flag bei mindestens einer der beiden
     * Kante gesetzt ist. Folgende Faelle koennen auftreten
     * 1. Bei keiner gesetzt: Die Kanten werden aufgrund des auf ihnen vorhandenen Verkehrsstroms
     * verglichen. Es wird die Kante zurueckgegeben, auf welcher der groessere Verkehrsstrom liegt.
     * 2. Bei einer gesetzt: Diese Kante wurde als feste Verbindung markiert, die Compare-Funktion waehlt
     * diese Kante.
     * 3. Bei beiden gesetzt: Wenn das Flag bei beiden Kanten gesetzt ist, wird diejenige mit dem
     * hoeheren Verkehrsstrom gewaehlt.
     */
 
    int  vergleichenKantenkosten( edge const  , edge const);

   
    
    /**
    * Mit dieser Methode wird die gesamte netzbedingte Wartezeit der Zusammenhangskomponente berechnet und gespeichert
    */
    void berechnenNetzbedingteWartezeit();
    /**
     * Taktzeit
     */
    int takt;
    /**
     * Diese Methode berechnet die Kanten, welche mit der angegebenen Kante auf einem Zyklus liegen.
     * Als Rueckgabe liefert die Methode eine Liste von 2-Tupeln. Der erste Wert des 2-Tupels gibt an,
     * ob die Kante in Gegenrichtung (FALSE) oder Richtung (TRUE) des Kreises liegt. Der zweite Wert ist dann die
     * Kante.
     */
    list<two_tuple<bool,edge> >* berechnenZyklusKanten(edge);
    /**
     * Dieses Array hasht von der internen Knoten ID auf den entsprechenden Knoten in dieser
     * Zusammenhangskomponente.
     */
    h_array<int, node>* zuordnungIdNachKnoten;

    /**
     * Die fuer die Zusammenhangskomponente benoetigte FahrzeugAnzahl
     */
    int fahrzeugAnzahl;

    /**
     * Die Gesamtdauer des zu erstellenden Fahrplans in Minuten
     */
    int gesamtDauerFahrplan;

    /**
     * Fahrplanwirkungsgrad der Zusammenhangskomponente
     */
    double fahrplanWirkungsgrad;

    /**
     * ID der Zusammenhangskompnente
     */  
    int zusammenhangskomponentenID;

    /**
     * Speicherung der Summe der netzbedingten Wartezeiten
     * in dieser Zusammenhangskomponenten
     */
    double summeNetzbedingteWartezeiten;

    /**
     * Speicherung des Wartezeit-Aufwandes bezueglich
     * dieser Zusammenhangskomponente
     * Summe der Produkte aus netzbedingter Wartezeit
     * und Reisendenzahl
     */
    double wartezeitAufwand;

    /**
     * Speicherung des Mindestwartezeit-Aufwandes bezueglich
     * dieser Zusammenhangskomponente
     * Summe der Produkte aus Summe der Mindestumsteigezeit und
     * Mindesthaltezeit
     * und Reisendenzahl
     */
    double mindestWartezeitaufwand;
    
    /**
     * Summe aus mindestWartezeitaufwand und wartezeitAufwand
     */
    double gesamtWartezeitAufwand;

    /**
     * Die Zusammenhangskomponente als gerichtete Liste
     * von Haltestellen + zugehoeriger Liniennummer
     */
    list<three_tuple<int, int, int>* >* listeDerHaltestellen;

    /**
     * In diesem Hash Array wird zu jeder Linie (der int - Wert) der jeweilige Startknoten in der
     * Zusammenhangskomponente gespeichert.
     */
    h_array<int,node>* startHaltestellenArray;
    h_array<int,node>* endHaltestellenArray;
    h_array<int, int>* absoluteFahrtzeitLinienArray;
    int relativeGesamtFahrtzeit;
    h_array<int, list<node>*>* startHaltestellenNachLinie; 
    GRAPH<four_tuple<int,int,int,int>, int >* umlaufGraph;
    list<int>* linienListe;

    /**
     * Die Methode Berechnet zu einer gegeben HaltestellenID und einer gegeben LinienID die Haltestelle, mit
     * der die Linie beginnt
     */
    
    node berechnenLinienStartHaltestelle(node*);

   

    void berechnenAbfahrtszeitenPerBFS(const node*);

    /**
     * Initialisieren der Wartezeit-Resultate in dieser
     * Zusammenhangskomponente, d.h. auf Null setzen der
     * Werte von summeNetzbedingteWartezeit, wartezeitAufwand,
     * mindestWartezeitAufwand und gesamtWartezeitAufwand
     */

    //void initWartezeitResultate();

    /**
     * Diese Methode baut den im Verfahren von Voelker vorgestellten Graphen, der die 
     * Grundlage der Fahrzeugumlaufplannung darstellt auf.
     */
    void aufbauUmlaufGraph();
    /** 
     * Diese Methode wird dazu benutzt, um festzustellen, ob eine Kante zwischen
     * einem Endknoten einer Fahrgastfahrt und dem Anfangsknoten eingefuegt
     * werden kann. Ist dies der Fall, wird der entsprechende Knoten zurueckliefert.
     */
     node vergleichenAbfahrtszeiten(list<node>*, node, bool &); 
    // two_tupel berechnenStundeMinute (int,int);

    array<edge>* kantenlisteDesGraphen;

    array<int>* feldDerBestenLoesung;

    double besteWartezeit;

    int aktuelleGeruestGroesse;

    int aktuelleNichtGeruestGroesse;

    int maximaleGeruestGroesse;

    int fahrgastfahrten;

    /**
     * Mit dieser Methode wird der Entscheidungsbaum berechnet.
     * Der Parameter ist dabei die KantenID aus dem Feld kantenlisteDesGraphen und eine liste wie der aktuelle Ast 
     * aussieht
     */
    void berechnenEntscheidungsbaum(int,array<int>*,int);

    CReisebewertung bewerteReise (four_tuple<node,node,int,list<edge>*>*);

};  
    /** 
     * Diese Funktion ist die Vergleichsfunktion, welche zur Sortierung der berechneten Fahrplan
     * liste benoetigt wird. Diese Sortierung ist Bestandteil der Fahrzeugumlaufplannung.
     */
     
    int sortierenNachLinie(CFahrplanElement* const &, CFahrplanElement* const &);

    int sortierenNachFahrzeit(CFahrplanElement* const &, CFahrplanElement* const &);

#endif //CZUSAMMENHANGSKOMPONENTEN_H









