#include "CVisualisierung.h"

//___________________________________________________________________________

CVisualisierung::CVisualisierung(const char* titel, int posx = 0, int posy = 0, int breite=200, int hoehe=200): window(breite,hoehe,titel) {
  if ((posx==0) && (posy==0)) {
    display();
  } else {
    display(posx,posy);
  }
  draw_box(0,0,100,100,color(white));
} // CVisualisierung

//___________________________________________________________________________

CVisualisierung::~CVisualisierung() {
  close();
} // ~CVisualisierung

//___________________________________________________________________________

void CVisualisierung::zeigenVersorgungsmatrix(matrix * M) {
  double wert;
  for (int i=0; i<M->dim1(); i++) {
    for (int j=0; j<M->dim2(); j++) {
      wert = (*M)(i,j);
      double grauton = 255;
      for (int k=0; k<(int)wert; k++) {
        grauton = grauton / 0.9;
      }
      draw_box((int)95-(j*90/M->dim2()), (int)95-(i*90/M->dim1()), (int)95-((j+1)*90/M->dim2()), (int)95-(((i+1)*90/M->dim1())), color((int)grauton,(int)grauton,(int)grauton));
    }
  }
} // zeigenVersorgungsmatrix

//___________________________________________________________________________

void CVisualisierung::zeigenFitness(list<double> * Werte) {
  double maximum = 0;
  double wert;
  double xSkalierung, ySkalierung;
  clear();
  for(int i=0; i<Werte->size(); i++) {
    wert = Werte->contents(Werte->get_item(i));
    if (wert>maximum) {
      maximum = wert;
    }
  }
  ySkalierung = height()/maximum;
  xSkalierung = width()/Werte->size();
  for (int i=0; i<Werte->size()-1; i++) {
    draw_segment(i*xSkalierung,height()-(Werte->contents(Werte->get_item(i))*ySkalierung),(i+1)*xSkalierung,height()-(Werte->contents(Werte->get_item(i))*ySkalierung));
  }
} // zeigenFitness;

//___________________________________________________________________________

void CVisualisierung::zeigenStrassen(GRAPH<rat_point,rat_segment> * Strassennetz, double Westen, double Norden, double Osten, double Sueden) {
  edge kante;
  double Bereich, xBereich, yBereich;
  double xVersatz, yVersatz;
//  Debug(7,"Bereich: " << xmin() << " " << xmax() << " " << ymin() << " " << ymax());
//  Debug(7,"Faktor : " << scale());
//  wartenTaste();
  draw_box(0,0,100,100,color(white));
  xVersatz = -Osten;
  yVersatz = -Norden;
  xBereich = (Westen-Osten);
  yBereich = (Sueden-Norden);
  if (abs(xBereich)>=abs(yBereich)) {
    Bereich = xBereich;
  } else {
    Bereich = yBereich;
  };
  forall_edges(kante, *Strassennetz) {
    double x1 = Strassennetz->inf(kante).xcoord1D();
    if (x1<0) x1=-x1;
    x1 += xVersatz;
    double y1 = Strassennetz->inf(kante).ycoord1D();
    if (y1<0) y1=-y1;
    y1 += yVersatz;
    double x2 = Strassennetz->inf(kante).xcoord2D();
    if (x2<0) x2=-x2;
    x2 += xVersatz;
    double y2 = Strassennetz->inf(kante).ycoord2D();
    if (y2<0) y2=-y2;
    y2 += yVersatz;
    draw_segment(95-(x1*90/Bereich),95-(y1*90/Bereich),95-(x2*90/Bereich),95-(y2*90/Bereich));
//    Debug(9,"[" << x1*90/Bereich+5 << "/" << y1*90/Bereich+5 << "] -> [" << x2*90/Bereich+5 << "/" << y2*90/Bereich+5 << "]");
//    Debug(9,"[" << x1-xVersatz << "/" << y1-yVersatz << "] -> [" << x2-yVersatz << "/" << y2-yVersatz << "]");
  }
/*  node knoten;
  forall_nodes(knoten, *Strassennetz) {
    double x = Strassennetz->inf(knoten).xcoordD()+xVersatz;
    double y = Strassennetz->inf(knoten).ycoordD()+yVersatz;
    draw_point(95-(x*90/Bereich),95-(y*90/Bereich),color(red));
  }*/
} // zeigenStrassen

//___________________________________________________________________________

void CVisualisierung::zeigenHaltestellen(list<CWegenetzKnoten*> * Haltestellen, double Westen, double Norden, double Osten, double Sueden) {
  double Bereich, xBereich, yBereich;
  double xVersatz, yVersatz;
//  Debug(7,"Bereich: " << xmin() << " " << xmax() << " " << ymin() << " " << ymax());
//  Debug(7,"Faktor : " << scale());
//  wartenTaste();
  xVersatz = -Osten;
  yVersatz = -Norden;
  xBereich = (Westen-Osten);
  yBereich = (Sueden-Norden);
  if (abs(xBereich)>=abs(yBereich)) {
    Bereich = xBereich;
  } else {
    Bereich = yBereich;
  };
  for (int i=0; i<Haltestellen->size(); i++) {
    double x = Haltestellen->contents(Haltestellen->get_item(i))->holenPosition()->holenX()+xVersatz;
    double y = Haltestellen->contents(Haltestellen->get_item(i))->holenPosition()->holenY()+yVersatz;
    draw_point(95-(x*90/Bereich),95-(y*90/Bereich),color(red));
  }
} // zeigenStrassen

//___________________________________________________________________________

void CVisualisierung::zeigenMatrixRelativ(matrix * M, double Maximum) {
  double wert;
  for (int i=0; i<M->dim1(); i++) {
    for (int j=0; j<M->dim2(); j++) {
      wert = (*M)(i,j);
      int farbton = (int)((255*wert)/Maximum);
      if (farbton>=0) {
        draw_box((int)5+(j*90/M->dim2()), (int)95-(i*90/M->dim1()), (int)5+((j+1)*90/M->dim2()), (int)95-((i+1)*90/M->dim1()), color((int)farbton,(int)255-farbton,0));
      } else {
//        farbton /= 2;
        if (farbton<-255) {
          farbton = -255;
        }
        draw_box((int)5+(j*90/M->dim2()), (int)95-(i*90/M->dim1()), (int)5+((j+1)*90/M->dim2()), (int)95-((i+1)*90/M->dim1()), color(0,(int)255+farbton,(int)-farbton));
      }
    }
  }
} // zeigenMatrixRelativ

//___________________________________________________________________________

void CVisualisierung::zeigenLegende(void) {
  int detail = 9;
  for(int i=0;i<detail; i++) {
    detail--;
    draw_box(5,i*50/detail+45,10,(i+1)*50/detail+45,color((int)i*255/detail,(int)255-(i*255/detail),0));
    detail++;
  }
  for(int i=0;i<detail-1; i++) {
    detail--;
    i++;
    draw_box(5,i*50/(-detail)+55,10,(i+1)*50/(-detail)+55,color(0,(int)255-(i*255/detail),(int)i*255/detail));
    i--;
    detail++;
  }
  draw_ctext(45,90,string("Unterversorgung"));
  draw_ctext(45,50,string("Optimalversorgung"));
  draw_ctext(45,10,string("berversorgung"));
} // zeigenLegende

//___________________________________________________________________________

