#ifndef CVISUALISIERUNG_H
#define CVISUALISIERUNG_H

#include <LEDA/window.h>
#include <LEDA/matrix.h>
#include <LEDA/graph.h>
#include <LEDA/rat_point.h>
#include <LEDA/rat_segment.h>
#include "Debug.h"
#include "../Datenstruktur/CWegenetzKnoten.h"
/**
 * Dient der Visualisierung diverser Kenngoessen innerhalb
 * der Haltestelle.
 * Hierzu zaehlen:
 * - Bevoelkerungsabdeckung
 * - Fitnesswerte
 */
class CVisualisierung: window {
public:

  /**
   * Knostruktor, welcher ein Fenster oeffnet.
   */
  CVisualisierung(const char* titel, int posx = 0, int posy = 0, int breite = 200, int hoehe = 200);

  /**
   * Destruktor, welcher das Fenster schliesst.
   */
  ~CVisualisierung();

   /**
    * Zeigt die Matrix in Form von Grautoenen.
    * Dabei ist die Kodierung wie folgt:
    * 0  - weiss
    * 1  - sanftes grau
    * 2+ - immer dunkler werdend
    */
   void zeigenVersorgungsmatrix(matrix * M);

   /**
    */
   void zeigenMatrixRelativ(matrix * M, double Maximum);

   /**
    * Zeigt eine Liste von Fitnesswerten an,
    * wobei selbige durch das hoechste skaliert werden
    * und das Minimum bei Null angenommen wird.
    * Die Punkte werden dabei durch Linien verbunden.
    */
   void zeigenFitness(list<double> * Werte);

   void zeigenStrassen(GRAPH<rat_point,rat_segment> * Strassennetz, double Westen, double Norden, double Osten, double Sueden);

   void zeigenHaltestellen(list<CWegenetzKnoten*> * Haltestellen, double Westen, double Norden, double Osten, double Sueden);

   void zeigenLegende(void);
};

#endif //CVISUALISIERUNG_H
