/* Generated by Together */

#ifndef ABSCHLUSSVERKNUEPFUNG_H
#define ABSCHLUSSVERKNUEPFUNG_H

#include "CNetzplanKnoten.h"
#include "CNetzplanKante.h"
#include "CLinie.h"
#include "CNetzplan.h"
#include "CBasislinie.h"
#include "CAufblinie.h"
#include "Debug.h"
#include <LEDA/matrix.h>
#include <LEDA/array.h>

/**
 * Diese Klsse optimiert eine gegebene Liste von Linien (wobei es egal
 * ist, ob diese Linien extern erstellt worden oder aber gerade in den
 * vorhigen Phasen erzeugt worden sind). Zu diesem Zweck benutzt es die
 * Funktionen Mehrfachverknuefung und Aufbruchverknuepfung um alle moeglichen
 * Verknuepfungsarten zwischen den Linien untereinander zu bewerten, das
 * Optimum auszuwaehlen und anschliessend die naechste, eine Verbesserung
 * erzeugende Verknuepfung zu bestimmen.
 */
class CAbschlussverknuepfung {
public:

    /**
     * Eingabe: CNetzplan als globale Datenstruktur 
     */
    CAbschlussverknuepfung(CNetzplan * meinNetzplan);

    CAbschlussverknuepfung(CNetzplan * meinNetzplan, matrix *meinBedarf);


    ~CAbschlussverknuepfung();


    /**
     * Bei dieser Methode werden all diejenigen Kanten betrachtet, auf denen es
     * mehrere verschiedene Verknuepfungsmoeglichkeiten zwischen den Linien
     * gibt. Der Reihe nach werden all diese Verknuepfungen, die aktuell
     * moeglich sind, mit Hilfe der Attraktivitiaetsberechnung bewertet, das
     * Optimum ausgewaehlt und die entsprechende Kante fuer weitere Verknuepfungen
     * gesperrt. Anschliessend wird der Algorithmus auf diesem Linienplan erneut
     * aufgerufen und die naechste optimale Verknuepfung errechnet. Eine
     * detailierte Beschreibung des verwendeten Algorithmus ist im
     * Zwischenbericht auf Seite XYZ zu finden.
     */
    void bildenMehrdeutigeLinienverknuepfung();

    /**
     * Bei dieser Methode erfolgt eine Kosten-Nutzen-Rechnung der Gestalt, dass
     * der Attraktivitaets-Gewinn, der durch eine neue Verknuepfung entsteht,
     * greoesser sein muss, als der Verlust, der dadurch entsteht dass eine andere
     * Verknuepfung aufgebrochen wird, um so Gesamt-Laengen-Kapazitaet fuer die
     * neue Verknuepfung zu erhalten. Als erstes wird deswegen entlang einer
     * Linie (Grundlinie) eine sogenannte Verknuepfungsdiskussion durgefuehrt, um
     * eine moegliche neue sinnvolle Verknuepfung mit einer anderen Linie zu
     * finden. Ist diese bestimmt, so wird der Gewinn (durch die
     * Attraktivitaetsberechnung) und die damit verbundenen Kosten (= neu
     * benoetigte Entfernung fuer die Gesamtlinienlaenge) bestimmt. Dieses
     * Verfahren setzt sich so lange fort, bis wieder eine optimale Verknuepfung
     * diese Art gefunden worden ist. Anschliessend werden so lange diejenigen
     * Kante bestimmt, die den guenstigsten Grad zwischen "Verlust" (=negativer
     * Attraktivitaetsgewinn) und "Kostengewinn" (verfuegbare Entfernung fuer
     * Gesamtlaenge) aufweisen, bis die benoetigte Kapazitaet erreicht ist, die
     * die optimale Verknuepfung fordern wuerde. Anschliessend wird der Verlust
     * durch diese Aufbrueche berechnet und wenn der Gewinn groesser als der
     * Verlust ist, so wird diese Aufbruchverknuepfung durchgefuehrt. Eine
     * detailierte Beschreibung des verwendeten Algorithmus ist im
     * Zwischenbericht auf Seite XYZ zu finden. 
     */
    void durchfuehrenAufbruchsverknuepfung();

    /**
     * ist die Frage, ob wir diese Methode ueberhaupt implementieren, da sie
     * kaum nutzen bringt... verschieben auf den 2. Prototypen
    */
    void bildenLinienendverknuepfung();

    void loeschedoppelteBasislinien();

private:    


    	node umwandelnHaltestelleZuKnoten(int haltestelle);

		CNetzplanKnoten* CAbschlussverknuepfung::umwandelnHaltestelleZuNetzplanKnoten (int haltestelle);
				
		edge suchenKanteZwischenKnoten(node startKnoten, node endKnoten);

		bool zyklusfreiheit(list<int> *linkeHListe,list_item linksstart, bool linksrichtung, list<int> *mittlereHListe,list<int> *rechteHListe,list_item rechtsstart, bool rechtsrichtung);

		
		void indirekterStromZwischenHaltestellen(int startKnoten, int endKnoten, int* versackenderStrom, int* verkehrsstrom, int* linienanzahl);
		
		edge suchenKanteZwischenHaltestellen(int startKnoten, int endKnoten);

    /**
     * Diese Methode wird von sowohl von den beiden Verknuepfungsmethoden
     * benoetigt, um den Wert der Verknuepfung zweier Linien auf einer Kante zu
     * bestimmen. Zu diesem Zweck berechnet diese Methode wieviele Fahrten
     * durch diese Verknuepfung nun keinen Umstiege mehr erfordern, wieviele
     * Direktfahrten bzw. Teilindirektfahrten dadurch realisiert werden
     * konnten. Eine detailierte Beschreibung des verwendeten Algorithmus ist
     * im Zwischenbericht auf Seite XYZ zu finden.
     */
		int berechnenAttraktivitaet(int linkeEndHStelle, int rechteEndHStelle, list<int> *linkeHListe, bool linksanfang, list<int> *rechteHListe, bool rechtsanfang);


		int berechneAufbAttraktivitaet(int anknuepfAnfHStelle, int anknuepfEndHStelle, bool anknuepfAnfang, list<int> *anknuepfHListe,int anknuepflinieID, int verknuepfAnfHStelle, int verknuepfEndHStelle, bool verknuepfAnfang, list<int> *verknuepfHListe,int verknuepflinieID, int aufbruchlaenge, int grundlinieID);

		int CAbschlussverknuepfung::berechneAufbruchAttraktivitaet(list_item verknuepfAItem, list_item verknuepfEItem, list<int> *grundHListe, bool grundrichtung, list_item grundLinieItem, list<int> *verknuepfHListe, bool verknuepfAnfang, list_item verknuepfLinieItem, bool* maxAnknuepfAnfang, list_item* maxAnknuepfLinieItem, list_item* maxAnknuepfGrundItem, int* maxAufbruchlaenge);
   int berechnenStrom(int linkeStelle, int mittlereStelle, int rechteStelle);

   bool verknuepfungslaengeZulaessig(CLinie* startLinie,CLinie* endLinie,int gemeinsam);

    bool zyklusfrei(int vergleichnummer, list<int> *linkeHListe,list<int> *rechteHListe,bool anfangLinke, bool anfangRechte);

    list<CAufblinie*>* erstellenAllerEndAufblinien(void);

    CLinie * lnkLinie;

    CNetzplan *netzplan;
    matrix *qzmatrix;
    list<CLinie*>* linienListe;
    list<CAufblinie*>* aufbruchlinienListe;
    UGRAPH<CNetzplanKnoten*,CNetzplanKante*>* wegenetzplan;
    array<int> vergleichsFeld;
    array<node*> nodeVektor;

    CNetzplanKante * lnkCNetzplanKante;

    CNetzplanKnoten * lnkCNetzplanKnoten;
    int vergleichnummer;

};
#endif //ABSCHLUSSVERKNUEPFUNG_H
