/* Generated by Together */

#ifndef AUFBEREITUNGSPHASE_H
#define AUFBEREITUNGSPHASE_H

#include "CNetzplanKnoten.h"
#include "CNetzplanKante.h"
#include "CNetzplan.h"
#include "Debug.h"
#include <LEDA/ugraph.h>
#include "LEDA/matrix.h"
#include <LEDA/shortest_path.h> 
#include <LEDA/basic_graph_alg.h>
//#include <LEDA/templates/shortest_path.t>
#include <LEDA/edge_map.h>
#include "CWegenetzKnoten.h"
#include "CWegenetzKante.h"
#include <LEDA/graph_misc.h>
#include <math.h>
#include "CFortschritt.h"

/**
 * Vorbereiten der Datenstrukturen auf die Verknuepfungsphase:
 * - CNetzplan aufbauen
 * - Verkehrsbedarfe als Verkehrsstrom an die Kanten legen
 * - Reduzieren von ueberfluessigen Knoten (nicht im ersten Prototyp) 
 */
class CAufbereitungsphase {
public:

    /**
     * Entgegennahme der Parameter
     * Eingaben:
     * - Wegenetzgraph
     * - Verkehrsbedarfsmatrix
     * - CNetzplan (mit leerem Graphen und leerer Liste) 
     */
    CAufbereitungsphase(CNetzplan * meinNetzplan, UGRAPH<CWegenetzKnoten*,CWegenetzKante*> * wegenetz, matrix * qzmatrix);

    ~CAufbereitungsphase();

    /**
     * Aufbau des Netzplangraphen
     * Im grossen Kopie des Wegenetzgraphen mit
     * folgenden Unterschieden:
     * - Entfernungen statt Fahrzeiten
     *   (teilen durch konstanten Faktor)
     * - an die Kanten zusaetzlich Verkehrsstrom
     *   und Anzahl der Basislinien 
     */
    void starten();

    /**
     * Verkehrsstroeme an die Kanten legen:
     * - kuerzeste Wege zwischen allen Haltestellen ermitteln
     * - auf diese kuerzesten Wege den Verkehrsbedarf addieren 
     */
    void umlegen();

    /**
     * !!!!!!!!!!!! WIRD IM ERSTEN PROTOTYP NOCH NICHT VERWENDET!!!!!!!!!!!!
     * 
     * - Loeschen von Knoten, die keinen Umstieg ermoeglichen
     * (= Knoten mit Grad 2)
     * - verhindern von Multigraphen:
     *       Wenn Knoten eigentlich geloescht werden soll, pruefen,
     *       ob schon eine Kante zwischen den beiden Nachbarkanten verlaeuft 
     */
    void reduzieren();

private:    

    /**
     * @supplierCardinality 1 
     */
    CNetzplan * netzplan;

    /**
     * @supplierCardinality 0..* 
     */
    CNetzplanKante * netzplanKante;

    /**
     * @supplierCardinality 0..* 
     */
    CNetzplanKnoten * netzplanKnoten;

    /**
     * @supplierCardinality 1 
     */
    CWegenetzKante * wegenetzKante;

    /**
     * @supplierCardinality 1 
     */
    CWegenetzKnoten * wegenetzKnoten;

    UGRAPH<CNetzplanKnoten*,CNetzplanKante*>* netzplangraph;

    UGRAPH<CWegenetzKnoten*,CWegenetzKante*>* wegenetzgraph;

    matrix* quellezielmatrix;

    CFortschritt* fortschritt;
};
#endif //AUFBEREITUNGSPHASE_H
