/* Generated by Together */

#ifndef BEWERTUNGSPHASE_H
#define BEWERTUNGSPHASE_H

#include "../Datenstruktur/CFPlanGraphKnoten.h"
#include "../Datenstruktur/CFPlanGraphKante.h"
#include "../Datenstruktur/CLinie.h"
#include "Debug.h"
#include "CNetzplan.h"
#include "LEDA/graph.h"
#include "LEDA/matrix.h"
#include "LEDA/list.h"
#include "LEDA/edge_map.h"
#include "LEDA/array.h"
#include "LEDA/ugraph.h"
#include "LEDA/shortest_path.h"
#include "LEDA/basic_graph_alg.h"
#include <math.h>
#include "CFortschritt.h"
#include "../Datenstruktur/CGuetemasse.h"

//#include "LEDA/templates/shortest_path.t"

/**
 * Diese Klasse bewertet einen vorgegebenen Linienplan auf der Grundlage
 * der Verkehrsbedarfs-Matrix, indem sie die Verkehrwuensche auf den
 * vorgegebenen Linienplan umlegt und dabei relevante Werte
 * mitprotokolliert. Ausserdem erzeugt sie den von der Fahrplangruppe
 * verlangten Fahrplangenerator. 
 */
class CBewertungsphase {
public:

    /**
     * Eingabe: CWegenetzplan (wenn es keine Reduktion gibt, dann entfaellt
     * dieser Schritt), CNetzplan, Verkehrsbedarfsmatrix 
     */
    CBewertungsphase(CNetzplan * meinNetzplan,  matrix * qzmatrix);

      ~CBewertungsphase();

    /**
     * Alle Felder der Verkehrsbedarfsmatrix muessen "abgearbeitet" und auf den
     * Linienplan umgelegt werden. Zu diesem Zweck wird zuerst die durch das
     * Feld der Matrix (a,b) bestimmte Verkehrsbeziehung zwischen a und b auf
     * dem kuerzesten Weg festgelegt. Diesmal wird aber im Gegensatz zu der
     * Umlage in der Aufbereitungsphase nicht der gesamte Wegenetzplan, sondern
     * nur diejenigen Kanten aus dem Wegenetzplan als Weg zugelassen, auf denen
     * auch laut Linienplan tatsaechlich ein Bus faehrt. Wenn es entlang dieses
     * kuerzesten Weges zu einem Umsteigevorgang zwischen zwei Buslinien kommt,
     * dann wird zu der Fahrtzeit fuer diese Strecke eine Umsteige- und
     * Wartezeit addiert, ansonsten nur die normale Fahrtzeit auf dieser
     * Strecke. Ausserdem werden alle Fahrtwuensche entlang dieser Strecke zu den
     * Fahrgaesten in den betroffenen Buslinien dazuaddiert. Was genau noch
     * alles mit protokolliert wird (umsteige-anzahl) ist noch nicht
     * festgelegt.
     * Ausgabe: CFahrplangraph
     */
      GRAPH<CFPlanGraphKnoten*,CFPlanGraphKante*>* umlegenVerkehrsbedarfAufLinienplan(CGuetemasse* guetemasse);




private:    


    /**
     * @supplierCardinality 1 
     */
    CNetzplan * netzplan;

    /**
     * @supplierCardinality 1 
     */
    CLinie * linie;
    GRAPH<CFPlanGraphKnoten*,CFPlanGraphKante*>* fahrplangraph;

    /**
     * @supplierCardinality 1 
     */
    CFPlanGraphKante * fPlanGraphKante;

    /**
     * @supplierCardinality 1 
     */
    CFPlanGraphKnoten * fPlanGraphKnoten;
    
    matrix* bedarfsmatrix;

    int entfernungsteiler; //Wert, durch den die Entfernung geteilt wird, um die Fahrtzeit zu erhalten
    // Default 250 m/min, entspricht einer Geschwindigkeit von 15 km/h

    int umstiegstrafe; // Anzahl von Minuten, die als Strafe fuers Umsteigen auf die Reisezeit aufgeschlagen werden

    array< list<int> > kantenArray;

    CFortschritt* fortschritt;

    void bauenFahrplangraph();

    double berechnenVerknuepfungsgrad();
};
#endif //BEWERTUNGSPHASE_H

