/* Generated by Together */

#ifndef LINIENPLAN_H
#define LINIENPLAN_H

#include "CBewertungsphase.h"
#include "CVerknuepfungsphase.h"
#include "CAufbereitungsphase.h"
#include "CNetzplan.h"
#include "CWegenetzKnoten.h"
#include "CWegenetzKante.h"
#include "Debug.h"
#include "../Programm/CLPSchnittstelle.h"
#include "CLPSchnittstelleTest.h"
#include "../Datenstruktur/CFPlanGraphKante.h"
#include "../Datenstruktur/CFPlanGraphKnoten.h"
#include <LEDA/matrix.h>
#include <LEDA/graphwin.h>
#include <LEDA/color.h>
#include <LEDA/window.h>

/**
 * Dies ist die Hauptklasse des Linienplan - Moduls.
 * Die komplette Kommunikation vom Programm - Modul
 * zum Linienplan - Modul geschieht ueber diese Klasse.
 * Desweiteren wird von dieser Klasse aus das Linienplan - Modul gesteuert.
 */
class CLinienplan {
public:

    /**
     * Der Konstruktor der Klasse CLinienplan. Er wird beim generieren der Klasse
     * aufgerufen und sorgt fuer die Initialisierung der Klasse.
     * 
     * Eingabe:
     * - Ein Zeiger auf ein Objekt der Klasse CLPSchnittstelle.
     *   Dieses Objekt stellt die Verbindung zum Modul Programm dar,
     *   an das auch spaeter das Ergebins geliefert werden soll.
     * 
     * Rueckgabe:
     * - nichts 
     */
    CLinienplan(CLPSchnittstelle* programmSchnittstelle);

    /**
     * Der Konstruktor fuer die Testumgebung
     */
    CLinienplan(CLPSchnittstelleTest* programmSchnittstelle);

    /**
     * Destruktor der Klasse CLinienplan. Das Objekt wird nicht mehr benoetigt und
     * muss nun alle von ihm erzeugten Objekte loeschen.
     * 
     * Eingabe:
     * - nichts
     * 
     * Rueckgabe:
     * - nichts 
     */
    ~CLinienplan();

    /**
     * Stoesst das Erzeugen eines Linienplans an.
     *
     * Eingabe:
     * - nichts (werden vom Programm abgeholt)
     *
     * Rueckgabe:
     * - nichts
     */
    void starten();

    /**
     * Dieses Linienplanobjekt hat schon einen Linienplan erzeugt, aber es soll ein neuer
     * Linienplan unter Beruecksichtigung veraenderter Parameter berechnet werden. Dabei
     * sind der Wegenetzgraph und die Verkehrsbedarfsmatrix von den Aenderungen
     * ausgenommen.
     *
     * Eingabe:
     * - nichts (werden vom Programm abgeholt)
     *
     * Rueckgabe:
     * - nichts
     */
    void rueckkoppeln();

    /**
     * Mit dieser Methode ist es moeglich zum naechst moeglichem Zeitpunkt
     * die Berechnung des Linienplans abzubrechen. Alle bisher getaetigten
     * Berechungen sind nicht nutzbar. Alle vom Linienplan erzeugten Objekte
     * werden geloescht.
     * 
     * Eingabe:
     * - nichts
     * 
     * Rueckgabe:
     * - nichts 
     */
    void stoppen();

private:    

    /**
     * Anhand diesen Typs erkennt metastart (s.u.) welchen Programmablauf es
     * nehmen soll:
     * startMitL:  Starten mit Pflichlinien
     * startOhneL: Starten ohne Pflichlinien
     *
     */
    enum Startart {startMitL, startOhneL, rueckMitL, rueckOhneL};

    /**
     * Enthaelt die Funktionalitaet der startmethoden. Diese uebergeben ihr Parameter
     * und ihren Namen (art) metastart.
     *
     * Eingabe:
     * - siehe start
     * 
     * Rueckgabe:
     * - nichts
     */

    void metastarten(Startart art, UGRAPH<CWegenetzKnoten*,CWegenetzKante*> * wegenetzgraph, list<CLinie*> * linienplan);

    GraphWin *zeigenWegenetzgraph(UGRAPH<CWegenetzKnoten*,CWegenetzKante*> *wegenetzgraph);

    GraphWin *zeigenNetzgraph();

    GraphWin *zeigenLinienplanGraph(GRAPH<CFPlanGraphKnoten*,CFPlanGraphKante*>* fertigerLinienplanGraph);

    /**
     * Zeiger auf einen Netzplan. Diese Datenstruktur enthaelt Daten
     * die von den meisten anderen Klassen benoetigt werden.
     * CLinienplan erzeugt eine Instanz, speichert in ihr die
     * uebergebenen Daten und uebergibt sie den anderen Klassen.
     * @supplierCardinality 1 
     */
    CNetzplan * netzplan;

    /**
     * @supplierCardinality 1 
     */
    CAufbereitungsphase * aufbereitungsphase;

    /**
     * @supplierCardinality 1 
     */
    CVerknuepfungsphase * verknuepfungsphase;

    /**
     * @supplierCardinality 1 
     */
    CBewertungsphase * bewertungsphase;

    /**
     * @supplierCardinality 1 
     */
    CLPSchnittstelle * lPSchnittstelle;
    CLPSchnittstelleTest * lPSchnittstelleTest;

    /**
     * Im Status wird beschrieben, mit welcher Aufgabe die Klasse sich gerade
     * beschaeftigt. Dieses ist notwendig, da wir durch das benutzen von Threads
     * unter Umstaenden oefter aufgerufen werden koennen oder bei einem Stoppen
     * wissen muessen, ob noch ein Prozess in unserem Paket laeuft.
     * unbenutzt:       Die Klasse wurde erstellt, aber sonst noch nicht benutzt
     * gestarten:       Die Methode starten wurde aufgerufen, ist aber noch nicht beendet
     * fertiggestartet: Die Methode starten oder rueckkopplung wurde beendet
     * rueckkopplung:   Die Methode rueckopplung wurde gestartet, ist aber noch nicht beendet
     * stop:            Jetzt sind wir gestoppt und wollen nichts mehr tun
     */

    enum Status {unbenutzt, gestartet, fertiggestartet, rueckkopplung, stop} status;

    /**
     * Die Verkehrsbedarfsmatrix muss fuer eine moegliche rueckkopplung gespeichert werden.
     */
    matrix* quellezielmatrix_lp;

};
#endif //LINIENPLAN_H
