/* Generated by Together */

#ifndef STARTVERKNUEPFUNG_H
#define STARTVERKNUEPFUNG_H

#include "CNetzplanKnoten.h"
#include "CNetzplanKante.h"
#include "CLinie.h"
#include "CNetzplan.h"
#include "CBasislinie.h"
#include "Debug.h"
/**
 * Diese Klasse erzeugt aus einem Wegenetzgraphen anhand
 * der Verknuepfungen "Basislinienwahl" und "eindeutige Linienwahl"
 * einen groben Linienplan, der durch CAbschlussverknuepfung
 * weiterbearbeitet wird.
 * 
 * Siehe Sonntag 108
 * 
 * Wenn ein Linienplan vorgegeben wurde, wird er in die interne Liste
 * der Linien (CLinienliste) eingefuegt. 
 */
class CStartverknuepfung {
public:

    /**
     * Der Konstruktor der Klasse. Er wird beim generieren der Klasse
     * aufgerufen und sorgt fuer die Initialisierung der Klasse.
     * 
     * Eingabe:
     * - Ein Zeiger auf ein Objekt der Klasse CNetzplan.
     *   Dieses Objekt enthaelt die fuer die Linienplanung noetigen
     *   Daten bereit.
     * 
     * Rueckgabe:
     * - nichts
     */
    CStartverknuepfung(CNetzplan* meinNetzplan);

    /**
     * Der Destruktor wird aufgerufen, wenn die Klasse geloescht werden soll. In
     * CStartverknuepfung werden die Basislinien wieder geloescht.
     */
    ~CStartverknuepfung();

    /**
     * Basislinien sind Grundelemente des Verknuepfungsansatzes von Sonntag
     * (siehe CBasislinie). Die Basislinienwahl erzeugt nun Basislinien, so dass
     * jede Kante des Graphen mindestens einmal enthalten ist, die
     * maximale Gesamtlinienlaenge nicht ueberschritten wird und
     * Basislinien mit hohem Strom bevorzugt werden.
     * 
     * Weitere Infos siehe Flussdiagramm Sonntag Seite 112 und Text
     * Sonntag ab Seite 108. 
     */
    void waehlenBasislinie(int algo);

    /**
     * Aus der Menge der Basislinen sollen diejenigen miteinander verknuepft
     * werden, die eine gemeinsame Endverknuepfung mit einfachem
     * Belegungsgrad besitzen.
     * 
     * Weitere Infos siehe Flussdiagramm Sonntag Seite 114 und Text Sonntag
     * ab Seite 113. 
     */
    void bildenEindeutigeLinienverknuepfung();

    /**
     * Wenn die Linienplanerstellung mit Pflichtlinien aufgerufen wird,
     * werden diese in die interenen Strukturen des Paketes Linienplan
     * eingefuegt. 
     */
    void erstellenLinien();

private:    

    /**
         * @supplierCardinality 0..*
         * @clientCardinality 1 
         */
//    list<CBasislinie*> * basislinien;

    /**
     * @supplierCardinality 1 
     */
//    list<CLinie*> * linien;

    /**
     * @supplierCardinality 1 
     */
    CNetzplan * netzplan;

    int aktGesamtlinienlaenge;

    void CStartverknuepfung::linieVerknBeideVorne(list<int>* hListe1, list<int>* hListe2);

    void CStartverknuepfung::linieVerknBeideHinten(list<int>* hListe1, list<int>* hListe2);

    void CStartverknuepfung::linieVerknConcat(list<int>* hListe1, list<int>* hListe2);

    list<CBasislinie*>* CStartverknuepfung::erstellenAllerBasislinien();

    list<CBasislinie*>* CStartverknuepfung::waehlenBasislinienMichael(list<CBasislinie*>* alleBasislinien);

    list<CBasislinie*>* CStartverknuepfung::waehlenBasislinienMichael2(list<CBasislinie*>* alleBasislinien);

    void CStartverknuepfung::umwandelnBasislinienInLinien(list<CBasislinie*>* gewaehlteBasislinien);
};

#endif //STARTVERKNUEPFUNG_H
