/* Generated by Together */

#ifndef CDATENBANK_H
#define CDATENBANK_H

#include "CGRASSSchnittstelle.h"
#include "CDBSchnittstelle.h"
#include "CImport.h"

#include "LEDA/list.h"
#include "LEDA/matrix.h"
#include "LEDA/integer_matrix.h"
#include "LEDA/tuple.h"
#include "LEDA/array.h"
#include "LEDA/h_array.h"
#include "LEDA/ugraph.h"
#include <LEDA/map2.h>

#include "CPunkt.h"
#include "CBevoelkerungsdichteElement.h"
#include "CFahrplanElement.h"
#include "CQuelleZielListeElement.h"
#include "CPendlerListeElement.h"
#include "CLinie.h"
#include "CFPlanGraphKnoten.h"
#include "CFPlanGraphKante.h"
#include "CWegenetzKnoten.h"
#include "CWegenetzKante.h"
#include "CGuetemasse.h"


//#include "CNetzplangenerator.h"

class CImport;


/**
 * Schnittstelle zur Datenbank 
 * verarbeitet Datenzugriffe und vermittelt diese auf die Datenquellen (DB, GRASS) .
 *
 *
 *
 * Das Vorgehen ist dabei Folgendes: Solange diese einzelnen Funktionen noch nicht implementiert
 * sind, werden automatisch die gleichnamigen Funktionen aus der Basisklasse verwendet, die dann
 * entsprechend ihre Testdaten absondern.
 * 
 * Immer wenn die entsprechenden Punkte implementiert wurden, braucht man sie hier nur zu berschreiben.
 *
 *
 *
 */
class CDatenbank {
private:    

    /** @link aggregation 
     * @supplierCardinality 1*/
    //CGRASSSchnittstelle lnkCGRASSSchnittstelle;

    /** @link aggregation 
     * @supplierCardinality 1*/
    //CDBSchnittstelle lnkCDBSchnittstelle;

    /** @link aggregation 
     * @supplierCardinality 1*/
    //CImport* lnkCImport;

protected:    

    /** @link aggregation 
     * @supplierCardinality 1*/
    CGRASSSchnittstelle* lnkCGRASSSchnittstelle;

    /** @link aggregation 
     * @supplierCardinality 1*/
    CDBSchnittstelle* lnkCDBSchnittstelle;

    /** @link aggregation 
     * @supplierCardinality 1*/
    CImport* lnkCImport;

    CGuetemasse* temporaere_guetemasse; // die Guetemasse werden hier zwischengepuffert

    GRAPH<CWegenetzKnoten*, CWegenetzKante*>* temporaerer_wegenetzgraph; // wird von den Funktionen zum Speichern der Pflichtlinien gebraucht

    /**
    * Testet einen Punkt, ob dieser in einem durch die Grenzpunkte definierten Rechteck liegt
    */
     virtual bool seinInRechteck(double xKoord_grenzeLO, double yKoord_grenzeLO,
	             double xKoord_grenzeRU, double yKoord_grenzeRU, CPunkt * testPunkt);

    /**
     * Diese Methode clippt alle Strassen, die in dem durch die Grenzpunkte definierten Rechteck liegen
     */
     virtual list<list<CPunkt* >*>* clippenStrassennetz(list<list<CPunkt* > * > * strassennetz, double grenzeLinksOben_X, double grenzeLinksOben_Y, double grenzeRechtsUnten_X, double grenzeRechtsUnten_Y );

     // die folgenden Werte werden hier ziwischengespeichert, damit sie die GUI abfragen kann
     // ueber die Datenbank brauchen wir bei diesen Daten nicht gehen.
     list<CFahrplanElement*>* derFahrplan;
     double summeNetzbedingteWartezeiten, wartezeitAufwand, mindestWartezeitAufwand, gesamtWarteaufwand;
     int anzahlFahrzeuge;

     bool batchbetrieb;

public:

	CDatenbank(char*, bool bbetrieb=false);
	virtual ~CDatenbank();

        // -------------------- Allgemein: --------------------------------------------------------------------------
	// die folgende Funktion speichert die Guetemasse in der Datenbank.
        virtual void speichernGuetemasse(CGuetemasse * );

        // die folgende Funktion holt bereits gefuellte Guetemassklasse aus der Datenbank.
	virtual CGuetemasse* holenGuetemasse();



	
        // -------------------- Haltestellen: -----------------------------------------------------------------------
        virtual void speichernDaten(GRAPH<CWegenetzKnoten*, CWegenetzKante*>* , matrix* ); // Ergebnisse des Haltestellenmoduls
	// speichert die vom Benutzer eingegebenen Startwerte  
	virtual void speichernHaltestellenModulParameter(int aufruf, int algorithmus, int zufallsgeneratorInitialisierung,
							double minRadius, double maxRadius, double verbindungsRadiusStadt,
							double verbindungsRadiusLand, int radiustyp, double haltestellenLage,
							double toleranzgrenze, int bevoelkerungsRaster, double koeffizientBevoelkerungsueberversorgung,
							double koeffizientBevoelkerungsunterversorgung, double koeffizientFlaechenueberversorgung,
							double koeffizientFlaechenunterversorgung, double koeffizientFahrtwunscheinbeziehung,
							double koeffizientStrassenanpassung,
							int AnzahlHaltestellen, bool AnzStrassenkarte,
							bool AnzBevoelkerungsversorgung, bool AnzFlaechenversorgung, bool Anzkomplettunterdruecken,
							int Strategie, int AnzahlEltern, int AnzahlKinder, int Generationen, 
							bool AbbruchGueteunterschreitung, double GrenzwGueteunterschreitung);
	// holt die Startwerte der Haltestellen:
	virtual void holenHaltestellenModulParameter(int aufruf, int& algorithmus, int& zufallsgeneratorInitialisierung,
							double& minRadius, double& maxRadius, double& verbindungsRadiusStadt,
							double& verbindungsRadiusLand, int& radiustyp, double& haltestellenLage,
							double& toleranzgrenze, int& bevoelkerungsRaster, double& koeffizientBevoelkerungsueberversorgung,
							double& koeffizientBevoelkerungsunterversorgung, double& koeffizientFlaechenueberversorgung,
							double& koeffizientFlaechenunterversorgung, double& koeffizientFahrtwunscheinbeziehung,
							double& koeffizientStrassenanpassung,
							int& AnzahlHaltestellen, bool& AnzStrassenkarte,
							bool& AnzBevoelkerungsversorgung, bool& AnzFlaechenversorgung, bool& Anzkomplettunterdruecken,
							int& Strategie, int& AnzahlEltern, int& AnzahlKinder, int& Generationen, 
							bool& AbbruchGueteunterschreitung, double& GrenzwGueteunterschreitung);
        /**
	 * hole Daten fuer die Haltestellen: 
         *
         * Parameter bedeuten dabei von links nach rechts:
         * @param Pflichthaltestellen
         * @param Strassennetz
         * @param Bevoelkerungsdichte
         * @param Quelle-Ziel-Matrix
         * @param Quelle-Ziel-Liste
         * @param Pendlermatrix
         * @param Pendlerliste
         */
        virtual void holenDaten(list<CPunkt*>*&, list<list<CPunkt*>*>*&, list<list<CPunkt*>*>*& wege,
		list<CBevoelkerungsdichteElement*>*&, matrix*&,
		list<CQuelleZielListeElement*>*&, matrix*&,
		list<CPendlerListeElement*>*&,
		list<list<CPunkt*>*>*&);



        // -------------------- Linienplan: -------------------------------------------------------------------------
        virtual void speichernDaten(GRAPH<CFPlanGraphKnoten*, CFPlanGraphKante*>*, list<CLinie*>*); // Ergebnisse des Linienplans
	// holt die Startwerte des Linienplans:
	virtual void holenLinienplanModulParameter(int aufruf, int& gesamtlaenge, int& maxLinienLaenge, 
							int& maxLinienAnzahl, double& linienplanID,
							int& geschwindigkeit, int& algorithmus, int& anzeigegraphen,
							bool& EindeutigeLinienverkn, bool& MehrdeutigeLinienverkn, 
							bool& Aufbruchsverknuepfung, bool& Linienendverknuepfung,
							bool& Basislinienwahl);
	// speichert die vom Benutzer eingegebenen Startwerte:
	virtual void speichernLinienplanModulParameter(int aufruf, int gesamtlaenge, int maxLinienLaenge, 
							int maxLinienAnzahl, double linienplanID,
							int geschwindigkeit=15, int algorithmus=0, int anzeigegraphen=0,
							bool EindeutigeLinienverkn=false, bool MehrdeutigeLinienverkn=false, 
							bool Aufbruchsverknuepfung=false, bool Linienendverknuepfung=false,
							bool Basislinienwahl=false);
        /** hole Daten fuer den Linienplan: 
         *
         * Parameter bedeuten dabei von links nach rechts:
         * QuelleZielMatrix
         * vorgegebene Linien
	 * Wegenetzgraph
         */
	virtual void holenDaten(matrix*& , list<CLinie*>*& , UGRAPH<CWegenetzKnoten*,CWegenetzKante*>*&);
	virtual matrix* holenLinienplanQuelleZielMatrix();
	virtual UGRAPH<CWegenetzKnoten*,CWegenetzKante*>* holenLinienplanWegenetzgraph();

        virtual void speichernPflichtlinienDatenbank(list<CLinie*>* pflichtlinien); // speichert alle Pflichtlinien in Oracle 
	virtual void speichernPflichtlinienGRASS(list<CLinie*>* pflichtlinien); // speichert alle Pflichtlinien in GRASS
	virtual void speichernEinePflichtlinie(list<int>* pflichtlinie, int id);  // speichert genau eine Pflichtlinie mit der ID "id" in GRASS
	virtual list<CLinie*>* holenPflichtlinien(); // holt die Pflichtlinien aus Oracle
	
	


        // -------------------- Fahrplan: ---------------------------------------------------------------------------
        virtual void speichernDaten(array<list<three_tuple<int,int,int>* >* >*); // Ergebnisrueckgabe des Fahrplns: Zusammenhangskomponenten
	virtual void speichernDaten(list<CFahrplanElement*>*); // Ergebnisrueckgabe des Fahrplans: Fahrplan
	// Der Starwert: der von dem Linienplan errechnete Fahrplan:
        virtual GRAPH<CFPlanGraphKnoten*, CFPlanGraphKante*>* fahrplanLinienplan();
        /** holt Mindestumsteigezeit
	 * @return Zeiger auf das LEDA-h_array mit der Mindestumsteigezeit
	 */
	virtual h_array<int,list<three_tuple<int,int,int>* >*>* fahrplanMindestumsteigezeit(); 
	/** speichert die Mindestumsteigezeit
	 */
	virtual void speichernFahrplanMindestumsteigezeit(h_array<int,list<three_tuple<int,int,int>* >*>*);
	/** holt die Mindesthaltezeit
	 * @return Zeiger auf das LEDA-h_array mit den Mindesthaltezeiten
	 */
	virtual h_array<int,int>* fahrplanMindesthaltezeit(); 
        /** speichert die Mindestumsteigezeit
	 */
        virtual void speichernFahrplanMindesthaltezeiten(h_array<int,int>*);
	/** liefert den direkten Anschluss
	 * @return Zeiger auf die Liste direkter Anschluesse fuer die Testdaten
	 */
	virtual list<four_tuple<int,int,int,int>* >* fahrplanDirekterAnschluss();
	/** liefert die maximale Fahrzeuganzahl
	 * @return Maximale Fahrzeuganzahl
	 */
	virtual int fahrplanMaximaleFahrzeuganzahl(); 
	/** holt die Abfahrtszeiten
	 * @return Zeiger auf das LEDA-Array mit den Abfahrtszeiten
	 */
	virtual array<three_tuple<int,int,int>* >* fahrplanAbfahrtszeiten(); 
        /** holt das Fahrplanintervall
	 * @return Zeiger auf das Fahrplanintervall
	 */
	virtual four_tuple<int,int,int,int>* fahrplanIntervall(); 
	/** speichert die Wartezeiten in der Datenbanktabelle Ausgangsloesung
	 */
        virtual void fahrplanerstellungWartezeiten (double input_summeNetzbedingteWartezeiten, double input_wartezeitAufwand,
						 double input_mindestWartezeitAufwand, double input_gesamtWarteaufwand);
	/** speichert die Anzahl benoetigter Fahrzeuge in der Datenbanktabelle Ausgangsloesung
	 */
	virtual void fahrplanerstellungAnzahlBenoetigterFahrzeuge (int input_fahrzeugAnzahl);
	/** diese Funktion holt die Ausgangsloesung (= Guetemasse des Fahrplans)
	 */
	virtual void holenAusgangsloesung(double& input_summeNetzbedingteWartezeiten, double& input_wartezeitAufwand,
	   					double& input_mindestWartezeitAufwand, double& input_gesamtWarteaufwand);
	/** diese Funktion holt die Anzahl der benoetigten Fahrzeuge:
	 */
	virtual void holenAnzahlFahrzeuge(int& input_anzahlFahrzeuge);
	/**
	  * Diese Methode gibt eine Liste des Fahrplans aus
	  * Die Liste ist geordnet nach: HaltestellenID, LinienID, AbfahrtszeitStd, AbfahrtszeitMin
	  */
	virtual list<CFahrplanElement*>* holenFahrplanHaltestellen();
	/**
	  * Diese Methode gibt eine Liste des Fahrplans aus
	  * Die Liste ist geordnet nach: LinienID, HaltestellenID, AbfahrtszeitStd, AbfahrtszeitMin
	  */
	virtual list<CFahrplanElement*>* holenFahrplanLinien();
        /* speichert alle Parameter von der GUI
         */
	virtual void speichernFahrplanModulParameter(int maxfahrzeuganzahl, int allgemeinertakt, int mindesthaltezeit, int mindestumsteigezeit);
        /* holt alle Parameter aus der Datenbank
        */
        virtual void holenFahrplanModulParameter(int& maxfahrzeuganzahl, int& allgemeinertakt, int& mindesthaltezeit, int& mindestumsteigezeit);
	

        // -------------------- GUI: --------------------------------------------------------------------------------
	/**
	 * Folgende Funktionen dienen der GUI:
         * sie kann auf diese Weise Daten in die Datenbank schieben und
	 * auch wieder herausholen.
	*/
	
	// speichert die von der GUI importierte Bevoelkerungsdichte
	virtual void speichernBevoelkerungsDichte(list<CBevoelkerungsdichteElement*>*);
	// Speichert die von der GUI importierte Quelle-Ziel-Matrix plus die dazugehoerige Liste
	virtual void speichernUrsprungsQZMatrix(matrix* quelleZielMatrix, list<CQuelleZielListeElement*>* quelleZielListe);
	// Speichert die von der GUI importierte Pendlermatrix plus die dazugehoerige Liste
	virtual void speichernPendlermatrix(matrix* pendlerMatrix, list<CPendlerListeElement*>* pendlerListe);
        
	// speichere die vorgegebenen Linien:
	virtual void speichernVorgegebeneLinien();

        // Holt die Haltestellen-ID zu einem Knoten:
	virtual int holenKnotenID(double, double);
	// Holt die Haltestellen-Koordinaten zu einem Knoten: (liefert NULL, wenn ID nicht vorhanden) 
	virtual CPunkt* holenKnotenKoordinaten(int);
	// Holt die LinienID zu einer Haltestellen-ID
 	virtual list<int>* holenLinienID(int);
	// liefert zu einer Haltestellen ID alle anderen Haltestellen, die von der uebergebenen
	// Haltestelle im Wegenetzgraphen erreichbar sind
	virtual list<int>* holenMoeglicheZielHaltestellen(int);
	// holt eine Zusammenhangskomponente zu einer Haltestelle und zwei Linien
	virtual int holenZusammenhangsKomponentenID(int,int); //Parameter: Haltestelle, Linie1
	// holt alle eingehenden Linien in eine Haltestelle (als ID uebergeben)
	virtual list<int>* holenEingehendeLinien(int); 
	// hole alle ausgehenden Linien aus einer Haltestelle (als ID uebergeben)
  	virtual list<int>* holenAusgehendeLinien(int);
	// holt alle Haltestellen, ueber die die Linie mit der linienid fuehrt
	virtual list<int>* holenLinienHaltestellen(int linienid); 	
	// diese Funktion speichert einen Teil des Strassennetzes in einem eigenen Layer
	// hiermit wird der Strassenplan in den einzelnen Testfaellen zu Anzeige abgespeichert
	// wird u.a. von der GUI aufgerufen
        virtual void speichernAusschnittStrassennetz();
	// loesche die Buslinie mit der ID id:
	virtual void loeschenLinie(int id);
	// gibt die kuerzeste Verbindung zw. zwei HS aus; WICHTIG: prueft nicht, ob FP schon durchgelaufen ist!
	virtual CReisebewertung* routenplanung(int starths, int endhs);
	
	
	// -------------------- Versionierung: ----------------------------------------------------------------------
	virtual void setzenAktuelleProjektID(int id); // hiermit wird ein Projekt ausgewaehlt
  	virtual int holenAktuelleProjektID(); // gibt die ID des aktuellen Projektes zurueck
	virtual void setztenPhase(int phase); // speichert den Schritt ab, indem sich der Benutzer im aktuellen Projekt gerade befindet
	virtual int holenPhase();	// liefert den Schritt, indem sich der Benutzer gerade befindet.
	virtual int anlegenNeuesProjekt(int id, char* beschreibung, int phase); // legt ein neues Projekt an, liefert die gespeicherte ID zurueck
	virtual void holenProjektInformationen(int id, char*& beschreibung, int& phase); // holt die Informationen des Projektes mit der ID "id"
	virtual list<two_tuple<int,char*>*>* holenAlleProjektInformationen(); // liefert die Informationen aller Projekte zurueck (gibt die ganze Projekttabelle aus)
	virtual int holenAnzahlProjekte(); // gibt die Anzahl der in Oracle gespeicherten Projekte zurueck
	virtual int holenErsteFreieProjektID(); // holt die erste Projekt-ID, die noch von keinem Projekt belegt wurde
	virtual void loeschenProjekt(int id); // loescht das Projekt mit der ID "id"
	virtual int kopierenProjekt(int quellid, int zielid, char* beschreibung=0); // kopiert das Projekt mit der ID "quellid" in das Projekt mit der ID "zielid", liefert die ID des Zielprojektes zuerueck
		// Datenaustausch-User Funktionen (mit ihnen koennen Daten Gruppenuebergreifend ausgetauscht werden)
  	virtual int anlegenNeuesProjektPG369(int id, char* beschreibung, int phase); // legt ein neues Projekt im Datenaustausch-Users an, liefert die gespeicherte ID zurueck
	virtual void holenProjektInformationenPG369(int id, char*& beschreibung, int& phase); // holt die Informationen des Projektes mit der ID "id" des Datenaustausch-Users
  	virtual list<two_tuple<int,char*>*>* holenAlleProjektInformationenPG369(); // liefert die Informationen aller Projekte des Datenaustausch-Users zurueck (gibt die ganze Projekttabelle aus)
	virtual int holenErsteFreieProjektIDPG369(); // holt die erste Projekt-ID, die noch von keinem Projekt belegt wurde vom Datenaustausch-User
  	virtual void loeschenProjektPG369(int id); // loescht das Projekt mit der ID "id" im Datenaustausch-Users
  	virtual int kopierenProjektVonPG369(int quellid, int zielid, char* beschreibung=0); // kopiert das Projekt mit der ID "quellid" des Datenaustausch-Users pg369 in das Projekt mit der ID "zielid", liefert die ID des Zielprojektes zuerueck
  	virtual int kopierenProjektNachPG369(int quellid, int zielid, char* beschreibung=0); // kopiert das Projekt mit der ID "quellid" in das Projekt mit der ID "zielid" des Datenaustauschusers pg369, liefert die ID des Zielprojektes zuerueck


	virtual void speichernPflichthaltestellen(); // speichert die Koordinaten der Pflichthaltestellen in Oracle  
	virtual void holenPflichthaltestellen(); // holt die Pflichthaltestellen wieder aus der DB und schreibt sie in GRASS
	virtual void speichernAusschnitt(double links, double oben, double rechts, double unten); // speichert die Abmessungen des zu beruecksichtigenden Ausschnitts
	virtual void holenAusschnitt(double& links, double& oben, double& rechts, double& unten); // holt die Abmessungen des zu beruecksichtigendesn Ausschnitts

	virtual void aktuallisierenAnzeigen(int phase); // nach dem Wechsel eines Projektes stellt dies wieder alle Anzeigen in GRASS her


};
#endif //CDATENBANK_H
