/* Generated by Together */

// Implementierung: Tim Schuermann

#ifndef CFPSCHNITTSTELLE_H
#define CFPSCHNITTSTELLE_H
#include "CSchnittstelleMitGUI.h"
#include "LEDA/list.h"
#include "LEDA/matrix.h"
#include "LEDA/integer_matrix.h"
#include "LEDA/tuple.h"
#include "LEDA/array.h"
#include "LEDA/h_array.h"
#include "LEDA/ugraph.h"
#include "../Datenstruktur/CFahrplanElement.h"
#include "../Datenstruktur/CGuetemasse.h"
#include "CGUI.h"

class Fahrplan;

/**
 * Diese Klasse bildet die Schnittstelle zwischen dem Fahrplanmodul, der GUI und der Datenbank. Sie leitet Anforderungen und Daten zwischen den anfragenden und den antwortenden Klassen weiter. Diese Klasse reagiert nur auf Anfrage des Fahrplanmoduls und arbeitet nicht selbststaendig. 
 */
class CFPSchnittstelle : public CSchnittstelleMitGUI {
public:    

   CFPSchnittstelle(CDatenbank* datenbank, CGUI* gui);


   // die folgende Funktion schieben die vom Fahrplanmodul
   // berechneten Guetemasse in die Datenbank
   void speichernGuetemasse(CGuetemasse * );

   // die folgende Funktion holt eine bereits teilweise gefuellte
   // Guetemassklasse, die der Fahrplan ausfuellt
   CGuetemasse* holenGuetemasse();


// ----------------------------- Werteanforderungsfunktionen: ----------------------------------------------------------------------

   /**
   * Diese Methode liefert in einer Liste alle tupel zurueck die eine feste Bindung haben
   * sollen. Dabei wird die ZusammenhangskomponentenID, die ID der Linie 1, die ID der 2. Linie und die Umsteigehaltestellen ID uebergeben.
   */
  // GUI
  list<four_tuple<int,int,int,int>* >* fahrplanerstellungDirekterAnschluss();

  /**
   * Hier soll die Maximale Fahrzeuganzahl uebergeben werden.
   */
  // GUI
  int fahrplanerstellungMaximaleFahrzeuganzahl();

  /**
   * Bei dieser Methode wird der Allgemeine Takt (Minuten - Wert) uebergeben.
   */
  // GUI
  int fahrplanerstellungAllgemeinerTakt();
    
          
  /**
   * Diese Methode erhaelt die Bewertung der Ausgangsloesung
   *
   *
   *!!!!!!!!!!!!!!!!!!!!! NOCH BEARBEITEN !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   *
   */
  // geht in die Datenbank
  //  void fahrplanerstellungAusgangsloesung(int,int,int,int,int);
    
  /**
   * Diese Methode erhaelt die Zusammenhangskomponente
   * Als Parameter wird ein Array der Zusammenhangskomponenten uebergeben, in dem wiederum eine Liste von three_tuple steht
   * Die Werte sind dabei 1. HaltestellenID, 2. HaltestellenID und LinienID
   */
  //geht in die Datenbank
  void fahrplanerstellungZusammenhangskomponenten(array<list<three_tuple<int,int,int>* >* >* );

  /**
   * Die Methode liefert ein Array von four_tuple zurueck, welche folgende Werte haben:
   *  - HaltestellenID
   *  - Liniennr
   *  - minute (Reicht aus, da in dem Intervall ein fester Takt gilt);
   *
   * Das Array muss dabei die gleiche Reihenfolge haben, wie das Array der Zusammenhangskomponenten
   */
  // GUI
  array<three_tuple<int,int,int>* >* fahrplanerstellungAbfahrtszeiten();
    
  /**
   * Methode wird zum Aktivieren/Deaktivieren von Menueintraegen benoetigt
   */
  // GUI
  void fahrplanerstellungMenueaktivierung();
    
  /**
   * Diese Methode uebergibt die Mindesthaltezeit fuer jede Haltestelle
   * Die Daten werden in einem Hash_Array zurueckgegeben, indem der erste Wert
   * die HaltestellenID und der zweite Wert die Haltezeit an dieser Haltestelle
   * zurueckgegeben werden.
   */
  // GUI
  h_array<int,int>* fahrplanerstellungMindesthaltezeit();
    
  /**
   * Diese Methode uebergibt die Mindestumsteigezeit fuer jede Kombination von
   * Haltestelle und Liniennummern.
   * Der Rueckgabewert baut sich wie folgt auf:
   *  Das erste Element im Hash_Array ist die HaltestellenID bei dem zweiten Element handelt es sich
   *  um eine Liste bestehend aus Liniennummer1, Liniennummer2 und die Mindestumsteigezeit zwischen diesen Linien.
   */
  // GUI
  h_array<int,list<three_tuple<int,int,int>* >*>* fahrplanerstellungMindestumsteigezeit();
    
  /**
   * WIRD ZURZEIT NOCH NICHT BENOETIGT !!!!!!!!!
   */
  //  void fahrplanerstellungNeuerLinienplan();

   /**
   * Bei dieser Methode wird das Intervall zurueckgegeben, fuer das der Fahrplan gueltig sein soll
   * Dabei gilt folgende Reihenfolge bei den int - Werten :
   * Stunde Von, Minute Von, Stunde Bis, Minute Bis
   */
  // GUI
  four_tuple<int,int,int,int>* fahrplanerstellungFahrplanIntervall();	


  /**
   * Setzen der Liste von festen Verbindungen
   */
  //GUI
  list<four_tuple<int, int, int, int>* >* fahrplanerstellungFesteVerbindungen();

  /**
   * holen der Fahrplan Modul Parameter, die ueber die GUI eingegeben werden
   */
  void holenFahrplanModulParameter(int& maxfahrzeuganzahl, int& allgemeinertakt, int& mindesthaltezeit, int& mindestumsteigezeit);


// ----------------------------- Werterueckgabefunktionen: -------------------------------------------------------------------------

   /**
   * Diese Methode erhaelt den erstellten Fahrplan zur Speicherung in die Datenbank.
   */
  // geht in Datenbank
  void fahrplanerstellungErgebnisrueckgabe(list<CFahrplanElement*>*);

  /**
   * Uebergabe der Ergebnisse der Fahrplanberechnung
   * hinsichtlich der Wartezeiten
   * 1. Summe der netzbedingten Wartezeiten
   * 2. Wartezeit-Aufwand
   * 3. Mindestwartezeit-Aufwand
   * 4. Gesamt-Wartezeit-Aufwand
   */
  void fahrplanerstellungWartezeiten (double, double, double, double);
        
  /**
   * Uebergabe der benoetigten Fahrzeug-Anzahl
   */
  void fahrplanerstellungAnzahlBenoetigterFahrzeuge (int);	

private:    

    /**
     * @supplierCardinality 1
     * @clientCardinality 1
     * @undirected 
     */
};
#endif //CFPSCHNITTSTELLE_H