/* Generated by Together */

#ifndef CGUI_H
#define CGUI_H

//#include "CGUI.h" Wieso hat Together das angelegt?
#include "CKontrolle.h"
#include "LEDA/list.h"
#include "LEDA/matrix.h"
#include "LEDA/string.h"
#include "CQuelleZielListeElement.h"

class CGUISchnittstelle;

/**
 * stellt die grafische Oberflaeche dar und reicht Benutzereingaben weiter an 
 */
class CGUI {
private:    

	 /**
     * @supplierCardinality 1 
     */
    CKontrolle* lnkCKontrolle;
    
    /**
     * @link aggregation
     * @supplierCardinality 1 
     */
    CGUISchnittstelle * lnkCGUISchnittstelle;
    
public:

    /** 
     * Konstruktor legt ein neues Kontroll-Objekt an, das seinerseits wiederum
     * die Schnittstellen erzeugt (vgl. Konstruktor von CKontrolle in CKontrolle.h) 
     * Dem Kontrollmodul muss dabei ein Zeiger auf das GUI-Objekt selbst (this)
     * uebergeben werden. Dieses reicht den Zeiger an die Schnittstellen
     * weiter
     *
     */
    CGUI(int argc=0, char ** argv=0);
        
    /** Destruktor dient zum Aufraeumen, bevor das Programm terminiert
     */
    ~CGUI();
    
    /**
     * Diese Funktion startet die Ereigniswarteschleife. Wird beim Programmstart benoetigt:
     * die main-Funktion erzeugt ein Objekt GUI. Anschliessend ruft die main-Funktion diese Funktion
     * laufe() auf, mit der die Ereigniswarteschleife gestartet wird. Sobald diese Funktion verlassen
     * wird, Terminiert das Programm. Entspricht der sonst ueblichen "Run" Funktion
     */
    void laufen();
    
    

    // FAHRPLAN-METHODEN:	

    /** 
     * abfrageAusgangsloesung zeigt die vom Fahrplanmodul generierten
     * Ausgangsloesungen dem Benutzer an und fordert diesen auf, sich fuer
     * eine dieser Ausgangsloesungen zu entscheiden. Die vom Benutzer
     * gewaehlte Loesung wird dann an das Fahrplanmodul zurueckgeliefert
     * (Rueckgabewert).
     */
    void abfragenAusgangsloesung();

    /**
     * direkterAnschluss wird vom Farhplanmodul aufgerufen.
     * fragt den Benutzer nach den direkten Anschluessen
     * four_tuple (= ein Anschluss) hat die folgende Werte:
     * ZusammenhangskomponentenID, die ID der Linie 1, die ID der 2. Linie und die Umsteigehaltestellen ID
     */
    list<four_tuple<int,int,int,int>* >* direkterAnschluss();
	
    /**
     * maximaleFahrzeuganzahl wird vom Farhplanmodul aufgerufen.
     * fragt den Benutzer nach der maximal verfuegabren Fahrzeuganzahl
     */
    int maximaleFahrzeuganzahl();

    /**
     * allgemeinerTakt wird vom Farhplanmodul aufgerufen.
     * fragt den Benutzer nach dem allgemeinen Takt
     */
    int allgemeinerTakt();

     /**
     * abfahrtszeiten wird vom Farhplanmodul aufgerufen.
     * fragt den Benutzer nach den Abfahrtszeiten
     * Werte der Tupel dabei:
     *  - HaltestellenID
     *  - Liniennr
     *  - minute (Reicht aus, da in dem Intervall ein fester Takt gilt);
     */
    array<three_tuple<int,int,int>* >* abfahrtszeiten();

    /**
     * mindesthaltezeit wird vom Farhplanmodul aufgerufen.
     * fragt den Benutzer nach den mindesthaltezeit
     * Die Daten werden in einem Hash_Array zurueckgegeben, indem der erste Wert
     * die HaltestellenID und der zweite Wert die Haltezeit an dieser Haltestelle
     * zurueckgegeben werden.
     */
     h_array<int,int>* mindesthaltezeit();

    /**
     * mindestumsteigezeit wird vom Farhplanmodul aufgerufen.
     * fragt den Benutzer nach den mindestumsteigezeiten
     * Der Rueckgabewert baut sich wie folgt auf:
     *  Das erste Element im Hash_Array ist die HaltestellenID bei dem zweiten Element handelt es sich
     *  um eine Liste bestehend aus Liniennummer1, Liniennummer2 und die Mindestumsteigezeit zwischen diesen Linien.
     */
    h_array<int,list<three_tuple<int,int,int>* >*>* mindestumsteigezeit();

    /**
     * fahrplanintervall wird vom Farhplanmodul aufgerufen.
     * fragt den Benutzer nach den fahrplanintervallen
     * Dabei gilt folgende Reihenfolge bei den int - Werten :
     * Stunde Von, Minute Von, Stunde Bis, Minute Bis
     */
    four_tuple<int,int,int,int>* fahrplanIntervall();

    /** Aktiviert bzw. deaktiviert einen entsprechenden Menuepunkt
     */
    void aktivierenMenue();

    



   /** Die folgenen Funktionen dienen zur Ausgabe, der von den Modulen
     * uebergebenen Texte
     */
    void ausgebenMeldung(char* );
    void ausgebenLaufzeitmeldung(char* );

    /** 
     * Die folgenden Funktionen dienen dazu, der GUI mitzuteilen,
     * dass und wie das entsprechende Modul seine Berechnungen beendet hat.
     * Die GUI kann daraufhin seine Anzeige aktuallisieren
     */
    void beendenHaltestellenberechnung(int,char* s=0);
    void beendenLinienplanberechnung(int,char* s=0);
    void beendenFahrplaninit(int,char* s=0);
    void beendenFahrplanberechnung(int,char* s=0);
    void beendenFahrplanfertigstellen(int,char* s=0);
    void beendenFahrplanausgebenBewertung(int,char* s=0);
	 
	 /** 
	  * Hole Daten fuer die Haltestellen
	  * QuelleZielMatrix und QuelleZielListe
	  */
	 void holenDaten(matrix*&, list<CQuelleZielListeElement*>*&);
	 
	 /** 
	  * Hole Daten fuer die Haltestellen
	  * PendlerMatrix und PendlerListe
	  */
	 void holenDaten(matrix*&, list<CPendlerListeElement*>*&);
	 
};
#endif //CGUI_H
