/* Generated by Together */

// Implementierung: Tim Schuermann

#ifndef CKONTROLLE_H
#define CKONTROLLE_H

#include <pthread.h>
#include <signal.h>
#include <iostream>
#include <new>	// Exceptions von new werden hier definiert

#include "CHSSchnittstelle.h"
#include "CLPSchnittstelle.h"
#include "CFPSchnittstelle.h"
#include "CGUISchnittstelle.h"
#include "CDatenbank.h"
#include "CTestklasse0.h"
#include "CTestklasse1.h"
#include "CTestklasse2.h"
#include "CTestklasse3.h"
#include "CTestklasse4.h"
#include "CTestklasse5.h"

#include "../Fahrplan/CFahrplan.h"
#include "../Haltestelle/CHaltestelle.h"
#include "../Linienplan/CLinienplan.h"
#include "../Datenstruktur/Fehler.h"

class CLinienplan;
class CHaltestelle;
class CGUI;
class CFPSchnittstelle;
class CFahrplan;


/**
 * verwaltet die Instanzen der einzelnen Module (Erschaffung, Init, Aufruf, Zerstoerung)
 * verwaltet die Datenfluesse 
 */
class CKontrolle {

protected:    
    /** @link aggregation
     * @supplierCardinality 1*/
    CDatenbank* lnkCDatenbank;

	 CGUI* lnkCGUI;

    int isttestversion; // bei 1 wird Testdatengenerator genommen, bei 0 wird die Datenbank genommen

protected:   
    /** @link aggregation 
     * @supplierCardinality 1*/
    CGUISchnittstelle* lnkCGUISchnittstelle;    

    /** @link aggregation 
     * @supplierCardinality 1*/
    //CHSSchnittstelle* lnkCHSSchnittstelle;

    /** @link aggregation 
     * @supplierCardinality 1*/
    //CLPSchnittstelle* lnkCLPSchnittstelle;

    /** @link aggregation 
     * @supplierCardinality 1*/
    //CFPSchnittstelle* lnkCFPSchnittstelle;

    /**
     * @supplierCardinality 1..* 
     */
    //CHaltestelle* lnkHS;

    /**
     * @supplierCardinality 1..* 
     */
    //CFahrplan * lnkCFahrplan;

    /**
     * @supplierCardinality 1..* 
     */
    //CLinienplan * lnkCLinienplan;


	
     /**
      * Schnittstelle und Modul werden in einer Struktur gespeichert,
      * damit es a) uebersichtlicher wird und b) damit der Datenaustausch
      * ueber die Threads klappt (die Thread-Funktion erlaubt nur ein
      * Argument
      * 
      */
     	struct haltestelleStruct{
		CHSSchnittstelle* lnkCHSSchnittstelle;
		CHaltestelle* lnkHS; // damit kommen wir vom Programmmodul an das Objekt im Thread heran
		CGUI* lnkCGUI; // die GUI um zu sagen: ich bin fertig
	} lnkHaltestelle;

	struct linienplanStruct {
         	CLPSchnittstelle* lnkCLPSchnittstelle;
		CLinienplan* lnkLP;  // damit kommen wir vom Programmmodul im Thread heran
		CGUI* lnkCGUI; // die GUI um zu sagen: ich bin fertig
		// Daten, die beim Starten uebergeben werden muessen:
//		UGRAPH<CWegenetzKnoten*,CWegenetzKante*>* graph;
//		list<CLinie*>* vorgegebeneLinien;
//		matrix* verkehrsbedarfsmatrix;
//		int gesamtlaenge;
//		int maxLinienLaenge; 
//		int maxLinienAnzahl;
	} lnkLinienplan;

	struct fahrplanStruct {
		CFPSchnittstelle* lnkCFPSchnittstelle;
		CFahrplan* lnkFP;  // Damit kommen wir vom Programmmodul an das Objekt im Thread heran
		// Daten, die beim Starten uebergeben werden:
		GRAPH<CFPlanGraphKnoten*, CFPlanGraphKante*>* linienplan;
		int zusammenhangskomponente; // dieser Parameter wird in einigen Funktionen des Fahrplans benoetigt
		int entscheidungsbaumtiefe;
		CGUI* lnkCGUI; // die GUI um zu sagen: ich bin fertig
	} lnkFahrplan;



     /* Die Folgenden Attribute werden fuer die Threads benoetigt (s.u.)*/	
     int res;	
     pthread_t haltestellen_thread;
     pthread_t linienplan_thread;
     pthread_t fahrplan_thread;
     void *thread_result;

    /**
     * Diese folgenden drei Funktionen dienen als Hilffunktionen fuer die
     * Threadausfuehrung. Das Vorgehen ist dabei Folgendermassen:
     * Die start-Funktionen rufen eine Betriebssystemfunktion zum starten
     * eines Threads auf. DIese BS-Funktion erwatet als Uebergabeparamter
     * eine Funktion, die im Folgenden als eigenstaendiger Thread ausgefuehrt
     * wird. Hier uebernehmen die drei Folgenden Funktionen diese Aufgabe.
     * Innerhalb dieser Funktionen werden die Objekte mit ihren jeweiligen
     * Funktionen angelegt und ausgefuehrt.
     *
     * Als Argumente werden die Schnittstellen uebergeben, die dann an die
     * einzelnen Module weiterreichen
     * 
     */

      static void* threadHaltestelle(void*);
      static void* threadLinienplan(void* );
      static void* threadFahrplan(void* );
      static void* threadFahrplanBerechnenHeuristischeFahrplanerstellung(void*);
      static void* threadFahrplanAusgebenFahrplan(void*);
      static void* threadFahrplanSukzessiveLoesungsverbesserung(void*);
      static void* threadFahrplanEntscheidungsbaumverfahren(void*);
      static void* threadFahrplanAusgebenBewertung(void*);

public:

    /**
     * Konstruktor leitet die Erzeugung der entsprechenden Schnittstellen ein,
     * von denen ja immer nur ein Objekt zur Laufzeit exisitieren darf.
     * Die GUI erzeugt ein CKontroll-Objekt, das seinerseits die Schnittstellen
     * und die angeschlossenen Module verwaltet. Die GUI bekommt aber ebenfalls
     * eine Schnittstelle, um auf die Datenbank zugreifen zu koennen.
     * Die vom Kontrollobjekt erzeugte GUISchnittstelle wird dabei explizit 
     * durch den uebergebenen Zeiger an das erzeugende GUI-Objekt 
     * 
     * GUI muss mit uebergeben werden, damit diese an die Schnittstellen
     * weitergereicht werden kann
     */
    CKontrolle(CGUISchnittstelle*&, CGUI*, int, char*);

    /** Destruktor dient zum Aufraeumen, bevor das Programm terminiert
     */
    ~CKontrolle();


    /**
     * die folgenden Funktionen dienen
     * zum Starten des entsprechenden Moduls
    */
    void startenHaltestellenModul();
    void startenLinienplanModul();
    void startenFahrplanModul();
    // Rueckkopplung Fahrplan
    void startenFahrplanBerechnenHeuristischeFahrplanerstellung(int zusammenhangskomponente);
    void startenFahrplanAusgebenFahrplan();
    void startenFahrplanSukzessiveLoesungsverbesserung(int zusammenhangskomponente);
    void startenFahrplanEntscheidungsbaumverfahren(int zusammenhangskomponente, int entscheidungsbaumtiefe=10);
    void startenFahrplanAusgebenBewertung();

    /**
     * die Folgenden Funktionen dienen zum Anhalten und anschliessendem
     * zerstoeren der entsprechenden Module
     */

    void stoppenHaltestellenModul(); // veranlasst das Modul anzuhalten
    void zerstoerenHaltestellenModul(); // zerstoert das Objekt
    void stoppenLinienplanModul();
    void zerstoerenLinienplanModul();

    void stoppenFahrplanModul();
    void stoppenFahrplanBerechnenHeuristischeFahrplanerstellung();
    void stoppenFahrplanAusgebenFahrplan();
    void stoppenFahrplanSukzessiveLoesungsverbesserung();
    void stoppenFahrplanEntscheidungsbaumverfahren();
    void stoppenFahrplanAusgebenBewertung();
    void zerstoerenFahrplanModul(); // zerstoert das Fahrplan-Objekt
    void ausgebenWartezeiten(); // veranlasst das Fahrplanmodul die Wartezeiten an die Schnittstelle auszugeben
    void ausgebenAnzahlFahrzeuge(); // veranlasst das Fahrplanmodul die Anzahl der benoetigten Fahrzeuge auszugeben

    /**
     * die Folgenden Funktionen dienen als Hilffunktionen fuer den Batchmodus
     * 
     */
    void setzenPhaseBatchModus(int phase); 

};
#endif //CKONTROLLE_H
